Package: TrenchR
Title: Tools for Microclimate and Biophysical Ecology
Description: Tools for translating environmental change into organismal response. Microclimate models to vertically scale weather station data to organismal heights. The biophysical modeling tools include both general models for heat flows and specific models to predict body temperatures for a variety of ectothermic taxa. Additional functions model and temporally partition air and soil temperatures and solar radiation. Utility functions estimate the organismal and environmental parameters needed for biophysical ecology. 'TrenchR' focuses on relatively simple and modular functions so users can create transparent and flexible biophysical models. Many functions are derived from Gates (1980) <doi:10.1007/978-1-4612-6024-0> and Campbell and Norman (1988) <isbn:9780387949376>.
Version: 1.2.1
Maintainer: Lauren Buckley <lbuckley@uw.edu>
Authors@R: c(
    person("Lauren", "Buckley", role = c("aut", "cre"), email = "lbuckley@uw.edu", comment = c(ORCID = "0000-0003-1315-3818")),
    person("Bryan", "Briones Ortiz", role = c("aut")),
    person("Isaac", "Caruso", role = c("aut")),
    person("Aji", "John", role = c("aut"), comment = c(ORCID = "0000-0002-4401-1401")),
    person("Ofir", "Levy", role = c("aut"), comment = c(ORCID = "0000-0003-0920-1207")),
    person("Abigail", "Meyer", role = c("aut")),
    person("Eric", "Riddell", role = c("aut"), comment = c(ORCID = "0000-0002-4229-4911")),
    person("Yutaro", "Sakairi", role = c("aut")),
    person("Juniper", "Simonis", role = c("aut"), comment = c(ORCID = "0000-0001-9798-0460")),
    person("Brian", "Helmuth", role = c("ctb"), comment = c(ORCID = "0000-0003-0180-3414"))
  )
License: MIT + file LICENSE
Date: 2026-04-28
URL: https://trenchproject.github.io/TrenchR/,
        https://github.com/trenchproject/TrenchR
BugReports: https://github.com/trenchproject/TrenchR/issues
VignetteBuilder: knitr
Depends: R (>= 3.3.1)
Imports: deSolve, msm, Rdpack (>= 0.7), stats, zoo
Suggests: covr, knitr, rmarkdown, testthat
RdMacros: Rdpack
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2026-04-29 01:59:18 UTC; lbuckley
Author: Lauren Buckley [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1315-3818>),
  Bryan Briones Ortiz [aut],
  Isaac Caruso [aut],
  Aji John [aut] (ORCID: <https://orcid.org/0000-0002-4401-1401>),
  Ofir Levy [aut] (ORCID: <https://orcid.org/0000-0003-0920-1207>),
  Abigail Meyer [aut],
  Eric Riddell [aut] (ORCID: <https://orcid.org/0000-0002-4229-4911>),
  Yutaro Sakairi [aut],
  Juniper Simonis [aut] (ORCID: <https://orcid.org/0000-0001-9798-0460>),
  Brian Helmuth [ctb] (ORCID: <https://orcid.org/0000-0003-0180-3414>)
Repository: CRAN
Date/Publication: 2026-04-29 02:30:02 UTC
Built: R 4.6.0; ; 2026-05-01 21:26:53 UTC; unix

Package: TrenchR
Title: Tools for Microclimate and Biophysical Ecology
Description: Tools for translating environmental change into organismal response. Microclimate models to vertically scale weather station data to organismal heights. The biophysical modeling tools include both general models for heat flows and specific models to predict body temperatures for a variety of ectothermic taxa. Additional functions model and temporally partition air and soil temperatures and solar radiation. Utility functions estimate the organismal and environmental parameters needed for biophysical ecology. 'TrenchR' focuses on relatively simple and modular functions so users can create transparent and flexible biophysical models. Many functions are derived from Gates (1980) <doi:10.1007/978-1-4612-6024-0> and Campbell and Norman (1988) <isbn:9780387949376>.
Version: 1.2.1
Maintainer: Lauren Buckley <lbuckley@uw.edu>
Authors@R: c(
    person("Lauren", "Buckley", role = c("aut", "cre"), email = "lbuckley@uw.edu", comment = c(ORCID = "0000-0003-1315-3818")),
    person("Bryan", "Briones Ortiz", role = c("aut")),
    person("Isaac", "Caruso", role = c("aut")),
    person("Aji", "John", role = c("aut"), comment = c(ORCID = "0000-0002-4401-1401")),
    person("Ofir", "Levy", role = c("aut"), comment = c(ORCID = "0000-0003-0920-1207")),
    person("Abigail", "Meyer", role = c("aut")),
    person("Eric", "Riddell", role = c("aut"), comment = c(ORCID = "0000-0002-4229-4911")),
    person("Yutaro", "Sakairi", role = c("aut")),
    person("Juniper", "Simonis", role = c("aut"), comment = c(ORCID = "0000-0001-9798-0460")),
    person("Brian", "Helmuth", role = c("ctb"), comment = c(ORCID = "0000-0003-0180-3414"))
  )
License: MIT + file LICENSE
Date: 2026-04-28
URL: https://trenchproject.github.io/TrenchR/,
        https://github.com/trenchproject/TrenchR
BugReports: https://github.com/trenchproject/TrenchR/issues
VignetteBuilder: knitr
Depends: R (>= 3.3.1)
Imports: deSolve, msm, Rdpack (>= 0.7), stats, zoo
Suggests: covr, knitr, rmarkdown, testthat
RdMacros: Rdpack
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2026-04-29 01:59:18 UTC; lbuckley
Author: Lauren Buckley [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1315-3818>),
  Bryan Briones Ortiz [aut],
  Isaac Caruso [aut],
  Aji John [aut] (ORCID: <https://orcid.org/0000-0002-4401-1401>),
  Ofir Levy [aut] (ORCID: <https://orcid.org/0000-0003-0920-1207>),
  Abigail Meyer [aut],
  Eric Riddell [aut] (ORCID: <https://orcid.org/0000-0002-4229-4911>),
  Yutaro Sakairi [aut],
  Juniper Simonis [aut] (ORCID: <https://orcid.org/0000-0001-9798-0460>),
  Brian Helmuth [ctb] (ORCID: <https://orcid.org/0000-0003-0180-3414>)
Repository: CRAN
Date/Publication: 2026-04-29 02:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 06:24:50 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DRYAIR)
export(Grashof_number)
export(Grashof_number_Gates)
export(Nusselt_from_Grashof)
export(Nusselt_from_Reynolds)
export(Nusselt_number)
export(Prandtl_number)
export(Qconduction_animal)
export(Qconduction_substrate)
export(Qconvection)
export(Qemitted_thermal_radiation)
export(Qevaporation)
export(Qmetabolism_from_mass)
export(Qmetabolism_from_mass_temp)
export(Qnet_Gates)
export(Qradiation_absorbed)
export(Qthermal_radiation_absorbed)
export(Reynolds_number)
export(TPC)
export(TPC_beta)
export(T_sky)
export(Tb_CampbellNorman)
export(Tb_Gates)
export(Tb_Gates2)
export(Tb_butterfly)
export(Tb_grasshopper)
export(Tb_limpet)
export(Tb_limpetBH)
export(Tb_lizard)
export(Tb_lizard_Fei)
export(Tb_mussel)
export(Tb_salamander_humid)
export(Tb_snail)
export(Tbed_mussel)
export(Tsoil)
export(VAPPRS)
export(WETAIR)
export(actual_vapor_pressure)
export(air_temp_profile)
export(air_temp_profile_neutral)
export(air_temp_profile_segment)
export(airpressure_from_elev)
export(azimuth_angle)
export(boundary_layer_resistance)
export(celsius_to_kelvin)
export(day_of_year)
export(daylength)
export(dec_angle)
export(degree_days)
export(degrees_to_radians)
export(direct_solar_radiation)
export(diurnal_radiation_variation)
export(diurnal_temp_variation_sine)
export(diurnal_temp_variation_sineexp)
export(diurnal_temp_variation_sinesqrt)
export(external_resistance_to_water_vapor_transfer)
export(fahrenheit_to_celsius)
export(fahrenheit_to_kelvin)
export(free_or_forced_convection)
export(heat_transfer_coefficient)
export(heat_transfer_coefficient_approximation)
export(heat_transfer_coefficient_simple)
export(kelvin_to_celsius)
export(latent_heat_vaporization_h2o)
export(mass_from_length)
export(monthly_solar_radiation)
export(partition_solar_radiation)
export(proportion_diffuse_solar_radiation)
export(proportion_silhouette_area)
export(proportion_silhouette_area_shapes)
export(radians_to_degrees)
export(saturation_vapor_pressure)
export(saturation_water_vapor_pressure)
export(soil_conductivity)
export(soil_specific_heat)
export(soil_temperature)
export(soil_temperature_equation)
export(soil_temperature_function)
export(soil_temperature_integrand)
export(solar_noon)
export(solar_radiation)
export(specific_heat_h2o)
export(stefan_boltzmann_constant)
export(surface_area_from_length)
export(surface_area_from_mass)
export(surface_area_from_volume)
export(surface_roughness)
export(volume_from_length)
export(von_karman_constant)
export(wind_speed_profile_neutral)
export(wind_speed_profile_segment)
export(zenith_angle)
importFrom(Rdpack,reprompt)
importFrom(deSolve,ode)
importFrom(msm,rtnorm)
importFrom(stats,integrate)
importFrom(stats,lm)
importFrom(stats,runif)
importFrom(stats,uniroot)
importFrom(zoo,na.approx)

# TrenchR: an R package for transparent environmental and ecological biophysics <img src="https://raw.githubusercontent.com/trenchproject/TrenchR/main/img/TrenchRIcon.png" width="200px" alt="The TrenchR logo invokes an energy budget for a grasshopper. A tan and blue hexagon is centered on a white square, with white thick arrows within pointing towards the center and then towards the bottom. The word TrenchR is in a salmon orange color with arrows pointing up and down from the h. Below the text is a blue-green and orange grasshopper, on a blue ground, with colors alluding to temperature." align = "right">


<!-- badges: start -->
[![R-CMD-check](https://github.com/trenchproject/TrenchR/actions/workflows/r_command_check.yaml/badge.svg)](https://github.com/trenchproject/TrenchR/actions/workflows/r_command_check.yaml)
[![Codecov test coverage](https://codecov.io/gh/trenchproject/TrenchR/branch/main/graph/badge.svg)](https://app.codecov.io/gh/trenchproject/TrenchR/branch/main)
[![NSF-1349865](https://img.shields.io/badge/NSF-1349865-blue.svg)](https://www.nsf.gov/awardsearch/show-award/?AWD_ID=1349865)
[![License](http://img.shields.io/badge/license-MIT-blue.svg)](https://raw.githubusercontent.com/trenchproject/TrenchR/main/LICENSE)
<!-- badges: end -->

**Author:** [TrEnCh project, Buckley Lab, Department of Biology, University of Washington](https://www.trenchproject.com/)<br>

[Package website](https://trenchproject.github.io/TrenchR/)

### Description
The [TrenchR package](https://github.com/trenchproject/TrenchR) aids in Translating Environmental Change into organismal responses. The package facilitates microclimate modeling to translate weather station data into the environmental conditions experienced by organisms and biophysical modeling to predict organismal body temperatures given the environmental conditions. The package aims to introduce and enable microclimate and biophysical modeling to improve ecological and evolutionary forecasting and includes tutorials and well as a series of educational modules introducing microclimate and biophysical modeling. The package focuses on transparent and modular functions but also includes some biophysical models for particular organisms. The package complements and integrates with the [NicheMapR package](https://github.com/mrke/NicheMapR), which contains more complex functions that are generally not intended for modular use. 

### Installation
You can install [the TrenchR package](https://CRAN.R-project.org/package=TrenchR) from [CRAN](https://cran.r-project.org/):
```r
install.packages("TrenchR")  
```

The latest version of the package can be installed from the [github repository](https://github.com/trenchproject/TrenchR):
```r
install.packages("devtools")   
devtools::install_github("trenchproject/TrenchR")
```

### Using the package
The package encompasses simple functions that can be combined to estimate environmental conditions and their impacts on organisms. 
Many of the functions are adapted from biophysical ecology texts including the following:  

* Gates DM. 1980. Biophysical Ecology.  
* Campbell GS and Norman JM. 2000. An introduction to environmental biophysics.

### Package Vignettes
We introduce functions in categorically grouped tutorials.  A good place to start is the Allometry and conversions tutorial, which provides tools for preparing data such as estimating additional dimensions of organisms from measured dimensions. 

```r
vignette("AllometryAndConversionsTutorial.Rmd", package = "TrenchR")

```

The Estimating microclimates tutorial provides resources for estimating the environmental conditions experienced by organisms.  This includes estimating solar radiation and its components, diurnal variation in temperature and radiation, temperature and wind speed profiles, and soil temperatures and profiles. 

```r
vignette("MicroclimateTutorial", package = "TrenchR")

```

Finally, the core biophysical modeling functions are described in a tutorial on Using energy balances to estimate body temperatures. Components of an energy budget can be estimated using individual functions and then operative environmental temperatures, Te, can be solved for using either a generic energy balance or taxa specific biophysical models.

```r
vignette("TeTutorial", package = "TrenchR")

```

### Future Directions
We welcome code contributions, fixes, and comments. Code (scripts and functions) will be accepted in any programming language and thorough commenting will be appreciated.  We would also appreciate your including a header that describes the intent, input, and output of your scripts and functions. 

### Citation
If you use this package, We would appreciate a citation. You can see an up to date citation information with `citation("TrenchR")`. You can cite either the package or the accompanying journal article.

### Developer notes
Please see the [Contributor Notes](https://github.com/trenchproject/TrenchR/blob/main/CONTRIBUTING.md) and [Code of Conduct](https://github.com/trenchproject/TrenchR/blob/main/CODE_OF_CONDUCT.md).

We use [Rdpack](https://cran.r-project.org/package=Rdpack) to facilitate package documentation with [BibTeX](https://www.bibtex.org) citations. See the [documentation workflow document](https://github.com/trenchproject/TrenchR/blob/main/documentation_workflow.md) for more details about building package manual components.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## -----------------------------------------------------------------------------

# Load TrenchR package

 library(TrenchR)


## -----------------------------------------------------------------------------
Qnet_Gates(Qabs  = 500, 
           Qemit = 10, 
           Qconv = 100, 
           Qcond = 100,
           Qmet  = 10, 
           Qevap = 5)

## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = seq(100, 1000, 100), 
     y    = Qradiation_absorbed(a       = 0.9, 
                                A       = 1, 
                                psa_dir = 0.4, 
                                psa_dif = 0.4,
                                psa_ref = 0.4, 
                                S_dir   = seq(100, 1000, 100), 
                                S_dif   = 200, 
                                rho     = 0.5), 
     type = "l", 
     xlab = expression("direct solar radiation" ~ (W/m^{2})), 
     ylab = "solar radiation absorbed (W)")

points(x    = seq(100, 1000, 100), 
       y    = Qradiation_absorbed(a       = 0.9, 
                                  A       = 1, 
                                  psa_dir = 0.2, 
                                  psa_dif = 0.4,
                                  psa_ref = 0.4, 
                                  S_dir   = seq(100, 1000, 100), 
                                  S_dif   = 200, 
                                  rho     = 0.5), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(100, 1000, 100), 
       y    = Qradiation_absorbed(a       = 0.45, 
                                  A       = 1, 
                                  psa_dir = 0.4, 
                                  psa_dif = 0.4,
                                  psa_ref = 0.4, 
                                  S_dir   = seq(100, 1000, 100), 
                                  S_dif   = 200, 
                                  rho     = 0.5), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(100, 1000, 100), 
       y    = Qradiation_absorbed(a       = 0.9, 
                                  A = 1, 
                                  psa_dir = 0.4, 
                                  psa_dif = 0.4,
                                  psa_ref = 0.4, 
                                  S_dir = seq(100, 1000, 100), 
                                  S_dif = 200, 
                                  rho = 0.2), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topleft", 
       title  = "parameters", 
       legend = c("a = 0.9, psa_dir = 0.4, rho = 0.5", "a = 0.9, sa_dir = 0.2, rho = 0.5", "a = 0.45, psa_dir = 0.4, rho = 0.5", "a = 0.9, psa_dir = 0.4, rho = 0.2"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## -----------------------------------------------------------------------------
T_sky(T_a=293, formula="Swinbank")

## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 293:313, 
     y    = Qemitted_thermal_radiation(epsilon  = 0.96, 
                                       A        = 1, 
                                       psa_dir  = 0.4, 
                                       psa_ref  = 0.5,  
                                       T_b      = 293:313,  
                                       T_g      = 293,  
                                       T_a      = 298,  
                                       enclosed = FALSE), 
     type = "l", 
     xlab = "body surface temperature, Tb (K)", 
     ylab = "emitted thermal radiation, Qemit (W)")
points(x    = 293:313, 
       y    = Qemitted_thermal_radiation(epsilon  = 0.96, 
                                         A        = 1, 
                                         psa_dir  = 0.2, 
                                         psa_ref  = 0.5, 
                                         T_b      = 293:313, 
                                         T_g      = 293, 
                                         T_a      = 298, 
                                         enclosed = TRUE), 
       type = "l", 
       lty  = "dashed")
points(x    = 293:313, 
       y    = Qemitted_thermal_radiation(epsilon  = 0.96,
                                         A        = 1,
                                         psa_dir  = 0.4, 
                                         psa_ref  = 0.5, 
                                         T_b      = 293:313, 
                                         T_g      = 283, 
                                         T_a      = 298, 
                                         enclosed = FALSE), 
       type = "l", 
       lty  = "dotted")
points(x    = 293:313, 
       y    = Qemitted_thermal_radiation(epsilon= 0.96, 
                                         A = 1, 
                                         psa_dir = 0.2, 
                                         psa_ref = 0.5, 
                                         T_b      = 293:313, 
                                         T_g      = 283, 
                                         T_a      = 298, 
                                         enclosed = FALSE), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topleft", 
       title  = "parameters", 
       legend = c("psa_dir= 0.4, T_g = 293", "psa_dir= 0.2, T_g = 293", "psa_dir= 0.4, T_g = 283", "psa_dir= 0.2, T_g = 283"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 293:313, 
     y    = Qconvection(T_a        = 303,
                        T_b        = 293:313,
                        H          = 5,
                        A          = 0.0025, 
                        proportion = 0.6), 
     type = "l", 
     xlab = "ground temperature (K)", 
     ylab = "Q convection (W)")
points(x    = 293:313, 
       y    = Qconvection(T_a        = 303,
                          T_b        = 293:313,
                          H          = 15,
                          A          = 0.0025,
                          proportion = 0.6), 
       type = "l",
       lty  = "dashed")
points(x    = 293:313, 
       y    = Qconvection(T_a        = 303,
                          T_b        = 293:313,
                          H          = 5,
                          A          = 0.0025,
                          proportion = 0.3), 
       type = "l",
       lty  = "dotted")
points(x    = 293:313, 
       y    = Qconvection(T_a        = 303,
                          T_b        = 293:313,
                          H          = 15,
                          A          = 0.0025,
                          proportion = 0.3), 
       type = "l",
       lty  = "dotdash")

legend(x      = "topleft", 
       title  = "parameters", 
       legend = c("H = 5, proportion = 0.6", "H = 15, proportion = 0.6", "H = 5, proportion = 0.3", "H = 15, proportion = 0.3"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 1:30, 
     y    = Nusselt_number(H_L = 1:30, 
                           D   = 0.01,
                           K   = 0.5), 
     type = "l", 
     xlab = expression("heat transfer coefficient" ~ (W ~ m^{-2} ~ K^{-1})), 
     ylab = "Nusselt number")

points(x    = 1:30, 
       y    = Nusselt_number(H_L = 1:30,
                             D   = 0.01, 
                             K   = 1.5), 
       type = "l", 
       lty  = "dashed")

points(x    = 1:30, 
       y    = Nusselt_number(H_L = 1:30,
                             D   = 0.05,
                             K   = 0.5), 
       type = "l", 
       lty  = "dotted")

points(x    = 1:30, 
       y    = Nusselt_number(H_L = 1:30, 
                             D   = 0.05, 
                             K   = 1.5), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topleft", 
      title  = "parameters", 
       legend = c("D = 0.01, K = 0.5", "D = 0.01, K = 1.5", "D = 0.05, K = 0.5", "D = 0.05, K = 1.5"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 1:30, 
     y    = Prandtl_number(c_p = 1:30,
                           mu  = 0.00001, 
                           K   = 0.5), 
     type = "l", 
     xlab = expression("specific heat at constant pressure" ~ (J ~ mol^{-1} ~ K^{-1})), 
     ylab = "Prandtl number")

points(x    = 1:30, 
       y    = Prandtl_number(c_p = 1:30, 
                             mu  = 0.00002, 
                             K   = 0.5), 
       type = "l", 
       lty  = "dashed")

points(x    = 1:30, 
       y    = Prandtl_number(c_p = 1:30, 
                             mu  = 0.00001, 
                             K   = 1.5), 
       type = "l", 
       lty  = "dotted")

points(x    = 1:30, 
       y    = Prandtl_number(c_p = 1:30, 
                             mu  = 0.00002, 
                             K   = 1.5), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topleft", 
      title  = "parameters", 
       legend = c("mu = 0.00001, K = 0.5", "mu = 0.00002, K = 0.5", "mu = 0.00001, K = 1.5", "mu = 0.00002, K = 1.5"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = seq(0, 5, 0.2), 
     y    = Reynolds_number(u  = seq(0, 5, 0.2), 
                            D  = 0.001, 
                            nu = 1), 
     type = "l", 
     xlab = "wind speed (m/s)", 
     ylab = "Reynolds number")

points(x    = seq(0, 5, 0.2), 
       y    = Reynolds_number(u  = seq(0, 5, 0.2),
                            D  = 0.001,
                            nu = 1.5), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 5, 0.2), 
       y    = Reynolds_number(u  = seq(0, 5, 0.2),
                              D  = 0.002,
                              nu = 1), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 5, 0.2), 
       y    = Reynolds_number(u  = seq(0, 5, 0.2),
                              D  = 0.002,
                              nu = 1.5), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topleft", 
       title  = "parameters", 
       legend = c("D = 0.001, nu = 1", "D = 0.001, nu = 1.5", "D = 0.002, nu = 1", "D = 0.002, nu = 1.5"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))



## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = seq(0, 0.01, 0.001), 
     y    = Grashof_number(T_a = 30, 
                           T_g = 35,  
                           D   = seq(0, 0.01, 0.001), 
                           nu  = 1.0), 
     type = "l", 
     xlab = "characteristic dimension (m)", 
     ylab = "Grashof number")

points(x    = seq(0, 0.01, 0.001), 
       y    = Grashof_number_Gates(T_a  = 30,  
                                   T_g  = 35,  
                                   beta = 0.00367,  
                                   D    = seq(0, 0.01, 0.001), 
                                   nu   = 1.0), 
       type = "l", 
       col = "red")

points(x    = seq(0, 0.01, 0.001), 
       y    = Grashof_number(T_a  = 30, 
                             T_g  = 35,  
                             D  = seq(0, 0.01, 0.001),  
                             nu   = 1.4), 
       type = "l", 
       lty = "dashed")

points(x    = seq(0, 0.01, 0.001), 
       y    = Grashof_number_Gates(T_a  = 30,  
                                   T_g  = 35, 
                                   beta = 0.00367,  
                                   D    = seq(0, 0.01, 0.001), 
                                   nu   = 1.4), 
       type = "l", 
       col = "red", 
       lty = "dashed")

legend(x      = "topleft", 
       title  = "parameters", 
       legend = c("Grashof_number, nu = 1", "Grashof_number_Gates, nu = 1", "Grashof_number, nu = 1.4", "Grashof_number_Gates, nu = 1.4"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 1:10, 
     y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                  taxon = "sphere"), 
     type = "l", 
     xlab = "Re", 
     ylab = "Nu", 
     ylim = c(0,4))

points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                    taxon = "cylinder"), 
       type = "l", 
       lty  = "dashed")

points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                    taxon = "frog"), 
       type = "l", 
       lty  = "dotted")

points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10, 
                                    taxon = "lizard_traverse_to_air_flow"), 
       type = "l", 
       lty  = "dotdash")

points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                    taxon = "lizard_parallel_to_air_flow"), 
       type = "l", 
       lty  = "dotdash", 
       col = "red") 
                  
points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                    taxon = "lizard_surface"), 
       type = "l", 
       lty  = "dotdash", 
       col = "orange")

points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                    taxon = "lizard_elevated"), 
       type = "l", 
       lty  = "dotdash", 
       col = "green")

points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                    taxon = "flyinginsect"), 
       type = "l", 
       lty  = "longdash")

points(x    = 1:10, 
       y    = Nusselt_from_Reynolds(Re    = 1:10,  
                                    taxon = "spider"), 
       type = "l", 
       lty  = "twodash")

legend(x      = "topright", 
       title  = "taxa", 
       legend = c("sphere", "cylinder", "frog", "lizard_traverse_to_air_flow", "lizard_parallel_to_air_flow", "lizard_surface", "lizard_elevated", "flyinginsect", "spider"), 
       lty    = c("solid", "dashed", "dotted", "dotdash", "dotdash", "dotdash", "dotdash", "longdash", "twodash"), 
       col    = c("black", "black", "black", "black", "red", "orange", "green", "black", "black" ))


## -----------------------------------------------------------------------------

Nusselt_from_Grashof(Gr = 5)


## -----------------------------------------------------------------------------

free_or_forced_convection(Gr = 100,  
                          Re = 5)


## ---- fig.height = 4, fig.width = 6-------------------------------------------

oldpar <- par()

par(mar = c(5,  5, 3, 2))

plot(x    = seq(0, 3, 0.25), 
     y    = heat_transfer_coefficient(u     = seq(0, 3, 0.25),
                                      D     = 0.05,
                                      K     =  25.7 * 10^(-3),
                                      nu    =  15.3 * 10^(-6), 
                                      taxon = "sphere"), 
     type = "l", 
     xlab = "Air velocity (m/s)", 
     ylab = expression("heat transfer coefficient," ~ H[L] ~ (W ~ m^{-2} ~ K^{-1})))

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient(u     = seq(0, 3, 0.25),
                                        D     = 0.05,
                                        K     =  25.7 * 10^(-3),
                                        nu    =  15.3 * 10^(-6), 
                                        taxon = "cylinder"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient(u     = seq(0, 3, 0.25),
                                        D     = 0.05,
                                        K     =  25.7 * 10^(-3),
                                        nu    =  15.3 * 10^(-6), 
                                        taxon = "frog"), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient(u     = seq(0, 3, 0.25),
                                        D     = 0.05,
                                        K     =  25.7 * 10^(-3),
                                        nu    =  15.3 * 10^(-6), 
                                        taxon = "lizard_surface"), 
       type = "l", 
       lty  = "dotdash")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient(u     = seq(0, 3, 0.25),
                                        D     = 0.05,
                                        K     =  25.7 * 10^(-3),
                                        nu    =  15.3 * 10^(-6), 
                                        taxon = "lizard_elevated"), 
       type = "l", 
       lty  = "dotdash")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient(u     = seq(0, 3, 0.25),
                                        D     = 0.05,
                                        K     =  25.7 * 10^(-3),
                                        nu    =  15.3 * 10^(-6), 
                                        taxon = "flyinginsect"), 
       type = "l", 
       lty  = "longdash")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient(u     = seq(0, 3, 0.25),
                                        D     = 0.05,
                                        K     =  25.7 * 10^(-3),
                                        nu    =  15.3 * 10^(-6), 
                                        taxon = "spider"), 
       type = "l", 
       lty  = "twodash")

legend(x      = "bottomright", 
       title  = "taxa", 
       legend = c("sphere", "cylinder", "frog", "lizard_surface", "lizard_elevated", "flyinginsect", "spider"), 
       lty    = c("solid", "dashed", "dotted", "dotdash", "dotdash", "dotdash", "dotdash", "longdash", "twodash"))


## ---- fig.height = 3, fig.width = 6-------------------------------------------

par(mar = c(5,  5, 3, 2))

plot(x    = seq(0, 3, 0.25), 
     y    = heat_transfer_coefficient_approximation(u     = seq(0, 3, 0.25),
                                                    D     = 0.05,
                                                    K     =  25.7 * 10^(-3),
                                                    nu    =  15.3 * 10^(-6), 
                                                    taxon = "sphere"), 
     type = "l", 
     xlab = "Air velocity (m/s)", 
     ylab = expression("heat transfer coefficient," ~ H[L] ~ (W ~ m^{-2} ~ K^{-1})))

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient_approximation(u     = seq(0, 3, 0.25),
                                                      D     = 0.05,
                                                      K     =  25.7 * 10^(-3),
                                                      nu    =  15.3 * 10^(-6), 
                                                      taxon = "frog"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient_approximation(u     = seq(0, 3, 0.25),
                                                      D     = 0.05,
                                                      K     =  25.7 * 10^(-3),
                                                      nu    =  15.3 * 10^(-6), 
                                                      taxon = "lizard"), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient_approximation(u     = seq(0, 3, 0.25),
                                                      D     = 0.05,
                                                      K     =  25.7 * 10^(-3),
                                                      nu    =  15.3 * 10^(-6), 
                                                      taxon = "flyinginsect"), 
       type = "l", 
       lty  = "dotdash")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient_approximation(u     = seq(0, 3, 0.25),
                                                      D     = 0.05,
                                                      K     =  25.7 * 10^(-3),
                                                      nu    =  15.3 * 10^(-6), 
                                                      taxon = "spider"), 
       type = "l", 
       lty  = "longdash")

legend(x      = "bottomright", 
       title  = "taxa", 
       legend = c("sphere", "frog", "lizard", "flyinginsect", "spider"), 
       lty    = c("solid", "dashed", "dotted", "dotdash", "longdash"))


## ---- fig.height = 3, fig.width = 5-------------------------------------------

plot(x    = seq(0, 3, 0.25), 
     y    = heat_transfer_coefficient_simple(u    = seq(0, 3, 0.25),
                                             D    = 0.05, 
                                             type = "Spotila"), 
     type = "l", 
     xlab = "Air velocity (m/s)", 
     ylab = expression("heat transfer coefficient," ~ H[L] ~ (W ~ m^{-2} ~ K^{-1})))

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient_simple(u    = seq(0, 3, 0.25),
                                               D    = 0.03, 
                                               type = "Spotila"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 3, 0.25), 
       y    = heat_transfer_coefficient_simple(u    = seq(0, 3, 0.25),
                                               D    = 0.07, 
                                               type = "Spotila"), 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright", 
       title  = "characteristic dimension (m)", 
       legend = c(0.03, 0.05, 0.07), 
       lty    = c("dashed", "solid", "dotted"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 293:313, 
     y    = Qconduction_animal(T_g        = 293:313,
                               T_b        = 303,
                               d          = 0.025,
                               K          = 0.3,
                               A          = 10^-3,
                               proportion = 0.2), 
     type = "l", 
     xlab = "ground temperature (Tb, K)", ylab = "Q conductance (W)")
points(x    = 293:313, 
       y    = Qconduction_animal(T_g        = 293:313,
                                 T_b        = 303,
                                 d          = 0.025,
                                 K          = 0.5,
                                 A          = 10^-3,
                                 proportion = 0.2), 
       type = "l", 
       lty  = "dashed")
points(x    = 293:313, 
       y    = Qconduction_animal(T_g        = 293:313,
                                 T_b        = 303,
                                 d          = 0.025,
                                 K          = 0.3,
                                 A          = 10^-3,
                                 proportion = 0.4), 
       type = "l", 
       lty  = "dotted")
points(x    = 293:313, 
       y    = Qconduction_animal(T_g        = 293:313,
                                 T_b        = 303,
                                 d          = 0.025,
                                 K          = 0.5,
                                 A          = 10^-3,
                                 proportion = 0.4), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topright", 
       title  = "parameters", 
       legend = c("K = 0.3, proportion = 0.2", "K = 0.5, proportion = 0.2", "K = 0.3, proportion = 0.4", "K = 0.5, proportion = 0.4"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 293:313, 
     y    = Qconduction_substrate(T_g        = 293:313,
                                  T_b        = 303,
                                  D          = 0.01,
                                  K_g        = 0.3,
                                  A          = 10^-2,
                                  proportion = 0.2), 
     type = "l", 
     xlab = "ground temperature (Tb, K)", 
     ylab = "Q conductance (W)")
points(x    = 293:313, 
       y    = Qconduction_substrate(T_g        = 293:313,
                                    T_b        = 303,
                                    D          = 0.01,
                                    K_g        = 0.5,
                                    A          = 10^-2,
                                    proportion = 0.2), 
       type = "l", 
       lty  = "dashed")
points(x    = 293:313, 
       y    = Qconduction_substrate(T_g        = 293:313,
                                    T_b        = 303,
                                    D          = 0.01,
                                    K_g        = 0.3,
                                    A          = 10^-2,
                                    proportion = 0.4), 
       type = "l", 
       lty  = "dotted")
points(x    = 293:313, 
       y    = Qconduction_substrate(T_g        = 293:313,
                                    T_b        = 303,
                                    D          = 0.01,
                                    K_g        = 0.5,
                                    A          = 10^-2,
                                    proportion = 0.4), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topright", 
       title  = "parameters", 
       legend = c("K = 0.3, proportion = 0.2", "K = 0.5, proportion = 0.2", "K = 0.3, proportion = 0.4", "K = 0.5, proportion = 0.4"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

plot(x    = 1:100, 
     y    = Qmetabolism_from_mass(m     = 1:100,
                                  taxon = "bird"), 
     type = "l", 
     xlab = "mass (g)", 
     ylab = "metabolism (W)", 
     log  = "xy", 
     ylim = c(0.1, 2))

points(x    = 1:100, 
       y    = Qmetabolism_from_mass(m     = 1:100,
                                    taxon = "mammal"), 
       type = "l", 
       lty  = "dashed")

points(x    = 1:100, 
       y    = Qmetabolism_from_mass(m     = 1:100,
                                    taxon = "reptile"), 
       type = "l", 
       lty  = "dotted")

legend(x      = "topleft", 
       title  = "taxa", 
       legend = c("bird", "mammal", "reptile"), 
       lty    = c("solid", "dashed", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

plot(x    = 1:100, 
     y    = Qmetabolism_from_mass_temp(m     = 1:100, 
                                       T_b   = 30,
                                       taxon = "bird"), 
     type = "l", 
     xlab = "mass (g)", 
     ylab = "metabolism (W)", 
     ylim = c(0, 0.1))

points(x    = 1:100, 
       y    = Qmetabolism_from_mass_temp(m     = 1:100, 
                                         T_b   = 30,
                                         taxon = "reptile"), 
       type = "l", 
       lty  = "dotted")

points(x    = 1:100, 
       y    = Qmetabolism_from_mass_temp(m     = 1:100, 
                                         T_b   = 30,
                                         taxon = "amphibian"), 
       type = "l", 
       lty  = "dotdash")

points(x    = 1:100, 
       y    = Qmetabolism_from_mass_temp(m     = 1:100,
                                         T_b   = 30,
                                         taxon = "invertebrate"), 
       type = "l", 
       lty  = "twodash")

legend(x      = "top", 
       title  = "taxa", 
       legend = c("bird,mammal", "reptile", "amphibian", "invertebrate"), 
       lty    = c("solid", "dotted", "dotdash", "twodash"))

plot(x    = 20:40, 
     y    = Qmetabolism_from_mass_temp(m     = 5, 
                                       T_b   = 20:40,
                                       taxon = "bird"), 
     type = "l", 
     xlab = "temperature (C)", 
     ylab = "metabolism (W)", 
     ylim = c(0,0.1))

points(x    = 20:40, 
       y    = Qmetabolism_from_mass_temp(m     = 5, 
                                         T_b   = 20:40,
                                         taxon = "reptile"), 
       type = "l", 
       lty  = "dotted")

points(x    = 20:40, 
       y    = Qmetabolism_from_mass_temp(m     = 5, 
                                         T_b   = 20:40,
                                         taxon = "amphibian"), 
       type = "l", 
       lty  = "dotdash")

points(x    = 20:40, 
       y    = Qmetabolism_from_mass_temp(m     = 5, 
                                         T_b   = 20:40,
                                         taxon = "invertebrate"), 
       type = "l", 
       lty  = "twodash")

legend(x      = "topleft", 
       title  = "taxa", 
       legend = c("bird,mammal", "reptile", "amphibian", "invertebrate"), 
       lty    = c("solid", "dotted", "dotdash", "twodash"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

# kPa

  vp <- saturation_vapor_pressure(293:313)

# convert to kg/m^3

  e_s <- vp * 0.032 

# kPa

  vp <- saturation_vapor_pressure(293:313-10)

# convert to kg/m^3

  e_a <-  vp * 0.032 


temps      <- 293:313
Qevaps     <- rep(NA,21)
Qevaps_wet <- rep(NA,21)

for (ind in 1:21) {
  Qevaps[ind]     <- Qevaporation(A     = 0.1, 
                                  T_b   = temps[ind],  
                                  taxon = "amphibian",  
                                  e_s = e_s[ind],  
                                  e_a = e_a[ind],  
                                  hp     = 0.5,  
                                  H     = 20,  
                                  r_i   = 50) 

  Qevaps_wet[ind] <- Qevaporation(A     = 0.1,  
                                  T_b   = temps[ind],  
                                  taxon = "amphibian_wetskin",  
                                  e_s = e_s[ind],  
                                  e_a = e_a[ind],  
                                  hp     = 0.5,  
                                  H     = 20,  
                                  r_i   = 50) 
}

plot(x    = temps, 
     y    = Qevaps, 
     type = "l", 
     xlab = "body temperature (K)", 
     ylab = "evaporative heat loss (W)", 
     lty  = "dashed", 
     ylim = c(0, 400))

points(x    = temps, 
       y    = Qevaps_wet, 
       type = "l", 
       lty  = "dotted")

Qevap <- unlist(lapply(293:313, 
                       FUN  = Qevaporation, 
                       A = 0.1,  
                       taxon = "lizard"))

points(x    = 293:313, 
       y    = Qevap, 
       type = "l" )

legend(x      = "right", 
       title  = "taxa", 
       legend = c("amphibian", "amphibian_wetskin", "lizard"), 
       lty    = c("dashed", "dotted", "solid"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar = c(5, 5, 3, 2))

plot(x    = 10:30, 
     y    = external_resistance_to_water_vapor_transfer(H = 10:30), 
     type = "l", 
     xlab = expression("heat transfer coefficient" ~ (W ~ m^{-2} ~  K^{-1})), 
     ylab = expression("external resistance, " ~ r[e] ~ (sm^{-1})))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = 10:30, 
     y    = saturation_water_vapor_pressure(T_a = 10:30), 
     type = "l", 
     xlab = "temperature (°C)", 
     ylab = "saturation water vapor pressure (Pa)")


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40, 
                FUN     = Tb_Gates, 
                A       = 0.1, 
                D       = 0.025, 
                psa_dir = 0.6, 
                psa_ref = 0.4, 
                psa_air = 0.6, 
                psa_g   = 0.0, 
                T_g     = 30, 
                Qabs    = 10, 
                epsilon = 0.95, 
                H_L     = 10, 
                ef      = 1.3, 
                K       = 0.5)

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature, Ta (°C)", 
     ylab = "body temperature, Tb (°C)", 
     xlim = c(22, 42), 
     ylim = c(22, 42))

abline(a   = 0,
       b   = 1, 
       col = "gray")

t.seq <- lapply(20:40, 
                FUN     = Tb_Gates, 
                A       = 0.1, 
                D       = 0.025, 
                psa_dir = 0.6, 
                psa_ref = 0.4, 
                psa_air = 0.6, 
                psa_g   = 0.0, 
                T_g     = 30, 
                Qabs    = 0, 
                epsilon = 0.95, 
                H_L     = 10, 
                ef      = 1.3, 
                K       = 0.5)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40, 
                FUN     = Tb_Gates, 
                A       = 0.1, 
                D       = 0.025, 
                psa_dir = 0.6, 
                psa_ref = 0.4, 
                psa_air = 0.6, 
                psa_g   = 0.0, 
                T_g     = 30, 
                Qabs    = 20, 
                epsilon = 0.95, 
                H_L     = 10, 
                ef      = 1.3, 
                K       = 0.5)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright", 
       title  = "Radiation, Qabs", 
       legend = c(0, 10, 20), 
       lty    = c("dashed", "solid", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

plot(x    = 20:40, 
     y    = Tb_CampbellNorman(T_a     = 20:40, 
                              T_g     = 30, 
                              S       = 600, 
                              a_s     = 0.7, 
                              a_l     = 0.96, 
                              epsilon = 0.96, 
                              c_p     = 29.3, 
                              D       = 0.17, 
                              u       = 1), 
     type = "l", 
     xlab = "air temperature (C)", 
     ylab = "body temperature (C)", 
     xlim = c(23,37), 
     ylim = c(17,67))

abline(a   = 0, 
       b   = 1, 
       col = "gray")

points(x    = 20:40, 
       y    = Tb_CampbellNorman(T_a     = 20:40,
                                T_g     = 30, 
                                S       = 200, 
                                a_s     = 0.7, 
                                a_l     = 0.96, 
                                epsilon = 0.96, 
                                c_p     = 29.3, 
                                D       = 0.17, 
                                u       = 1), 
       type = "l", 
       lty  = "dashed")

points(x    = 20:40, 
       y    = Tb_CampbellNorman(T_a     = 20:40, 
                                T_g     = 30, 
                                S       = 400, 
                                a_s     = 0.7, 
                                a_l     = 0.96, 
                                epsilon = 0.96, 
                                c_p     = 29.3, 
                                D       = 0.17, 
                                u       = 1), 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright", 
       title  =  expression("Radiation, Sabs" ~ (W ~ m^{-2})), 
       legend = c(200,400,600), 
       lty    = c("dashed", "dotted", "solid"))


## ---- fig.height = 4, fig.width = 6-------------------------------------------

# sun, surface

  t.seq <- lapply(20:40, 
                  FUN       = Tb_lizard, 
                  T_g       = 30,  
                  u         = 0.1, 
                  svl       = 60, 
                  m         = 10, 
                  psi       = 34, 
                  rho_s     = 0.7, 
                  elev      = 500, 
                  doy       = 200, 
                  sun       = TRUE, 
                  surface   = TRUE, 
                  a_s       = 0.9, 
                  a_l       = 0.965, 
                  epsilon_s = 0.965, 
                  F_d       = 0.8, 
                  F_r       = 0.5, 
                  F_a       = 0.5, 
                  F_g       = 0.5)
  
  plot(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       xlab = "ambient temperature (°C)", 
       ylab = "body temperature (°C)", 
       xlim = c(20,40), 
       ylim = c(20,51))
  abline(a   = 0,
         b   = 1, 
         col = "gray")

# shade, surface

  t.seq <- lapply(20:40, 
                  FUN       = Tb_lizard, 
                  T_g       = 30, 
                  u         = 0.1, 
                  svl       = 60, 
                  m         = 10, 
                  psi       = 34, 
                  rho_s     = 0.7, 
                  elev      = 500, 
                  doy       = 200, 
                  sun       = FALSE, 
                  surface   = TRUE, 
                  a_s       = 0.9, 
                  a_l       = 0.965,
                  epsilon_s = 0.965, 
                  F_d       = 0.8, 
                  F_r       = 0.5, 
                  F_a       = 0.5, 
                  F_g       = 0.5)

  points(x    = 20:40, 
         y    = t.seq, 
         type = "l", 
         lty  = "dashed")

# sun, above surface

  t.seq <- lapply(20:40, 
                  FUN       = Tb_lizard, 
                  T_g       = 30, 
                  u         = 0.1, 
                  svl       = 60, 
                  m         = 10, 
                  psi       = 34, 
                  rho_s     = 0.7, 
                  elev      = 500, 
                  doy       = 200, 
                  sun       = TRUE, 
                  surface   = FALSE, 
                  a_s       = 0.9, 
                  a_l       = 0.965, 
                  epsilon_s = 0.965, 
                  F_d       = 0.8, 
                  F_r       = 0.5, 
                  F_a       = 0.5, 
                  F_g       = 0.5)

  points(x    = 20:40, 
         y    = t.seq, 
         type = "l", 
         lty  = "dotted")

# shade, above surface

  t.seq <- lapply(20:40, 
                  FUN       = Tb_lizard,
                  T_g       = 30, 
                  u         = 0.1, 
                  svl       = 60, 
                  m         = 10, 
                  psi       = 34, 
                  rho_s     = 0.7, 
                  elev      = 500, 
                  doy       = 200, 
                  sun       = FALSE, 
                  surface   = FALSE, 
                  a_s       = 0.9, 
                  a_l       = 0.965, 
                  epsilon_s = 0.965, 
                  F_d       = 0.8, 
                  F_r       = 0.5, 
                  F_a       = 0.5, 
                  F_g       = 0.5)
  points(x    = 20:40, 
         y    = t.seq, 
         type = "l", 
         lty  = "dotdash")

legend(x      = "topright",
       title  = "parameters", 
       legend = c("sun, surface", "shade, surface", "sun, above surface", "shade, above surface"), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40, 
                FUN       = Tb_lizard_Fei,  
                T_g       = 27,  
                S         = 800,  
                lw        = 30,  
                shade     = 0.5,  
                m         = 10.5,  
                Acondfact = 0.05,  
                Agradfact = 0.4)

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", 
     ylab = "body temperature (°C)")

abline(a   = 0,
       b   = 1, 
       col = "gray")

t.seq <- lapply(20:40,  
                FUN       = Tb_lizard_Fei,  
                T_g       = 27,  
                S         = 800,  
                lw        = 30,  
                shade     = 0.0,  
                m         = 10.5,  
                Acondfact = 0.05,  
                Agradfact = 0.4)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40,  
                FUN       = Tb_lizard_Fei,  
                T_g       = 27,  
                S         = 800,  
                lw        = 30,  
                shade     = 1.0,  
                m         = 10.5,  
                Acondfact = 0.05,  
                Agradfact = 0.4)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright", 
       title  = "proportion shade", 
       legend = c(0.0,0.5,1.0), 
       lty    = c("dashed", "solid", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40, 
                FUN    = Tb_butterfly,  
                T_g    = 25,  
                T_sh   = 20,  
                u      = 0.4,  
                S_sdir = 300,  
                S_sdif = 100,  
                z      = 30,  
                D      = 0.36,  
                delta  = 1.46,  
                alpha  = 0.6,  
                r_g    = 0.3)

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", 
     ylab = "body temperature (°C)")

abline(a   = 0,
       b   = 1, 
       col = "gray")

t.seq <- lapply(20:40,  
                FUN    = Tb_butterfly,  
                T_g    = 25,  
                T_sh   = 20,  
                u      = 0.4,  
                S_sdir = 100,  
                S_sdif = 100,  
                z      = 30,  
                D      = 0.36,  
                delta  = 1.46,  
                alpha  = 0.6,  
                r_g    = 0.3)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40,  
                FUN    = Tb_butterfly,  
                T_g    = 25,  
                T_sh   = 20,  
                u      = 0.4,  
                S_sdir = 500,  
                S_sdif = 100,  
                z      = 30,  
                D      = 0.36,  
                delta  = 1.46,  
                alpha  = 0.6,  
                r_g    = 0.3)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright",
       title= expression("direct radiation," ~ S[sdif]), 
       legend = c(100,300,500), 
       lty    = c("dashed", "solid", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40, 
                FUN       = Tb_grasshopper,  
                T_g       = 25,  
                u         = 0.4,  
                S         = 800,  
                K_t       = 0.7,  
                psi       = 30,  
                l         = 0.05,  
                Acondfact = 0.00,  
                z         = 0.001,  
                abs       = 0.7,  
                r_g       = 0.3)

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", 
     ylab = "body temperature (°C)", 
     xlim = c(20,40), 
     ylim = c(20,40))

abline(a   = 0,
       b   = 1, 
       col = "gray")

t.seq <- lapply(20:40,  
                FUN       = Tb_grasshopper,  
                T_g       = 25,  
                u         = 0.4,  
                S         = 400,  
                K_t       = 0.7,  
                psi       = 30,  
                l         = 0.05,  
                Acondfact = 0.00,  
                z         = 0.001,  
                abs       = 0.7,  
                r_g       = 0.3)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40,  
                FUN       = Tb_grasshopper,  
                T_g       = 25,  
                u         = 0.4,  
                S         = 600,  
                K_t       = 0.7,  
                psi       = 30,  
                l         = 0.05,  
                Acondfact = 0.00,  
                z         = 0.001,  
                abs       = 0.7,  
                r_g       = 0.3)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright",
       title  = expression("direct radiation, S"), 
       legend = c(200, 400, 600), 
       lty    = c("dashed", "dotted", "solid"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40, 
                FUN   = Tb_mussel,  
                l     = 0.1,  
                h     = 0.05,  
                T_g   = 30, 
                S     = 500,  
                k_d   = 0.2,  
                u     = 0.2,  
                psi   = 30,  
                evap  = TRUE,  
                cl    = 0.5,  
                group = "solitary")

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", 
     ylab = "body temperature (°C)", 
     ylim = c(25, 45))

t.seq <- lapply(20:40,  
                FUN   = Tb_mussel,  
                l     = 0.1,  
                h     = 0.05,  
                T_g   = 30,  
                S     = 500,  
                k_d   = 0.2,  
                u     = 1,  
                psi   = 30,  
                evap  = TRUE,  
                cl    = 0.5,  
                group = "solitary")

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40,  
                FUN   = Tb_mussel,  
                l     = 0.1,  
                h     = 0.05,  
                T_g   = 30,  
                S     = 500, 
                k_d   = 0.2,  
                u     = 2, 
                psi   = 30, 
                evap  = TRUE, 
                cl    = 0.5, 
                group = "solitary")

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

abline(a   = 0,
       b   = 1, 
       col = "gray")

legend(x      = "bottomright",
       title  = expression("Wind speed, m/s"), 
       legend = c(0.2,1,2), 
       lty    = c("solid", "dashed", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(seq(0.02, 0.14, 0.01), 
                FUN   = Tb_mussel,  
                T_a   = 25,  
                h     = 0.05,  
                T_g   = 30,  
                S     = 500,  
                k_d   = 0,  
                u     = 0.5,  
                psi   = 60,  
                evap  = FALSE,  
                cl    = 0,  
                group = "solitary")

plot(x    = seq(0.02, 0.14, 0.01), 
     y    = t.seq, 
     type = "l", 
     xlab = "mussel length (m)", 
     ylab = "body temperature (°C)", 
     ylim = c(25,32))

t.seq <- lapply(seq(0.02, 0.14, 0.01),  
                FUN   = Tb_mussel,  
                T_a   = 25,  
                h     = 0.05,  
                T_g   = 30,  
                S     = 500,  
                k_d   = 0,  
                u     = 1,  
                psi   = 60,  
                evap  = FALSE, 
                cl    = 0,  
                group = "solitary")

points(x    = seq(0.02, 0.14, 0.01), 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(seq(0.02, 0.14, 0.01),  
                FUN   = Tb_mussel,  
                T_a   = 25,  
                h     = 0.05,  
                T_g   = 30,  
                S     = 500,  
                k_d   = 0,  
                u     = 3,  
                psi   = 60,  
                evap  = FALSE,  
                cl    = 0,  
                group = "solitary")

points(x    = seq(0.02, 0.14, 0.01), 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

t.seq <- lapply(seq(0.02, 0.14, 0.01),  
                FUN   = Tb_mussel,  
                T_a   = 25,  
                h     = 0.05,  
                T_g   = 30,  
                S     = 500,  
                k_d   = 0,  
                u     = 1,  
                psi   = 60,  
                evap  = TRUE,  
                cl    = 0,  
                group = "solitary")

points(x    = seq(0.02, 0.14, 0.01), 
       y    = t.seq, 
       type = "l", 
       col  = "red")

legend(x      = "bottomright",
       title  = expression("Wind speed, m/s"), 
       legend = c(0.5,1,3), 
       lty    = c("solid", "dashed", "dotted"))

## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40,  
                FUN  = Tbed_mussel,  
                l    = 0.1,  
                S    = 500,  
                k_d  = 0.2,  
                u    = 0.2,  
                evap = FALSE)

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", 
     ylab = "body temperature (°C)", 
     ylim = c(20, 50))

t.seq <- lapply(20:40,  
                FUN  = Tbed_mussel,  
                l    = 0.1,  
                S    = 500,  
                k_d  = 0.2,  
                u    = 1,  
                evap = FALSE)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40,  
                FUN  = Tbed_mussel,  
                l    = 0.1,  
                S    = 500,  
                k_d  = 0.2,  
                u    = 2,  
                evap = FALSE)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

t.seq <- lapply(20:40,  
                FUN  = Tbed_mussel,  
                l    = 0.1,  
                S    = 500,  
                k_d  = 0.2,  
                u    = 0.2,  
                evap = TRUE)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "solid", 
       col  = "red")

abline(a   = 0,
       b   = 1, 
       col = "gray")

legend(x      = "bottomright",
       title  = expression("Wind speed, m/s"), 
       legend = c(0.2,1,2), 
       lty    = c("solid", "dashed", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40,  
                FUN      = Tb_limpet,  
                T_r      = 30,  
                l        = 0.0176,  
                h        = 0.0122,  
                S        = 1000,  
                u        = 1,  
                psi      = 30,  
                c        = 1,  
                position = "anterior")

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", 
     ylab = "body temperature (°C)", 
     ylim = c(25, 50))

t.seq <- lapply(20:40,  
                FUN      = Tb_limpet,  
                T_r      = 30,  
                l        = 0.0176,  
                h        = 0.0122,  
                S        = 1300,  
                u        = 1,  
                psi      = 30,  
                c        = 1,  
                position = "anterior")

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40,  
                FUN      = Tb_limpet,  
                T_r      = 30,  
                l        = 0.0176,  
                h        = 0.0122,  
                S        = 1600,  
                u        = 1,  
                psi      = 30,  
                c        = 1,  
                position = "anterior")

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

abline(a   = 0,
       b   = 1, 
       col = "gray")

legend(x      = "bottomright",
       title  = expression("Solar irradiance, W/m^2"), 
       legend = c(1000, 1300, 1600), 
       lty    = c("solid", "dashed", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

t.seq <- lapply(20:40, 
                FUN      = Tb_limpet,  
                T_r      = 30,  
                l        = 0.0176,  
                h        = 0.0122,  
                S        = 800,  
                u        = 1,  
                psi      = 30,  
                c        = 1,  
                position = "anterior")

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", 
     ylab = "body temperature (°C)", 
     ylim = c(25, 50))

t.seq <- lapply(20:40,  
                FUN      = Tb_limpetBH,  
                T_r      = 30,  
                l        = 0.0176,  
                h        = 0.0122,  
                S        = 800,  
                u        = 1,  
                s_aspect = 90,  
                s_slope  = 60,
                c        = 1)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dashed")

t.seq <- lapply(20:40,  
                FUN   = Tb_snail,  
                l     = 0.012,  
                S = 800,  
                u    = 1,  
                CC    = 0.5,  
                WL    = 0,  
                WSH   = 10)

points(x    = 20:40, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

abline(a   = 0,
       b   = 1, 
       col = "gray")

legend(x      = "bottomright",
       title  = expression("function"), 
       legend = c("Tb_limpet", "Tb_limpetBH", "Tb_Snail"), 
       lty    = c("solid", "dashed", "dotted"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------
par(mar = c(5,  5, 3, 2))
blr.seq= lapply(293:313, FUN=boundary_layer_resistance, e_s= 2.5, e_a = 2.3, elev = 500, D = 0.007, u = 2)
plot(x    = 293:313, 
     y    = blr.seq, 
     type = "l", 
     xlab = "air temperature (K)", 
     ylab = expression("boundary layer resistance" ~ (s ~ cm^{-1})))

## ---- fig.height = 4, fig.width = 6-------------------------------------------

plot(x    = 1:24, 
     y    = Tsoil(T_g_max = 30, 
                  T_g_min = 15, 
                  hour    = 1:24,  
                  depth   = 0), 
     type = "l", 
     xlab = "hour", 
     ylab = "soil temperature (°C)")

points(x    = 1:24, 
       y    = Tsoil(T_g_max = 30,  
                    T_g_min = 15,  
                    hour    = 1:24,  
                    depth   = 5), 
       type = "l", 
       lty  = "dashed")

points(x    = 1:24, 
       y    = Tsoil(T_g_max = 30,   
                    T_g_min = 15,   
                    hour    = 1:24,   
                    depth   = 10), 
       type = "l", 
       lty  = "dotted")

points(x    = 1:24, 
       y    = Tsoil(T_g_max = 30,   
                    T_g_min = 15,   
                    hour    = 1:24,   
                    depth   = 20), 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topleft",
       title  = "depth (cm)", 
       legend = c(0,5,10,20), 
       lty    = c("solid", "dashed", "dotted", "dotdash"))


## ---- fig.height = 4, fig.width = 4-------------------------------------------

rad.seq <- sapply(20:40, 
                  FUN            = Qthermal_radiation_absorbed,  
                  T_g            = 30,  
                  epsilon_ground = 0.97,  
                  a_longwave     = 0.965)

t.seq <- mapply(FUN     = Tb_salamander_humid,  
                T_a     = 20:40,  
                r_i     = 4, 
                r_b     = 1, 
                D       = 0.01, 
                elev    = 500, 
                e_a     = 1.5, 
                e_s     = 2.5, 
                Qabs    = rad.seq, 
                epsilon = 0.96)

plot(x    = 20:40, 
     y    = t.seq, 
     type = "l", 
     xlab = "ambient temperature (°C)", ylab = "humid operative temperature (°C)")

abline(a    = 0, 
       b    = 1, 
       lty  = "dotted")

# check higher skin resistance-> Te closer to air due to reduced cooling

  t.seq <- mapply(FUN     = Tb_salamander_humid,  
                  T_a     = 20:40,  
                  r_i     = 20, 
                  r_b     = 1, 
                  D       = 0.01, 
                  elev    = 500, 
                  e_a     = 1.5, 
                  e_s     = 2.5, 
                  Qabs    = rad.seq,  
                  epsilon = 0.96)
  points(x    = 20:40, 
         y    = t.seq, 
         type = "l", 
         col  = "red")

# check higher vpd-> lower Te due to greater cooling

  t.seq <- mapply(FUN     = Tb_salamander_humid,  
                  T_a     = 20:40,  
                  r_i     = 4, 
                  r_b     = 1, 
                  D       = 0.01, 
                  elev    = 500, 
                  e_a     = 1.0, 
                  e_s     = 2.5, 
                  Qabs    = rad.seq,  
                  epsilon = 0.96)
  points(x    = 20:40, 
         y    = t.seq, 
         type = "l", 
         col  = "blue")

legend(x      = "topleft",
       title  = NULL, 
       legend = c("base scenario", "higher skin resistance", "higher vpd"), 
       lty    = c("solid"), 
       col    = c("black", "red", "blue"))


## -----------------------------------------------------------------------------

# set up input data as variables
# latitude and longitude (degrees) and elevation (m)

  lat  <- 35.69
  lon  <- -105.944
  elev <- 2121 

# minimum and maximum of air and soil temperatures, respectively (C)

  Tmin   <- 10
  Tmax   <- 25 
  Tmin_s <- 15 
  Tmax_s <- 30 

# wind speed (m/s)

  u <- 0.5 

# assumptions
#  atmospheric transmissivity and albedo

  tau <- 0.7
  rho <- 0.6 
 
# initial assumption of body temperature (C)

  Tb0 <- 25

  doy   <- day_of_year("2021-06-01", 
                       format= "%Y-%m-%d") 
  snoon <- solar_noon(lon = lon, 
                      doy = doy) 
  dayl  <- daylength(lat = lat,
                     doy = doy) 

# time of sunrise

  tr <- snoon - dayl / 2 

# time of sunset

  ts <- snoon + dayl / 2 


## ---- fig.height = 4, fig.width = 6-------------------------------------------

psi_deg <- sapply(0:23, 
                  FUN = zenith_angle, 
                  doy = doy, 
                  lat = lat, 
                  lon = lon)

psi_rad <- degrees_to_radians(psi_deg)

Srad <- sapply(psi_rad, 
               FUN  = solar_radiation, 
               doy  = doy, 
               tau  = tau, 
               elev = elev, 
               rho  = rho)

# Separate into direct, diffuse, and reflected solar radiation

  Sdir <- Srad[1,]
  Sdif <- Srad[2,]
  Sref <- Srad[3,]
 
plot(x    = 0:23, 
     y    = Sdir, 
     type = "l", 
     xlab = "hour", 
     ylab = expression(radiation ~ (W/m^{2})), 
     ylim = c(0,1200))

points(x    = 0:23, 
       y    = Sdif, 
       type = "l", 
       lty  = "dotted")

points(x    = 0:23, 
       y    = Sref, 
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "Solar radiation component", 
       legend = c("direct", "diffuse", "reflected"), 
       lty    = c("solid", "dotted", "dashed"))


## -----------------------------------------------------------------------------

Ta <- sapply(0:23, 
             FUN   = diurnal_temp_variation_sineexp, 
             T_max = Tmax,  
             T_min = Tmin,  
             t_r   = tr,  
             t_s   = ts,  
             alpha = 2.59,  
             beta  = 1.55,  
             gamma = 2.2) #air temperature (C)

Ts <- sapply(0:23,  
             FUN   = diurnal_temp_variation_sine,  
             T_max = Tmax_s,  
             T_min = Tmin_s) #soil temperature (C)


## -----------------------------------------------------------------------------

mass <- 10 #grams

# solar absorptivity
 
  a <- 0.9 

# estimate surface area (m^2) from mass (g)

  A <- surface_area_from_mass(m     = mass, 
                              taxon = "lizard")

# estimate projected (silhouette) area as a portion of surface area

  psa <- sapply(psi_deg, 
                FUN     = proportion_silhouette_area, 
                taxon   = "lizard", 
                posture = "elevated")

# change negative values to zero

  psa[psa < 0] <- 0

# proportion of surface area in contact with ground, assume 0.25   

  psa_g= 0.25  

# Total radiation 

  Qabs <- psa * Sdir + 0.5 * Sdif + (0.5-psa_g) * Sref
 

## -----------------------------------------------------------------------------

# Use DRYAIR from NicheMapR to estimate the thermal conductivity of air and kinematic viscosity
# estimate Barometric pressure (pascal)

  ap <- airpressure_from_elev(elev) * 1000 
 
  DRYAIRout <- DRYAIR(db  = Ta, 
                      bp  = ap, 
                      alt = elev)

  # thermal conductivity (Wm^-2K^-1) 
  
    K   <- DRYAIRout$thcond 
  
  # kinematic viscosity (m2 s-1)  
    
    nu  <- DRYAIRout$viskin 

  # approximate snout vent length (m) for Sceloporus

    svl <- 0.006 

  # approximate characteristic dimension as cube root of volume, assuming density of water as 1000kg/m^3
    
    D <- ((mass / 1000) / 1000) ^ (1 / 3)
    
  # We will use the average of K and nu across the day for simplicity and since there's not a substantial variation

    K  <-  mean(K)
    nu <- mean(nu)

  # Estimate the heat transfer coefficient using an empirical relationship for lizards

    H_L <- heat_transfer_coefficient(u     = u,
                                     D     = D,
                                     K     = 25.7 * 10^(-3), 
                                     nu    = 15.3 * 10^(-6), 
                                     taxon = "lizard_surface")

  # Also illustrate estimations using a spherical approximation and a simplified version of the approximation.

    heat_transfer_coefficient_approximation(u     = u,
                                            D     = D,
                                            K     =  25.7 * 10^(-3), 
                                            nu    = 15.3 * 10^(-6), 
                                            taxon = "lizard")

    heat_transfer_coefficient_simple(u    = 0.5,
                                     D    = D, 
                                     type = "Spotila")


## -----------------------------------------------------------------------------

epsilon_s <- 0.965

TeGates <- rep(NA, 24)

for (hr in 1:24) {
  TeGates[hr] <- Tb_Gates(A       = A, 
                          D       = D, 
                          psa_dir = psa[hr], 
                          psa_ref = 0.5 -psa_g, 
                          psa_air = 0.5, 
                          psa_g   = psa_g, 
                          T_g     = Ts[hr], 
                          T_a     = Ta[hr], 
                          Qabs    = Qabs[hr]*A, 
                          epsilon = epsilon_s, 
                          H_L     = H_L, 
                          ef      = 1.3, 
                          K       = 0.5)
}


## -----------------------------------------------------------------------------

TeCN <- rep(NA, 24)

for (hr in 1:24) {
  TeCN[hr] <- Tb_CampbellNorman(T_a     = Ta[hr], 
                                T_g     = Ts[hr], 
                                S       = Qabs[hr], 
                                a_l     = 0.96, 
                                epsilon = epsilon_s, 
                                c_p     = 29.3, 
                                D       = D, 
                                u       = u)
}

#S is solar radiation flux (W m^-2), so we divide by surface area, A


## -----------------------------------------------------------------------------

TeLiz <- rep(NA, 24)

for (hr in 1:24) {
  TeLiz[hr] <- Tb_lizard(T_a       = Ta[hr], 
                         T_g       = Ts[hr], 
                         u         = u, 
                         svl       = svl * 1000, 
                         m         = mass, 
                         psi       = psi_deg[hr], 
                         rho_s     = rho, 
                         elev      = elev, 
                         doy       = doy, 
                         sun       = TRUE, 
                         surface   = TRUE, 
                         a_s       = a, 
                         a_l       = 0.965, 
                         epsilon_s = epsilon_s, 
                         F_d       = 0.8, 
                         F_r       = 0.5, 
                         F_a       = 0.5, 
                         F_g       = 0.5)
}

TeFei <- rep(NA, 24)

for (hr in 1:24) {
  TeFei[hr] <- Tb_lizard_Fei(T_a       = Ta[hr], 
                             T_g       = Ts[hr], 
                             S         = Qabs[hr], 
                             lw        = 30, 
                             shade     = 0.0, 
                             m         = 10.5,
                             Acondfact = 0.05,
                             Agradfact = 0.4)
}


## ---- fig.height = 4, fig.width = 6-------------------------------------------

par(mar = c(4, 4, 1, 4), 
    mgp = c(2, 1, 0), 
    las = 0)

# Gates

  plot(x    = 1:24, 
       y    = TeGates, 
       type = "l", 
       xlab = "Hour", 
       ylab = "Temperature (°C)", 
       col = "blue", 
       ylim = c(10,50)) 

# Campbell

  points(x    = 1:24,  
         y    = TeCN, 
         type = "l", 
         col  = "blue", 
         lty  = "dotted") 

# Buckley 2008

  points(x    = 1:24,  
         y    = TeLiz, 
         type = "l", 
         col  = "blue", 
         lty  = "dashed") 

# Fei et al.

  points(x    = 1:24,  
         y    = TeFei, 
         type = "l", 
         col  = "blue", 
         lty  = "dotdash") 

points(x    = 1:24,  
       y    = Ta, 
       type = "l", 
       col  = "orange")

points(x    = 1:24,  
       y    = Ts, 
       type = "l", 
       col  = "purple")
 
# add additional axis with radiation

  par(new = TRUE)

  plot(x    = 1:24, 
       y    = Sdir, 
       pch  = 16, 
       axes = FALSE, 
       xlab = NA, 
       ylab = NA, 
       type = "l", 
       col  = "green")

  axis(side = 4)

  mtext(side = 4, 
        line = 2, 
        text = "Radiation", 
        col  = "green")

legend(x      = "topleft",
       legend = c("Ta", "Ts", "Tb_Gates", "Tb_CampbellNorman", "Tb_lizard", "Tb_lizard_Fei"),
       lty    = c("solid", "solid", "solid", "dotted", "dashed", "dotdash"), 
       pch    = NA, 
       col    = c("orange", "purple", "blue", "blue", "blue", "blue"))


## -----------------------------------------------------------------------------

Qabs <- rep(NA, 24)

for (hr in 1:24) {
  Qabs[hr] <- Qradiation_absorbed(a       = a, 
                                  A       = A, 
                                  psa_dir = psa[hr],
                                  psa_dif = 0.5,
                                  psa_ref = 0.5-psa_g, 
                                  S_dir   = Sdir[hr], 
                                  S_dif   = Sdif[hr], 
                                  rho     = rho)
}


## -----------------------------------------------------------------------------

# Use Gates model as Te estimate

  Te <- TeGates

Qemit <- rep(NA, 24)

for (hr in 1:24) {
  Qemit[hr] <- Qemitted_thermal_radiation(epsilon  = epsilon_s, 
                                          A        = A, 
                                          psa_dir  = 0.5, 
                                          psa_ref  = 0.5-psa_g, 
                                          T_b      = Te[hr] + 273, 
                                          T_g      = Ts[hr] + 273, 
                                          T_a      = Ta[hr] + 273, 
                                          enclosed = FALSE)
}


## -----------------------------------------------------------------------------

Qconv <- rep(NA, 24)

for (hr in 1:24) {
  Qconv[hr] <- Qconvection(T_a        = Ta[hr] + 273.15,
                           T_b        = Te[hr] + 273.15,
                           H          = H_L,
                           A          = A, 
                           proportion = 0.67, 
                           ef         = 1.3)
}


## -----------------------------------------------------------------------------

Qcond <- rep(NA, 24)

for (hr in 1:24) {
  Qcond[hr] <- Qconduction_animal(T_g        = Ts[hr] + 273,
                                  T_b        = Te[hr] + 273,
                                  d          = 0.025, #assume for conductive heat exchange occurs down to 2.5cm
                                  K          = 0.5,
                                  A          = A, 
                                  proportion = psa_g)
}


## -----------------------------------------------------------------------------

Qmet  <- 0
Qevap <- 0


## ---- fig.height = 4, fig.width = 7-------------------------------------------

Qnet <- Qnet_Gates(Qabs  = Qabs, 
                   Qemit = Qemit, 
                   Qconv = Qconv, 
                   Qcond = Qcond, 
                   Qmet  = Qmet, 
                   Qevap = Qevap)

par(mar = c(5, 5, 3, 5))

plot(x    = 1:24, 
     y    = Qnet, 
     type = "l", 
     xlab = "hour", 
     ylab = expression("heat flux" ~ (W/m^{2})), 
     ylim = c(-1,4))

points(x    = 1:24, 
       y    = Qabs, 
       type = "l", 
       lty  = "dotted")

points(x    = 1:24, 
       y    = Qemit, 
       type = "l", 
       lty  = "dashed")

points(x    = 1:24, 
       y    = Qconv, 
       type = "l", 
       lty  = "dotdash")

points(x    = 1:24, 
       y    = Qcond, 
       type = "l", 
       lty  = "twodash")

legend(x      = "topright", 
       title  = "component", 
       legend = c("net radition, Qnet", "solar radiation, Qabs", "thermal radiation, Qemit", "convection, Qconv", "conduction, Qcond"), 
       lty    = c("solid", "dotted", "dashed", "dotdash", "twodash"))


## ----include=FALSE------------------------------------------------------------

suppressWarnings(par(oldpar))

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## -----------------------------------------------------------------------------

# Load TrenchR package

  library(TrenchR)


## -----------------------------------------------------------------------------

day_of_year(day    = "2017-04-22", 
            format = "%Y-%m-%d")


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = 1:365, 
     y    = dec_angle(1:365), 
     type = "l", 
     xlab = "day of year", 
     ylab = "declination angle (radian)")


## ---- fig.height = 3, fig.width = 6-------------------------------------------

zenith <- zenith_angle(hour = 1:24, 
                       doy  = 200, 
                       lat  = 47.61, 
                       lon  = -122.33)

plot(x    = 1:24, 
     y    = zenith, 
     type = "l", 
     ylab = "zenith angle (°)", 
     xlab = "hour")

zenith <- zenith_angle(hour = 1:24, 
                       doy  = 1, 
                       lat  = 47.61, 
                       lon  = -122.33)  
points(x    = 1:24, 
       y    = zenith, 
       type = "l", 
       lty  = "dotted")

zenith <- zenith_angle(hour = 1:24, 
                       doy  = 100, 
                       lat  = 47.61, 
                       lon  = -122.33)
points(x    = 1:24, 
       y    = zenith, 
       type = "l", 
       lty  = "dotdash")
  
zenith <- zenith_angle(hour = 1:24, 
                       doy  = 300, 
                       lat  = 47.61, 
                       lon  = -122.33)
points(x    = 1:24, 
       y    = zenith, 
       type = "l", 
       lty  = "dashed")

legend(x      = "bottomleft", 
       title  = "day of year", 
       legend = c(1, 100, 200, 300), 
       lty    = c("dotted", "dotdash", "solid", "dashed"))


## ---- fig.height = 3, fig.width = 6-------------------------------------------

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 60, 
                       lon  = -122.33)

plot(x    = 1:365, 
     y    = zenith, 
     type = "l", 
     ylim = range(0,90), 
     ylab = "zenith angle (°)", 
     xlab = "day of year")

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 40, 
                       lon  = -122.33)
points(x    = 1:365, 
       y    = zenith, 
       type = "l", 
       lty  = "dotted")

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 20, 
                       lon  = -122.33)

points(x    = 1:365, 
       y    = zenith, 
       type = "l", 
       lty  = "dotdash")

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 0, 
                       lon  = -122.33)

points(x    = 1:365, 
       y    = zenith, 
       type = "l", 
       lty  = "dashed")

legend(x      = "topleft", 
       title  = "latitude (°)", 
       legend = c(0, 20, 40, 60), 
       lty    = c("dashed", "dotdash", "dotted", "solid"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = 1:365, 
     y    = solar_noon(lon = 150, doy = 1:365), 
     type = "l", 
     ylim = c(11.3,12.7), 
     ylab = "time of solar noon (hour)", 
     xlab = "day of year")

points(x    = 1:365, 
       y    = solar_noon(lon = 155, doy = 1:365), 
       type = "l", 
       lty  = "dotted")

points(x    = 1:365, 
       y    = solar_noon(lon = 145, doy = 1:365), 
       type = "l", 
       lty  = "dashed")

abline(a = 12,
       b = 0)

legend(x      = "topright", 
       title  = "longitude (°)", 
       legend = c(145, 150, 155), 
       lty    = c("dashed", "solid", "dotted"))


## ---- fig.height = 3, fig.width = 6-------------------------------------------

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 173, 
                    lat = 47.61, 
                    lon = -122.33))

plot(x    = 5:18, 
     y    = az, 
     type = "l", 
     xlab = "hour", 
     ylab = "azimuth angle (°)")

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 356, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 5:18, 
       y    = az, 
       type = "l",
       lty  = "dashed")

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 266, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 5:18, 
       y    = az, 
       type = "l",
       lty  = "dotted")

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 228, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 5:18, 
       y    = az, 
       type = "l",
       lty  = "dotdash")

legend(x      = "topleft", 
       title  = "day of year", 
       legend = c(173, 228, 266, 356), 
       lty    = c("solid", "dotdash", "dotted", "dashed"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

az <- unlist(lapply(1:365, 
                    FUN = azimuth_angle, 
                    hour = 12, 
                    lat = 47.61, 
                    lon = -122.33))

plot(x    = 1:365, 
     y    = az, 
     type = "l", 
     xlab = "day of year", 
     ylab = "azimuth angle (°)", ylim=c(0,120))

az <- unlist(lapply(1:365, 
                    FUN = azimuth_angle, 
                    hour = 9, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 1:365, 
       y    = az, 
       type = "l",
       lty  = "dashed")

az <- unlist(lapply(1:365, 
                    FUN = azimuth_angle, 
                    hour = 6, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 1:365, 
       y    = az, 
       type = "l",
       lty  = "dotted")

legend(x      = "topright", 
       title  = "hour", 
       legend = c(6, 9, 12), 
       lty    = c("dotted", "dashed", "solid"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = 1:365,  
     y    = daylength(lat = 10, doy = 1:365), 
     type = "l", 
     ylim = c(8, 22), 
     xlab = "day of year",
     ylab = "day length (hour)")

points(x    = 1:365,  
       y    = daylength(lat = 35, doy = 1:365),  
       type = "l",  
       lty  = "dashed")

points(x    = 1:365, 
       y    = daylength(lat = 60, doy = 1:365),  
       type = "l",  
       lty   = "dotted")

legend(x      = "topright", 
       title  = "latitude (°)", 
       legend = c(10, 35, 60), 
       lty    = c("solid", "dashed", "dotted"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = 1:4000, 
     y    = airpressure_from_elev(elev = 1:4000), 
     type = "l", 
     xlab = "elevation (m)", 
     ylab = "air pressure (kPa)")


## ---- fig.height = 3, fig.width = 6-------------------------------------------

oldpar<-par()

par(mar=c(5, 5, 3, 2))

zen <- unlist(lapply(1:24, 
                     FUN = zenith_angle, 
                     doy = 200, 
                     lat = 47.61, 
                     lon = -122.33))

zen <- zen * 2 * pi / 360

rd <- lapply(zen, 
             FUN  = solar_radiation, 
             doy  = 200, 
             tau  = 0.6, 
             elev = 1500, 
             rho  = 0.7)

rd <- matrix(unlist(rd), 
             nrow  = 3, 
             byrow = FALSE)

plot(x    = 1:24, 
     y    = rd[1,], 
     type = "l", 
     xlab = "hour", 
     ylab = expression(radiation ~ (W/m^{2})), 
     ylim = c(0, 800))

points(x    = 1:24, 
       y    = rd[2,], 
       type = "l", 
       lty = "dotted")

points(x    = 1:24, 
       y    = rd[3,], 
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "solar radiation flux", 
       legend = c("direct", "diffuse", "reflected"), 
       lty    = c("solid", "dotted", "dashed"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar = c(5, 5, 3, 2))

psi.seq <- seq(from = 0, 
               to   = 90, 
               by   = 5)

rd <- lapply(psi.seq * 2 * pi / 360, 
             FUN  = solar_radiation, 
             doy  = 200, 
             tau  = 0.75, 
             elev = 1500, 
             rho  = 0.7)

rd <- matrix(unlist(rd), 
             nrow  = 3, 
             byrow = FALSE)

plot(x    = psi.seq, 
     y    = rd[1,], 
     type = "l", 
     xlab = "zenith angle (°)", 
     ylab = expression(radiation ~ (W/m^{2})), 
     ylim = c(0, 1200))

points(x    = psi.seq, 
       y    = rd[2,], 
       type = "l", 
       lty = "dotted")

points(x    = psi.seq, 
       y    = rd[3,], 
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "solar radiation flux", 
       legend = c("direct", "diffuse", "reflected"), 
       lty    = c("solid", "dotted", "dashed"))


## ---- fig.height = 3, fig.width = 6-------------------------------------------

par(mar = c(5, 5, 3, 2))

r.seq <- lapply(seq(4, 20), 
                FUN    = direct_solar_radiation, 
                lat    = 43.57,
                doy    = 112,
                elev   = 866,
                t0     = 12, 
                method = "Campbell 1977")

r.seq <- unlist(r.seq)

plot(x    = seq(4, 20), 
     y    = r.seq, 
     type = "l", 
     xlab = "hour", 
     ylab = expression(radiation ~ (W/m^{2})))

r.seq <- lapply(seq(4, 20), 
                FUN    = direct_solar_radiation, 
                lat    = 43.57,
                doy    = 112,
                elev   = 866,
                t0     = 12, 
                method = "Gates 1962")

points(x    = seq(4, 20), 
       y    = r.seq,
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "Radiation algorithm", 
       legend = c("Campbell 1977", "Gates 1962"), 
       lty    = c("solid", "dashed"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar = c(5, 5, 3, 2))

r.seq <- lapply(seq(1, 365, 31), 
                FUN  = monthly_solar_radiation, 
                lat  = 43.57,
                lon  = -116.22,
                elev = 866,
                T    = 20,
                hp   = 15,
                P    = 15)

r.seq <- unlist(r.seq)

plot(x    = seq(1, 365, 31), 
     y    = r.seq, 
     type = "l", 
     xlab = "day of year", 
     ylab = expression(radiation ~ (W/m^{2})))


## ---- fig.height = 3, fig.width = 6-------------------------------------------

par(mar=c(5, 5, 3, 2))

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Erbs", 
                lat      = 40, 
                sol.elev = 60)

plot(x    = seq(0, 1, 0.1), 
     y    = r.seq, 
     type = "l", 
     xlab = expression("clearness index" ~ k[t]), 
     ylab = "diffuse fraction", 
     ylim = c(0, 1))

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Liu_Jordan", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "dashed")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Orgill_Hollands", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "dotted")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Olyphant", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "dotdash")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Reindl-1", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "longdash")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Reindl-2", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "longdash")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Lam_Li", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "twodash")

legend(x      = "topright", 
       title  = "Partitioning method", 
       legend = c("Erbs", "Liu_Jordan", "Orgill_Hollands", "Olyphant", "Reindl-1", "Reindl-2", "Lam_Li"), 
       lty    = c("solid", "dashed", "dotted", "dotdash", "longdash", "longdash", "twodash"))


## ---- fig.height = 3, fig.width = 6-------------------------------------------

par(mar = c(5, 5, 3, 2))
r.seq <- lapply(seq(20, 85), 
                FUN = proportion_diffuse_solar_radiation, 
                p_a = 86.1, 
                rho = 0.25)

plot(x    = seq(20,85), 
     y    = r.seq,
     type = "l", 
     xlab = "zenith angle (°)", 
     ylab = "diffuse fraction")

r.seq <- lapply(seq(20, 85), 
                FUN = proportion_diffuse_solar_radiation, 
                p_a = 96.1, 
                rho = 0.25)

points(x    = seq(20,85), 
       y    = r.seq, 
       type = "l", 
       lty  = "dashed")

r.seq <- lapply(seq(20, 85), 
                FUN = proportion_diffuse_solar_radiation, 
                p_a = 76.1, 
                rho = 0.25)

points(x    = seq(20,85), 
       y    = r.seq, 
       type = "l", 
       lty = "dotted")


## ---- fig.height = 3, fig.width = 4-------------------------------------------

u_r <- c(0.01, 0.025, 0.05, 0.1, 0.2)
zr  <- c(0.1, 0.25, 0.5, 0.75, 1)

surface_roughness(u_r = u_r, 
                  zr  = zr)


## ---- fig.height = 3, fig.width = 4-------------------------------------------

zs <- seq(0, 2, 0.1) 

us <- wind_speed_profile_neutral(u_r = 1, 
                                 zr  = 2, 
                                 z0  = 0.2,
                                 z   = zs)

plot(x    = us,
     y    = zs, 
     type = "l", 
     xlab = "wind speed, u (m/s)", 
     ylab = "height, z (m)")

points(x    = wind_speed_profile_neutral(u_r = 0.5,
                                         zr  = 2, 
                                         z0  = 0.2, 
                                         z   = zs), 
       y    = zs, 
       type = "l", 
       lty  = "dashed")

points(x    = wind_speed_profile_neutral(u_r = 0.25, 
                                         zr  = 2, 
                                         z0  = 0.2, 
                                         z   = zs), 
       y    = zs, 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright", 
       title  = "wind speed at 2m", 
       legend = c(0.25, 5, 1), 
       lty    = c("dotted", "dashed", "solid"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

z.seq <- seq(0, 2, by = 0.1)
t.seq <- air_temp_profile_neutral(T_r = 20, 
                                  zr  = 1, 
                                  z0  = 0.02, 
                                  z   = z.seq, 
                                  T_s = 25)

plot(x    = t.seq, 
     y    = z.seq, 
     type = "l", 
     xlab = "Temperature (°C)", 
     ylab = "z (height above ground in m)")

t.seq <- air_temp_profile_neutral(T_r = 20, 
                                  zr  = 1, 
                                  z0  = 0.05, 
                                  z   = z.seq, 
                                  T_s = 25)

points(x    = t.seq, 
       y    = z.seq, 
       type = "l", 
       lty  = "dotted")

t.seq <- air_temp_profile_neutral(T_r = 20, 
                                  zr  = 1, 
                                  z0  = 0.08, 
                                  z   = z.seq, 
                                  T_s = 25)
points(x    = t.seq, 
       y    = z.seq, 
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "surface roughness", 
       legend = c(0.02, 0.05, 0.08), 
       lty    = c("solid", "dotted", "dashed"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

z.seq <- seq(0, 2, by = 0.1)

t.seq <- air_temp_profile(T_r = 20, 
                          u_r = 0.5, 
                          zr  = 2, 
                          z0  = 0.02, 
                          z   = z.seq, 
                          T_s = 25)

plot(x    = t.seq, 
     y    = z.seq, 
     type = "l", 
     xlab = "Temperature (°C)", 
     ylab = "z (height above ground in m)")

t.seq <- air_temp_profile(T_r = 20, 
                          u_r = 0.5, 
                          zr  = 2, 
                          z0  = 0.05, 
                          z   = z.seq, 
                          T_s = 25)

points(x    = t.seq, 
       y    = z.seq, 
       type = "l", 
       lty  = "dotted")

t.seq <- air_temp_profile(T_r = 20, 
                          u_r = 0.5, 
                          zr  = 2, 
                          z0  = 0.08, 
                          z   = z.seq, 
                          T_s = 25)

points(t.seq, 
       z.seq, 
       type = "l", 
       lty = "dashed")

legend(x      = "topright", 
       title  = "surface roughness", 
       legend = c(0.02, 0.05, 0.08), 
       lty    = c("solid", "dotted", "dashed"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

z.seq <- seq(0, 2, by = 0.1)
t.seq <- lapply(z.seq, 
                FUN = air_temp_profile_segment, 
                T_r = c(25, 22, 20),
                u_r = c(0.01, 0.025, 0.05), 
                zr  = c(0.05, 0.25, 0.5), 
                z0  = c(0.01, 0.05, 0.1), 
                T_s = 27)

plot(x    = t.seq, 
     y    = z.seq, 
     type = "l",  
     xlab = "Temperature (°C)",  
     ylab = "z (height above ground in m)")

points(x = c(27, 25, 22, 20),
       y = c(0, 0.05, 0.25, 0.5))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

z.seq <- seq(0, 2, by = 0.1)
u.seq <- lapply(z.seq, 
                FUN = wind_speed_profile_segment, 
                u_r = c(0.01, 0.025, 0.05), 
                zr  = c(0.05, 0.25, 0.5),
                z0  = c(0.01, 0.05, 0.1))

plot(x    = u.seq, 
     y    = z.seq, 
     type = "l", 
     xlab = "Wind speed (m/s)", 
     ylab = "z (height above ground in m)",
     xlim=c(0,0.05))

points(x = c(0.01, 0.025, 0.05),
       y = c(0.05, 0.25, 0.5))


## ---- fig.height = 3, fig.width = 6-------------------------------------------

plot(x    = 1:23, 
     y    = diurnal_temp_variation_sine(T_max = 30, T_min = 10, t=1:23), 
     type = "l", 
     xlab = "hour", 
     ylab = "temperature (°C)")

t.seq <- lapply(1:23, 
                FUN   = diurnal_temp_variation_sineexp, 
                T_max = 30, 
                T_min = 10, 
                t_r   = 6, 
                t_s   = 18)

points(x    = 1:23, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

points(1:23, 
       y = diurnal_temp_variation_sinesqrt(t      = 1:23, 
                                           t_r    = 6, 
                                           t_s    = 18, 
                                           T_max  = 30, 
                                           T_min  = 10, 
                                           T_minp = 12), 
       type = "l", 
       lty = "dashed")

legend(x      = "topleft", 
       title  = "function", 
       legend = c("sine", "sine-exp", "sine-sqrt"),
       lty    = c("solid", "dotted", "dashed"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar=c(5, 5, 3, 2))

r.seq <- lapply(1:24, 
                FUN = diurnal_radiation_variation, 
                doy = 172, 
                S = 8000, 
                lon = -112.07, 
                lat = 33.45)

plot(x    = 1:24, 
     y    = r.seq, 
     type = "l", 
     xlab = "hour", 
     ylab = expression(radiation ~ (W/m^{2})))

r.seq <- lapply(1:24, 
                FUN    = diurnal_radiation_variation, 
                doy    = 356, 
                S      = 4000, 
                lon    = -112.07, 
                lat    = 33.45)

points(x    = 1:24, 
       y    = r.seq, 
       type = "l", 
       lty  = "dotted")

r.seq <- lapply(1:24, 
                FUN    = diurnal_radiation_variation, 
                doy    = 266, 
                S      = 6000, 
                lon    = -112.07, 
                lat    = 33.45)

points(x    = 1:24, 
       y    = r.seq, 
       type = "l", 
       lty  = "dashed")

r.seq <- lapply(1:24, 
                FUN    = diurnal_radiation_variation, 
                doy    = 228, 
                S      = 6000, 
                lon    = -112.07, 
                lat    = 33.45)

points(x    = 1:24, 
       y    = r.seq, 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topright", 
       title  = "day of year", 
       legend = c(172, 228, 266, 356), 
       lty    = c("solid", "dotdash", "dashed", "dotted"))


## -----------------------------------------------------------------------------

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "single.sine")

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "double.sine")

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "single.triangulation")

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "double.triangulation")


## ---- fig.height = 3, fig.width = 4-------------------------------------------

temp_vector <- diurnal_temp_variation_sine(T_max = 20, 
                                           T_min = -10, 
                                           t     = rep(1:24,4))

wind_speed_vector <- runif(n   = 96,
                           min = 0, 
                           max = 0.4)

time_vector <- rep(1:24, 4)
solrad_vector <- unlist(lapply(1:24, 
                               FUN    = diurnal_radiation_variation,
                               doy    = 172, 
                               S = 8000, 
                               lon    = -112.07, 
                               lat    = 33.45))

solrad_vector <- rep(solrad_vector, 4)

params <- list(SSA        = 0.7, 
               epsilon_so = 0.98, 
               k_so       = 0.293, 
               c_so       = 800, 
               dz         = 0.05, 
               z_r        = 1.5, 
               z0         = 0.02, 
               S          = solrad_vector, 
               T_a        = temp_vector, 
               u          = wind_speed_vector, 
               rho_a      = 1.177,
               rho_so     = 1620, 
               c_a        = 1006, 
               TimeIn     = time_vector, 
               dt         = 60*60, 
               shade      = FALSE)

plot(x    = 1:96, 
     y    = temp_vector, 
     type = "l", 
     xlab = "hour", 
     ylab = "temperature (°C)", 
     ylim = c(-10,50))

T_soil <- soil_temperature(z_r.intervals = 12,
                           z_r           = 1.5, 
                           z             = 2, 
                           T_a           = temp_vector, 
                           u             = wind_speed_vector, 
                           Tsoil0        = 20, 
                           z0            = 0.02, 
                           SSA           = 0.7, 
                           TimeIn        = time_vector, 
                           S             = solrad_vector, 
                           water_content = 0.2, 
                           air_pressure  = 85, 
                           rho_so        = 1620, 
                           shade         = FALSE)

points(x    = 1:96, 
       y    = T_soil, 
       lty  = "dashed", 
       type = "l")

T_soil <- soil_temperature(z_r.intervals = 12,
                           z_r           = 1.5, 
                           z             = 4, 
                           T_a           = temp_vector, 
                           u             = wind_speed_vector, 
                           Tsoil0        = 20, 
                           z0            = 0.02, 
                           SSA           = 0.7, 
                           TimeIn        = time_vector, 
                           S             = solrad_vector, 
                           water_content = 0.2, 
                           air_pressure  = 85, 
                           rho_so        = 1620, 
                           shade         = FALSE)

points(x    = 1:96, 
       y    = T_soil, 
       lty  = "dotted", 
       type = "l")

T_soil <- soil_temperature(z_r.intervals = 12,
                           z_r           = 1.5, 
                           z             = 6, 
                           T_a           = temp_vector, 
                           u             = wind_speed_vector, 
                           Tsoil0        = 20, 
                           z0            = 0.02, 
                           SSA           = 0.7, 
                           TimeIn        = time_vector, 
                           S             = solrad_vector, 
                           water_content = 0.2, 
                           air_pressure  = 85, 
                           rho_so        = 1620, 
                           shade         = FALSE)

points(x    = 1:96, 
       y    = T_soil, 
       lty  = "dotdash", 
       type = "l")

legend(x      = "topright", 
       title  = "soil depth interval", 
       legend = c(2, 4, 6), 
       lty    = c("dashed", "dotted", "dotdash"))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = seq(500,1700,100), 
     y    = soil_specific_heat(x_o    = 0.01, 
                               x_m    = 0.6, 
                               x_w    = 0.2, 
                               rho_so = seq(500,1700,100)), 
     type = "l", 
     xlab = expression("bulk density" ~ (kg/m^{3})), 
     ylab = expression("soil specific heat" ~ (J ~ kg^{-1} ~ K^{-1})))

suppressWarnings(par(oldpar))


## -----------------------------------------------------------------------------

soil_conductivity(x      = c(0.10,0.40,0.11,0.01,0.2, 0.18), 
                  lambda = c(0.10,0.40,0.11,0.01,0.2, 0.18), 
                  g_a    = 0.125)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## -----------------------------------------------------------------------------

# Load TrenchR package

  library(TrenchR)


## ---- fig.height = 3, fig.width = 4-------------------------------------------

oldpar<- par()

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0.01, 0.25, 0.01), 
     y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                             taxon = "lizard"), 
     type = "l", 
     xlab = "length (m)", 
     ylab = "mass (g)", 
     ylim = c(0, 200), 
     xlim = c(0, 0.25), 
     las  = 1, 
     bty  = "L", 
     lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "salamander"), 
       type = "l", 
       col  = "blue", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01),
                               taxon = "frog"), 
       type = "l", 
       col  = "red", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "snake"), 
       type = "l", 
       col  = "green", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "turtle"), 
       type = "l", 
       col  = "orange", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "insect"), 
       type = "l", 
       col  = "purple", 
       lwd  = 2)


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar = c(5, 5, 2, 2))

plot(x    = 1:50, 
     y    = surface_area_from_mass(m     = 1:50, 
                                   taxon = "lizard"), 
     type = "l", 
     xlab = "mass (g)", 
     ylab = "", 
     las  = 1, 
     bty  = "L", 
     lwd  = 2)

mtext(text = expression("surface area" ~ (m^{2})), 
      side = 2, 
      line = 3.5)

points(x    = 1:50, 
       y    = surface_area_from_mass(m     = 1:50, 
                                     taxon = "frog"), 
       type = "l", 
       lty  = "dashed", 
       lwd  = 2)

points(x    = 1:50, 
       y    = surface_area_from_mass(m     = 1:50,  
                                     taxon = "salamander"), 
       type = "l", 
       lty  = "dotted", 
       lwd  = 2)
points(x    = seq(0.1, 5, 0.2), 
       y    = surface_area_from_mass(m     = seq(0.1,5,0.2),  
                                     taxon = "insect"), 
       type = "l", 
       lty  = "dotdash", 
       lwd   = 2)

box(bty = "L",
    lwd = 2)


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = seq(0.001, 0.01, 0.001), 
     y    = surface_area_from_volume(V     = seq(0.001, 0.01, 0.001), 
                                     taxon = "lizard"), 
     type = "l", 
     xlab = expression(volume ~ (m^{3})),
     ylab = expression("surface area" ~ (m^{2})))

points(x    = seq(0.001, 0.01, 0.001), 
       y    = surface_area_from_volume(V     = seq(0.001, 0.01, 0.001), 
                                       taxon = "frog"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0.001, 0.01, 0.001),
       y    = surface_area_from_volume(V     = seq(0.001, 0.01, 0.001), 
                                       taxon = "sphere"),
       type = "l", 
       lty  = "dotted")


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0.01, 0.05, 0.001), 
     y    = volume_from_length(l     = seq(0.01, 0.05, 0.001), 
                               taxon = "lizard"), 
     type = "l", 
     xlab = "length (m)", 
     ylab = expression(volume ~ (m^{3})))

points(x    = seq(0.01, 0.05, 0.001), 
       y    = volume_from_length(l     = seq(0.01, 0.05, 0.001), 
                                 taxon = "frog"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0.01, 0.05, 0.001), 
       y    = volume_from_length(l     = seq(0.01, 0.05, 0.001), 
                                 taxon = "sphere"), 
       type = "l", 
       lty  = "dotted")


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0.01, 0.05, 0.001), 
     y    = surface_area_from_length(l = seq(0.01, 0.05, 0.001)), 
     type = "l", 
     xlab = "length (m)", 
     ylab = expression("surface area" ~ (m^{2})))


## ---- fig.height = 3, fig.width = 4-------------------------------------------

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area(psi   = seq(0, 90, 10), 
                                       taxon = "frog"), 
     type = "l", 
     xlab = "zenith angle (°)", 
     ylab = "proportion silhouette area", 
     ylim = range(0, 0.5))

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area(psi     = seq(0, 90, 10), 
                                         taxon   = "lizard", 
                                         raz     = 0, 
                                         posture = "prostrate"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area(psi     = seq(0, 90, 10), 
                                         taxon   = "lizard", 
                                         raz     = 0, 
                                         posture = "elevated"), 
       type = "l", 
       lty  = "dotted")
points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area(psi   = seq(0, 90, 10), 
                                         taxon =  "grasshopper"), 
       type = "l", 
       lty  = "dotdash")

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area_shapes(shape = "spheroid", 
                                              theta = seq(0, 90, 10), 
                                              h     = 0.4, 
                                              d     = 0.39), 
     type = "l", 
     xlab = "theta (°)", 
     ylab = "proportion silhouette area", 
     ylim = c(0, 0.35), 
     lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "spheroid", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.3), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "spheroid", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.2), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "spheroid",
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.1), 
       type = "l", 
       lty  = "dotdash")

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                              theta = seq(0, 90, 10), 
                                              h     = 0.4, 
                                              d     = 0.4), 
     type = "l", 
     lty  = "dashed", 
     xlab = "theta (°)", 
     ylab = "proportion silhouette area", 
     ylim = c(0, 0.35))

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.2), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.1), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.05), 
       type = "l", 
       lty  = "dotdash")

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                              theta = seq(0, 90, 10), 
                                              h     = 0.4, 
                                              d     = 0.4), 
     type = "l", 
     xlab = "theta (°)", 
     ylab = "proportion silhouette area", 
     ylim = c(0, 0.35))

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.2), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.1), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.05), 
       type = "l", 
       lty  = "dotdash")


## -----------------------------------------------------------------------------

degrees_to_radians(47.608)
radians_to_degrees(0.831)
fahrenheit_to_celsius(85)
fahrenheit_to_kelvin(85)
kelvin_to_celsius(270)


## ---- fig.height = 3, fig.width = 4-------------------------------------------

plot(x    = 0:60, 
     y    = TPC(T      = 0:60, 
                T_opt  = 30, 
                CT_min = 10, 
                CT_max = 40), 
     type = "l", 
     ylim = c(0,3), 
     ylab = "performance", 
     xlab = "temperature (°C)")

points(x    = 0:60, 
       y    = TPC_beta(T         = 0:60, 
                       shift     = -1, 
                       breadth   = 0.1, 
                       aran      = 0, 
                       tolerance = 43, 
                       skew      = 0.7), 
       type = "l", 
       lty  = "dashed")

suppressWarnings(par(oldpar))

#' @title Operative Environmental Temperature of a Limpet
#'
#' @description The function estimates body temperatures (C, operative environmental temperatures) of a limpet based on \insertCite{Denny2006;textual}{TrenchR}.
#' 
#' @param T_a \code{numeric} air temperature (C).
#'
#' @param T_r \code{numeric} rock surface temperature (C) in the sunlight.
#'
#' @param l \code{numeric} limpet length (anterior/posterior axis, m).
#'
#' @param h \code{numeric} limpet height (dorsal/ventral axis, m).
#'
#' @param S \code{numeric} solar irradiance (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#'
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param psi \code{numeric} solar zenith angle (degrees). Can be calculated from \code{\link{zenith_angle}} function.
#'
#' @param c \code{numeric} fraction of the sky covered by cloud (proportion).
#'
#' @param position \code{character} direction of the limpet that is facing upwind. Options are \code{"anterior"}, \code{"posterior"}, and \code{"broadside"}.
#'
#' @return \code{numeric} predicted body (operative environmental) temperature (C).
#'
#' @family biophysical models 
#'
#' @details The original equation uses a finite-difference approach where they divide the rock into series of chunks, and calculate the temperature at each node to derive the conductive heat. For simplification, here it takes the rock temperature as a parameter, and conductive heat is calculated as a product of the area, thermal conductivity of rock and the temperature difference between the rock and the body.
#'   \cr \cr
#'   Limpets are simulated as cones following and using solar emissivity values from \insertCite{Campbell1998;textual}{TrenchR}.
#'   \cr \cr
#'   The area of the limpet's shell (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) is projected according to the direction at which sunlight strikes the organism \insertCite{Pennell1989}{TrenchR}.
#'   \cr \cr
#'   Air conductivity values (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) are calculated following \insertCite{Denny2006;textual}{TrenchR}.
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Tb_limpet(T_a      = 25, 
#'             T_r      = 30, 
#'             l        = 0.0176, 
#'             h        = 0.0122, 
#'             S        = 1300, 
#'             u        = 1, 
#'             psi      = 30, 
#'             c        = 1, 
#'             position = "anterior")
#' 
Tb_limpet <- function (T_a, 
                       T_r, 
                       l, 
                       h, 
                       S, 
                       u, 
                       psi, 
                       c, 
                       position = "anterior") {
  
  stopifnot(l   >  0,
            h   >  0, 
            S   >  0, 
            u   >= 0, 
            psi >= 0, 
            psi <= 90, 
            c   >= 0, 
            c   <= 1, 
            length(position) == 1,
            position %in% c("anterior", "posterior", "broadside"))
  
  # Conversions

    T_a <- celsius_to_kelvin(T_a)
    T_r <- celsius_to_kelvin(T_r)

    psi <- degrees_to_radians(psi)
    r   <- l / 2            
  
  # Calculations

    # Short wave heat transfer
  
      # Area of the limpet's shell (m^2) projected in the direction at which sunlight strikes the organism (Pennell and Deignan 1989)

        if (tan(psi) < r / h) {

          Ap <- pi * r^2 * cos(psi) + h * r * sin(psi) - pi * r^2 / 2 * cos(psi)

        } else {

          Ap <- pi * r^2 * cos(psi)

        }

      # short-wave absorptivity of the shell (the fraction of light energy that is absorbed) 

        alpha_sw <- 0.68
  

  
    # Long-wave energy transfer
  
      # View factor simulating limpets as a cone (Campbell and Norman 1998). 
    
        Vs <- cos(psi) * r / sqrt(r^2 + h^2)
    
      # lateral area of a limpet shell (m^2)

        Al <- pi * r * sqrt(h^2 + r^2) 

      #  long-wave emissivity of the shell (proportion)

        eps_ws <- 0.97  

      sigma <- stefan_boltzmann_constant()

      # clear sky emissivity (Campbell and Norman 1998, 10.11)

        eps_ac <- 9.2 * 10^-6 * T_a^2 

      # emissivity of air with clouds (same as above, 10.12)

        eps_wa <- (1 - 0.84 * c) * eps_ac + 0.84 * c  

  

    # Convective heat transfer
  
      # conductivity of air (W m^-1 K^-1)

        Ka <- 0.00501 + 7.2 * 10^-5 * T_a       

      # kinematic viscosity of air (m^2 s^-1)

        v <- -1.25 * 10^-5 + 9.2 * 10^-8 * T_a  
  
      # Reynolds number

        Re <- u * l / v
  
      if (position == "anterior") {

        a <- 1.955
        b <- 0.371

      } else if (position == "posterior") {

        a <- 1.881
        b <- 0.376

      } else if (position == "broadside"){

        a <- 1.304
        b <- 0.404

      }
  
      # Nusselt number

        Nu <- a * Re^b    

      # Heat transfer coefficient (W m^-2 K^-1)

        hc <- a * Ka * ((u / v)^b)*(l^(b - 1)) 
  
      # area of the shell in convective contact with the air (m^2)

        A_cv <- Al  
  
  # Conductive heat transfer
  
    # area of conductive contact between the limpet's foot and the rock (m^2)

      A_cd <- pi * r^2  

    # thermal conductivity of rock (W m^-1 K^-1)

      Kr <- 3.06


  q1  <- Ap * alpha_sw * S
  q2  <- Vs * Al * eps_ws * sigma * T_a^4 * (eps_wa - 1)
  q3  <- 4 * Vs * Al* eps_ws * sigma * T_a^3
  q4  <- hc * A_cv
  q5  <- A_cd * Kr
  
  T_b <- (q1 + q2 + (q3 + q4) * T_a + q5 * T_r) / (q3 + q4 + q5)
  
  kelvin_to_celsius(T_b)

}

#' @title Operative Environmental Temperature of a Lizard
#' 
#' @description The function estimates body temperature (C, operative environmental temperature) of a lizard based on \insertCite{Campbell1998;textual}{TrenchR}. The function was designed for Sceloporus lizards and described in \insertCite{Buckley2008;textual}{TrenchR}.
#' 
#' @details The proportion of radiation that is direct is determined following \insertCite{Sears2011;textual}{TrenchR}.
#'   \cr \cr 
#'   Boundary conductance uses a factor of 1.4 to account for increased convection \insertCite{Mitchell1976}{TrenchR}.
#'
#' @param T_a \code{numeric} air temperature (C).
#' 
#' @param T_g \code{numeric} surface temperature (C).
#' 
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @param svl \code{numeric} lizard snout vent length (mm).
#' 
#' @param m \code{numeric} lizard mass (g); note that it can be estimated as in \code{\link{mass_from_length}}: \ifelse{html}{\out{3.55 x 10<sup>-5</sup> x length<sup>3</sup>}}{\eqn{3.55*10^-5 * length^3}{ASCII}}
#' 
#' @param psi \code{numeric} solar zenith angle (degrees).
#' 
#' @param rho_s \code{numeric} surface albedo (proportion). ~ 0.25 for grass, ~ 0.1 for dark soil, > 0.75 for fresh snow \insertCite{Campbell1998}{TrenchR}.
#' 
#' @param elev \code{numeric} elevation (m).
#' 
#' @param doy \code{numeric} day of year (1-366).
#' 
#' @param sun \code{logical} indicates whether lizard is in sun (\code{TRUE}) or shade (\code{FALSE}).
#' 
#' @param surface \code{logical} indicates whether lizard is on ground surface (\code{TRUE}) or above the surface (\code{FALSE}, e.g. in a tree).
#' 
#' @param a_s \code{numeric} lizard solar absorptivity (proportion), \code{a_s = 0.9} \insertCite{Gates1980}{TrenchR} (Table 11.4).
#' 
#' @param a_l \code{numeric} lizard thermal absorptivity (proportion), \code{a_l = 0.965} \insertCite{Bartlett1967}{TrenchR}.
#' 
#' @param epsilon_s \code{numeric} surface emissivity of lizards (proportion), \code{epsilon_s = 0.965} \insertCite{Bartlett1967}{TrenchR}.
#' 
#' @param F_d \code{numeric} the view factor between the surface of the lizard and diffuse solar radiation (proportion). i.e., the portion of the lizard surface that is exposed to diffuse solar radiation \insertCite{Bartlett1967}{TrenchR}.
#' 
#' @param F_r \code{numeric} the view factor between the surface of the lizard and reflected solar radiation (proportion).
#' 
#' @param F_a \code{numeric} the view factor between the surface of the lizard and atmospheric radiation (proportion).
#' 
#' @param F_g \code{numeric} the view factor between the surface of the lizard and ground thermal radiation (proportion).
#' 
#' @return T_e \code{numeric} predicted body (operative environmental) temperature (C).
#' 
#' @family biophysical models
#' 
#' @export 
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   Tb_lizard(T_a       = 25, 
#'             T_g       = 30, 
#'             u         = 0.1, 
#'             svl       = 60, 
#'             m         = 10, 
#'             psi       = 34, 
#'             rho_s     = 0.24, 
#'             elev      = 500, 
#'             doy       = 200, 
#'             sun       = TRUE, 
#'             surface   = TRUE, 
#'             a_s   = 0.9, 
#'             a_l   = 0.965, 
#'             epsilon_s = 0.965, 
#'             F_d       = 0.8, 
#'             F_r       = 0.5, 
#'             F_a       = 0.5, 
#'             F_g       = 0.5)
#' 
Tb_lizard <- function (T_a, 
                       T_g, 
                       u, 
                       svl, 
                       m, 
                       psi, 
                       rho_s, 
                       elev, 
                       doy, 
                       sun       = TRUE, 
                       surface   = TRUE, 
                       a_s   = 0.9, 
                       a_l   = 0.965, 
                       epsilon_s = 0.965, 
                       F_d       = 0.8, 
                       F_r       = 0.5, 
                       F_a       = 0.5, 
                       F_g       = 0.5) {
  
  stopifnot(u         >= 0, 
            svl       >= 0, 
            m         >= 0, 
            rho_s     >= 0, 
            rho_s     <= 1, 
            elev      >= 0, 
            doy       >  0, 
            doy       <  367, 
            a_s   >= 0, 
            a_s   <= 1, 
            a_l   >= 0, 
            a_l   <= 1, 
            epsilon_s >= 0, 
            epsilon_s <= 1, 
            F_d       >= 0, 
            F_d       <= 1, 
            F_r       >= 0, 
            F_r       <= 1, 
            F_a       >= 0, 
            F_a       <= 1, 
            F_g       >= 0, 
            F_g       <= 1,
            is.logical(sun),
            is.logical(surface))
  
  psi <- degrees_to_radians(psi) 
  
  # constants

    sigma <- stefan_boltzmann_constant() 
    c_p   <- 29.3 # specific heat of air, J/mol C (p.279) Parentheses all from Campbell & Norman 1998
    tau   <- 0.65 # atmospheric transmissivity
    S_p0  <- 1360 # extraterrestrial flux density, W/m^2 (p.159)
  
  # Calculate radiation
  # view angles, parameterized for animal suspended above ground (p181), on ground- adjust F_e, F_r, and F_g

    h <- svl / 1000 # convert snout vent length to m
  
    A <- 0.121 * m^0.688   # total lizard area, Roughgarden (1981)
    A_p <- (-1.1756810^-4 * psi^2 - 9.2594 * 10^-2 * psi + 26.2409) * A / 100 # projected area
    F_p <- A_p / A
  
  # Radiation

    p_a <- 101.3 * exp (-elev / 8200)  # atmospheric pressure
    m_a <- p_a / (101.3 * cos(psi))  # (11.12) optical air mass
    m_a[(psi > (80 * pi / 180))] <- 5.66
  
  # Flux densities

    epsilon_ac <- 9.2 * 10^-6 * (T_a + 273)^2 # (10.11) clear sky emissivity
    L_a <- sigma * (T_a + 273)^4  # (10.7) long wave flux densities from atmosphere 
    L_g <- sigma * (T_g + 273)^4  # (10.7) long wave flux densities from ground
  
    S_d <- 0.3 * (1 - tau^m_a) * S_p0 * cos(psi)  # (11.13) diffuse radiation
  
    dd2 <- 1 + 2 * 0.1675 * cos(2 * pi * doy / 365)
    S_p <- S_p0 * tau^m_a * dd2 * cos(psi)  # Sears and Angilletta 2012 #dd is correction factor accounting for orbit
    S_b <- S_p * cos(psi)
    S_t <- S_b + S_d
    S_r <- rho_s * S_t # (11.10) reflected radiation
  
  # Conductance
  
    # characteristic dimension in meters based on mass in g
    # calculate as cube root of volume as recommended by Mitchell 1976
    # assume volume V= m/p, where p is density of body and assumed to be that of water (p=1000kg/m3)
      dim <- ((m / 1000) / 1000) ^ (1 / 3)

      g_r <- 4 * epsilon_s * sigma * (T_a + 273)^3 / c_p # (12.7) radiative conductance
  
      g_Ha <- 1.4 * 0.135 * sqrt(u / dim) # boundary conductance, factor of 1.4 to account for increased convection (Mitchell 1976)
  
  # Operative environmental temperature
  
  # Calculate with both surface and air temp (on ground and in tree)

    sprop <- 1 # proportion of radiation that is direct, Sears and Angilletta 2012
    R_abs <- sprop * a_s * (F_p * S_p + F_d * S_d + F_r * S_r) + a_l * (F_a * L_a + F_g * L_g) # (11.14) Absorbed radiation

    Te      <- T_a + (R_abs - epsilon_s * sigma * (T_a + 273)^4) / (c_p * (g_r + g_Ha)) # (12.19) Operative temperature            
    Te_surf <- T_g + (R_abs - epsilon_s * sigma * (T_g + 273)^4) / (c_p * (g_r + g_Ha))        
  
  # Calculate in shade, no direct radiation

    sprop <- 0 # proportion of radiation that is direct, Sears and Angilletta 2012
    R_abs <- sprop * a_s * (F_p * S_p + F_d * S_d + F_r * S_r) + a_l * (F_a * L_a + F_g * L_g) # (11.14) Absorbed radiation

    TeS      <- T_a + (R_abs - epsilon_s * sigma * (T_a + 273)^4) / (c_p * (g_r + g_Ha)) # (12.19) Operative temperature                        
    TeS_surf <- T_g + (R_abs - epsilon_s * sigma * (T_g + 273)^4) / (c_p * (g_r + g_Ha))  
  

  if (sun & surface) {
    
    Te <- Te_surf
    
  } else if (sun & !surface) {
    
    Te <- Te
    
  } else if (!sun & surface) {
    
    Te <- TeS_surf
    
  } else if (!sun & !surface) {
    
    Te <- TeS
    
  }
  
  Te
  
}

context("biophysmodel_sceloporus")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}


test_that("Tb_lizard function works as expected", {
  expect_similar(Tb_lizard(T_a=25, T_g=30, u=1, svl=60, m=10, psi=34, rho_s=0.7, elev=500, doy=200, sun=TRUE, surface=TRUE, a_s=0.9, a_l=0.965, epsilon_s=0.965, F_d=0.8, F_r=0.5, F_a=0.5, F_g=0.5), 38.1195)
  expect_similar(Tb_lizard(T_a=25, T_g=30, u=0.1, svl=60, m=10, psi=34, rho_s=0.7, elev=2000, doy=100, sun=FALSE, surface=FALSE, a_s=0.9, a_l=0.965, epsilon_s=0.965, F_d=0.8, F_r=0.5, F_a=0.5, F_g=0.5), 25.83812)
expect_similar(Tb_lizard(T_a=25, T_g=30, u=0.1, svl=60, m=10, psi=34, rho_s=0.7, elev=2000, doy=100, sun=TRUE, surface=FALSE, a_s=0.9, a_l=0.965, epsilon_s=0.965, F_d=0.8, F_r=0.5, F_a=0.5, F_g=0.5), 52.35219)
expect_similar(Tb_lizard(T_a=25, T_g=30, u=0.1, svl=60, m=10, psi=34, rho_s=0.7, elev=2000, doy=100, sun=FALSE, surface=TRUE, a_s=0.9, a_l=0.965, epsilon_s=0.965, F_d=0.8, F_r=0.5, F_a=0.5, F_g=0.5), 29.161889)
})

#' @title Properties of Dry Air
#'
#' @description The function calculates several properties of dry air and related characteristics shown as output variables below. The program is based on equations from \insertCite{List1971;textual}{TrenchR} and code implementation from NicheMapR \insertCite{Kearney2017,Kearney2020}{TrenchR}.
#'   \cr \cr
#'  The user must supply values for the input variables \code{db}, \code{bp}, and \code{alt}. If \code{alt} is known (-1000 < alt < 20000) but not BP, then set \code{bp = 0}.
#' 
#' @param db \code{numeric} dry bulb temperature (C).
#' 
#' @param bp \code{numeric} barometric pressure (Pa).
#' 
#' @param alt \code{numeric} altitude (m).
#' 
#' @return Named \code{list} with elements:
#'   \describe{
#'     \item{\code{patmos}}{\code{numeric} standard atmospheric pressure (Pa)}
#'     \item{\code{density}}{\code{numeric} density (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}})}
#'     \item{\code{visdyn}}{\code{numeric} dynamic viscosity (\ifelse{html}{\out{kg m<sup>-1</sup> s<sup>-1</sup>}}{\eqn{kg m^-1 s^-1}{ASCII}})}
#'     \item{\code{viskin}}{\code{numeric} kinematic viscosity (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}})}
#'     \item{\code{difvpr}}{\code{numeric} diffusivity of water vapor in air (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}})}
#'     \item{\code{thcond}}{\code{numeric} thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}})}
#'     \item{\code{htovpr}}{\code{numeric} latent heat of vaporization of water (\ifelse{html}{\out{J kg<sup>-1</sup>}}{\eqn{J kg^-1}{ASCII}})}
#'     \item{\code{tcoeff}}{\code{numeric} temperature coefficient of volume expansion (\ifelse{html}{\out{K<sup>-1</sup>}}{\eqn{K^-1}{ASCII}})}
#'     \item{\code{ggroup}}{\code{numeric} group of variables in Grashof number (\ifelse{html}{\out{m<sup>-3</sup> K<sup>-1</sup>}}{\eqn{m^-3 K^-1}{ASCII}})}
#'     \item{\code{bbemit}}{\code{numeric} black body emittance (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}})}
#'     \item{\code{emtmax}}{\code{numeric} wave length of maximum emittance (m)}
#'   }
#'
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   DRYAIR(db = 30, 
#'          bp = 100*1000, 
#'          alt = 0)
#' 
DRYAIR <- function(db, 
                   bp = 0, 
                   alt = 0) {
  
  stopifnot(alt >= 0, 
            bp  >=  0)

  tstd <- 273.15
  
  pstd   <- 101325.
  patmos <- pstd*((1 - (0.0065 * alt / 288))^(1 / 0.190284))
  
  bp          <- rep(bp, length(patmos))
  bp[bp <= 0] <- patmos[bp <= 0]
  
  density <- bp / (287.04 * (db + tstd))
  
  visnot <- 1.8325E-5
  tnot   <- 296.16
  c      <- 120.
  
  visdyn <- (visnot * ((tnot + c) / (db + tstd + c))) * (((db + tstd) / tnot)^1.5)
  viskin <- visdyn / density
  
  difvpr <- 2.26E-5 * (((db + tstd) / tstd)^1.81) * (1.E5 / bp)
  thcond <- 0.02425 + (7.038E-5 * db)
  htovpr <- 2.5012E6 - 2.3787E3 * db
  tcoeff <- 1. / (db + tstd)
  ggroup <- 0.0980616 * tcoeff / (viskin * viskin)
  bbemit <- 5.670367E-8 * ((db + tstd)^4)
  emtmax <- 2.897E-3 / (db + tstd)
  
  list(patmos  = patmos, 
       density = density, 
       visdyn  = visdyn, 
       viskin  = viskin, 
       difvpr  = difvpr, 
       thcond  = thcond, 
       htovpr  = htovpr, 
       tcoeff  = tcoeff, 
       ggroup  = ggroup, 
       bbemit  = bbemit, 
       emtmax  = emtmax)

}


#' @title Saturation Vapor Pressure
#'
#' @description The function calculates saturation vapor pressure for a given air temperature. The program is based on equations from \insertCite{List1971;textual}{TrenchR} and code implementation from NicheMapR \insertCite{Kearney2017,Kearney2020}{TrenchR}.
#' 
#' @param db \code{numeric} Dry bulb temperature (C)
#' 
#' @return esat \code{numeric} Saturation vapor pressure (Pa)
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   VAPPRS(db = 30)
#' 
VAPPRS <- function(db) {
  
  t <- celsius_to_kelvin(db)
  
  loge <- t
  loge[t <= 273.16] <- -9.09718 * (273.16 / t[t <= 273.16] - 1.) - 3.56654 * log10(273.16 / t[t <= 273.16]) + 0.876793 * (1. - t[t <= 273.16] / 273.16) + log10(6.1071)
  loge[t > 273.16]  <- -7.90298 * (373.16 / t[t > 273.16] - 1.) + 5.02808 * log10(373.16 / t[t > 273.16]) - 1.3816E-07 * (10.^(11.344 * (1. - t[t > 273.16] / 373.16)) - 1.) + 8.1328E-03 * (10.^(-3.49149 * (373.16 / t[t > 273.16] - 1.)) - 1.) + log10(1013.246)
  
  (10.^loge) * 100
  
}

#' @title Properties of Wet Air
#'
#' @description The function calculates several properties of humid air described as output variables below. The program is based on equations from \insertCite{List1971;textual}{TrenchR} and code implementation from NicheMapR \insertCite{Kearney2017,Kearney2020}{TrenchR}.
#'  \cr 
#'  WETAIR must be used in conjunction with \code{\link{VAPPRS}}. Input variables are shown below. See Details. 
#'
#' @details The user must supply known values for DB and BP (BP at one standard
#'   atmosphere is 101,325 pascals). Values for the remaining variables are
#'   determined by whether the user has either (1) psychrometric data (WB or RH),
#'   or (2) hygrometric data (DP):
#'   \describe{
#'     \item{Psychrometric data}{If WB is known but not RH, then set RH = -1
#'       and DP = 999. If RH is known but not WB then set WB = 0 and DP = 999.}
#'     \item{Hygrometric data}{If DP is known, set WB = 0 and RH = 0.}
#'   }
#'
#' @param db \code{numeric} dry bulb temperature (C).
#' 
#' @param wb \code{numeric} wet bulb temperature (C).
#' 
#' @param rh \code{numeric} relative humidity (\%).
#' 
#' @param dp \code{numeric} dew point temperature (C).
#' 
#' @param bp \code{numeric} barometric pressure (Pa).
#' 
#' @return Named \code{list} with elements:
#'   \describe{
#'     \item{\code{e}}{\code{numeric} saturation vapor pressure (Pa)}
#'     \item{\code{vd}}{\code{numeric} vapor density (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}})}
#'     \item{\code{rw}}{\code{numeric} mixing ratio (\ifelse{html}{\out{kg kg<sup>-1</sup>}}{\eqn{kg kg^-1}{ASCII}})}
#'     \item{\code{tvir}}{\code{numeric} virtual temperature (K)}
#'     \item{\code{tvinc}}{\code{numeric} virtual temperature increment (K)}
#'     \item{\code{denair}}{\code{numeric} density of the air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}})}
#'     \item{\code{cp}}{\code{numeric} specific heat of air at constant pressure (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}})}
#'     \item{\code{wtrpot}}{\code{numeric} water potential (Pa)}
#'     \item{\code{rh}}{\code{numeric} relative humidity (\%)}
#'   }
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   WETAIR(db = 30, 
#'          wb = 28, 
#'          rh = 60, 
#'          bp = 100 * 1000)
#' 
WETAIR <- function (db, 
                    wb = db, 
                    rh = 0, 
                    dp = 999, 
                    bp = 101325) {
  
  stopifnot(rh <= 100, 
            bp >  0)
  
  tk    <- celsius_to_kelvin(db)
  esat  <- VAPPRS(db)

  if(dp < 999.0) {
    
    e  <- VAPPRS(dp)
    rh <- (e / esat) * 100
    
  } else {
    
    if(min(rh) > -1){
      
      e <- esat * rh / 100
      
    } else{
      
      wbd   <- db - wb
      wbsat <- VAPPRS(wb)
      dltae <- 0.000660 * (1.0 + 0.00115 * wb) * bp * wbd
      e     <- wbsat - dltae
      rh    <- (e / esat) * 100
      
    }
  }
  
  rw     <- ((0.62197 * 1.0053 * e) / (bp - 1.0053 * e))
  vd     <- e * 0.018016 / (0.998 * 8.31434 * tk)
  tvir   <- tk * ((1.0 + rw / (18.016 / 28.966)) / (1.0 + rw))
  tvinc  <- tvir - tk
  denair <- 0.0034838 * bp / (0.999 * tvir)
  cp     <- (1004.84 + (rw * 1846.40)) / (1.0 + rw)
  
  if (min(rh) <= 0.0) {
    
    wtrpot <- -999
    
  } else {
    
    wtrpot <- 4.615e+5 * tk * log(rh / 100.0)
    
  }
  
  list(e      = e, 
       esat   = esat, 
       vd     = vd, 
       rw     = rw, 
       tvinc  = tvinc, 
       denair = denair, 
       cp     = cp, 
       wtrpot = wtrpot, 
       rh     = rh)
  
}

context("DRYWETAIR")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("DRYAIR function works as expected", {
  expect_equal(length(DRYAIR(db=30, bp=100*1000, alt=0)), 11)
  expect_identical(class(DRYAIR(db=30, bp=100*1000, alt=0)), "list")
  expect_similar(DRYAIR(db=30, bp=100*1000, alt=0)[[2]], 1.149212)
})

test_that("VAPRS function works as expected", {
  expect_similar(VAPPRS(db=30), 4240.599)
})

test_that("WETAIR function works as expected", {
  expect_equal(length(WETAIR(db=30, wb=28, rh=60, bp=100*1000)), 9)
  expect_identical(class(WETAIR(db=30, wb=28, rh=60, bp=100*1000)), "list")
  expect_similar(WETAIR(db=30, wb=28, rh=60, bp=100*1000)[[5]], 2.961613)
  expect_similar(WETAIR(db=30, wb=28, rh=60, dp = 9, bp=100*1000)[[5]], 1.32676)
  expect_similar(WETAIR(db=30, wb=28, rh=-0.5, bp=100*1000)[[8]], -999)
expect_similar(WETAIR(db=30, wb=28, rh=-1.1, bp=100*1000)[[5]], 4.2537)
})
       

#' @title Estimate the Three Components of Solar Radiation (Direct, Diffuse and Reflected)
#' 
#' @description The function estimate direct, diffuse, and reflected components of solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}) as a function of day of year using the model in \insertCite{Campbell1998;textual}{TrenchR}. 
#' 
#' @param doy \code{numeric} the day of year; \code{\link{day_of_year}}.
#' 
#' @param psi \code{numeric} zenith angle (radians).
#' 
#' @param tau \code{numeric} atmospheric transmissivity (proportion), which is ratio of global solar radiation at ground level to extra-terrestrial solar radiation.
#' 
#' @param elev \code{numeric} elevation (meters).
#' 
#' @param rho \code{numeric} albedo as a proportion (0-1). 
#' 
#' @return \code{numeric} radiation components - direct, diffused and reflected (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   solar_radiation(doy  = 112, 
#'                   psi  = 1, 
#'                   tau  = 0.6, 
#'                   elev = 1500, 
#'                   rho  = 0.7)
#'
solar_radiation <- function (doy, 
                             psi, 
                             tau, 
                             elev, 
                             rho = 0.7){
  
  stopifnot(doy >  0, 
            doy <  367, 
            psi <= 2 * pi, 
            tau >= 0, 
            tau <= 1, 
            elev > 0, 
            rho >= 0, 
            rho <= 1)
  
  sigma <- stefan_boltzmann_constant()
  c_p <- 29.3 # Specific heat of air, J/mol degrees K or C
  S_p0 <- 1360 # extraterrestrial flux density, W/m^2 (p159)
  
  # Radiation adjusted for elevation
  p_a <- 101.3 * exp (-elev / 8200)  # atmospheric pressure
  m_a <- p_a / (101.3 * cos(psi))  # (11.12) optical air mass
  m_a[which(psi > (80 * pi / 180))] <- 5.66
  
  # Flux densities
  # S_p is direct radiation reaching the earth's surface
  S_p <- S_p0 * tau^m_a * cos(psi)
  
  # S_d is diffuse radiation
  S_d <- 0.3 * (1 - tau^m_a) * S_p0 * cos(psi)
  
  # S_d is reflected radiation
  S_r <- rho * (S_p + S_d) # (11.10) reflected radiation
  
  # Return direct, diffuse, reflected solar radiation
  c(S_p, S_d, S_r)
  
}

#' @title Hourly Solar Radiation
#' 
#' @description The function estimates hourly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> hr<sup>-1</sup>}}{\eqn{W m^-2 hr^-1}{ASCII}}) as a function of daily global solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> d<sup>-1</sup>}}{\eqn{W m^-2 d^-1}{ASCII}}). Based on \insertCite{Tham2010;textual}{TrenchR} and \insertCite{AlRawahi2011;textual}{TrenchR}.
#' 
#' @param doy \code{numeric} the day of year.
#' 
#' @param S \code{numeric} solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> d<sup>-1</sup>}}{\eqn{W m^-2 d^-1}{ASCII}}).
#' 
#' @param hour \code{numeric} hour (0-24). 
#' 
#' @param lon \code{numeric} longitude (degrees).
#' 
#' @param lat \code{numeric} latitude (degrees).
#' 
#' @return \code{numeric} hourly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#' diurnal_radiation_variation(doy    = 112, 
#'                             S = 8000, 
#'                             hour   = 12, 
#'                             lon    = -122.33, 
#'                             lat    = 47.61)
#'
diurnal_radiation_variation <- function(doy, 
                                        S, 
                                        hour, 
                                        lon, 
                                        lat) { 

  stopifnot(doy    >  0, 
            doy    <  367, 
            S >  0, 
            hour   >= 0, 
            hour   <= 24, 
            lon    >- 180, 
            lon    <= 180, 
            lat    >= -90, 
            lat    <= 90)
  
  # Calculate solar time
  rd <- 180 / pi  # factor to convert radians into degrees
  RevAng <- 0.21631 + 2 * atan(0.967 * tan(0.0086 * (-186 + doy))) # Revolution angle in radians
  DecAng <- asin(0.39795 * cos(RevAng))  # Declination angle in radians      
  
  f <- (279.575 + 0.9856 * doy) / rd  # f in radians
  ET <- (-104.7 * sin(f) + 596.2 * sin(2 * f) + 4.3 * sin(3 * f) - 12.7 * sin(4 * f) - 429.3 * cos(f) - 2.0 * cos(2 * f) + 19.3 * cos(3 * f)) / 3600   # (11.4) Equation of time
  LC <- 1 / 15 * (15 - lon%%15) # longitude correction, 1/15h for each degree east of standard meridian
  hour_sol <- 12 + LC + ET
  
  # W: hour angle of the sun (in radians)
  W <- pi * (hour - hour_sol) / 12  # Brock 1981
  
  # Ws: sunset hour angle (in radians)
  inner<- -tan(lat / rd) * tan(DecAng)
  #restrict between -1 and 1
  inner<- ifelse(inner > 1, 1, ifelse(inner < -1, -1, inner))
  Ws <- acos(inner)
  
  d <- 0.409 + 0.5016 * sin(Ws - 1.047)
  b <- 0.6609 - 0.4767 * sin(Ws - 1.047) # papers differ in whether sign before 0.4767 is negative or positive
  
  # rG: ratio of hourly to daily global radiation
  rG <- pi / 24 * (d + b * cos(W)) * (cos(W) - cos(Ws)) / (sin(Ws) - Ws * cos(Ws))   # (Liu and Jordan, Collares-Pereira and Rable 1979)
  # rG <- pi / 24 * (d + b * cos(W) - cos(Ws)) / (sin(Ws) - Ws * cos(Ws))           # Brock 1981
  
  #handle negative or infinite values
  rG <- ifelse(rG < 0, 0, rG)
  rG <- ifelse(!is.finite(rG), 0, rG)
  
  rG * S 
  
}

#' @title Average Monthly Solar Radiation
#' 
#' @description The function estimates average monthly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> d<sup>-1</sup>}}{\eqn{W m^-2 d^-1}{ASCII}}) using basic topographic and climatic information as input. Cloudiness is stochastically modeled, so output will vary between functional calls. Based on \insertCite{Nikolov1992;textual}{TrenchR}.
#' 
#' @param lat \code{numeric} latitude (degrees).
#' 
#' @param lon \code{numeric} longitude (degrees).
#' 
#' @param doy \code{numeric} day of year (1-366).
#' 
#' @param elev \code{numeric} elevation (meters).
#' 
#' @param T_a \code{numeric} mean monthly air temperature (C).
#' 
#' @param hp \code{numeric} mean month relative humidity (percentage).
#' 
#' @param P \code{numeric} total monthly precipitation (mm).
#' 
#' @return \code{numeric} average monthly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   monthly_solar_radiation(lat  = 47.61, 
#'                           lon  = -122.33, 
#'                           doy  = 112, 
#'                           elev = 1500, 
#'                           T_a  = 15, 
#'                           hp   = 50, 
#'                           P   = 50)
#'
monthly_solar_radiation <- function (lat,
                                     lon, 
                                     doy, 
                                     elev, 
                                     T_a, 
                                     hp, 
                                     P) {

  stopifnot(lat  >= -90, 
            lat  <= 90, 
            lon  >  -180, 
            lon  <= 180, 
            doy  >  0, 
            doy  <  367, 
            elev >  0, 
            hp   >= 0, 
            hp   <= 100, 
            P    >  0)
  
  rd <- 180 / pi  # factor to convert radians into degrees
  
  # functions to calculate sin and cos for angles in degrees
  cos.deg <- function(deg) { 
    
    cos(degrees_to_radians(deg))
  
  }
  
  sin.deg <- function(deg) { 
    
    sin(degrees_to_radians(deg))
    
  }

  
  # Sc is the solar constant (2.0 cal cm ^-2 min ^-1)
  S_c <- 2 # Cal cm^-2 min^-1  1367 #W/m2   #2 
  
  # Calculate solar declination in radians (Swift 1976) 
  D_s <- asin(0.39785*sin(4.868961 + 0.017203*doy+0.033446*sin(6.224111 + 0.017202*doy)))      
  
  # h_s the sunrise/sunset hour angle (degree)
  # Keith and Kreider 1978  
  h_s <- acos(-tan(lat / rd) * tan(D_s)) * 180 / pi
  
  # convert solar declination to degrees
  D_s <- D_s * 180 / pi
  
  # Daily potential radiation
  R_0 <- S_c * 24 * 60 / pi * (1 + 0.033 * cos.deg(360 * doy / 365)) * (cos.deg(lat) * cos.deg(D_s) * sin.deg(h_s) + (h_s * 2 * pi / 360) * sin.deg(lat) * sin.deg(D_s))
  
  # Assume relationships to translate to surface
  # ev: mean monthly water vapor pressure in the atmosphere (Pa)
  ev <- hp * 6.1078 * exp(17.269 * T_a/(T_a + 237.3))
  
  # C is the mean monthly cloudiness (tenths)
  C <- 10 - 1.155 * (ev / P)^0.5
  
  # beta: clear-sky atmospheric transmittance
  beta <- 0.715 - 0.3183 * (1 - 1.3614 * cos.deg(lat))
  
  # omega: light absorption effect of clouds
  omega <- 0.03259
  
  # alpha: diffuse solar radiation
  alpha <- 32.9835 - 64.88411 * (1 - 1.3614 * cos.deg(lat))
  
  # VARY CLOUDINESS: USE CLOUDINESS DATA TO ESTIMATE MEAN AND SD
  C1 <- rtnorm(1, mean = C, sd = 1.5, lower = 0, upper = 10)
  
  # Average portion of the sky covered by clouds (Bonan 1988)
  R <- R_0 * (beta - omega * C1) - alpha
  
  # Function of elevation
  t <- 1:24 # time in hours from midnight
  h <- (t - 12) * 15 # solar hour angle (degrees)
  
  # E_sm: mean monthly solar altitude angle calculated by dividing the monthly integral of hourly estimates of solar elevation by the number of hours in a month when the sun is above the horizon
  daylength <- daylength(lat, doy)
  # E_s: solar elevation (in degrees)
  E_s <- asin(sin.deg(lat) * sin.deg(D_s) + cos.deg(lat) * cos.deg(D_s) * cos.deg(h)) * rd
  E_s[E_s < 0] <- 0
  E_sm <- sum(E_s) / daylength
  
  # k: basic atmospheric radiation coefficient
  k <- 0.014 #cal cm^-2 m^-1
  
  # Solar radiation as a function of site elevation
  Ra <- R + ((R_0 + 1) - R) * (1 - exp(-k / sin.deg(E_sm) * (elev - 274) / 274))
  
  Ra * 0.4845 #convert to W/m^2
  
}


#' @title Direct Solar Radiation
#' 
#' @description The function estimates direct solar radiation (\ifelse{html}{\out{W/m<sup>2</sup>}}{\eqn{W/m^2}{ASCII}}) based on latitude, day of year, elevation, and time. The function uses two methods \insertCite{McCullough1971,Campbell1998}{TrenchR} compiled in \insertCite{Tracy1983;textual}{TrenchR}.
#' 
#' @param lat \code{numeric} latitude (degrees).
#' 
#' @param doy \code{numeric} day of year (1-366).
#' 
#' @param elev \code{numeric} elevation (m).
#' 
#' @param t \code{numeric} local time (decimal hours).
#' 
#' @param t0 \code{numeric} time of local noon (decimal hours), can be estimated using \code{\link{solar_noon}}.
#' 
#' @param method \code{character} method for estimating direct solar radiation, options: \code{"Campbell 1977"} (default), \code{"Gates 1962"}.
#' 
#' @return \code{numeric} direct solar radiation (\ifelse{html}{\out{W/m<sup>2</sup>}}{\eqn{W/m^2}{ASCII}}).
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   direct_solar_radiation(lat    = 47.61, 
#'                          doy    = 112, 
#'                          elev   = 1500, 
#'                          t      = 9, 
#'                          t0     = 12, 
#'                          method = "Campbell 1977")
#'
direct_solar_radiation <- function (lat, 
                                    doy, 
                                    elev, 
                                    t, 
                                    t0, 
                                    method = "Campbell 1977") {

  stopifnot(lat  >= -90, 
            lat  <= 90, 
            doy  >  0, 
            doy  <  367, 
            elev >  0, 
            t    >= 0, 
            t    <= 24, 
            t0   >= 0, 
            t0   <= 24,
            length(method) == 1,
            method %in% c("Campbell 1977", "Gates 1962"))
  
  # estimate needed quantities
  # elliptical longitude
  E <- 0.01675 # eccentricity of the earth's orbit about the sun
  
  ELon <- (2 * pi / 365) * (doy - 80) + 2 * E * (sin(2 * pi / 365 * doy) - sin(2 * pi / 365 * 80))
  
  # solar declination angle (radians), McCullough and Porter 1971
  DecAng <- asin(0.39795 * sin(ELon))           
  
  # extraterrestrial solar flux
  S_po <- 1.36 # kW m^{-2}
  
  # optimal air mass
  # adjust atmospheric pressure for elevation
  p_a <- 101.3 * exp(-elev / 8200)  # kPa, atmospheric pressure from Campbell & Norman (1998)
  
  # geographical latitude
  geo.lat <- lat * pi / 180
  
  # sine of sun's altitude angle
  sin_alt_ang <- sin(geo.lat) * sin(DecAng) + cos(geo.lat) * cos(DecAng) * cos(0.2618 * (t - t0))
  
  m_a <- p_a / 101.3 / sin_alt_ang
  
  a <- 0.83 # transmissivity of the atmosphere, between 0 and 1 "The atmospheric transmission coefficient. Varies from 0.9 for a very clear atmosphere to around 0.6 for a hazy or smoggy atmosphere. A typical value for a clear day would be around 0.84." (Campbell, 1977)
  
  # radius vector of the earth in atmospheric units (AU)
  # sunr function from insol package
  Temp    <- (doy - 2451545) / 36525
  epsilon <- (23 + 26 / 60 + 21.448 / 3600) - (46.815 / 3600) * Temp - (0.00059 / 3600) * Temp^2 + (0.001813 / 3600) * Temp^3
  M       <- 357.5291 + 35999.0503 * Temp - 0.0001559 * Temp^2 - 4.8e-07 * Temp^3
  e       <- 0.016708617 - 4.2037e-05 * Temp - 1.236e-07 * Temp^2
  C       <- (1.9146 - 0.004817 * Temp - 1.4e-05 * Temp^2) * sin(degrees_to_radians(M)) + (0.019993 - 0.000101 * Temp) * sin(2 * degrees_to_radians(M)) + 0.00029 * sin(3 * degrees_to_radians(M))
  v       <- M + C
  r       <- (1.000001018 * (1 - e^2)) / (1 + e * cos(degrees_to_radians(v)))
  
  #-------
  #Campbell 1977 - direct radiation
  if (method=="Campbell 1977") {

    Sb <- a^m_a * S_po * sin_alt_ang

  } else if (method == "Gates 1962") { 
  
    w <- 6.93 #precipitable water vapor (mm)
    #"The amount of water vapor in the atmosphere in the zenith direction. Varies from I m for very cold dry atmospheres to about 20 mm in warm moist atmosphere. It can get as high as 30 mm." (Gates, 1962)
  
    d <- 0.896 #haze-dust concentration (particles cm^{-3})
    #"The concentration of dust and other particulates in the air. Number varies from 0.2-3.0. On clear days it will be 0.6-1.0. Around big cities it will be 1.4-2.0." (Gates, 1962)
  
    Sb <- (S_po / r^2) * sin_alt_ang * exp(-0.089 * (p_a * m_a / 101.3)^0.75 - 0.174 * (w * m_a / 20)^0.6 - 0.083 * (d * m_a)^0.9)
  
  }
  
  Sb <- Sb * 1000 # Convert from kW/m^2 to W/m^2
  
  # Set negative values before and after sunrise to zero
  Sb[Sb < 0] <- 0
  
 Sb
 
}  
  

context("RadiationFunctions")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("solar_radiation function works as expected", {
  expect_equal(length(solar_radiation(doy=112, psi=1, tau=0.6, elev=1500, rho=0.7)),3)
  expect_similar(solar_radiation(doy=112, psi=1, tau=0.6, elev=1500, rho=0.7)[1], 334.3606)
  expect_similar(solar_radiation(doy=112, psi=1, tau=0.6, elev=1500, rho=0.7)[2], 120.1351)
  expect_similar(solar_radiation(doy=112, psi=1, tau=0.6, elev=1500, rho=0.7)[3], 318.1471)
})

test_that("diurnal_radiation_variation function works as expected", {
  expect_similar(diurnal_radiation_variation(doy=112, S=8000, hour=12, lon=-122.33, lat=47.61), 1006.89)
  expect_similar(diurnal_radiation_variation(doy=112, S=1, hour=1, lon=-122.33, lat=47.61), 0)

})

test_that("monthly_solar_radiation function works as expected", {
  expect_lt(monthly_solar_radiation(lat=47.61, lon=-122.33, doy=112, elev=1500, T_a=15, hp=50, P=50), 268.9506)
  expect_gt(monthly_solar_radiation(lat=47.61, lon=-122.33, doy=112, elev=1500, T_a=15, hp=50, P=50), 165.1397)
})

test_that("direct_solar_radiation function works as expected", {
expect_similar(direct_solar_radiation(lat    = 47.61, 
                          doy    = 112, 
                          elev   = 1500, 
                          t      = 9, 
                          t0     = 12, 
                          method = "Campbell 1977"), 656.1941)

expect_similar(direct_solar_radiation(lat    = 47.61, 
                          doy    = 112, 
                          elev   = 1500, 
                          t      = 9, 
                          t0     = 12, 
                          method = "Gates 1962"), 598.4864)
})

#' @title Diffuse Fraction for Partitioning Solar Radiation
#' 
#' @description The function partitions solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}) into direct and diffuse components by estimating the diffuse fraction (k_d). The function uses the models presented in \insertCite{Wong2001;textual}{TrenchR}.
#' 
#' @param method \code{character} method to use for estimating the diffuse fraction, currently available: \code{"Liu_Jordan"}, \code{"Orgill_Hollands"}, \code{"Erbs"}, \code{"Olyphant"}, \code{"Spencer"}, \code{"Reindl-1"}, \code{"Reindl-2"}, \code{"Lam_Li"}.
#' 
#' @param kt \code{numeric} the clearness index (dimensionless), which is the ratio of the global solar radiation measured at the surface to the total solar radiation at the top of the atmosphere. (0-1)
#' 
#' @param lat \code{numeric} latitude (degrees). Needed only if method is \code{"Spencer"}.
#' 
#' @param sol.elev \code{numeric} the solar elevation angles (degrees). Needed only if method is \code{"Reindl-2"}. 
#' 
#' @return \code{numeric} diffuse fraction.
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   partition_solar_radiation(method   = "Erbs", 
#'                             kt       = 0.5, 
#'                             lat      = 40, 
#'                             sol.elev = 60)
#'
partition_solar_radiation <- function (method, 
                                       kt, 
                                       lat = NA, 
                                       sol.elev = NA){  
 
  stopifnot(length(method) == 1,
            method %in% c("Liu_Jordan", "Orgill_Hollands", "Erbs", "Olyphant", "Spencer", "Reindl-1", "Reindl-2", "Lam_Li"), 
            kt >= 0, 
            kt <= 1)
  
  #6.1 Liu and Jordan 
  if (method == "Liu_Jordan") {
    
    kd <- (0.271 - 0.294 * kt) / kt
    
    if (kd > 1) { 
 
      kd <- 1

    }

  }
    
  #6.2 Orgill and Hollands
  if (method == "Orgill_Hollands") {
    
    if (kt < 0.35) {

     kd <- 1 - 0.249 * kt

    }

    if (kt >= 0.35 & kt <= 0.75) {

      kd <- 1.577 - 1.84 * kt
    
    }

    if (kt >= 0.75) {

      kd <- 0.177 
    
    }


  }
  
  #6.3 Erbs et al.
  if (method == "Erbs") {
    
    if(kt <= 0.22) {

      kd <- 1 - 0.09 * kt
    
    }

    if(kt > 0.22 & kt < 0.8) {

      kd <- 0.9511 - 0.1604 * kt + 4.388 * kt^2 - 16.638 * kt^3 + 12.336 * kt^4
    
    }

    if(kt >= 0.8) {

      kd <- 0.165 # Correction from 0.125 for CO from Olyphant 1984
  
    }

  }
  
  if(method == "Olyphant") { 
    # Correction for Colorado from Olyphant 1984
    
    if(kt <= 0.22) {

      kd <- 1 - 0.09 * kt
    
    }

    if(kt > 0.22 & kt < 0.8) {

      kd <- 0.9511 - 0.1604 * kt + 4.388 * kt^2 - 16.638 * kt^3 + 12.336 * kt^4
    
    }

    if(kt >= 0.8) {

      kd <- 0.125 
  
    }

  }
  
  #6.4 Spencer
  if(method=="Spencer"){
    
    a3 <- 0.94 + 0.0118 * abs(lat)
    
    b3 <- 1.185 + 0.0135 * abs(lat)
    
    # method assumes constant kd if kt outside the below range
    kd <- NA
    
    if(kt >= 0.35 & kt <= 0.75) {

      kd <- a3 - b3 * kt
    
    }

  }
  
  #6.5 Reindl et al.
  if(method == "Reindl-1"){
    
    if(kt <= 0.3) {

      kd <- 1.02 - 0.248 * kt
    
    }

    if(kt > 0.3 & kt < 0.78) {

      kd <- 1.45 - 1.67 * kt
    
    }

    if(kt >= 0.78) {

      kd <- 0.147
  
    }

  }
  
  if(method == "Reindl-2") {
    
    if(kt <= 0.3) {

      kd <- 1.02 - 0.254 * kt
    
    }

    if(kt > 0.3 & kt < 0.78) {

      kd <- 1.4 - 1.749 * kt + 0.177 * sin(sol.elev * 180 / pi)
    
    }

    if(kt >= 0.78) {

      kd <- 0.486 * kt - 0.182 * sin(sol.elev * 180 / pi)
  
    }

  }
  
  #6.6 Lam and Li
  if(method == "Lam_Li") {
    
    if(kt <= 0.15) {

      kd <- 0.977
    
    }

    if(kt > 0.15 & kt <= 0.7) {

      kd <- 1.237 - 1.361 * kt
    
    }

    if(kt > 0.7) {

      kd <- 0.273
  
    }

  }
  
  #direct and diffuse radiation is c(rad*(1-kd),rad*(kd))
  
  kd

}  

#' @title Ratio of Diffuse to Direct Solar Radiation
#' 
#' @description The function estimates the ratio of diffuse to direct solar radiation based on the approximation of the SOLRAD model \insertCite{McCullough1971}{TrenchR} described in \insertCite{Tracy1983;textual}{TrenchR}.
#' 
#' @param psi \code{numeric} Zenith angle of the sun (degrees).
#' 
#' @param p_a \code{numeric} Atmospheric pressure (kPa).
#' 
#' @param rho \code{numeric} albedo of the substrate (fraction of 1).
#' 
#' @return \code{numeric} diffuse fraction.
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   proportion_diffuse_solar_radiation(psi = 60, 
#'                                      p_a = 86.1, 
#'                                      rho   = 0.25)
#'
proportion_diffuse_solar_radiation <- function(psi, p_a, rho) {
  
  # Input validation (vectorized-safe)
  if (!all(psi >= 0 & psi <= 89.5, na.rm = TRUE))
    stop("'psi' must be between 0 and 89.5 degrees.")
  if (!all(p_a > 0, na.rm = TRUE))
    stop("'p_a' must be > 0.")
  if (!all(rho >= 0 & rho <= 1, na.rm = TRUE))
    stop("'rho' must be between 0 and 1.")
  
  # Low-angle branch (psi <= 50)
  prop_low <- (5.67e-2 +
                 1.698e-5  * psi +
                 1.917e-6  * psi^2 +
                 1.028e-7  * psi^3) *
    (1 + 0.01  * (p_a - 86.1) +
       0.12  * (rho - 0.25))
  
  # High-angle branch (psi > 50)
  prop_high <- (5.83819968 -
                  0.390636004   * psi +
                  9.79200778e-3 * psi^2 -
                  1.0786077e-4  * psi^3 +
                  4.42915464e-7 * psi^4) *
    (1 + 0.009           * (p_a - 86.1) +
       (0.8 - 0.015 * psi) * (rho - 0.25))
  
  # Select branch element-wise
  ifelse(psi <= 50, prop_low, prop_high)
}

context("RadiationPartitioningFunctions.R")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("partition_solar_radiation function works as expected", {
  expect_similar(partition_solar_radiation(method="Liu_Jordan", kt=0.5, lat=40, sol.elev=60),0.248)
  expect_similar(partition_solar_radiation(method="Liu_Jordan", kt=0.1, lat=40, sol.elev=60),1)
  expect_similar(partition_solar_radiation(method="Orgill_Hollands", kt=0.05, lat=40, sol.elev=60),0.98755)
  expect_similar(partition_solar_radiation(method="Orgill_Hollands", kt=0.8, lat=40, sol.elev=60),0.177)
  expect_similar(partition_solar_radiation(method="Orgill_Hollands", kt=0.5, lat=40, sol.elev=60),0.657)
  expect_similar(partition_solar_radiation(method="Erbs", kt=0.1, lat=40, sol.elev=60),0.997)
  expect_similar(partition_solar_radiation(method="Erbs", kt=0.5, lat=40, sol.elev=60),0.65915)
  expect_similar(partition_solar_radiation(method="Erbs", kt=0.85, lat=40, sol.elev=60),0.165)
  expect_similar(partition_solar_radiation(method="Olyphant", kt=0.1, lat=40, sol.elev=60),0.991)
  expect_similar(partition_solar_radiation(method="Olyphant", kt=0.5, lat=40, sol.elev=60),0.65915)
  expect_similar(partition_solar_radiation(method="Olyphant", kt=0.85, lat=40, sol.elev=60),0.125)
  expect_similar(partition_solar_radiation(method="Spencer", kt=0.5, lat=40, sol.elev=60),0.5495)
  expect_similar(partition_solar_radiation(method="Reindl-1", kt=0.1, lat=40, sol.elev=60),0.9952)
  expect_similar(partition_solar_radiation(method="Reindl-1", kt=0.5, lat=40, sol.elev=60),0.615)
  expect_similar(partition_solar_radiation(method="Reindl-1", kt=0.8, lat=40, sol.elev=60),0.147)
  expect_similar(partition_solar_radiation(method="Reindl-2", kt=0.1, lat=40, sol.elev=60),0.9946)
  expect_similar(partition_solar_radiation(method="Reindl-2", kt=0.5, lat=40, sol.elev=60),0.6578213)
  expect_similar(partition_solar_radiation(method="Reindl-2", kt=0.8, lat=40, sol.elev=60),0.2527408)
  expect_similar(partition_solar_radiation(method="Lam_Li", kt=0.1, lat=40, sol.elev=60),0.977)
  expect_similar(partition_solar_radiation(method="Lam_Li", kt=0.5, lat=40, sol.elev=60),0.5565)
  expect_similar(partition_solar_radiation(method="Lam_Li", kt=0.8, lat=40, sol.elev=60),0.273)
})

test_that("proportion_diffuse_solar_radiation function works as expected", {
  expect_similar(proportion_diffuse_solar_radiation(psi = 60, p_a = 86.1, rho = 0.25),0.09352554)
  expect_similar(proportion_diffuse_solar_radiation(psi = 10, p_a = 86.1, rho = 0.25),0.0571643)
})

#' @title Operative Environmental Temperature of a Mussel Bed
#' 
#' @description The function estimates body temperature of a mussel (C). The function implements a steady-state model, which assumes unchanging environmental conditions. Based on \insertCite{Helmuth1999;textual}{TrenchR}.
#' 
#' @param l \code{numeric} mussel length (anterior/posterior axis) (m).
#' 
#' @param T_a \code{numeric} air temperature at 4 m above ground (C).
#' 
#' @param S \code{numeric} direct solar flux density (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#' 
#' @param k_d \code{numeric} diffuse fraction, proportion of solar radiation that is diffuse.
#' 
#' @param u \code{numeric} wind speed at 4 m above ground (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @param evap \code{logical} Are mussels gaping to evaporatively cool? If \code{TRUE}, assumes constant mass loss rate of 5 percent of initial body mass per hour. 
#' 
#' @param cl \code{numeric} fraction of the sky covered by cloud, optional.
#' 
#' @details Conduction is considered negligible due to a small area of contact.
#'   \cr \cr
#'   Thermal radiative flux is calculated following \insertCite{Helmuth1998;textual}{TrenchR}, \insertCite{Helmuth1999;textual}{TrenchR}, and \insertCite{Idso1969;textual}{TrenchR}.
#'
#' 
#' @return \code{numeric} predicted body (operative environmental) temperature (C).
#' 
#' @family biophysical models
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'  Tbed_mussel(l    = 0.1, 
#'              T_a  = 25, 
#'              S    = 500, 
#'              k_d  = 0.2, 
#'              u    = 1, 
#'              evap = FALSE)
#' 
Tbed_mussel <- function(l, 
                        T_a, 
                        S, 
                        k_d, 
                        u, 
                        evap = FALSE, 
                        cl   = NA) {
  
  stopifnot(l   >= 0,
            S   >= 0, 
            k_d >= 0, 
            k_d <= 1, 
            u   >  0, 
            is.logical(evap))

  if (!is.na(cl)) {

    stopifnot(cl  >= 0, 
              cl  <= 1)
  }
  
  T_a <- celsius_to_kelvin(T_a)
  
  sigma  <- stefan_boltzmann_constant()
  lambda <- latent_heat_vaporization_h2o()
  c      <- specific_heat_h2o()
  
  # Areas

    # total mussel shell surface area (m^2)

      A <- 1.08 * l^2 + 0.0461 * l - 0.0016   

    # projected area for tightly packed bed

      Ap <- 0.15 * A 
  
  
    # Assume 40% exposed to ground

      A_radSky <- 0.4 * A 

    # Assume 40% exposed to sky

      A_radGround <- 0.4 * A 

    # Assume 40% exposed to diffuse solar radiation

      A_solDiff <- 0.4 * A 

    # Assume 50% exposed to convection

      Aconv <- 0.5 * A 
  
  # Convection 

    # m/s, shear velocity

      u_star <- 0.03 * u 

    # c_p * rho <- 1200 J m^{-3}*K^{-1}

      c_prho <- 1200

      k      <- von_karman_constant()

    # environmental input is at height of 4m

      z  <- 4 
  
    # m, roughness height

      z0 <- 0.0017
  
    C <- (Aconv / Ap) * (u_star * c_prho * k) / (log(z / z0))
  
   
  
  # Emissivities

    eps_ac <- 0.72 + 0.005 * (T_a - 273) # Helmuth 1999 from Idso and Jackson 1969
    eps_sky <- ifelse( is.na(cl), 0.9, eps_ac + cl * (1 - eps_ac - 8 / T_a))  # Helmuth 1999, assume 0.9 if no cloudiness data
    eps_org <- 1.0   # infrared emissivity of organism (same as above, p.163)
  
  # Evaporation - assume constant mass loss rate of 5% initial body mass per hour, assuming a density of 700 mussels/m2 and L=0.075

    mflux <- ifelse(evap == FALSE, 0, 1.99 * 10^{-4})  #kg/sm^2
  
  # Radiation

    alpha <- 0.75   # solar absorptivity
  
    S_dir  <- S * (1 - k_d) # direct radiation
    S_diff <- S * (k_d) # diffuse radiation
  
    solar <- alpha * (S_dir + (A_solDiff / Ap) * S_diff)
    Rad_s <- 4 * sigma * (A_radSky / Ap) * eps_sky^0.75
    Rad_g <- 4 * sigma * (A_radGround / Ap)
  

  # Steady-state heat balance model
 
  # Solve steady state energy balance equation:
  # T_b*mflux*c= Q_rad,sol +- Q_rad,sky +- Q_rad,ground +- Q_conduction +- Qconvection -Qevaporation
  
  T_bed <- (solar + T_a^4 * (Rad_s + Rad_g) + C * T_a - lambda * mflux) / (mflux * c + T_a^3 * (Rad_s + Rad_g) + C)
 
  kelvin_to_celsius(T_bed)
  
}  

context("biophysmodel_mussel bed")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}


test_that("Tbed_mussel function works as expected", {
  expect_similar(Tbed_mussel(l     = 0.1, 
                           T_a   = 25, 
                           S     = 500, 
                           k_d   = 0.2, 
                           u     = 1, 
                           evap  = FALSE), 38.44283)

  expect_similar(Tbed_mussel(l     = 0.1, 
                           T_a   = 25, 
                           S     = 500, 
                           k_d   = 0.2, 
                           u     = 1, 
                           evap  = TRUE), 31.6204)

  expect_similar(Tbed_mussel(l     = 0.1, 
                           T_a   = 25, 
                           S     = 500, 
                           k_d   = 0.2, 
                           u     = 1, 
                           cl    = 0.5,
                           evap  = FALSE), 38.4008)

})

#' @title Operative Environmental Temperature of a Mussel
#' 
#' @description The function estimates body temperature (C, operative environmental temperature) of a mussel. The function implements a steady-state model, which assumes unchanging environmental conditions. 
#' 
#' @param l \code{numeric} mussel length (anterior/posterior axis, m).
#' 
#' @param h \code{numeric} mussel height (dorsal/ventral axis, m). It is reasonable to assume \code{h = 0.5 * l}.
#' 
#' @param T_a \code{numeric} air temperature (C).
#' 
#' @param T_g \code{numeric} ground temperature (C).
#' 
#' @param S \code{numeric} direct solar flux density (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#' 
#' @param k_d \code{numeric} diffuse fraction, proportion of solar radiation that is diffuse.
#' 
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @param psi \code{numeric} solar zenith angle (degrees): can be calculated from \code{\link{zenith_angle}}.
#' 
#' @param evap \code{logical} Whether mussel is gaping to evaporatively cool. If \code{TRUE}, the function assumes a constant mass loss rate of 5 percent of the initial body mass per hour. 
#' 
#' @param cl \code{numeric} fraction of the sky covered by cloud. 
#' 
#' @param group \code{character}; options are \code{"aggregated"}: mussels living in beds; \code{"solitary"}: solitary individual, anterior or posterior end facing upwind; and \code{"solitary_valve"}: solitary individual, valve facing upwind.
#'
#' @details Thermal radiative flux is calculated following \insertCite{Helmuth1998;textual}{TrenchR}, \insertCite{Helmuth1999;textual}{TrenchR}, and \insertCite{Idso1969;textual}{TrenchR}.
#'
#' @return \code{numeric} predicted body (operative environmental) temperature (C).
#' 
#' @family biophysical models
#' 
#' @references
#'  \insertAllCited{}
#'
#' @export
#' 
#' @examples
#' 
#' Tb_mussel(l     = 0.1, 
#'           h     = 0.05, 
#'           T_a   = 25, 
#'           T_g   = 30, 
#'           S     = 500, 
#'           k_d   = 0.2, 
#'           u     = 2, 
#'           psi   = 30, 
#'           evap  = FALSE, 
#'           cl    = 0.5, 
#'           group = "solitary")
#' 
Tb_mussel <- function (l, 
                       h, 
                       T_a, 
                       T_g, 
                       S, 
                       k_d, 
                       u, 
                       psi, 
                       cl, 
                       evap  = FALSE, 
                       group = "solitary") {
  
  stopifnot(l   >  0, 
            h   >  0, 
            u   >= 0, 
            S   >= 0, 
            psi >= 0, 
            psi <= 90, 
            cl  >= 0, 
            cl  <= 1, 
            k_d >= 0, 
            k_d <= 1, 
            is.logical(evap), 
            length(group) == 1,
            group %in% c("aggregated", "solitary", "solitary_valve"))
  
  T_a <- celsius_to_kelvin(T_a)
  T_g <- celsius_to_kelvin(T_g)

  # total mussel shell surface area (m^2)

    A <- 1.08 * l^2 + 0.0461 * l - 0.0016   

  # mussel body mass, kg
  
    m <- 191 * l^3.53  

  psi <- degrees_to_radians(psi)
  
  # constants

    sigma <- stefan_boltzmann_constant()  
    lambda <- latent_heat_vaporization_h2o()                   
  
  #  Short-wave solar flux  

    # solar absorptivity

      alpha <- 0.75           
      k1 <- alpha / sin(psi)
  
  # direct radiation

    S_p <- S * (1 - k_d) 

  # diffuse radiation

    S_d <- S * (k_d)

  # omit reflected radiation
  
  # Long-wave radiaion
  
    # emissivities

      # Helmuth 1999 from Idso and Jackson 1969

        eps_ac <- 0.72 + 0.005 * (T_a - 273) 

      # Helmuth 1999

        eps_sky <- eps_ac + cl * (1 - eps_ac - 8 / T_a)

      # infrared emissivity of organism (same as above, p.163)

        eps_org <- 1.0         
  
  # Estimate lumped coefficients

    k2 <- 4 * sigma * eps_org * eps_sky^(3/4)
    k3 <- eps_sky^(1/4)
    k4 <- 4 * sigma * eps_org
  
  # Conduction (Coefficient)
  #   thermal conductivity of heat in body (W m^-2 K^-1). 
  #   Approximated to that of water because mussels are mostly made of water

    kb <- 0.6      
    k5 <- kb / (0.5 * h)
  
  # Convection
  # Denny and Harley. 2006, Hot limpets: predicting body temperature in a conductance-mediated thermal system 

  # Conductivity of air (W m^-1 K^-1)

    Ka <- 0.00501 + 7.2 * 10^-5 * T_a        

  # Kinematic viscosity of air (m^2 s^-1)

    v <- -1.25 * 10^-5 + 9.2 * 10^-8 * T_a   
  
  # average body dimensions (Helmuth 1998 p.74)

    d <- l * 2 / 3   

  # Reynolds number

    Re <- u * d / v
  
  if (group == "aggregated") {

    a <- 0.67
    b <- 0.42

  } else if (group == "solitary"){

    a <- 0.38
    b <- 0.51

  } else {

    a <- 0.63
    b <- 0.47

  }

  # Nusselt number

    Nu <- a * Re^b   

  # heat transfer coefficient (W m^-2 K^-1)

    hc <- Nu * Ka / d     
  
  # evaporative mass loss
  #
  #   set maximum mflux rate to 5% of body mass over 1 hour, level that results in dessication
  #   see Helmuth 1998 for a more detailed evaporation model based on mussel gaping

    mflux <- ifelse(evap == FALSE, 0, m * 0.05 / (60 * 60))

  # Calculating areas of interest

    # surface area subject to long-wave radiation from sky (m^2). Half facing the sky, the other half facing the ground

      A_radSky <- A / 2    

    # surface area subject to long-wave radiation from ground (m^2)

      A_radGround <- A / 2 

    # area of contact between mussel and ground (m^2)

      A_cond <- 0.05 * A

    # surface area exposed to convective heat loss (m^2)

      A_conv <- A          

    # surface area exposed to diffuse and albedo flux (m^2)

      A_d <- A / 2         
  
  if (group == "aggregated") {

    # projected area in direction of sun (m^2)
    
    A_sol <- 0.15 * A

  } else {

    A_sol <- 0.25 * A

  }
  
  # Steady-state heat balance model
 
  # Solve steady state energy balance equation:

  # T_b*mflux*c <- Q_rad,sol +- Q_rad,sky +- Q_rad,ground +- Q_conduction +- Qconvection -Qevaporation
  
  T_b <- (k1 * (A_sol * S_p + A_d * S_d) + k2 * A_radSky * k3 * T_a^4 + k4 * A_radGround * T_g^4 + k5 * A_cond * T_g + hc * A_conv * T_a - lambda * mflux) / 
         (k2 * A_radSky * T_a^3 + k4 * A_radGround * T_g^3 + k5 * A_cond + hc * A_conv + mflux * specific_heat_h2o())
  
  kelvin_to_celsius(T_b)

}

context("biophysmodel_mussel")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("Tb_mussel function works as expected", {
  expect_similar(Tb_mussel(l     = 0.1, 
                           h     = 0.05, 
                           T_a   = 25, 
                           T_g   = 30, 
                           S     = 500, 
                           k_d   = 0.2, 
                           u     = 2, 
                           psi   = 30, 
                           evap  = FALSE, 
                           cl    = 0.5, 
                           group = "aggregated"), 33.66166)
  expect_similar(Tb_mussel(l     = 0.1, 
                           h     = 0.05, 
                           T_a   = 25, 
                           T_g   = 30, 
                           S     = 500, 
                           k_d   = 0.2, 
                           u     = 2, 
                           psi   = 30, 
                           evap  = FALSE, 
                           cl    = 0.5, 
                           group = "solitary"), 34.97406)
  expect_similar(Tb_mussel(l     = 0.1, 
                           h     = 0.05, 
                           T_a   = 25, 
                           T_g   = 30, 
                           S     = 500, 
                           k_d   = 0.2, 
                           u     = 2, 
                           psi   = 30, 
                           evap  = FALSE, 
                           cl    = 0.5, 
                           group = "solitary_valve"), 34.03166)
})

#' @title Operative Environmental Temperature of an Ectotherm based on Campbell and Norman (1988)
#' 
#' @description The function estimates body temperatures (C, operative environmental temperature) of an ectotherm using an approximation based on \insertCite{Campbell1998;textual}{TrenchR} and \insertCite{Mitchell1976;textual}{TrenchR}.
#' 
#' @param T_a \code{numeric} air temperature (C).
#' 
#' @param T_g \code{numeric} ground temperature (C).
#' 
#' @param S \code{numeric} flux density of solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}), combining direct, diffuse, and reflected radiation accounting for view factors.
#' 
#' @param a_s \code{numeric} organismal solar absorptivity (proportion). 
#' 
#' @param a_l \code{numeric} organismal thermal absorptivity (proportion); 0.965 for lizards \insertCite{Bartlett1967}{TrenchR}.
#' 
#' @param epsilon \code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.
#' 
#' @param c_p \code{numeric} specific heat of air (\ifelse{html}{\out{J mol<sup>-1</sup> C<sup>-1</sup>}}{\eqn{J mol^-1 C^-1}{ASCII}}). 
#' 
#' @param D \code{numeric} characteristic dimension of the animal (m).
#' 
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @details Boundary conductance uses a factor of 1.4 to account for increased convection \insertCite{Mitchell1976}{TrenchR}. The function assumes forced conduction.
#'
#' @return \code{numeric} operative environmental temperature, \code{T_e} (C).
#' 
#' @family biophysical models
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples 
#' Tb_CampbellNorman (T_a     = 30, 
#'                    T_g     = 30, 
#'                    S       = 823, 
#'                    a_s = 0.7, 
#'                    a_l = 0.96, 
#'                    epsilon = 0.96, 
#'                    c_p     = 29.3, 
#'                    D       = 0.17, 
#'                    u       = 1)
#'
Tb_CampbellNorman <- function (T_a, 
                               T_g, 
                               S, 
                               a_s = 0.7, 
                               a_l = 0.96, 
                               epsilon = 0.96, 
                               c_p = 29.3, 
                               D, 
                               u) {
    
  stopifnot(T_a     >  -50, 
            T_a     <  100, 
            epsilon >= 0.5, 
            epsilon <= 1, 
            a_s >= 0, 
            a_l >= 0, 
            c_p     >= 0, 
            D       >  0, 
            u       >= 0)
  
  sigma <- stefan_boltzmann_constant()
  
  #convert temperatures to Kelvin
    T_a = T_a +273.15
    T_g = T_g +273.15
  
  # solar and thermal radiation absorbed
  # (10.7) long wave flux densities from atmosphere, ground 

    L_a <- sigma * T_a^4  
    L_g <- sigma * T_g^4 

  # proportion of organism exposure to air, ground

    F_a <- 0.5 
    F_g <- 0.5

  # (11.14) Absorbed radiation

    R_abs <- a_s * S + a_l * (F_a * L_a + F_g * L_g) 
  
  # thermal radiation emitted

    Qemit <- epsilon * sigma * T_a^4
  
  # conductance: boundary, ground
  #   boundary conductance, factor of 1.4 to account for increased convection (Mitchell 1976), assumes forced conduction

    g_Ha <- 1.4 * 0.135 * sqrt(u / D) 
    g_r <- 4 * epsilon * sigma * T_a^3 / c_p 
  
  # operative environmental temperature in C

    T_a + (R_abs - Qemit) / (c_p * (g_r + g_Ha)) -273.15
  
}

#' @title Net Energy Exchange Between an Animal and the Environment
#' 
#' @description The function estimates the net energy exchange (W) between an animal and the environment. The function follows \insertCite{Gates1980;textual}{TrenchR} and others.
#' 
#' @param Qabs \code{numeric} solar radiation absorbed (W).
#' 
#' @param Qemit \code{numeric} thermal radiation emitted (W).
#' 
#' @param Qconv \code{numeric} energy exchange due to convection; Energy exchange from an animal to its surrounding environment (air or water) (W).
#' 
#' @param Qcond \code{numeric} energy exchange due to conduction; Energy exchange from animal to a surface if they are in contact  (W).
#' 
#' @param Qmet \code{numeric} energy emitted due to metabolism (W).
#' 
#' @param Qevap \code{numeric} energy emitted due to evaporative water loss (W).
#' 
#' @return \code{numeric} net energy exchange (W).
#' 
#' @family biophysical models
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples 
#' Qnet_Gates(Qabs  = 500,
#'            Qemit = 10, 
#'            Qconv = 100, 
#'            Qcond = 100, 
#'            Qmet  = 10, 
#'            Qevap = 5)
#' 
Qnet_Gates <- function (Qabs, 
                        Qemit, 
                        Qconv, 
                        Qcond, 
                        Qmet, 
                        Qevap) {
  
  stopifnot(Qabs  >= 0, 
            Qmet  >= 0)
  
  Qabs - Qemit - Qconv - Qcond - Qmet - Qevap
  
}


#' @title Operative Environmental Temperature of an Ectotherm Based on Gates (1980) 
#' 
#' @description The function predicts body temperatures (C, operative environmental temperature) of an ectotherm using the approximation in \insertCite{Gates1980;textual}{TrenchR}. The functions omits evaporative and metabolic heat loss \insertCite{Mitchell1976,Kingsolver1983}{TrenchR}.
#' 
#' @param A \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#' 
#' @param D \code{numeric} characteristic dimension for conduction (meters).
#' 
#' @param psa_dir \code{numeric} view factor for the proportion surface area exposed to direct radiation from the sky (or enclosure) (0-1).
#' 
#' @param psa_ref \code{numeric} view factor for proportion surface area exposed to reflected radiation from the ground (0-1).
#' 
#' @param psa_air \code{numeric} proportion surface area exposed to air (0-1).
#' 
#' @param psa_g \code{numeric} proportion surface area in contact with substrate (0-1).
#' 
#' @param T_g \code{numeric} ground surface temperature (C).
#' 
#' @param T_a \code{numeric} ambient air temperature (C).
#' 
#' @param Qabs \code{numeric} Solar radiation absorbed (W).
#' 
#' @param epsilon \code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.
#' 
#' @param H_L \code{numeric} Convective heat transfer coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}).
#' 
#' @param ef \code{numeric} enhancement factor used to adjust H_L to field conditions using h_L approximation from \insertCite{Mitchell1976;textual}{TrenchR}.  Approximated as 1.23 by default, but see \insertCite{Mitchell1976;textual}{TrenchR} for relationship.
#' 
#' @param K \code{numeric} Thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}), K = 0.5 \ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}} for naked skin, K = 0.15 \ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}for insect cuticle \insertCite{Galushko2005;textual}{TrenchR}; conductivity of the ground is generally greater than that of animal tissues, so animal thermal conductivity is generally the rate limiting step. 
#' 
#' @return \code{numeric} operative environmental temperature, \code{T_e} (C).
#' 
#' @family biophysical models
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples 
#'  Tb_Gates (A       = 0.1, 
#'            D       = 0.025, 
#'            psa_dir = 0.6, 
#'            psa_ref = 0.4, 
#'            psa_air = 0.5, 
#'            psa_g   = 0.1, 
#'            T_g     = 30, 
#'            T_a     = 37, 
#'            Qabs    = 2, 
#'            epsilon = 0.95, 
#'            H_L     = 10, 
#'            ef      = 1.23, 
#'            K       = 0.5)
#' 
Tb_Gates <- function (A, 
                      D, 
                      psa_dir, 
                      psa_ref, 
                      psa_air, 
                      psa_g, 
                      T_g, 
                      T_a, 
                      Qabs, 
                      epsilon, 
                      H_L, 
                      ef = 1.3, 
                      K) {

  stopifnot(A       >  0, 
            D       >  0, 
            psa_dir >= 0, 
            psa_dir <= 1, 
            psa_ref >= 0, 
            psa_ref <= 1, 
            psa_air >= 0, 
            psa_air <= 1, 
            psa_g   >= 0, 
            psa_g   <= 1, 
            T_g     >  -50, 
            T_g     <  100, 
            T_a     >  -50, 
            T_a     <  100, 
            Qabs    >= 0, 
            epsilon >  0.5, 
            epsilon <= 1, 
            H_L     >  0, 
            K       >  0)
  
  sigma <- stefan_boltzmann_constant()
  
  # Areas

    A_s <- A * psa_dir 
    A_r <- A * psa_ref 
  
    # skin area exposed to air

      A_air <- A * psa_air 

    # area of contact

      A_contact <- A * psa_g 
  
  # effective radiant temperature of sky
  #  K, Gates 1980 Biophysical ecology based on Swinback 1960, Kingsolver (1983) estimates using Brunt equation

    T_sky <- celsius_to_kelvin(1.22 * T_a - 20.4)
  
  #convert to Kelvin
    T_a= celsius_to_kelvin(T_a)
    T_g= celsius_to_kelvin(T_g)
    
  # energy balance function for steady state conditions
  #  0 = Qabs - Qemit - Qconv - Qcond

    Qfn <- function(T_b, Qabs, epsilon, sigma, A_s, T_sky, A_r, T_g, H_L, A_air, T_a, A_contact, K, D) {

      Qemit <- epsilon * sigma * (A_s * (T_b^4 - T_sky^4) + A_r * (T_b^4 - T_g^4))
      Qconv <- ef * H_L * A_air * (T_b - T_a) 
      Qcond <- A_contact * K * (T_b - T_g) / D 

      Qabs - Qemit - Qconv - Qcond
    
    }
  
  T_e <- tryCatch(expr = uniroot(f         = Qfn, 
                                 interval  = c(273, 350), 
                                 Qabs      = Qabs, 
                                 epsilon   = epsilon, 
                                 sigma     = sigma, 
                                 A_s       = A_s, 
                                 T_sky     = T_sky, 
                                 A_r       = A_r, 
                                 T_g       = T_g, 
                                 H_L       = H_L, 
                                 A_air     = A_air, 
                                 T_a       = T_a, 
                                 A_contact = A_contact, 
                                 K         = K, 
                                 D         = D, 
                                 tol       = 0.0001)$root, 
                 
                  error = function(e) {
                            message("Unable to balance energy budget. One issue to check is whether absorbed solar radiation exceeds energy potentially lost to thermal radiation, convection, and conduction.")
                            NA})
  
  kelvin_to_celsius(T_e)

}

#' @title Operative Environmental Temperature of an Ectotherm Based on a Variant of Gates (1980) 
#' 
#' @description The function predicts body temperatures (C, operative environmental temperature) of an ectotherm using the approximation in \insertCite{Gates1980;textual}{TrenchR}. The function omits evaporative and metabolic heat loss.
#' 
#' @param A \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#' 
#' @param D \code{numeric} characteristic dimension for conduction (meters).
#' 
#' @param T_g \code{numeric} ground surface temperature (C).
#' 
#' @param T_a \code{numeric} ambient air temperature (C).
#' 
#' @param Qabs \code{numeric} Solar radiation absorbed (W).
#' 
#' @param u \code{numeric} Wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @param epsilon \code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.
#' 
#' @return \code{numeric} operative environmental temperature (C). 
#' 
#' @family biophysical models
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples 
#'   Tb_Gates2(A       = 1, 
#'             D       = 0.001, 
#'             T_g     = 27, 
#'             T_a     = 37, 
#'             Qabs    = 2, 
#'             u       = 0.1, 
#'             epsilon = 1) 
#' 
Tb_Gates2 <- function (A, 
                       D, 
                       T_g, 
                       T_a, 
                       Qabs, 
                       u, 
                       epsilon) {
  
  sigma <- stefan_boltzmann_constant()

  stopifnot(A       >  0, 
            D       >  0, 
            T_g     >  -50, 
            T_g     <  100, 
            T_a     >  -50, 
            T_a     <  100, 
            Qabs    >= 0, 
            epsilon >  0.5, 
            epsilon <= 1, 
            u       >  0)


  # skin area exposed to air

    A_air <- A
  
  # Convection coefficient from Gates (1980)

    H_L <- 3.49 * (u^0.5 / D^0.5)
  
  # effective radiant temperature of sky
  #  K, Gates 1980 Biophysical ecology based on Swinback 1960, Kingsolver (1983) estimates using Brunt equation

    T_sky <- celsius_to_kelvin(1.22 * (T_a - 20.4))
    
    #convert to Kelvin
    T_a= celsius_to_kelvin(T_a)
    T_g= celsius_to_kelvin(T_g)
    
  # Thermal radiation absorbed 

    QIR <- (A_air / 2) * (sigma * epsilon * T_g^4 + sigma * epsilon * T_sky^4)
  
  # energy balance function for steady state conditions
  #   0 = Qabs - Qemit - Qconv - Qcond

    Qfn <- function(T_b, Qabs, QIR, epsilon, sigma, T_sky, H_L, A_air, T_a) {
    
      Qemit <- A_air * sigma * epsilon * T_b^4
      Qconv <- H_L * A_air * (T_b - T_a)
    
      Qabs + QIR - Qemit - Qconv

    }
  
  T_e <- tryCatch(expr = uniroot(f        = Qfn,
                                 interval = c(273, 353),
                                 Qabs     = Qabs, 
                                 QIR      = QIR, 
                                 epsilon  = epsilon, 
                                 sigma    = sigma, 
                                 T_sky    = T_sky, 
                                 H_L      = H_L, 
                                 A_air    = A_air, 
                                 T_a      = T_a, 
                                 tol      = 0.0001)$root, 
                  error = function(e) {
                            message("Unable to balance energy budget. One issue to check is whether absorbed solar radiation exceeds energy potentially lost to thermal radiation, convection, and conduction.")
                            NA})
  
  kelvin_to_celsius(T_e)
  
}

context("OperativeTemperatureFunctions")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("Tb_CampbellNorman function works as expected", {
  expect_similar(Tb_CampbellNorman(T_a=30, T_g=30, S=823, a_s=0.7, epsilon=0.96, c_p=29.3, D=0.17, u=1), 59.54376)
})

test_that("Qnet_Gates function works as expected", {
  expect_equal(Qnet_Gates(Qabs=500, Qemit=10, Qconv=100, Qcond=100, Qmet=10, Qevap=5), 275)
})

test_that("Tb_Gates function works as expected", {
  expect_similar(Tb_Gates(A=0.1, D=0.025, psa_dir=0.6, psa_ref=0.4, psa_air=0.6, psa_g=0.2, T_g=30, T_a=37, Qabs=2, epsilon=0.95, H_L=10, ef=1.23, K=0.5), 33.04367)
expect_message(x <- Tb_Gates(A=0.1, D=0.025, psa_dir=0.6, psa_ref=0.4, psa_air=0.6, psa_g=0.2, T_g=77, T_a=-37, Qabs=2, epsilon=0.95, H_L=10, ef=1.23, K=0.01))
expect_true(is.na(x))
})

test_that("Tb_Gates2 function works as expected", {
  expect_similar(Tb_Gates2(A = 0.1, D = 0.025, T_g = 27, T_a = 37, Qabs = 2, u = 0.1, epsilon = 1), 32.25524)
expect_message(x <- Tb_Gates2(A = 0.1, D = 0.025, T_g = 90, T_a = -42, Qabs =1, u = 0.1, epsilon = 0.6))
expect_true(is.na(x))
})

#' @title Hourly Temperature Variation assuming Sine and Exponential Components
#'
#' @description The function estimates temperature across hours using a diurnal temperature variation function incorporating sine and exponential components \insertCite{Parton1981}{TrenchR}. 
#'
#' @details Default \code{alpha}, \code{beta}, and \code{gamma} values are the
#'   average of 5 North Carolina sites \insertCite{Wann1985}{TrenchR}.
#'   \cr \cr
#'   Other \code{alpha}, \code{beta}, and \code{gamma} parameterizations include
#'   values for Denver, Colorado from \insertCite{Parton1981;textual}{TrenchR}:
#'   \describe{
#'     \item{150 cm air temperature}{\code{alpha} = 1.86, \code{beta} = 2.20, \code{gamma} = -0.17}
#'     \item{10 cm air temperature}{\code{alpha} = 1.52, \code{beta} = 2.00, \code{gamma} = -0.18}
#'     \item{Soil surface temperature}{\code{alpha} = 0.50, \code{beta} = 1.81, \code{gamma} = 0.49}
#'     \item{10 cm soil temperature}{\code{alpha} = 0.45, \code{beta} = 2.28, \code{gamma} = 1.83}
#'   }
#'
#' @param T_max,T_min \code{numeric} maximum and minimum daily temperatures (C).
#'  
#' @param t_r,t_s \code{numeric} times of sunrise and sunset (hour).
#' 
#' @param t \code{numeric} time for temperature estimate (hour).
#' 
#' @param alpha \code{numeric} time difference between \code{t_x} (time of maximum temperature) and noon (hour). 
#' 
#' @param gamma \code{numeric} decay parameter for rate of \code{t} change from sunset to \code{t_n} (time of minimum temp). 
#' 
#' @param beta \code{numeric} time difference between \code{t_x} and sunrise (hour).
#' 
#' @return \code{numeric} temperature (C) at a specified hour. 
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   diurnal_temp_variation_sineexp(T_max = 30, 
#'                                  T_min = 10, 
#'                                  t     = 11:15, 
#'                                  t_r   = 6, 
#'                                  t_s   = 18)
#' 
diurnal_temp_variation_sineexp <- function (T_max, 
                                            T_min, 
                                            t, 
                                            t_r, 
                                            t_s, 
                                            alpha = 2.59, 
                                            beta  = 1.55, 
                                            gamma = 2.2) {

  # Input validation (vectorized-safe)
  if (!all(T_max >= T_min, na.rm = TRUE))
    stop("'T_max' must be >= 'T_min'.")
  if (!all(t_r >= 0 & t_r <= 24, na.rm = TRUE))
    stop("'t_r' must be between 0 and 24.")
  if (!all(t_s >= 0 & t_s <= 24, na.rm = TRUE))
    stop("'t_s' must be between 0 and 24.")
  if (!all(t >= 0 & t <= 24, na.rm = TRUE))
    stop("'t' must be between 0 and 24.")
  if (!all(alpha >= 0 & alpha <= 24, na.rm = TRUE))
    stop("'alpha' must be between 0 and 24.")
  if (!all(beta >= 0 & beta <= 24, na.rm = TRUE))
    stop("'beta' must be between 0 and 24.")
   
  # daylength

    l <- t_s - t_r 
  
  # time of maximum, minimum temperatures

   t_x <- 0.5 * (t_r + t_s) + alpha 
   t_n <- t_r +  beta 
  
  # if night or day

  # day
  temp_day <- T_min + (T_max - T_min) * sin((pi * (t - t_r - beta)) / (l + 2 * (alpha - beta)))
  
  # sunset
  T_sn <- T_min + (T_max - T_min) * sin((pi * (t_s - t_r - beta)) / (l + 2 * (alpha - beta)))
   
  # after sunset
  t_as <- ifelse(t <= (t_r + beta), t + 24 - t_s, t - t_s)
  
  # night
  temp_night <- T_min + (T_sn - T_min) * exp(-(gamma * t_as) / (24 - l + beta))
   
  is_daytime <- (t > (t_r + beta) & t < t_s)
   
  Temp <- ifelse(is_daytime, temp_day, temp_night)
   
  Temp
  
}


#' @title Hourly Temperature Variation assuming a Sine Interpolation
#'
#' @description The function estimates temperature for a specified hour using the sine interpolation in \insertCite{Campbell1998;textual}{TrenchR}.
#' 
#' @param T_max,T_min \code{numeric} maximum and minimum daily temperatures (C). 
#' 
#' @param t \code{numeric} time for temperature estimate (hour).
#' 
#' @return \code{numeric} temperature (C) at a specified hour. 
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   diurnal_temp_variation_sine(T_max = 30, 
#'                               T_min = 10, 
#'                               t     = 11:15)
#' 
diurnal_temp_variation_sine <- function (T_max, 
                                         T_min, 
                                         t) {
  
  stopifnot(t     >= 0, 
            t     <= 24, 
            T_max >= T_min)
  
  W <- pi / 12
  gamma <- 0.44 - 0.46 * sin(0.9 + W * t) + 0.11 * sin(0.9 + 2 * W * t)   
    
  T_max * gamma + T_min * (1 - gamma)
  
}


#' @title Hourly Temperature Variation using Sine and Square Root Functions
#'
#' @description  The function estimates temperature for a specified hour using sine and square root functions \insertCite{Cesaraccio2001}{TrenchR}.
#' 
#' @param t \code{numeric} hour or hours for temperature estimate.
#' 
#' @param t_r,t_s \code{numeric} sunrise and sunset hours (0-23).
#' 
#' @param T_max,T_min \code{numeric} maximum and minimum temperatures of current day (C).
#' 
#' @param T_minp \code{numeric} minimum temperature of following day (C).
#' 
#' @return \code{numeric} temperature (C) at a specified hour. 
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   diurnal_temp_variation_sinesqrt(t      = 11:15, 
#'                                   t_r    = 6, 
#'                                   t_s    = 18, 
#'                                   T_max  = 30, 
#'                                   T_min  = 10, 
#'                                   T_minp = 12)
#' 
diurnal_temp_variation_sinesqrt <- function (t, 
                                             t_r, 
                                             t_s, 
                                             T_max, 
                                             T_min, 
                                             T_minp) {
 
  stopifnot(t     >= 0, 
            t     <= 24, 
            t_r   >= 0, 
            t_r   <= 24, 
            t_s   >= 0, 
            t_s   <= 24, 
            T_max >= T_min)
  
  # Time estimates
  # Assume time of maximum temperature 4h before sunset

    t_p <- t_r + 24 
    t_x <- t_s - 4 
  
  # Temperature at sunset

    c  <- 0.39 
    To <- T_max - c * (T_max - T_minp)
  
  alpha <- T_max - T_min
  R <- T_max - To
  b <- (T_minp - To) / sqrt(t_p - t_s)
  
  Temp <- rep(NA, length(t))
  
  inds <- which(t <= t_r) 
  if(length(inds > 0))  {
    
    Temp[inds] <- To + b * sqrt(t[inds] - (t_s - 24))
    
  }

  inds <- which(t > t_r & t <= t_x) 
  if(length(inds > 0)) {
    
    Temp[inds] <- T_min + alpha * sin(pi / 2 * (t[inds] - t_r) / (t_x - t_r))
    
  }
  
  inds <- which(t > t_x & t < t_s) 
  if(length(inds > 0)) {
    
    Temp[inds] <- To + R * sin(pi / 2 * (1 + (t[inds] - t_x) / 4))
    
  }
  
  inds <- which(t >= t_s) 
  if(length(inds > 0))  {
    
    Temp[inds] <- To + b * sqrt(t[inds] - t_s)
    
  }

  Temp
  
}

context("DTRFunctions")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("diurnal_temp_variation_sineexp function works as expected", {
  expect_similar(diurnal_temp_variation_sineexp(T_max=30, T_min=10, t=11, t_r=6, t_s=18, alpha=2.59, beta= 1.55, gamma=2.2), 23.91954)
  expect_similar(diurnal_temp_variation_sineexp(T_max=30, T_min=10, t=20, t_r=6, t_s=18, alpha=2.59, beta= 1.55, gamma=2.2), 20.46866)
  expect_similar(diurnal_temp_variation_sineexp(T_max=30, T_min=10, t=3, t_r=6, t_s=18, alpha=2.59, beta= 1.55, gamma=2.2), 13.35971)
  
})

test_that("diurnal_temp_variation_sine function works as expected", {
  expect_similar(diurnal_temp_variation_sine(T_max=30, T_min=10, t=11), 25.08958)
})

test_that("diurnal_temp_variation_sinesqrt function works as expected", {

  expect_similar(diurnal_temp_variation_sinesqrt(t=8, t_r=6, t_s=18, T_max=30, T_min=10, T_minp=12), 17.65367)
  expect_similar(diurnal_temp_variation_sinesqrt(t=2, t_r=6, t_s=18, T_max=30, T_min=10, T_minp=12), 14.01487)
  expect_similar(diurnal_temp_variation_sinesqrt(t=16, t_r=6, t_s=18, T_max=30, T_min=10, T_minp=12), 27.94389)
  expect_similar(diurnal_temp_variation_sinesqrt(t=20, t_r=6, t_s=18, T_max=30, T_min=10, T_minp=12), 18.49743)

})

#' @title Tools for Microclimate and Biophysical Ecology
#'
#' @description Tools for translating environmental change into organismal
#'   response. Microclimate models to vertically scale weather station data
#'   to organismal heights. The biophysical modeling tools include both
#'   general models for heat flows and specific models to predict body
#'   temperatures for a variety of ectothermic taxa. Additional functions
#'   model and temporally partition air and soil temperatures and solar
#'   radiation. Utility functions estimate the organismal and environmental
#'   parameters needed for biophysical ecology. 'TrenchR' focuses on
#'   relatively simple and modular functions so users can create transparent
#'   and flexible biophysical models. Many functions are derived from
#'   Gates (1980) and Campbell and Norman (1988).
#'
#' @keywords internal
"_PACKAGE"

#' @importFrom deSolve ode
#' @importFrom msm rtnorm
#' @importFrom Rdpack reprompt
#' @importFrom stats integrate lm runif uniroot
#' @importFrom zoo na.approx
NULL

#' @title Organism Surface Area from Mass 
#' 
#' @description The function estimates surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) from mass (g) for one of a variety of taxa. 
#'
#' @param m \code{numeric} vector of mass (g). 
#'
#' @param taxon \code{character} taxonomic classification of organism, current choices: \code{"lizard"}, \code{"salamander"}, \code{"frog"}, \code{"insect"}.
#' 
#' @return \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#' 
#' @family allometric functions 
#' 
#' @details All models follow (\ifelse{html}{\out{SA = a M<sup>b</sup>}}{\eqn{SA = a M^b}{ASCII}}) with mass in grams and surface area in \ifelse{html}{\out{meters<sup>2</sup>}}{\eqn{meters^2}{ASCII}}.
#' \cr
#'  \itemize{
#'   \item Lizards \insertCite{Norris1965,Porter1979,Roughgarden1981,OConnor1999,Fei2012}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 0.000314 \pi} 
#'     \cr \eqn{b = 2/3}
#'   \item Salamanders \insertCite{Whitford1967,Riddell2017}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 0.000842} 
#'     \cr \eqn{b = 0.694}
#'   \item Frogs \insertCite{McClanahan1969}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 0.00099} 
#'     \cr \eqn{b = 0.56}
#'   \item Insects \insertCite{Lactin1997}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 0.0013} 
#'     \cr \eqn{b = 0.8}
#'  }
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   surface_area_from_mass(m     = 1:50, 
#'                          taxon = "lizard")
#'   surface_area_from_mass(m     = 1:50,  
#'                          taxon = "salamander")
#'   surface_area_from_mass(m     = 1:50,  
#'                          taxon = "frog")
#'   surface_area_from_mass(m     = seq(0.1, 5, 0.1),  
#'                          taxon = "insect")
#'
#' @export
#'
surface_area_from_mass <- function(m, taxon) {
  
  # Coefficient lookup tables
  a_vals <- c(
    lizard     = 0.000314 * pi,
    salamander = 0.000842,
    frog       = 0.00099,
    insect     = 0.0013
  )
  b_vals <- c(
    lizard     = 2/3,
    salamander = 0.694,
    frog       = 0.56,
    insect     = 0.8
  )
  
  # Input validation (vectorized-safe)
  unknown <- setdiff(unique(taxon), names(a_vals))
  if (length(unknown) > 0)
    stop("Unknown taxon: ", paste(unknown, collapse = ", "),
         ". Must be one of: ", paste(names(a_vals), collapse = ", "))
  if (!all(m > 0, na.rm = TRUE))
    stop("All values of 'm' must be greater than 0.")
  
  # Compute surface area
  unname(a_vals[taxon] * m ^ b_vals[taxon])
}

#' @title Organism Mass from Length 
#' 
#' @description The function estimates mass (g) from length (m) for a variety of taxa.
#'
#' @param l \code{numeric} vector of length (m). Can be 1 or more values.
#'  \cr
#'  Snout-vent length is used for amphibians and reptiles, except turtles where length is carapace length.
#'
#' @param taxon \code{character} taxon of organism, current choices: \code{"insect"}, \code{"lizard"}, \code{"salamander"}, \code{"frog"}, \code{"snake"}, \code{"turtle"}. 
#'
#' @return \code{numeric} mass (g).
#'
#' @family allometric functions
#' 
#' @details All models follow (\ifelse{html}{\out{m = a l<sup>b</sup>}}{\eqn{m = a * l^b}{ASCII}}) with mass in grams and length in meters.
#'  \itemize{
#'   \item Lizards: \insertCite{Meiri2010;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 16368.17} 
#'     \cr \eqn{b = 3.022}
#'   \item Salamanders: \insertCite{Pough1980;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 13654.4} 
#'     \cr \eqn{b = 2.94}
#'   \item Frogs: \insertCite{Pough1980;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 181197.1} 
#'     \cr \eqn{b = 3.24}
#'   \item Snakes: \insertCite{Pough1980;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 723.6756} 
#'     \cr \eqn{b = 3.02}
#'   \item Turtles: \insertCite{Pough1980;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 93554.48} 
#'     \cr \eqn{b = 2.69}
#'   \item Insects: \insertCite{Sample1993;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{a = 806.0827} 
#'     \cr \eqn{b = 2.494}
#'  }
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   mass_from_length(l     = 0.04,
#'                    taxon = "insect")
#'   mass_from_length(l     = 0.04,
#'                    taxon = "lizard")
#'   mass_from_length(l     = 0.04,
#'                    taxon = "salamander")
#'   mass_from_length(l     = 0.04,
#'                    taxon = "frog")
#'   mass_from_length(l     = 0.04, 
#'                    taxon = "snake")
#'   mass_from_length(l     = 0.04, 
#'                    taxon = "turtle")
#'
#' @export
#'
mass_from_length <- function(l, taxon) {
  
  # Coefficient lookup tables
  a_vals <- c(
    insect     = 806.0827,
    lizard     = 16368.17,
    salamander = 13654.4,
    frog       = 181197.1,
    snake      = 723.6756,
    turtle     = 93554.48
  )
  b_vals <- c(
    insect     = 2.494,
    lizard     = 3.022,
    salamander = 2.94,
    frog       = 3.24,
    snake      = 3.02,
    turtle     = 2.69
  )
  
  # Input validation (vectorized-safe)
  unknown <- setdiff(unique(taxon), names(a_vals))
  if (length(unknown) > 0)
    stop("Unknown taxon: ", paste(unknown, collapse = ", "),
         ". Must be one of: ", paste(names(a_vals), collapse = ", "))
  if (!all(l > 0, na.rm = TRUE))
    stop("All values of 'l' must be greater than 0.")
  
  # Compute mass
  unname(a_vals[taxon] * l ^ b_vals[taxon])
}


#' @title Organism Surface Area from Volume 
#' 
#' @description The function estimates surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) from volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}) for a variety of taxa following \insertCite{Mitchell1976;textual}{TrenchR}.
#' 
#' @param V \code{numeric} vector of volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}). Can be one or more values.
#'
#' @param taxon \code{character} taxon of organism, current choices: \code{"lizard"}, \code{"frog"}, \code{"sphere"}.
#' 
#' @return \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @family allometric functions
#' 
#' @details All models follow (\ifelse{html}{\out{SA = Ka V<sup>2/3</sup>}}{\eqn{SA = Ka * V^{2/3}}{ASCII}}) with surface area and volume in meters.
#'  \itemize{
#'   \item Lizards: \insertCite{Norris1965;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{Ka = 11.0} 
#'   \item Frogs: \insertCite{Tracy1972;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{Ka = 11.0} 
#'   \item Sphere: \insertCite{Mitchell1976;textual}{TrenchR}:
#'     \cr 
#'     \cr \eqn{Ka = 4.83} 
#'  }   
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   surface_area_from_volume(V     = 0.001, 
#'                            taxon = "lizard")
#'   surface_area_from_volume(V     = 0.001,  
#'                            taxon = "frog")
#'   surface_area_from_volume(V     = 0.001,  
#'                            taxon = "sphere")
#'
#' @export
#'
surface_area_from_volume <- function(V, taxon) {
  
  # Coefficient lookup table
  Ka_vals <- c(
    lizard = 11,
    frog   = 11,
    sphere = 4.83
  )
  
  # Input validation (vectorized-safe)
  unknown <- setdiff(unique(taxon), names(Ka_vals))
  if (length(unknown) > 0)
    stop("Unknown taxon: ", paste(unknown, collapse = ", "),
         ". Must be one of: ", paste(names(Ka_vals), collapse = ", "))
  if (!all(V > 0, na.rm = TRUE))
    stop("All values of 'V' must be greater than 0.")
  
  # Compute surface area
  unname(Ka_vals[taxon] * V^(2/3))
}

#' @title Organism Volume from Length
#' 
#' @description The function estimates volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}) from length (m) for a variety of taxa following \insertCite{Mitchell1976;textual}{TrenchR}.
#' 
#' @param l \code{numeric} length (m). 
#'  \cr
#'  Use snout-vent length for lizards and frogs.
#'
#' @param taxon \code{character} taxon of organism, current choices: \code{"lizard"}, \code{"frog"}, \code{"sphere"}.
#' 
#' @return \code{numeric} volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}).
#' 
#' @family allometric functions
#' 
#' @details Relationships come from \itemize{
#'   \item Lizards: \insertCite{Norris1965;textual}{TrenchR}
#'   \item Frogs: \insertCite{Tracy1972;textual}{TrenchR}
#'   \item Sphere: \insertCite{Mitchell1976;textual}{TrenchR}
#'  }
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   volume_from_length(l     = 0.05,  
#'                      taxon = "lizard")
#'   volume_from_length(l     = 0.05,   
#'                      taxon = "frog")
#'   volume_from_length(l     = 0.05,   
#'                      taxon = "sphere")
#'
#' @export
#'
volume_from_length <- function(l, taxon) {
  
  # Coefficient lookup table
  Kl_vals <- c(
    lizard = 3.3,
    frog   = 2.27,
    sphere = 1.24
  )
  
  # Input validation (vectorized-safe)
  unknown <- setdiff(unique(taxon), names(Kl_vals))
  if (length(unknown) > 0)
    stop("Unknown taxon: ", paste(unknown, collapse = ", "),
         ". Must be one of: ", paste(names(Kl_vals), collapse = ", "))
  if (!all(l > 0, na.rm = TRUE))
    stop("All values of 'l' must be greater than 0.")
  
  # Compute volume
  unname((l / Kl_vals[taxon])^3)
}

#' @title Organism Surface Area from Length
#' 
#' @description This function estimates surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) from length (m) by approximating the animal's body as a rotational ellipsoid with half the body length as the semi-major axis. 
#' 
#' @param l \code{numeric} length (m).
#'
#' @return \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @family allometric functions
#'
#' @details Following \insertCite{Samietz2005;textual}{TrenchR} and \insertCite{Lactin1998;textual}{TrenchR}.
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   surface_area_from_length(l = 0.04)
#'
surface_area_from_length <- function (l) {
  
  stopifnot(l > 0)
  
  # convert m to mm

  l_mm = l * 1000
  
  # inital units: mm
  #  c- semi-major axis (half of grasshopper length), 
  #  a- semi-minor axis (half of grasshopper width)

  c <- l_mm/2
  a <- (0.365 + 0.241 * l_mm * 1000) / 1000  
  e <- sqrt(1 - a^2 / c^2)
  sa <- 2 * pi * a^2 + 2 * pi * a * c / (e * asin(e))
  
  # to m^2

  sa / (1000^2)

}

#' @title Organism Silhouette Area
#' 
#' @description The function estimates the projected (silhouette) area as a portion of the surface area of the organism as a function of zenith angle. The function is useful for estimating absorbed solar radiation.
#' 
#' @param psi \code{numeric} zenith angle in degrees between \code{0} and \code{360}.
#' 
#' @param taxon \code{character} organism name. Current choices are \code{"lizard"}, \code{"frog"}, and \code{"grasshopper"}.
#' 
#' @param raz \code{numeric} relative solar azimuth angle (in degrees). Required if \code{taxon = "lizard"}. This is the horizontal angle of the sun relative to the head and frontal plane of the lizard and options currently include \code{0} (in front), \code{90} (to side), and \code{180} (behind) degrees.
#' 
#' @param posture \code{character} value describing posture. Required if \code{taxon = "lizard"}. Options include \code{"prostrate"} (default) and \code{"elevated"}.
#'
#' @return \code{numeric} silhouette area as a proportion.
#' 
#' @family allometric functions
#' 
#' @details Relationships come from \itemize{
#'   \item Lizards: \insertCite{Muth1977;textual}{TrenchR}
#'   \item Frogs: \insertCite{Tracy1976;textual}{TrenchR}
#'   \item Grasshoppers: \insertCite{Anderson1979;textual}{TrenchR}
#'  }
#'
#' @references
#'   \insertAllCited{}
#'
#' @export
#' 
#' @examples
#'   proportion_silhouette_area(psi     = 60,   
#'                              taxon = "frog")
#'   proportion_silhouette_area(psi     = 60, 
#'                              taxon = "grasshopper")
#'   proportion_silhouette_area(psi       = 60, 
#'                              taxon   = "lizard", 
#'                              posture = "prostrate", 
#'                              raz     = 90)
#'   proportion_silhouette_area(psi       = 60, 
#'                              taxon   = "lizard", 
#'                              posture = "elevated", 
#'                              raz     = 180)
#' 
proportion_silhouette_area <- function (psi,
                                        taxon, 
                                        raz     = 0, 
                                        posture = "prostrate") {
  
  stopifnot(length(taxon) == 1, 
            taxon %in% c("frog", "lizard", "grasshopper"), 
            psi >= 0, 
            psi <  360)
  
  if (taxon == "frog"){

    (1.38171e-6 * psi^4 - 1.93335e-4 * psi^3 + 4.75761e-3 * psi^2 - 0.167912 * psi + 45.8228) / 100
  
  } else if (taxon == "grasshopper") {

    0.19 - 0.00173 * psi 

  } else if (taxon == "lizard") {

    stopifnot(raz %in% c(0, 90, 180), posture %in% c("prostrate", "elevated"))

    if (posture == "prostrate") { 

      if (raz == 0) {

        A <- -2.3148*10^(-6) 
        B <- -2.1024*10^(-3)
        C <- -4.6162*10^(-2)
        D <- 30.7316

      } else if (raz == 90) {

        A <- -1.0185*10^(-5) 
        B <- 1.3574*10^(-3)
        C <- -9.5589*10^(-3)
        D <- 30.87255

      } else if (raz == 180) {

        A <- 0
        B <- -2.7105*10^(-3)
        C <- -6.3915*10^(-2)
        D <- 29.8534
  
      }

    } else if (posture == "elevated") {

      if (raz == 0) {

        A <- 3.6979*10^(-5)
        B <- -4.7752*10^(-3)
        C <- -6.4026*10^(-2)
        D <- 26.2831

      } else if (raz == 90) {

        A <- 0
        B <- -1.1756*10^(-4)
        C <- -9.2594*10^(-2)
        D <- 26.2409

      } else if (raz == 180) {

        A <- 0
        B <- -1.5662*10^(-3)
        C <- -5.6423*10^(-2)
        D <- 26.6833

      }

    }

    (A * psi^3 + B * psi^2 + C * psi + D) / 100 
  
  }

}



#' @title Organism Silhouette Area using Shape Approximations
#' 
#' @description The function estimates the projected (silhouette) area as a portion of the surface area of the organism. The function estimates the projected area as a function of the dimensions and the angle between the solar beam and the longitudinal axis of the solid, using Figure 11.6 in \insertCite{Campbell1998;textual}{TrenchR}. The function is useful for estimating absorbed solar radiation. 
#' 
#' @param shape \code{character} Shape to use to approximate an organism. Shapes are assumed to be prolate or have the longest axis parallel with the ground. Current choices are \code{"spheroid"}, \code{"cylinder flat ends"}, and \code{"cylinder hemisphere ends"}.
#' 
#' @param theta \code{numeric} angle between the solar beam and the longitudinal axis (degrees).
#' 
#' @param h \code{numeric} height (long axis in m). Cross section length for spheroid.
#' 
#' @param d \code{numeric} diameter (short axis in m). Cross section length for spheroid.
#'
#' @return \code{numeric} silhouette area as a proportion.
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @family allometric functions
#' 
#' @export
#' 
#' @examples
#'   proportion_silhouette_area_shapes(shape = "spheroid", 
#'                                     theta = 60,  
#'                                     h     = 0.01,  
#'                                     d     = 0.001)
#'   proportion_silhouette_area_shapes(shape = "cylinder flat ends",  
#'                                     theta = 60,  
#'                                     h     = 0.01, 
#'                                     d     = 0.001)
#'   proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends",  
#'                                     theta = 60,  
#'                                     h     = 0.01, 
#'                                     d     = 0.001)
#'
proportion_silhouette_area_shapes <- function(shape, 
                                              theta, 
                                              h, 
                                              d){
  
  stopifnot(length(shape) == 1, 
            shape %in% c("spheroid", "cylinder flat ends", "cylinder hemisphere ends"), 
            theta >= 0, 
            theta <  360, 
            h     >= 0, 
            d     >= 0)
  
  theta_r <- degrees_to_radians(theta)
  

  if (shape == "spheroid") {

    x <- d / h

    sqrt(1 + (x^2 - 1) * cos(theta_r)^2) / (2 * x+ (2 * asin(sqrt(1 - x^2)) / sqrt(1 - x^2)))

  }
  
  else if (shape == "cylinder flat ends") {

    (cos(theta_r) + 4 * h * sin(theta_r) / (pi * d)) / (2 + 4 * h / d)

  }
  
  else if (shape == "cylinder hemisphere ends") {

    (1 + 4 * h * sin(theta_r) / (pi * d)) / (4 + 4 * h / d)

  }
  
}

context("AllometricFunctions")

test_that("surface_area_from_mass function works as expected", {

  expect_equal(surface_area_from_mass(2, "insect"), 0.002263431, tolerance = 1e-4)
  expect_equal(surface_area_from_mass(50, "salamander"), 0.01271737, tolerance = 1e-4)
  expect_equal(surface_area_from_mass(400, "frog"), 0.02836539, tolerance = 1e-4)
  expect_equal(surface_area_from_mass(400, "lizard"), 0.0536, tolerance = 1e-4)

})

test_that("mass_from_length function works as expected", {

  expect_equal(mass_from_length(0.04, "insect"), 0.2629767, tolerance = 1e-4)
  expect_equal(mass_from_length(0.6, "turtle"), 23675.08, tolerance = 1e-4)
  expect_equal(mass_from_length(1.8, "snake"), 4270.384, tolerance = 1e-4)
  expect_equal(mass_from_length(1.8, "lizard"), 96701.59, tolerance = 1e-4)
  expect_equal(mass_from_length(1.8, "salamander"), 76872.99, tolerance = 1e-4)
  expect_equal(mass_from_length(1.8, "frog"), 1216842, tolerance = 1e-4)

})

test_that("surface_area_from_volume function works as expected", {

  expect_equal(surface_area_from_volume(V = 0.001, "lizard"), 0.11, tolerance = 1e-4)
  expect_equal(surface_area_from_volume(V = 0.01, "frog"), 0.5105748, tolerance = 1e-4)
  expect_equal(surface_area_from_volume(V = 1, "sphere"), 4.83, tolerance = 1e-4)

})

test_that("volume_from_length function works as expected", {

  expect_equal(volume_from_length(l = 0.05, "lizard"), 3.478309e-06, tolerance = 1e-4)
  expect_equal(volume_from_length(l = 0.15, "frog"), 0.0002885335, tolerance = 1e-4)
  expect_equal(volume_from_length(l = 1, "sphere"), 0.5244873, tolerance = 1e-4)

})

test_that("surface_area_from_length function works as expected", {

  expect_equal(surface_area_from_length(l = 0.04), 0.00187877, tolerance = 1e-4)

})

test_that("proportion_silhouette_area function works as expected", {

  expect_equal(proportion_silhouette_area(psi = 60, taxon = "frog"), 0.2902208, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon = "grasshopper"), 0.1381, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon = "lizard"), 0.2739208, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon= "lizard", raz = 0, posture = "prostrate"), 0.2739208, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon= "lizard", raz = 90, posture = "prostrate"), 0.3153245, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon= "lizard", raz = 180, posture = "prostrate"), 0.254965, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon= "lizard", raz = 0, posture = "elevated"), 0.2106307, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon= "lizard", raz = 90, posture = "elevated"), 0.2335728, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area(psi = 30, taxon= "lizard", raz = 180, posture = "elevated"), 0.2358103, tolerance = 1e-4)

})

test_that("proportion_silhouette_area_shapes function works as expected", {

  expect_equal(proportion_silhouette_area_shapes(shape = "spheroid", theta = 60, h = 0.01, d = 0.001), 0.2748564, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area_shapes(shape = "cylinder flat ends", theta = 60, h = 0.01, d = 0.001), 0.2744423, tolerance = 1e-4)
  expect_equal(proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", theta = 60, h = 0.01, d = 0.001), 0.2733313, tolerance = 1e-4)

})

#' @title Degree Days
#' 
#' @description The function calculates degree days using the following approximations: single or double sine wave, single or double triangulation \insertCite{ucipm}{TrenchR}. Double approximation methods assume symmetry, such that a day's thermal minimum is equal to that of the previous day. Double sine wave approximation of degree days from \insertCite{Allen1976;textual}{TrenchR}.
#' 
#' @param T_min \code{numeric} Minimum temperature of the day (C).
#' 
#' @param T_max \code{numeric} Maximum temperature of the day (C).
#' 
#' @param LDT \code{numeric} lower developmental threshold (C).
#' 
#' @param UDT \code{numeric} upper developmental threshold (C).
#' 
#' @param method \code{character} type of method being used. Current choices: \code{"single.sine"}, \code{"double.sine"}, \code{"single.triangulation"}, and \code{"double.triangulation"}.
#' 
#' @return \code{numeric} degree days (C).
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   degree_days(T_min  = 7, 
#'               T_max  = 14, 
#'               LDT    = 12, 
#'               UDT    = 33, 
#'               method = "single.sine")
#'   degree_days(T_min  = 7, 
#'               T_max  = 14, 
#'               LDT    = 12, 
#'               UDT    = 33, 
#'               method = "single.triangulation")
#' 
degree_days <- function (T_min, 
                         T_max, 
                         LDT    = NA, 
                         UDT    = NA, 
                         method = "single.sine") {
  
  stopifnot(T_max >= T_min, 
            UDT   >= LDT, 
            length(method) == 1,
            method %in% c("single.sine", "double.sine", "single.triangulation", "double.triangulation"))
  
  # See http://ipm.ucanr.edu/WEATHER/ddconcepts.html
  
  # Amplitude
  alpha <- (T_max - T_min) / 2 
  dd <- 0
  
  if (method == "single.sine") {
    
    if (T_min >= UDT && T_max >= UDT) { # entirely above both thresholds
      
      dd <- (T_max - T_min)
      
    } else  if ( T_min >= LDT  && T_max >= UDT) { #Intercepted by upper threshold 
      
      theta2 <- asin((UDT - (T_max + T_min) / 2) / alpha)
      
      dd <- 1 / pi * (((T_max + T_min) / 2 - LDT) * (theta2 + pi / 2) + (UDT - LDT) * (pi / 2 - theta2) - alpha * cos(theta2))
    
    } else  if (T_min <= LDT &&  T_max >= UDT ) {  #Intercepted by both thresholds
      
      theta2 <- asin((UDT - (T_max + T_min) / 2) / alpha)
      theta1 <- asin((LDT - (T_max + T_min) / 2) / alpha)
      
      dd <- 1 / pi * ((  ((T_max + T_min) / 2) - LDT) * (theta2 - theta1) + alpha * (cos(theta1) - cos(theta2)) + (UDT - LDT) * (pi / 2 - theta2))
      
    } else if (T_min >= LDT &&  T_max <= UDT ) { #Entirely between both thresholds
      
      dd <- ((T_max + T_min) / 2) - LDT
      
     } else if (T_min <= LDT && T_max >= LDT) {  # intercepted by LDT  
       
      #theta1=asin((LDT-(T_max+T_min)/2)/alpha)
      # credit - http://stackoverflow.com/questions/20998460/unexpected-behavior-in-asin
      #It's a floating point issue. The way floating point numbers work is that all 
      #numbers need to be mapped to the nearest one which can be expressed as a finite 
      #sum of powers of two and this may lead to small inaccuracies in the expected output 
      #and can be dependent upon how the numbers are calculated
       
      theta1 <- asin(pmax(-1, pmin(1, (LDT - (T_max + T_min) / 2) / alpha)))
      
      dd <- 1 / pi * (( ((T_max + T_min) / 2) - LDT) * ( (pi / 2) - theta1) + alpha * cos(theta1))
      
    } else if (T_min < LDT && T_max <= LDT) { # entirely below both thresholds
      
      dd <- 0
      
    }
    
  } else if (method == "double.sine") {
    
    if (T_min >= UDT && T_max >= UDT) { # entirely above both thresholds
      
      dd <- (UDT - LDT) / 2

    } else if (T_min >= LDT  && T_max >= UDT) { #Intercepted by upper threshold

      theta2 <- asin((UDT - (T_max + T_min) / 2) / alpha)
      dd     <- 1 / (2 * pi) * (((T_max + T_min) / 2 - LDT) * (theta2 + pi / 2) + (UDT - LDT) * (pi / 2 - theta2) - alpha * cos(theta2))

    } else if (T_min <= LDT &&  T_max >= UDT) { #Intercepted by both thresholds

      theta2 <- asin((UDT - (T_max + T_min) / 2) / alpha)
      theta1 <- asin((LDT - (T_max + T_min) / 2) / alpha)
      dd     <- 1 / (2 * pi) * (((T_max + T_min) / 2 - LDT) * (theta2 - theta1) + alpha * (cos(theta1) - cos(theta2)) + (UDT - LDT) * (pi / 2 - theta2))

    } else if (T_min >= LDT &&  T_max <= UDT) { #Entirely between both thresholds

      dd <- 0.5 * ((T_max + T_min) / 2 - LDT)

    } else if (T_min <= LDT && T_max >= LDT) { # intercepted by LDT

      theta1 <- asin(pmax(-1, pmin(1, (LDT - (T_max + T_min) / 2) / alpha)))
      dd     <- 1 / (2 * pi) * (((T_max + T_min) / 2 - LDT) * (pi / 2 - theta1) + alpha * cos(theta1))

    } else if (T_min <= LDT && T_max <= LDT) { # entirely below both thresholds

      dd <- 0
    }

    dd <- dd*2

  } else if (method == "single.triangulation") {
    
    MT <- (T_max + T_min) / 2
    
    if (T_min >= UDT && T_max >= UDT) { # entirely above both thresholds
      
      dd <- UDT - LDT
      
    } else  if ( T_min >= LDT  && T_max >= UDT) { # Intercepted by upper threshold 
      
      dd <- (MT - LDT) - ((T_max - UDT)^2 / ((T_max - T_min) * 2))
      
    } else  if (T_min <= LDT &&  T_max >= UDT ) {  # Intercepted by both thresholds
      
      dd <- ((T_max - LDT)^2 - (T_max - UDT)^2) / ((T_max - T_min) * 2)
      
    } else if (T_min >= LDT &&  T_max <= UDT ) { # Entirely between both thresholds
      
      dd <- MT - LDT
      
    } else if (T_min <= LDT && T_max >= LDT) {  # intercepted by LDT  
      
      dd <- (T_max - LDT)^2 / ((T_max - T_min) * 2)
      
    } else if (T_min <= LDT && T_max <= LDT) { # entirely below both thresholds
      
      dd <- 0
      
    }
  } else if (method == "double.triangulation") {
    
    MT <- (T_max + T_min) / 2
    
    if (T_min >= UDT && T_max >= UDT) { # entirely above both thresholds
      
      dd <- (UDT - LDT) / 2
      
    } else if ( T_min >= LDT  && T_max >= UDT) { # Intercepted by upper threshold 
      
      dd <- (MT - LDT) - ((T_max - UDT)^2 / ((T_max - T_min) * 4))
      
    } else if (T_min <= LDT &&  T_max >= UDT ) {  # Intercepted by both thresholds
      
      dd <- ((T_max - LDT)^2 - (T_max - UDT)^2) / ((T_max - T_min) * 4)
      
    } else if (T_min >= LDT &&  T_max <= UDT ) { # Entirely between both thresholds
      
      dd <- (MT / 4) - (LDT / 2)
      
    } else if (T_min <= LDT && T_max >= LDT) {  # intercepted by LDT 
      
      dd <- (T_max - LDT)^2 / ((T_max - T_min) * 4)
      
    } else if (T_min <= LDT && T_max <= LDT) { # entirely below both thresholds
      
      dd <- 0
      
    }
    
   dd <- dd * 2 
    
  }
  
  round(dd, 2)
  
}

context("Single Triangulation")
test_that("degree day function-single-triangulation work", {
#fpath <-   test_path("degree-days-single-triangle.csv")
 # singletriang <- read.csv(fpath)

   
  #singletriang$DD.Trench<-degree_days(singletriang$Air.min,singletriang$Air.max,12,33,"single.triangulation")
  #corrsingletrian=cor(singletriang$Degree.days,singletriang$DD.Trench,method = "pearson")
  #expect_gte(corrsingletrian, .94)
  #rm(singletriang)

expect_equal(degree_days(T_min=7, T_max=14, LDT=12, UDT=33, method="single.triangulation"),.29 )
})

context("Double Triangulation")
test_that("degree day function-double-triangulation work", {
#  fpath <- test_path("degree-days-double-triangle.csv")
#  doubletriang <- read.csv(fpath)

  #doubletriang$DD.Trench<-degree_days(doubletriang$Air.min,doubletriang$Air.max,12,33,"double.triangulation")
  #corrdoubletrian=cor(doubletriang$Degree.days,doubletriang$DD.Trench,method = "pearson")
  #expect_gte(corrdoubletrian, .94)
  #rm(doubletriang)
  expect_equal(degree_days(T_min=7, T_max=14, LDT=12, UDT=33, method="double.triangulation"), .29)
})

context("DDFunctions")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}


test_that("degree_days function works as expected", {

  expect_similar(degree_days(T_min=7, T_max=14, LDT=12, UDT=33, method="single.sine"), 0.47)
  expect_similar(degree_days(T_min=7, T_max=10, LDT=12, UDT=33, method="single.sine"), 0)
  expect_similar(degree_days(T_min=13, T_max=14, LDT=12, UDT=33, method="single.sine"), 1.5)
  expect_similar(degree_days(T_min=35, T_max=40, LDT=12, UDT=33, method="single.sine"), 5)
  expect_similar(degree_days(T_min=13, T_max=40, LDT=12, UDT=33, method="single.sine"), 12.94)
  expect_similar(degree_days(T_min=7, T_max=40, LDT=12, UDT=33, method="single.sine"), 10.94)


  expect_similar(degree_days(T_min=7, T_max=20, LDT=12, UDT=33, method="double.sine"), 2.87)
  expect_similar(degree_days(T_min=7, T_max=10, LDT=12, UDT=33, method="double.sine"), 0)
  expect_similar(degree_days(T_min=13, T_max=14, LDT=12, UDT=33, method="double.sine"), 1.5)
  expect_similar(degree_days(T_min=35, T_max=40, LDT=12, UDT=33, method="double.sine"), 21)
  expect_similar(degree_days(T_min=13, T_max=40, LDT=12, UDT=33, method="double.sine"), 12.94)
  expect_similar(degree_days(T_min=7, T_max=40, LDT=12, UDT=33, method="double.sine"), 10.94)

  expect_similar(degree_days(T_min=7, T_max=14, LDT=12, UDT=33, method="single.triangulation"), 0.29)
  expect_similar(degree_days(T_min=7, T_max=10, LDT=12, UDT=33, method="single.triangulation"), 0)
  expect_similar(degree_days(T_min=13, T_max=14, LDT=12, UDT=33, method="single.triangulation"), 1.5)
  expect_similar(degree_days(T_min=35, T_max=40, LDT=12, UDT=33, method="single.triangulation"), 21)
  expect_similar(degree_days(T_min=13, T_max=40, LDT=12, UDT=33, method="single.triangulation"), 13.59)
  expect_similar(degree_days(T_min=7, T_max=40, LDT=12, UDT=33, method="single.triangulation"), 11.14)

  expect_similar(degree_days(T_min=7, T_max=20, LDT=12, UDT=33, method="double.triangulation"), 2.46)
  expect_similar(degree_days(T_min=7, T_max=10, LDT=12, UDT=33, method="double.triangulation"), 0)
  expect_similar(degree_days(T_min=13, T_max=32, LDT=10, UDT=33, method="double.triangulation"), 1.25)
  expect_similar(degree_days(T_min=35, T_max=40, LDT=12, UDT=33, method="double.triangulation"), 21)
  expect_similar(degree_days(T_min=13, T_max=40, LDT=12, UDT=33, method="double.triangulation"), 28.09)
  expect_similar(degree_days(T_min=7, T_max=40, LDT=12, UDT=33, method="double.triangulation"), 11.14)
})

context("Double Sine")
test_that("degree day function-double-sine work", {
  #fpath <- test_path("degree-days-double-sine.csv")
  #doublesine <- read.csv(fpath)  

# doublesine$DD.Trench<-degree_days(doublesine$Air.min,doublesine$Air.max,12,33,"double.sine")
 #corrdoubsine=cor(doublesine$Degree.days,doublesine$DD.Trench,method = "pearson")
 #expect_gte(corrdoubsine, .94)
 #rm(doublesine)
expect_equal(degree_days(T_min=7, T_max=14, LDT=12, UDT=33, method="double.sine"), .47)
  
})

context("Single Sine")
test_that("degree day function-single-sine work", {
  #fpath <- test_path("degree-days-single-sine.csv")
#  singlesine <- read.csv(fpath)
 # singlesine$DD.Trench<-degree_days(singlesine$Air.min,singlesine$Air.max,12,33,"single.sine")
 # corrsingsine=cor(singlesine$Degree.days,singlesine$DD.Trench,method = "pearson")
 # expect_gte(corrsingsine, .94)
  #rm(singlesine)
  expect_equal(degree_days(T_min=7, T_max=14, LDT=12, UDT=33, method="single.sine"), .47)
})

#' @title General Use Constants
#'
#' @description Basic functions for numerical constants for conversions.
#'
#' @param units \code{character} indicating units \itemize{
#'    \item{specific_heat_h2o}: \itemize{
#'      \item{\code{"J_kg-1_K-1"}}: {\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}}
#'    }
#'    \item{latent_heat_vaporization_h2o}: \itemize{
#'      \item{\code{"J_kg-1"}}: {\ifelse{html}{\out{J kg<sup>-1</sup>}}{\eqn{J kg^-1}{ASCII}}}
#'    }
#'    \item{stefan_boltzmann_constant}: \itemize{
#'      \item{\code{"W_m-2_K-4"}}: {\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-4</sup>}}{\eqn{W m^-2 K^-4}{ASCII}}}
#'      \item{\code{"mW_cm-2_K-4"}}: {\ifelse{html}{\out{mW cm<sup>-2</sup> K<sup>-4</sup>}}{\eqn{mW cm^-2 K^-4}{ASCII}}}
#'    }
#'    \item{con_karman_constant}: \itemize{
#'      \item{\code{""}}: dimensionless
#'    }
#'  }
#'
#' @return \code{numeric} values in \code{units}.
#'
#' @examples
#'   specific_heat_h2o()
#'   latent_heat_vaporization_h2o()
#'   stefan_boltzmann_constant()
#'
#' @name constants
#'
#' @export
#'
specific_heat_h2o <- function (units = "J_kg-1_K-1") {

  stopifnot(units %in% "J_kg-1_K-1")

  4184

}

#' @rdname constants
#'
#' @export
#'
latent_heat_vaporization_h2o <- function (units = "J_kg-1") {

  stopifnot(units %in% "J_kg-1")

  2.48

}

#' @rdname constants
#'
#' @export
#'
stefan_boltzmann_constant <- function (units = "W_m-2_K-4") {

  stopifnot(units %in% c("W_m-2_K-4", "mW_cm-2_K-4"))

  if (units == "W_m-2_K-4") {
  
    5.673e-8

  } else if (units == "mW_cm-2_K-4"){

    5.673e-9
    
  }

}

#' @rdname constants
#'
#' @export
#'
von_karman_constant <- function (units = "") {

  stopifnot(units %in% c(""))

  0.41

}

context("Constants")
test_that("constants", {
  expect_equal(specific_heat_h2o(),  4184)
  expect_equal(latent_heat_vaporization_h2o(),  2.48)
  expect_equal(stefan_boltzmann_constant(),   5.67e-8)
  expect_equal(stefan_boltzmann_constant("mW_cm-2_K-4"), 5.67e-9)
})

#' @title Operative Temperature of a Lizard Using Fei et al. (2012)
#' 
#' @description The function predicts body temperature (C, operative environmental temperature) of a lizard based on \insertCite{Fei2012;textual}{TrenchR}.
#'
#' @param T_a \code{numeric} air temperature at lizard height (C).
#' 
#' @param T_g \code{numeric} surface temperature (C).
#' 
#' @param S \code{numeric} total (direct + diffuse) solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m-^2}{ASCII}}).
#' 
#' @param lw \code{numeric} downward flux of near-infrared radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m-^2}{ASCII}}).
#' 
#' @param shade \code{numeric} proportion of shade.
#' 
#' @param m \code{numeric} lizard mass (g).
#' 
#' @param Acondfact \code{numeric} proportion of the lizard projected area that is in contact with the ground. \code{Acondfact = 0.1} for standing and \code{Acondfact = 0.4} for lying on ground.
#' 
#' @param Agradfact \code{numeric} proportion of the lizard projected area exposed to radiation from the ground. \code{Agradfact = 0.3} for standing and \code{Agradfact = 0.0} for lying on ground.
#' 
#' @return \code{numeric} predicted body (operative environmental) temperature (K).
#' 
#' @family biophysical models
#' 
#' @author Ofir Levy
#'
#' @details Thermal radiative flux is calculated following \insertCite{Fei2012;textual}{TrenchR} based on \insertCite{Bartlett1967;textual}{TrenchR} and \insertCite{Porter1973;textual}{TrenchR}.
#'
#' 
#' @export
#'
#' @references
#'  \insertAllCited{}
#'  
#' @examples
#'   Tb_lizard_Fei(T_a       = 20,
#'                 T_g       = 27,
#'                 S         = 1300, 
#'                 lw        = 60, 
#'                 shade     = 0.5, 
#'                 m         = 10.5, 
#'                 Acondfact = 0.1, 
#'                 Agradfact = 0.3)
#' 
#' 
Tb_lizard_Fei <- function(T_a, 
                          T_g, 
                          S, 
                          lw, 
                          shade, 
                          m, 
                          Acondfact, 
                          Agradfact) {
  
  stopifnot(T_a       >  -50, 
            T_a       <  100, 
            T_g       >  -50, 
            T_g       <  100,
            S         >= 0, 
            lw        >= 0, 
            shade     >= 0, 
            shade     <= 1, 
            m         >= 0, 
            Acondfact >= 0, 
            Acondfact <= 1, 
            Agradfact >= 0, 
            Agradfact <= 1)
  
  # Constants

    # thermal absorptivity (proportion), Bartlett & Gates 1967

      alpha_L <- 0.965 

    # convective heat transfer ceofficient (W m-2 C-1) (Fei et al. 2012, Porter et al. 1973)

      h_L <- 10.45 

    # emissivity of lizard's skin (proportion)

      epsilon_lizard <- 0.95 


    # thermal conductivity (W C-1 m-1)

      K_lizard <- 0.5 

    # lizard mean thickness in meters (diameter)

      lambda <- 0.015

    # specific heat capacity (J kg-1 C-1)

      c_lizard <- 3762

    # time interval in seconds

      dt <- 120


    sigma <- stefan_boltzmann_constant() 
  
  # Convert mass to kg

    mass_kg <- m / 1000
  
  # Estimate areas

    # Surface area m2

      A_L <- 0.0314 * pi * mass_kg^(2 / 3) 

    # Estimate projected lizard area for radiation from the ground

      A_down <- Agradfact * A_L 

    # Estimate projected lizard area that contacts the ground
  
      A_contact <- Acondfact * A_L 

    # convert to Kelvin
      T_a= celsius_to_kelvin(T_a)
      T_g= celsius_to_kelvin(T_g)
      
  # Initial operative environmental temperature

    # Initial body temperature in kelvin, assume Ta 

      T_o <- T_a  
  
  # solar radiation

    # projected lizard area for direct and scattered solar radiation

      A_p <- 0.4 * A_L 
  
  # Check

    dQ_solar <- (1 - shade) * alpha_L * A_p * S
  

  # Net longwave radiation

    # Proportion of surface area facing toward the sky

      A_up <- 0.6 * A_L 

    # (10.11) clear sky emissivity

      epsilon_ac <- 9.2 * 10^-6 * (T_a)^2 
  
  # Iterate to calculate steady state

    for (i in c(1:50)) {
    
      # Thermal radiation

        dQ_IR <- epsilon_lizard * A_down * sigma * (T_g^4 - T_o^4) + epsilon_lizard * A_up * sigma * (T_a^4 - T_o^4)

        # alternative version
        #
        #   dQ_IR <- epsilon_lizard * A_down * sigma * (T_g^4 - T_o^4) + epsilon_lizard * A_up * ((1 - shade) * lw + shade * sigma * T_a^4) - epsilon_lizard * A_up * sigma * T_o^4
      

      # Conduction

        dQ_cond <- A_contact * K_lizard * (T_g - T_o) / (lambda / 2)
  
      # Convection, assuming no wind

        # skin area that is exposed to air

          Aair <- 0.9 * A_L
          dQ_conv <- h_L * Aair * (T_a - T_o)
  
      # Metabolism

        TinC <- kelvin_to_celsius(T_o)
        dQ_meta <- 0.348 * exp(0.022 * TinC - 0.132) * mass_kg

        # alternative version
        #
        #   ew <- exp(-10.0+0.51*log(m)+0.115*(T_o-273)) *3 #Buckley 2008
        #   dQ_meta <- ew/3600. #metabolic rate (j/s)

    
      # Respiratory loss

        if (TinC < 20) {

          dQ_resp <- 0.272 * mass_kg

        } else if (TinC > 36) {

          dQ_resp <- 0.003 * mass_kg * exp(0.1516 * TinC)

        } else {

          dQ_resp <- 0.0836 * mass_kg * exp(0.0586 * TinC)

        }
    
      dQe <- (dQ_solar + dQ_IR + dQ_meta + dQ_cond + dQ_conv - dQ_resp)
   
      dTe <- dQe / ((mass_kg) * c_lizard)
      T_o <- T_o + dTe * dt
    
    }
  
    kelvin_to_celsius(T_o)

}

context("calculate_Tb_Fei")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("Tb_lizard_Fei function works as expected", {
  expect_similar(Tb_lizard_Fei(T_a=20, T_g=27, S=600, lw=30, shade=0.5, m=10.5, Acondfact=0.05, Agradfact=0.4), 28.42987)
  expect_similar(Tb_lizard_Fei(T_a=20, T_g=27, S=600, lw=30, shade=0.5, m=10.5, Acondfact=0.35, Agradfact=0.0), 27.73462)
  expect_similar(Tb_lizard_Fei(T_a=20, T_g=27, S=300, lw=30, shade=0.5, m=10.5, Acondfact=0.35, Agradfact=0.0), 26.13779)
})

#' @title Operative Environmental Temperature of a Butterfly
#'
#' @description The function estimates body temperatures (C, operative environmental temperatures) of a butterfly based on \insertCite{Kingsolver1983;textual}{TrenchR} and \insertCite{Buckley2012;textual}{TrenchR}. The function is designed for butterflies that bask with closed wings such as Colias.
#' 
#' @param T_a \code{numeric} air temperature (C).
#'
#' @param T_g  \code{numeric} surface temperature (C) in the sunlight.
#'
#' @param T_sh \code{numeric} surface temperature (C) in the shade.
#'
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param S_sdir \code{numeric} direct solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#'
#' @param S_sdif \code{numeric} diffuse solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#'
#' @param z \code{numeric} solar zenith angle (degrees).
#'
#' @param D \code{numeric} thoracic diameter (cm).
#'
#' @param delta \code{numeric} thoracic fur thickness (mm).
#'
#' @param alpha \code{numeric} wing solar absorptivity (proportion). The range for Colias butterflies is 0.4 to 0.7.
#'
#' @param r_g \code{numeric} substrate solar reflectivity (proportion). See \insertCite{Kingsolver1983;textual}{TrenchR}.
#'
#' @param shade \code{logical} whether body temperature should be calculated in sun (\code{FALSE}) or shade (\code{TRUE}).
#'
#' @return \code{numeric} predicted body (operative environmental) temperature (C).
#'
#' @details 
#'  Thermal radiative flux is calculated following \insertCite{Gates1980;textual}{TrenchR} based on \insertCite{Swinbank1960;textual}{TrenchR}. \insertCite{Kingsolver1983;textual}{TrenchR} estimates using the Brunt equation with black body sky temperature from \insertCite{Swinbank1963;textual}{TrenchR}.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Tb_butterfly(T_a    = 25, 
#'                T_g    = 25, 
#'                T_sh   = 20, 
#'                u      = 0.4, 
#'                S_sdir = 300, 
#'                S_sdif = 100, 
#'                z      = 30, 
#'                D      = 0.36, 
#'                delta  = 1.46, 
#'                alpha  = 0.6, 
#'                r_g    = 0.3)
#'
Tb_butterfly <- function (T_a, 
                          T_g, 
                          T_sh, 
                          u, 
                          S_sdir, 
                          S_sdif, 
                          z, 
                          D, 
                          delta, 
                          alpha, 
                          r_g    = 0.3,
                          shade  = FALSE) {

  stopifnot(u      >= 0, 
            S_sdir >= 0, 
            S_sdif >= 0, 
            z      >= -90, 
            z      <= 90, 
            D      > 0, 
            delta  >= 0, 
            alpha  >= 0, 
            r_g    >= 0, 
            r_g    <= 1, 
            is.logical(shade))  
  
  # conversions

    T_a    <- celsius_to_kelvin(T_a)
    T_a_sh <- T_a
    T_g    <- celsius_to_kelvin(T_g)
   
    # wind speed m/s to cm/s

      u <- u *100  

    # solar flux W/m2 to mW/cm2

      S_sdir <- S_sdir / 10 
      S_sdif <- S_sdif / 10

    # thoracic fur thickness mm to cm

      delta <- delta / 10     

  # Total solar radiation

    S_sttl <- S_sdir + S_sdif


  # Butterfly Parameters

    # surface emissivity (proportion), ranges from 0.95-1

      epsilon_s <- 0.97 

      sigma <- stefan_boltzmann_constant(units = "mW_cm-2_K-4") # note units

    # butterfly thermal emissivity

      Ep <- 1 

    # k_e- thermal conductivity of the fur, 1.3 mW cm^-1*K^-1

      k_e <- 1.3 

    # r_i- body radius from Kingsolver 1983

      r_i <- 0.15 

    # approximate thermal conductivity of air, mWcm^-1*K^-1

      k_a <- 0.25

    #  kinematic viscosity of air, cm^2/s at 300K 
    #     http://www.engineeringtoolbox.com/air-absolute-kinematic-viscosity-d_601.html

      v <- 15.68 * 10^-2  


  # Calculations

    # total surface area cm^2 as area of cylinder without ends

      A_sttl <- pi * D * 2 

    # direct and reflected surface areas for butterflies basking with wings perpendicular to radiation 

      A_sdir <- A_sttl / 2
      A_sref <- A_sdir

    # Radiative Heat Flux, mW
  
      Q_s <- alpha * A_sdir * S_sdir / cos(z * pi / 180) + alpha * A_sref * S_sdif + alpha * r_g * A_sref * S_sttl  

    # Thermal Radiative Flux in K

 
    # effective radiant temperature of sky
    #  K, Gates 1980 Biophysical ecology based on Swinback 1960, Kingsolver (1983) estimates using Brunt equation

       T_sky <- celsius_to_kelvin(1.22 * kelvin_to_celsius(T_a) - 20.4)



    # Convective Heat Flux

      # Reynolds number- ratio of interval viscous forces

      R_e <- u * D / v

      # Nusselt number- dimensionless conductance

        N_u <- 0.6 * R_e^0.5

      # Kingsolver 1983

        # N_u <- 2.3 


      h_c <- N_u * k_a / D


      # total convective heat transfer coefficient

        h_T <- (1 / h_c + (r_i + delta) * log((r_i + delta) / r_i) / k_e)^-1;  

 
  # Shade Adjustments

    if (shade) {

      # Calculate without basking by dividing areas by two
      # Radiative Heat Flux in Shade, mW

        A_sdir <- A_sttl / 4    # only dorsal quarter absorbs diffuse solar
        A_sref <- A_sdir

        # No direct radiation, only diffuse and reflected
  
          S_sdir_sh <- 0
          S_sdif_sh <- S_sdif
          S_sttl <- S_sdir_sh + S_sdif_sh 

        Q_s <- alpha * A_sdir * S_sdir_sh / cos(z * pi / 180) + alpha * A_sref * S_sdif_sh + alpha * r_g * A_sref * S_sttl 

      # Use shaded surface temperature
      T_g <- celsius_to_kelvin(T_sh)  # shade ground temp in Kelvin

    }
               			
    # Energy balance coefficients
    a <- A_sttl * Ep * sigma
    b <- h_T * A_sttl
    d <- h_T * A_sttl * T_a +0.5 * A_sttl * Ep * sigma * T_sky^4 + 0.5 * A_sttl * Ep * sigma * (T_g)^4 + Q_s

      # Define the energy balance residual as a function of T_b (in Kelvin)
      # f(T_b) = 0 at steady-state body temperature
      energy_balance <- function(T_b) a * T_b^4 + b * T_b - d
      
      # Solve numerically using Brent's method
      # Bounds: 200K (-73°C) to 500K (227°C) — physically safe for any ectotherm
      T_e <- tryCatch(
        uniroot(energy_balance, interval = c(200, 500), tol = 1e-6)$root,
        error = function(e)
          stop("Could not solve energy balance for T_b. ",
               "Check that inputs produce a physically plausible temperature. ",
               "Original error: ", conditionMessage(e))
      )
      
    # in C

      kelvin_to_celsius(T_e)

} 

context("biophysmodel_Colias")


test_that("Tb_butterfly function works as expected", {

  expect_equal(Tb_butterfly(T_a = 25, T_g = 25, T_sh = 20, u = 0.4, S_sdir = 300, S_sdif = 100, z = 30, D = 0.36, delta = 1.46, alpha = 0.6, r_g = 0.3), 29.16614, tol = 1e-4)
  expect_equal(Tb_butterfly(T_a = 25, T_g = 25, T_sh = 20, u = 0.4, S_sdir = 300, S_sdif = 100, z = 30, D = 0.36, delta = 1.46, alpha = 0.6, r_g = 0.3, shade = TRUE), 23.80226, tol = 1e-4)

})

#' @title Surface Roughness from Empirical Measurements
#' 
#' @description The function estimates surface roughness (m) from empirical wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) data collected at a vector of heights (m) \insertCite{Kingsolver2015,Campbell1998,Porter1979}{TrenchR}.
#' 
#' @param u_r \code{numeric} wind velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at a vector of reference heights.
#' 
#' @param zr \code{numeric} vector of reference heights (m).
#' 
#' @return \code{numeric} surface roughness (m).
#' 
#' @family microclimate functions
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' 
#' @examples
#'   surface_roughness(u_r = c(0.01, 0.025, 0.05, 0.1, 0.2), 
#'                     zr  = c(0.05, 0.25, 0.5, 0.75, 1))
#' 
surface_roughness <- function (u_r, 
                               zr) {
 
 stopifnot(zr  >  0,
           u_r >= 0)
  
 mod1 <- lm(log(zr) ~ u_r)
 d <- exp(as.numeric(mod1$coefficients[1])) # Zero Plane displacement:height at which the wind speed is zero
 # can also assume d=0.63h (Monteith 1975)
 inds <- which(zr - d > 0)   # indices of measurements where zr-d>0
 
 z0 <- NA
 if(length(inds) > 0){
   
   mod1 <- lm(u_r[inds] ~ log(zr[inds] - d))
   b <- as.numeric(mod1$coefficients[1])
   n <- as.numeric(mod1$coefficients[2])
   z0 <- exp(-b / n)
 
 }
 
 z0
 
}


#' @title Wind Speed at a Specific Height Under Neutral Conditions
#' 
#' @description The function calculates wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at a specified height (m) within a boundary layer near the surface.  The profile assumes neutral conditions. The velocity profile is the neutral profile described by \insertCite{Sellers1965;textual}{TrenchR}. Function is equations (2) and (3) of \insertCite{Porter1973;textual}{TrenchR}. 
#' 
#' @param u_r \code{numeric} wind velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at reference height.
#' 
#' @param zr \code{numeric} initial reference height (m).
#' 
#' @param z0 \code{numeric} surface roughness (m).
#' 
#' @param z \code{numeric} height to scale (m).
#' 
#' @return \code{numeric} windspeed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @family microclimate functions
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' 
#' @examples
#'   wind_speed_profile_neutral(u_r = 0.1, 
#'                              zr  = 0.1, 
#'                              z0  = 0.2, 
#'                              z   = 0.15)
#'
 
wind_speed_profile_neutral <- function (u_r, 
                                        zr, 
                                        z0, 
                                        z) {

  stopifnot(u_r >= 0, 
            zr  >= 0, 
            z0  >= 0, 
            z   >= 0)
  
  u_r * log(z / z0 + 1) / log(zr / z0 + 1)

}

#' @title Air Temperature at a Specified Height Under Neutral Conditions
#' 
#' @description The function calculates air temperature (C) at a specified height (m) within a boundary layer near the surface.  The velocity profile is the neutral profile described by \insertCite{Sellers1965;textual}{TrenchR}. Function is included as equations (2) and (3) of \insertCite{Porter1973;textual}{TrenchR}.
#' 
#' @param T_r \code{numeric} air temperature (C) at reference height.
#' 
#' @param zr \code{numeric} initial reference height (m).
#' 
#' @param z0 \code{numeric} surface roughness (m).
#' 
#' @param z \code{numeric} height to scale to (m).
#' 
#' @param T_s \code{numeric} surface temperatures (C).
#' 
#' @return \code{numeric} air temperature (C).
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @examples
#'   air_temp_profile_neutral(T_r = 20, 
#'                            zr  = 0.1, 
#'                            z0  = 0.2, 
#'                            z   = 0.15, 
#'                            T_s = 25)
#'
#'
air_temp_profile_neutral <- function (T_r, 
                                      zr, 
                                      z0, 
                                      z, 
                                      T_s) {

  stopifnot(zr >= 0, 
            z0 >= 0, 
            z  >= 0)
  
  (T_r - T_s) * log(z / z0 + 1) / log(zr / z0 + 1) + T_s 

}

#' @title Air Temperature Profile using MICRO Routine
#' 
#' @description  The function estimates air temperature (C) at a specified height (m). Estimates a single, unsegmented temperature profile using the MICRO routine from NicheMapR \insertCite{Kearney2017}{TrenchR}.
#' 
#' @param T_r \code{numeric} air temperature (C) at reference height.
#' 
#' @param u_r \code{numeric} windspeed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at reference height.
#' 
#' @param zr \code{numeric} initial reference height (m).
#' 
#' @param z0 \code{numeric} surface roughness (m).
#' 
#' @param z \code{numeric} height to scale (m).
#' 
#' @param T_s \code{numeric} surface temperatures (C).
#' 
#' @return \code{numeric} air temperature (C).
#' 
#' @family microclimate functions
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' 
#' @examples
#'   air_temp_profile(T_r = 20,
#'                    u_r = 0.1, 
#'                    zr  = 0.1, 
#'                    z0  = 0.2, 
#'                    z   = 0.15, 
#'                    T_s = 25)
#'
air_temp_profile <- function (T_r, 
                              u_r, 
                              zr, 
                              z0,
                              z,
                              T_s) {
  
  stopifnot(u_r >= 0, 
            zr  >= 0, 
            z0  >= 0, 
            z   >= 0)
  
  k <- von_karman_constant()

  # friction velocity

    u_star <- k * u_r / log(zr / z0 + 1)  

  # sublayer Stanton number

    S_ts <- 0.62 / (z0 * u_star / 12)^0.45

  # bulk Stanton number

    S_tb <- 0.64 / log(zr / z0 + 1)

  # Temperature at roughness height, z0

    T_z0 <- (T_r * S_tb + T_s * S_ts) / (S_tb + S_ts)

  # Temperature at local height
  # Initial from Ecography paper but fixed in vignette: T_z= T_z0 + (T_r - T_z0)*log(z/z0+1)

    T_z0 + (T_r - T_z0) * log(z / z0 + 1) / log(zr / z0 + 1)

}

#' @title Air Temperature at a Specified Height 
#' 
#' @description The function calculates air temperature (C) at a specified height (m). Estimates a three segment velocity and temperature profile based on user-specified, experimentally determined values for 3 roughness heights and reference heights. Multiple heights are appropriate in heterogenous areas with, for example, a meadow, bushes, and rocks. Implements the MICROSEGMT routine from NicheMapR as described in \insertCite{Kearney2017;textual}{TrenchR}.
#' 
#' @param T_r \code{numeric} a vector of air temperatures (C) at the 3 reference heights.
#' 
#' @param u_r \code{numeric} a vector of wind speeds (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at the 3 reference heights.
#' 
#' @param zr \code{numeric} a vector of 3 reference heights (meters).
#' 
#' @param z0 \code{numeric} a vector of 3 experimentally determined roughness heights (meters).
#' 
#' @param z \code{numeric} height for air temperature estimation (meters).
#' 
#' @param T_s \code{numeric} surface temperatures (C).
#' 
#' @return \code{numeric} air temperature (C).
#' 
#' @family microclimate functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   air_temp_profile_segment(T_r = c(25, 22, 20), 
#'                            u_r = c(0.01, 0.025, 0.05), 
#'                            zr  = c(0.05, 0.25, 0.5), 
#'                            z0  = c(0.01, 0.15, 0.2), 
#'                            z   = 0.3, 
#'                            T_s = 27)
#'
air_temp_profile_segment <- function(T_r,
                                     u_r,
                                     zr,
                                     z0,
                                     z,
                                     T_s) {
  
  # Input validation
  if (!all(lengths(list(T_r, u_r, zr, z0)) == 3))
    stop("'T_r', 'u_r', 'zr', and 'z0' must each be length 3 ",
         "(one value per canopy segment).")
  if (!all(u_r >= 0, na.rm = TRUE))
    stop("All values of 'u_r' must be >= 0.")
  if (!all(zr >= 0, na.rm = TRUE))
    stop("All values of 'zr' must be >= 0.")
  if (!all(z0 > 0, na.rm = TRUE))
    stop("All values of 'z0' must be > 0 (roughness length cannot be zero).")
  if (!all(z >= 0, na.rm = TRUE))
    stop("'z' must be >= 0.")
  if (!all(zr > z0, na.rm = TRUE))
    stop("All reference heights 'zr' must be greater than roughness lengths 'z0'.")
  
  # Order segments by decreasing reference height
  zr.ord <- order(zr, decreasing = TRUE)
  zr     <- zr[zr.ord]
  z0     <- z0[zr.ord]
  u_r    <- u_r[zr.ord]
  T_r    <- T_r[zr.ord]
  
  # Friction velocity (von Karman constant = 0.4)
  u_star <- 0.4 * u_r / log(zr / z0 + 1)
  
  # Sublayer Stanton number (surface-to-air, uses lowest segment)
  S_ts <- 0.62 / (z0[3] * u_star[2] / 12)^0.45
  
  # Bulk Stanton number (air-to-air, uses mid segment)
  S_tb <- 0.64 / log(zr[2] / z0[3] + 1)
  
  # Select segment containing height z
  if (z >= zr[1]) {
    us_star <- u_star[1]
    z0s     <- z0[1]
    T_rs    <- T_r[1]
    zrs     <- zr[1]
  } else if (z >= zr[2]) {
    us_star <- u_star[2]
    z0s     <- z0[2]
    T_rs    <- T_r[2]
    zrs     <- zr[2]
  } else if (z >= 0) {
    us_star <- u_star[3]
    z0s     <- z0[3]
    T_rs    <- T_r[3]
    zrs     <- zr[3]
  } else {
    stop("Could not assign height 'z' to any canopy segment. ",
         "Check that 'z' and 'zr' values are consistent.")
  }
  
  # Wind speed at height z (m/s) 
  u_z <- 2.5 * us_star * log(z / z0s + 1)
  
  # Temperature at roughness height z0 (blended surface/air temp)
  T_z0 <- (T_rs * S_tb + T_s * S_ts) / (S_tb + S_ts)
  
  # Temperature at height z (°C)
  T_z0 + (T_rs - T_z0) * log(z / z0s + 1) / log(zrs / z0s + 1)
}


#' @title Wind Speed at a Specified Height 
#' 
#' @description The function calculates wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at a specified height (m). The function estimates a three segment velocity and temperature profile based on user-specified, experimentally determined values for 3 roughness heights and reference heights. Multiple heights are appropriate in heterogenous areas with, for example, a meadow, bushes, and rocks. Implements the MICROSEGMT routine from NicheMapR as described in \insertCite{Kearney2017;textual}{TrenchR}.
#' 
#' @param u_r \code{numeric} a vector of wind speeds (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at the 3 reference heights.
#' 
#' @param zr \code{numeric} a vector of 3 reference heights (m).
#' 
#' @param z0 \code{numeric} a vector of 3 experimentally determined roughness heights (m).
#' 
#' @param z \code{numeric} height to scale (m).
#' 
#' @return \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @family microclimate functions
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' 
#' @examples
#'   wind_speed_profile_segment(u_r = c(0.01, 0.025, 0.05), 
#'                              zr  = c(0.05, 0.25, 0.5), 
#'                              z0  = c(0.01, 0.15, 0.2), 
#'                              z   = 0.3)
#'
wind_speed_profile_segment <- function(u_r,
                                       zr,
                                       z0,
                                       z) {
  
  # Input validation
  if (!all(lengths(list(u_r, zr, z0)) == 3))
    stop("'u_r', 'zr', and 'z0' must each be length 3 ",
         "(one value per canopy segment).")
  if (!all(u_r >= 0, na.rm = TRUE))
    stop("All values of 'u_r' must be >= 0.")
  if (!all(zr >= 0, na.rm = TRUE))
    stop("All values of 'zr' must be >= 0.")
  if (!all(z0 > 0, na.rm = TRUE))
    stop("All values of 'z0' must be > 0 (roughness length cannot be zero).")
  if (!all(z >= 0, na.rm = TRUE))
    stop("'z' must be >= 0.")
  if (!all(zr > z0, na.rm = TRUE))
    stop("All reference heights 'zr' must be greater than roughness lengths 'z0'.")
  
  k <- von_karman_constant()
  
  # Order segments by decreasing reference height (zr[1] > zr[2] > zr[3])
  zr.ord <- order(zr, decreasing = TRUE)
  zr     <- zr[zr.ord]
  z0     <- z0[zr.ord]
  u_r    <- u_r[zr.ord]
  
  # Friction velocity per segment
  u_star <- k * u_r / log(zr / z0 + 1)
  
  # Select segment containing height z (consistent with air_temp_profile_segment)
  if (z >= zr[1]) {
    us_star <- u_star[1]
    z0s     <- z0[1]
  } else if (z >= zr[2]) {
    us_star <- u_star[2]
    z0s     <- z0[2]
  } else if (z >= 0) {
    us_star <- u_star[3]
    z0s     <- z0[3]
  } else {
    stop("Could not assign height 'z' to any canopy segment. ",
         "Check that 'z' and 'zr' values are consistent.")
  }
  
  # Log-linear wind profile: u_z = (u_star / k) * log(z / z0 + 1)
  # FIX: was 0.5 * us_star — should be (1/k) * us_star = 2.5 * us_star
  (1 / k) * us_star * log(z / z0s + 1)
}

#' @title Operative Environmental Temperature of a Marine Snail
#' 
#' @description The function estimates body temperature (C, operative environmental temperature) of a marine snail. The function implements a steady-state model, which assumes unchanging environmental conditions and is based on \insertCite{Iacarella2012}{TrenchR}. Body temperature and desiccation constrain the activity of Littoraria irrorata within the Spartina alterniflora canopy. The function was provided by Brian Helmuth and is a simplified version of the published model. 
#' 
#' @param temp \code{numeric} air temperature (C).
#' 
#' @param l \code{numeric} snail length (m).
#' 
#' @param S \code{numeric} direct solar flux density (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#' 
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @param CC \code{numeric} fraction of the sky covered by cloud (0-1).
#' 
#' @param WL \code{numeric} water loss rate (\ifelse{html}{\out{kg s<sup>-1</sup>}}{\eqn{kg s^-1}{ASCII}}), 5 percent loss of body mass over one hour is a reasonable maximum level \insertCite{Helmuth1999}{TrenchR}. 
#' 
#' @param WSH \code{numeric} wind sensor height (m).
#' 
#' @return \code{numeric} predicted body (operative environmental) temperature (C).
#'
#' @details Thermal radiative flux is calculated following \insertCite{Helmuth1998;textual}{TrenchR}, \insertCite{Helmuth1999;textual}{TrenchR}, and \insertCite{Idso1969;textual}{TrenchR}.
#' 
#' @family biophysical models
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @author Brian Helmuth et al.
#' 
#' @examples
#'   Tb_snail(temp  = 25, 
#'            l     = 0.012, 
#'            S = 800, 
#'            u    = 1, 
#'            CC    = 0.5, 
#'            WL    = 0, 
#'            WSH   = 10)
#' 
Tb_snail <- function (temp, 
                      l, 
                      S, 
                      u, 
                      CC, 
                      WL,
                      WSH) {
  
  stopifnot(l     >  0, 
            S >= 0, 
            u    >= 0, 
            CC    >= 0, 
            CC    <= 1, 
            WL    >= 0, 
            WSH   >= 0)
  
  sigma <- stefan_boltzmann_constant()   

  Ktemp <- celsius_to_kelvin(temp)
  Gtemp <- Ktemp # ground temperature, assume equal to air temperature
  
  # Areas

    PSA <- 3.1415 * (l / 2)^2   # 3.14*wid*Len # Projected SA (Short-wave) snails SA of circle
    SA <- 4 * 3.15 * (l / 2)^2  # Surface Area (Rad/Conv) for snails SA of sphere
  
  # Aradsky is surface area subject to long-wave radiation from sky (m^2)
  # Aradground is surface area subject to radiation from the ground (m^2)
  # Aproj is projected surface area (m^2)
  # Aground is surface area in contact with the ground (m^2)

  A1 <- (SA / 2) / PSA # Aradsky/Aproj
  A2 <- 0.05           # Aground/Aproj
  A3 <- (SA / 2) / PSA # Aradground/Aproj
  
  # Convection
  U      <- u * 0.03 # U * m/s, shear velocity
  c_prho <- 1200      # c_p * rho = 1200 J m^{-3}*K^{-1}
  k      <-von_karman_constant()
  z0 <- 0.0017       # m, roughness height
  C <- (A1 * 2 * U * c_prho * k) / (log(WSH / z0))
  
  # Absorptivities

  if (l >= 0.037){  # Absorptivity from Luke Miller 
    
    Abs <- 0.615
    
  } else {
    
    if (l <= 0.02225) {
      
      Abs <- 0.689
    
    } else {
      
      Abs <- 0.68 
      
    }
    
  } 
  
  # Emissivities

    eskyclear <- 0.72 + 0.005 * temp # Helmuth 1999 from Idso and Jackson 1969
    esky      <- eskyclear + CC * (1 - eskyclear - (8 / Ktemp))  # Helmuth 1999
    Emm       <- 0.97                                            # Emmissivity # long-wave emissivity shell
  
  # Steady-state heat balance model
  # Solve steady state energy balance equation:
  # T_b*mflux*c= Q_rad,sol +- Q_rad,sky +- Q_rad,ground +- Q_conduction +- Qconvection -Qevaporation

    com1 <- Abs * S + 4 * Emm * sigma * esky * A1 *(Ktemp^4)  + (Gtemp^4) * 4 * Emm * sigma * A3  + C * Ktemp + 0.6 * A2 * Gtemp / (l / 2) + 2.48 * WL
    com2 <- 4180 * WL + (Ktemp^3) * 4 * Emm * sigma *(esky^0.75) * A1 + 4 * Emm * sigma * A3 *(Gtemp^3) + C + 0.6 * A2 / (0.5 * l)
    T_b  <- com1 / com2
  
  kelvin_to_celsius(T_b)
  
}  

context("biophysmodel_snail")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("Tb_snail function works as expected", {
  expect_similar(Tb_snail(temp  = 25, 
                 l     = 0.012, 
                 S = 800, 
                 u    = 1, 
                 CC    = 0.5, 
                 WL    = 0, 
                 WSH   = 10), 38.79568)

})


test_that("Tb_snail function works as expected", {
  expect_similar(Tb_snail(temp  = 25, 
                 l     = 0.04, 
                 S = 800, 
                 u    = 1, 
                 CC    = 0.5, 
                 WL    = 0, 
                 WSH   = 10), 38.44257)

})

test_that("Tb_snail function works as expected", {
  expect_similar(Tb_snail(temp  = 25, 
                 l     = 0.03, 
                 S = 800, 
                 u    = 1, 
                 CC    = 0.5, 
                 WL    = 0, 
                 WSH   = 10), 39.88404)

})

#' @title Soil Thermal Conductivity
#' 
#' @description The function estimates soil thermal conductivity (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) using the methods of \insertCite{deVries1963;textual}{TrenchR}.
#' 
#' @param x \code{numeric} vector of volume fractions of soil constituents (e.g., clay, quartz, minerals other than quartz, organic matter, water, air). The volume fractions should sum to 1. Note that \code{x} and \code{lambda} values in the example correspond to these soil constituents.
#' 
#' @param lambda \code{numeric} vector of the thermal conductivities (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) of the soil constituents.
#' 
#' @param g_a \code{numeric} shape factor on soil particles. The soil particles are assumed to be ellipsoids with axes \code{g_a}, \code{g_b}, and \code{g_c}, where \code{g_a + g_b + g_c = 1} and \code{g_a = g_b}. \insertCite{deVries1952;textual}{TrenchR} suggests \code{g_a = g_b = 0.125}.
#' 
#' @return \code{numeric} soil thermal conductivity (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}).
#' 
#' @family soil temperature functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @author Joseph Grigg
#' 
#' @examples
#'   soil_conductivity(x      = c(0.10, 0.40, 0.11, 0.01, 0.2, 0.18), 
#'                     lambda = c(0.10, 0.40, 0.11, 0.01, 0.2, 0.18), 
#'                     g_a     = 0.125)
#'   
soil_conductivity <- function (x, 
                               lambda, 
                               g_a) {
  
  stopifnot(g_a > 0, 
            g_a < 1,
            x   > 0)
  
  # Estimate ellipsoid axis g_c assuming g_a = g_b.

    g_c <- 1 - 2 * g_a 
  
  k <- rep(NA, length(x))
  
  for(i in 1:length(x)){
    
    if(i != 6) {
      
      k[i] <- 1 / 3 * sum(2 / (1 + (lambda[i] / lambda[5] - 1) * g_a), 1 / (1 + (lambda[i] / lambda[1] - 1) * g_c))
      
    }
    
    if(i == 6) {
      
      k[i] <- 1 / 3 * sum(2 / (1 + (lambda[i] / lambda[5] - 1) * 0.2), 1 / (1 + (lambda[i] / lambda[1] - 1) * 0.6))
      
    }
    
  }
  
  sum(k * x * lambda) / sum(k * x)
  
}

#' @title Soil Specific Heat
#' 
#' @description The function estimates soil specific heat (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}) using the methods of \insertCite{deVries1963;textual}{TrenchR}. The function incorporates the volume fraction of organic material, minerals, and water in soil.
#' 
#' @param x_o \code{numeric} volume fraction of organic material (0-1).
#' 
#' @param x_m \code{numeric} volume fraction of minerals (0-1).
#' 
#' @param x_w \code{numeric} volume fraction of water (0-1).
#' 
#' @param rho_so \code{numeric} particle density of soil in (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) (bulk density).
#' 
#' @return \code{numeric} soil specific heat (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}).
#' 
#' 
#' @family soil temperature functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @author Joseph Grigg
#' 
#' @examples
#'   soil_specific_heat(x_o    = 0.01, 
#'                      x_m    = 0.6, 
#'                      x_w    = 0.2, 
#'                      rho_so = 1620)
#'
soil_specific_heat <- function (x_o, 
                                x_m, 
                                x_w, 
                                rho_so) {
  
  stopifnot(x_o    >= 0, 
            x_o    <= 1, 
            x_m    >= 0, 
            x_m    <= 1, 
            x_w    >= 0, 
            x_w    <= 1, 
            rho_so >  0)
  
  # 4.184 converts from cal/K to J/K
  # 1000000 converts from cm^-3 to m^-3
  # /rho_so converts from heat capacity per unit volume to per kg
  (1300 * 1920 * x_o + 2650 * 870 * x_m + 1.00 * 4.18 * x_w) / rho_so 
  
}

#' @title Solve Equation for Soil Temperature
#' 
#' @description This function is called by \code{\link{soil_temperature_equation}} to solve the equation for soil temperature from \insertCite{Beckman1973;textual}{TrenchR}. The function represents the integrand in the equation. It is not intended to be called directly.
#' 
#' @param x \code{numeric} vector of volume fractions of soil constituents (e.g., clay, quartz, minerals other than quartz, organic matter, water, air).  The volume fractions should sum to 1. Note that x and lambda values in the example correspond to these soil constituents.
#' 
#' @param L \code{numeric} Monin-Obukhov length, a measure of the instability of heat flow \insertCite{Beckman1973}{TrenchR}.
#' 
#' @param z0 \code{numeric} surface roughness (m).
#' 
#' @return \code{numeric} integrand for soil temperature function.
#' 
#' 
#' @family soil temperature functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @author Joseph Grigg
#' 
#' @examples
#'   soil_temperature_integrand(x  = c(0.10, 0.40, 0.11, 0.01, 0.2, 0.18), 
#'                              L  = -10, 
#'                              z0 = 0.2)
#'
soil_temperature_integrand <- function (x, 
                                        L, 
                                        z0) { 
  
  stopifnot(z0 > 0)
  
  (3 - 1.4 * exp(1.5 * x))^-1 * (exp(x + z0 / L) / (exp(x + z0 / L) - 1))

}


#' @title Core Function Called to Solve Equation for Soil Temperature
#'
#' @description The function called by \code{\link{soil_temperature_function}} to solve equation for soil temperature from \insertCite{Beckman1973;textual}{TrenchR}.
#' 
#' @param L \code{numeric} Monin-Obukhov length, a measure of the instability of heat flow (see \insertCite{Beckman1973;textual}{TrenchR}).
#' 
#' @param rho_a \code{numeric} density of air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}).
#' 
#' @param c_a \code{numeric} specific heat of air (\ifelse{html}{\out{J kg<sup>-1</sup> C<sup>-1</sup>}}{\eqn{J kg^-1 C^-1}{ASCII}}).
#' 
#' @param u_inst \code{numeric} instantaneous wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @param z_r \code{numeric} reference height (m).
#' 
#' @param z0 \code{numeric} surface roughness (m).
#' 
#' @param T_inst \code{numeric} instantaneous air temperature (C).
#' 
#' @param T_s \code{numeric} initial soil surface temperature (C). 
#' 
#' @return \code{numeric} soil temperature (C).
#' 
#' @family soil temperature functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @author Joseph Grigg
#' 
#' @examples
#'   soil_temperature_equation(L      = -10, 
#'                             rho_a  = 1.177, 
#'                             c_a    = 1006, 
#'                             u_inst = 0.3, 
#'                             z_r    = 1.5, 
#'                             z0     = 0.02, 
#'                             T_inst = 8, 
#'                             T_s    = 20)
#'
soil_temperature_equation <- function (L, 
                                       rho_a, 
                                       c_a, 
                                       u_inst, 
                                       z_r, 
                                       z0, 
                                       T_inst, 
                                       T_s) { 
  
  stopifnot(rho_a  > 0, 
            c_a    > 0, 
            T_inst > -50, 
            T_inst < 100, 
            z_r    > 0, 
            z0     > 0)

  k <- von_karman_constant()
  
  #convert to kelvin
  T_inst= celsius_to_kelvin(T_inst)
  ### check T_s
  
  rho_a * c_a * k * (u_inst * k / log((exp((z_r + z0) / L) - 1) / (exp(z0 / L) - 1))) * (T_inst - T_s) / integrate(soil_temperature_integrand, lower = 0, upper = z_r / L, L, z0)$value - (u_inst * k / log((exp((z_r + z0) / L) - 1)/(exp(z0 / L) - 1)))^3 * T_inst * rho_a * c_a / (k * 9.81 * L)
  
}


#' @title Core Function for Calculating Soil Temperature
#' 
#' @description This function is called to calculate soil temperature as in \insertCite{Beckman1973;textual}{TrenchR}. Parameters are passed as a list to facilitating solving the equations. This function is not intended to be called directly. The energy balance equations are from \insertCite{Porter1973;textual}{TrenchR} and \insertCite{Kingsolver1979;textual}{TrenchR}
#' 
#' @param j \code{numeric} the number of the iteration of running the model.
#' 
#' @param T_so \code{numeric} the initial soil temperature profile in C. 
#' 
#' @param params \code{list} containing the following param, which are described or calculated in \code{\link{soil_temperature}}: \code{SSA, epsilon_so, k_so, c_so, dz, z_r, z0, S, T_a, u, rho_a, rho_so, c_a, TimeIn, dt, shade}.   
#' 
#' @return Soil temperature profile as a \code{list}.
#' 
#' @family soil temperature functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @author Joseph Grigg
#' 
#' @examples
#'   set.seed(123)
#'   temp_vector       <- runif(96, min = -10, max = 10)
#'   wind_speed_vector <- runif(96, min = 0, max = 0.4)
#'   time_vector       <- rep(1:24, 4)
#'   solrad_vector     <- rep(c(rep(0, 6), 
#'                              seq(10, 700, length.out = 6), 
#'                              seq(700, 10, length.out = 6), 
#'                              rep(0, 6)),
#'                            4)
#'   params            <- list(SSA        = 0.7, 
#'                             epsilon_so = 0.98, 
#'                             k_so       = 0.293, 
#'                             c_so       = 800, 
#'                             dz         = 0.05, 
#'                             z_r        = 1.5, 
#'                             z0         = 0.02, 
#'                             S          = solrad_vector, 
#'                             T_a        = temp_vector, 
#'                             u          = wind_speed_vector, 
#'                             rho_a      = 1.177, 
#'                             rho_so     = 1620,
#'                             c_a        = 1006, 
#'                             TimeIn     = time_vector, 
#'                             dt         = 60 * 60, 
#'                             shade      = FALSE)
#' 
#' soil_temperature_function(j      = 1, 
#'                           T_so   = rep(20,13), 
#'                           params = params)
#' 
#'
#'
soil_temperature_function <- function (j, 
                                       T_so, 
                                       params) {
  
  sigma <- stefan_boltzmann_constant()
  k     <- von_karman_constant() 

  Tsoil_deep <- 20+273.15
  
  SSA <- params[[1]]
  epsilon_so <- params[[2]]
  k_so <- params[[3]]
  c_so <- params[[4]]
  dz <- params[[5]]
  z_r <- params[[6]]
  z0 <- params[[7]]
  S <- params[[8]]
  T_a <- params[[9]]
  u <- params[[10]]
  rho_a <- params[[11]]
  rho_so <- params[[12]]
  c_a <- params[[13]]
  TimeIn <- params[[14]]
  dt <- params[[15]]
  shade <- params[[16]]
  
  T_so <- celsius_to_kelvin(T_so)
  
  a <- 2 * dt / (rho_so * c_so * dz)  # eqn (12)
  h_inst1 <- k^2 * c_a * rho_a / log(z_r / z0 + 1)^2 # eqn (1) #calculate h at time t based on u_inst
  alpha2 <- k_so / (c_so * rho_so)
  
  # heat budget elements
  q_sun <- S[j]

  if(shade == TRUE) {
    
    q_sun= q_sun*0.5 #ASSUME 50% reduction in incoming solar radiation in shade
    
  }
  
  T_inst <- celsius_to_kelvin(T_a[j])
  u_inst <- u[j] 
  
  h_inst <- h_inst1 * u_inst #take u_inst out for easier passing to function
  T_sky  <- -0.0552 * T_inst^1.5 #eqn (4)
  
  # energy balance for the surface temperature node
  # multiplying by 'a' gives the change in temperature related to that particular energy source
  
  q_sol   <- a * SSA * q_sun #a*eqn(5) #surface temperature change as a result of solar radiation during the time step.
  q_therm <- a * epsilon_so * sigma * ((T_sky)^4 - (T_so[1])^4) # a*eqn(6) #surface temperature change as a result of thermal radiation during the time step.
  
  # Beckman's method of calculating the convective heat transfer coefficient
  u_shear <- u_inst * k / log(z_r / z0 + 1) #shear velocity
  
  if(j == 1){  # Cannot use Beckman's method for the first time step. Assumed neutral conditions instead. q_conv<-a*h_inst*(T_inst-T_vector[1]) is a*eqn(7) in notes
    
    q_conv <- a * h_inst * (T_inst - T_so[1])
    
  } 
  
  if(j != 1){
    
    if(T_inst < T_so[1]){
      
      # When soil temp is near air temp, an error occurs because L approaches infinity as soil temp approaches air temp. 
      # tryCatch executes alternate command if an error occurs.
      # the function goes to infinity near zero. 
      # The upper bound on this interval was selected to avoid errors that result from numbers approaching infinity. The lower bound can be any large number.
      # Assume neutral conditions if error occurs.

      suppressWarnings(tryCatch(expr  = {L <- uniroot(soil_temperature_equation, 
                                                      interval = c(-50, -.03), 
                                                      rho_a = 1.177, 
                                                      c_a = 1006, 
                                                      k = .41, 
                                                      u_inst = u_inst, 
                                                      z_r = z_r, 
                                                      z0 = z0, 
                                                      T_inst = T_inst, 
                                                      T_s = T_so)$root; 
                                        q_conv <- a * (u_inst * k / log((exp((z_r + z0) / L) - 1) / (exp(z0 / L) - 1)))^3 * T_inst * rho_a * c_a / (k * 9.81 * L)}, 
                                error = function(e){
                                          q_conv <<- a * h_inst * (T_inst - T_so[1])
                                        })) 
      
    } else{
      
      q_conv <- a * h_inst * (T_inst - T_so[1])
      
    }
  } 
  
  q_cond <- a * k_so / dz * (T_so[2] - T_so[1]) #a*eqn(8) #surface temperature change as a result of conduction during the time step.
  
  list(c(
    
    #surface temp
    q_sol + q_therm + q_conv + q_cond, ##this is exactly eqn(13) rescaled to hours as dt is in a
    
    #intermediate temps
    (alpha2 * dt / dz^2) * (T_so[3] + T_so[1] - 2 * T_so[2]),
    (alpha2 * dt / dz^2) * (T_so[4] + T_so[2] - 2 * T_so[3]),
    (alpha2 * dt / dz^2) * (T_so[5] + T_so[3] - 2 * T_so[4]),
    (alpha2 * dt / dz^2) * (T_so[6] + T_so[4] - 2 * T_so[5]),
    (alpha2 * dt / dz^2) * (T_so[7] + T_so[5] - 2 * T_so[6]),
    (alpha2 * dt / dz^2) * (T_so[8] + T_so[6] - 2 * T_so[7]),
    (alpha2 * dt / dz^2) * (T_so[9] + T_so[7] - 2 * T_so[8]),
    (alpha2 * dt / dz^2) * (T_so[10] + T_so[8] - 2 * T_so[9]),
    (alpha2 * dt / dz^2) * (T_so[11] + T_so[9] - 2 * T_so[10]),
    (alpha2 * dt / dz^2) * (T_so[12] + T_so[10] - 2 * T_so[11]),
    (alpha2 * dt / dz^2) * (Tsoil_deep + T_so[11] - 2 * T_so[12]),
    0
  )) 
} 

#' @title Calculate Soil Temperature using ODEs
#' 
#' @description This function is called to calculate soil temperature (C) as in \insertCite{Beckman1973;textual}{TrenchR}. This function calls \code{\link{soil_temperature_function}}, which uses ODEs to calculate a soil profile using equations from \insertCite{deVries1963;textual}{TrenchR}
#' 
#' @param z_r.intervals \code{numeric} the number of intervals in the soil profile to calculate, defaults to 12.
#' 
#' @param z_r \code{numeric} reference height (m).
#' 
#' @param z \code{numeric} interval of the soil profile to return (1 to \code{z_r.intervals}).
#' 
#' @param T_a \code{numeric} vector of air temperature (degrees C), Note: missing values will be linearly interpolated.
#' 
#' @param u \code{numeric} vector of wind speeds (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#' 
#' @param Tsoil0 \code{numeric} initial soil temperature (degrees C). 
#' 
#' @param z0 \code{numeric} surface roughness (m).
#' 
#' @param SSA \code{numeric} solar absorptivity of soil surface as a fraction.
#' 
#' @param TimeIn \code{numeric} vector of time periods for the model.
#' 
#' @param S \code{numeric} vector of solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#' 
#' @param water_content \code{numeric} percent water content (percent).
#' 
#' @param air_pressure \code{numeric} air pressure (kPa).
#' 
#' @param rho_so \code{numeric} particle density of soil.
#' 
#' @param shade \code{logical} whether or not soil temperature should be calculated in the shade.
#' 
#' @return \code{numeric} soil temperature (C).
#' 
#' @family soil temperature functions
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @author Joseph Grigg
#' 
#' @examples
#'   set.seed(123)
#'   temp_vector       <- runif(48, min = -10, max = 10)
#'   wind_speed_vector <- runif(48, min = 0, max = 0.4)
#'   time_vector       <- rep(1:24, 2)
#'   solrad_vector     <- rep(c(rep(0, 6), 
#'                              seq(10, 700, length.out = 6), 
#'                              seq(700, 10, length.out = 6), 
#'                              rep(0, 6)),
#'                            2)
#'
#'   soil_temperature(z_r.intervals = 12, 
#'                    z_r           = 1.5, 
#'                    z             = 2, 
#'                    T_a           = temp_vector, 
#'                    u             = wind_speed_vector, 
#'                    Tsoil0        = 20, 
#'                    z0            = 0.02, 
#'                    SSA           = 0.7, 
#'                    TimeIn        = time_vector, 
#'                    S             = solrad_vector, 
#'                    water_content = 0.2, 
#'                    air_pressure  = 85, 
#'                    rho_so        = 1620, 
#'                    shade         = FALSE)
#'
soil_temperature <- function (z_r.intervals = 12, 
                              z_r, 
                              z, 
                              T_a, 
                              u, 
                              Tsoil0, 
                              z0, 
                              SSA, 
                              TimeIn, 
                              S, 
                              water_content = 0.2, 
                              air_pressure, 
                              rho_so        = 1620, 
                              shade         = FALSE) {
  
  stopifnot(z_r.intervals >  0, 
            z_r           >= 0, 
            z0            >  0, 
            SSA           >= 0, 
            SSA           <= 1, 
            water_content >= 0, 
            water_content <= 1, 
            air_pressure  >  0, 
            rho_so        >  0, 
            is.logical(shade))
  
  # account for NAs at beginning of data
  first.dat <- min(which(!is.na(T_a)))
  # find last data
  last.dat <- max(which(!is.na(T_a)))
  
  # fill temperature data
  T_a <- na.approx(T_a, na.rm = FALSE) #Interpolate
  
  # parameters/constants:
  # SI units were used
  M_w <- 0.018 # kg/mol #molecular weight of water
  rho_w <- 1 * 10^3 # kg/m^3 #water density
  R <- 8.3143 # J/mol  # universal gas constant
  h <- 1 # relative humidity of air in the soil pores.
  
  rho_particle <- 2650 # average particle density of soil #kg/m^3
  rho_quartz   <- 2660 # density of quartz #kg/m^3 #Table 8.2 in Intro to Environmental Biophysics
  rho_o        <- 1300 # average density of organic matter in soil #kg/m^3
  
  # mineral fractions used here are from SCAN data at Nunn, CO
  f_clay <- 0.17277 
  f_sandsilt <- 1 - f_clay
  fraction_quartz <- 0.78 # percentage of solid sand/silt that is quartz
  fraction_other <- 1 - fraction_quartz # percentage of solid sand/silt that is minerals other than quartz
  
  # OrgC and the VanBemmelen factor are mainly useful when looking at data from SCAN (Soil Climate Analysis Network).
  OrgC        <- 0.0056 # organic carbon. this is a value available through SCAN or pedon soil reports. #.0117 corresponds to 2% organic matter.
  VanBemmelen <- 1.724 # VanBemmelen*OrgC is approximately the volume fraction of organic matter in soil. #The VanBemmelen factor is  based on the assumption that organic matter contains 58% organic C. Found information about this from the "Soil Survey Investigations Report No. 42".
  
  dz <- 0.6 / z_r.intervals # 60 cm/number of intervals #60cm= depth for which deep soil temp is measured
  k <- 0.41 # von Karman's constant
  c_a <- 1.006*1000 # specific heat of air (J/(kg*K))
  rho_a <- 1.177 # density of air (kg/m^3)
  epsilon_so <- 0.98
  sigma <- stefan_boltzmann_constant()
  
  # thermal conductivity values (W/mK) along with functions for finding the conductivity based on temperature.
  lambda_clay <- 2.92 # DeVries (1963)
  lambda_quartz <- 8.8 # Table 8.2 in Intro to Environmental Biophysics #9.103 - .028*Temp 
  lambda_other <- 2.93 # DeVries (1963)
  lambda_o <- 0.251 # DeVries (1963)
  lambda_w <- 0.56 + 0.0018 * 20 # .56+.0018*Temp(celsius) is an equation from Table 8.2 in Intro to Environmental Biophysics
  lambda_a <- 0.0237 + 0.000064 * 20 # .0237+.000064*Temp equation from paper by Boguslaw and Lukasz
  
  
  # finding the apparent conductivity of air in soil. These are the methods in DeVries (1963) and summarized in the paper by Boguslaw and Lukasz. variable names were based on those in the Boguslaw and Lukasz paper.
  P <- air_pressure 
  L1 <- 2490317-2259.4 * Tsoil0 #J/kg
  rho_svd <- 0.001 * exp(19.819 - 4975.9 / (Tsoil0 + 273.15)) 
  v <- P / (P - (h * rho_svd * R * (Tsoil0 + 273.15) / (1000 * M_w)))
  D_a <- 21.7 * 10^-6 * (101.325 / P) * ((Tsoil0 + 273.15) / 273.15)^1.88 #m^2/s
  drhoo_dT <- 4975.9 * rho_svd / (Tsoil0 + 273.15)^2 #kg/m^3
  
  lambda_v <- L1 * D_a * v * drhoo_dT
  lambda_app <-lambda_a + h * lambda_v
  
  # finding volume fraction of soil constituents.
  x_clay <- rho_so * f_clay * (1 - OrgC * VanBemmelen) / rho_particle # fraction clay
  x_quartz <- rho_so * f_sandsilt * (1 - OrgC * VanBemmelen) * fraction_quartz / rho_particle # fraction quartz in sand/silt
  x_other <- rho_so * f_sandsilt * (1 - OrgC * VanBemmelen) * fraction_other / rho_particle # fraction minerals other than quartz
  x_o <- rho_so * f_sandsilt * OrgC * VanBemmelen / rho_o # fraction organic matter
  x_solid <- x_clay + x_quartz + x_other + x_o # fraction solids
  x_w <- water_content # fraction water
  x_a <- 1 - (x_solid + x_w) # fraction air
  
  # set up a vector of volume fractions to be used in the 'conductivity.R' script.
  x <- c(x_clay, x_quartz, x_other, x_o, x_w, x_a)
  # set up a vector of thermal conductivities to be used in the 'conductivity.R' script.
  lambda <- c(lambda_clay, lambda_quartz, lambda_other, lambda_o, lambda_w, lambda_app)
  
  # finding soil thermal conductivity and specific heat
  k_so <- soil_conductivity(x, lambda, 0.125) #calculate soil thermal conductivity
  c_so <- soil_specific_heat(x_o, x_solid - x_o, x_w, rho_so) #calculate soil specific heat
  alpha2 <- k_so / (c_so * rho_so)
  
  #param for ODE
  dt <- 60 * 60
  
  #------------------
  
  # SOLVE ODE
  params <- list(SSA, epsilon_so, k_so, c_so, dz, z_r, z0, S, T_a, u, rho_a, rho_so, c_a, TimeIn, dt, shade)
  
  Tsoil <- suppressWarnings(ode(y = rep(Tsoil0, z_r.intervals + 1), func = soil_temperature_function, times = 1:length(S), params))
  
  return(Tsoil[, z])
  
} 

context("TsoilFunctions")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("soil_conductivity function works as expected", {
  x=c(0.10,0.40,0.11,0.01,0.2, 0.18) 
  lambda=c(0.10,0.40,0.11,0.01,0.2,0.18)
  expect_similar(soil_conductivity(x=x,lambda=lambda, g_a=0.125), 0.2336174)
})

test_that("soil_specific_heat function works as expected", {
  expect_similar(soil_specific_heat(x_o=0.01, x_m=0.6, x_w=0.2, rho_so=1620), 869.2968)
})

test_that("soil_temperature_integrand function works as expected", {
  expect_equal(length(soil_temperature_integrand(x=c(0.10,0.40,0.11,0.01,0.2, 0.18), L=-10, z0=0.2)),6) 
})

test_that("soil_temperature_equation function works as expected", {
  expect_similar(soil_temperature_equation(L=-10, rho_a=1.177, c_a=1006, u_inst=0.3, z_r=1.5, z0=0.02, T_inst= -8, T_s=20), 1341.527)
})


test_that("soil_temperature function works as expected", {
set.seed(123)
  temp_vector= runif(96, min=-10, max=10)
  wind_speed_vector= runif(96, min=0, max=0.4)
  time_vector= rep(1:24,4)
  solrad_vector= rep(c(rep(0,6),seq(10,700,length.out=6), seq(700,10,length.out=6),rep(0,6)),4)
  results <- soil_temperature(z_r.intervals=12,z_r=1.5, z=2, T_a=temp_vector, u=wind_speed_vector, Tsoil0= 20, z0=0.02, SSA=0.7, TimeIn=time_vector, S= solrad_vector, water_content=0.2, air_pressure=85, rho_so=1620, shade=FALSE)
  expect_equal(length(results), 96)
  expect_identical(class(results), "numeric")
  expect_equal(results[1], 20)
})

test_that("soil_temperature_function function words as expected", {
set.seed(123)
  temp_vector= runif(96, min=-10, max=10)
  wind_speed_vector= runif(96, min=0, max=0.4)
  time_vector= rep(1:24,4)
  solrad_vector= rep(c(rep(0,6),seq(10,700,length.out=6), seq(700,10,length.out=6),rep(0,6)),4)
  params=list(SSA=0.7, epsilon_so=0.98, k_so=0.293, c_so=800, dz=0.05, z_r=1.5, z0=0.02, S=solrad_vector, T_a=temp_vector, u=wind_speed_vector, rho_a=1.177,rho_so=1620, c_a=1006, TimeIn=time_vector, dt=60*60, shade=FALSE)
  expect_identical(class(soil_temperature_function(j=1,T_so= rep(20,13), params=params)), "list")
  expect_equal(length(soil_temperature_function(j=1,T_so= rep(20,13), params=params)[[1]]), 13)
  expect_gt(soil_temperature_function(j=1,T_so= rep(20,13), params=params)[[1]][1], -43)
  expect_lt(soil_temperature_function(j=1,T_so= rep(20,13), params=params)[[1]][1], -14)

  expect_gt(soil_temperature_function(j=2,T_so= rep(20,13), params=params)[[1]][1], -43)
  expect_lt(soil_temperature_function(j=2,T_so= rep(20,13), params=params)[[1]][1], -14)

  params=list(SSA=0.7, epsilon_so=0.98, k_so=0.293, c_so=800, dz=0.05, z_r=1.5, z0=0.02, S=solrad_vector, T_a=temp_vector, u=wind_speed_vector, rho_a=1.177,rho_so=1620, c_a=1006, TimeIn=time_vector, dt=60*60, shade=TRUE)
  expect_gt(soil_temperature_function(j=1,T_so= rep(20,13), params=params)[[1]][1], -43)
  expect_lt(soil_temperature_function(j=1,T_so= rep(20,13), params=params)[[1]][1], -14)

})

#' @title Conductance Assuming Animal Thermal Conductivity is Rate Limiting
#'
#' @description The function calculates conductance (W) of an ectothermic animal to its substrate. Method assumes the major resistance to conduction is within surface layers of the animal and that the interior of the animal is equal in temperature to its surface (thermally well mixed) \insertCite{Spotila1992}{TrenchR}.
#'
#' @param T_g \code{numeric} ground surface temperature (K).
#'
#' @param T_b \code{numeric} body temperature (K).
#'
#' @param d \code{numeric} mean thickness of the animal skin (surface) in (meters). The function assumes a well mixed interior.
#'
#' @param K \code{numeric} thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}). \code{K = 0.5} for naked skin and \code{K = 0.15} for insect cuticle \insertCite{Galushko2005}{TrenchR}. The conductivity of the ground is generally greater than that of animal tissues, so animal thermal conductivity is generally the rate limiting step.
#'
#' @param A \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @param proportion \code{numeric} proportion of body in contact with the surface (0-1).
#'
#' @return \code{numeric} conductance (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qconduction_animal(T_g        = 293,
#'                      T_b        = 303,
#'                      d          = 10^-6,
#'                      K          = 0.5,
#'                      A          = 10^-3,
#'                      proportion = 0.2)
#'
Qconduction_animal <- function (T_g,
                                T_b,
                                d,
                                K = 0.5,
                                A,
                                proportion) {

  stopifnot(T_g        >  200, 
            T_g        <  400,
            T_b        >  200, 
            T_b        <  400, 
            d          >= 0, 
            K          >  0, 
            A          >  0, 
            proportion >= 0, 
            proportion <= 1)


  A_contact <- A * proportion

  # Conduction
  # the heat loss (Difference between animal temperature and its environment)
  # m^2 * W K^-1 m^-1 * K / m

  A_contact * K * (T_b - T_g) / d

}

#' @title Conductance Assuming Substrate Thermal Conductivity is Rate Limiting
#'
#' @description The function calculates conductance (W) of an ectothermic animal to its substrate. The method assumes the major resistance to conduction is the substrate and that the interior of the animal is equal in temperature to its surface (thermally well mixed) \insertCite{Spotila1992}{TrenchR}.
#'
#' @param T_g \code{numeric} surface temperature (K).
#'
#' @param T_b \code{numeric} body temperature (K).
#'
#' @param D \code{numeric} characteristic dimension of the animal (m).
#'
#' @param K_g \code{numeric} thermal conductivity of substrate (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}).
#'
#' @param A \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @param proportion \code{numeric} proportion in contact to the surface.
#'
#' @return \code{numeric} conductance (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qconduction_substrate(T_g        = 293,
#'                         T_b        = 303,
#'                         D          = 0.01,
#'                         K_g        = 0.3,
#'                         A          = 10^-2,
#'                         proportion = 0.2)
#'
#'
Qconduction_substrate <- function (T_g,
                                   T_b,
                                   D,
                                   K_g = 0.5,
                                   A,
                                   proportion) {

  stopifnot(T_g        >  200, 
            T_g        <  400, 
            T_b        >  200, 
            T_b        <  400, 
            D          >= 0, 
            K_g        >  0, 
            A          >  0, 
            proportion >= 0, 
            proportion <= 1)


  A_contact <- A * proportion

  # Thermal conductance between animal and substrate
  H_g <- 2.0 * K_g / D

  # Conduction,  m^2 * W K^-1 m^-1 * K / m
  A_contact * H_g * (T_b - T_g)

}


#' @title Organismal Convection
#'
#' @description The function calculates convection from an organism to its environment as in \insertCite{Mitchell1976;textual}{TrenchR}. It includes an enhancement factor associated with outdoor environments.
#'
#' @param T_a \code{numeric} air temperature (K).
#'
#' @param T_b \code{numeric} initial body temperature (K).
#'
#' @param H_L \code{numeric} convective heat transfer coefficient (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-2</sup>}}{\eqn{W K^-1 m^-2}{ASCII}}).
#'
#' @param A \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @param proportion \code{numeric} proportion of surface area exposed to air.
#'
#' @param ef \code{numeric} is the enhancement factor, used to adjust H to field conditions.  Approximated as mean value of 1.23 by default, but see \insertCite{Mitchell1976;textual}{TrenchR} for further information.
#'
#' @return \code{numeric} convection (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qconvection(T_a        = 293,
#'               T_b        = 303,
#'               H_L        = 10.45,
#'               A          = 0.0025,
#'               proportion = 0.85)
#'
#'
Qconvection <- function (T_a,
                         T_b,
                         A,
                         proportion,
                         H_L         = 10.45,
                         ef          = 1.23) {

  stopifnot(T_a        >  200, 
            T_a        <  400, 
            T_b        >  200, 
            T_b        <  400, 
            H_L        >  0, 
            A          >  0, 
            proportion >= 0, 
            proportion <= 1, 
            ef         >= 1)

  # Calculate skin area exposed to air
  A_air <- A * proportion

  ef * H_L * A_air * (T_b - T_a)

}


#' @title Estimate the Heat Transfer Coefficient Empirically
#'
#' @description The function estimates the heat transfer coefficient for various taxa based on empirical measurements \insertCite{Mitchell1976}{TrenchR}.
#'
#' @param u \code{numeric} air velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param D \code{numeric} characteristic dimension (e.g., diameter or snout-vent length) (meters).
#'
#' @param K \code{numeric} thermal conductivity of air (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}), can calculate using \code{\link{DRYAIR}} or \code{\link{WETAIR}}.
#'
#' @param nu \code{numeric} kinematic viscosity of air (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}), can calculate using \code{\link{DRYAIR}} or \code{\link{WETAIR}}.
#'
#' @param taxon \code{character} which class of organism, current choices: \code{"sphere"}, \code{"cylinder"}, \code{"frog"}, \code{"lizard_surface"}, \code{"lizard_elevated"}, \code{"flyinginsect"}, \code{"spider"}. 
#'  \cr
#'  \code{"cylinder"} assumes 40 < Re < 4000. \code{"lizard_surface"} and \code{"lizard_elevated"} assume the lizard is prostrate on and elevated above the surface, respectively. The values are the average for lizards parallel and perpendicular to the air flow.
#'
#' @return \code{numeric} heat transfer coefficient, \code{H_L} (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-2</sup>}}{\eqn{W K^-1 m^-2}{ASCII}}).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   heat_transfer_coefficient(u     = 0.5,
#'                             D     = 0.05,
#'                             K     = 25.7 * 10^(-3),
#'                             nu    = 15.3 * 10^(-6),
#'                             taxon = "cylinder")
#'
heat_transfer_coefficient <- function (u,
                                       D,
                                       K,
                                       nu,
                                       taxon = "cylinder") {

  taxa <- c("sphere", "cylinder", "frog", "lizard_surface", "lizard_elevated", "flyinginsect", "spider")

  stopifnot(u  >= 0, 
            D  >= 0, 
            K  >= 0, 
            nu >= 0, 
            length(taxon) == 1,
            taxon %in% taxa)

  # Dimensionless constant (Cl)
  Cls <- c(0.37, 0.615, 0.258, 1.36, 1.91, 0.0749, 0.47)
  ns <- c(0.6, 0.466, 0.667, 0.39, 0.45, 0.78, 0.5)

  # Find index
  ind <- match(taxon, taxa)

  Re <- u * D / nu # Reynolds number
  Nu <- Cls[ind] * Re^ns[ind]  # Nusselt number

  Nu * K / D

}


#' @title Estimate the Heat Transfer Coefficient Using a Spherical Approximation
#'
#' @description The function estimates the heat transfer coefficient for various taxa. Approximates forced convective heat transfer for animal shapes using the convective relationship for a sphere \insertCite{Mitchell1976}{TrenchR}.
#'
#' @param u \code{numeric} air velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param D \code{numeric} characteristic dimension (e.g., diameter or snout-vent length) (meters).
#'
#' @param K \code{numeric} thermal conductivity of air (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}), can calculate using \code{\link{DRYAIR}} or \code{\link{WETAIR}}.
#'
#' @param nu \code{numeric} kinematic Viscosity of air (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}), can calculate using \code{\link{DRYAIR}} or \code{\link{WETAIR}}.
#'
#' @param taxon \code{character} of which class of organism, current choices: \code{"sphere"}, \code{"frog"}, \code{"lizard"}, \code{"flyinginsect"}, \code{"spider"}.
#'
#' @return \code{numeric} heat transfer coefficient, \code{H_L} (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 C^-1}{ASCII}}).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'  heat_transfer_coefficient_approximation(u     = 3,
#'                                          D     = 0.05,
#'                                          K     = 25.7 * 10^(-3),
#'                                          nu    = 15.3 * 10^(-6),
#'                                          taxon = "sphere")
#'
heat_transfer_coefficient_approximation <- function (u,
                                                     D,
                                                     K,
                                                     nu,
                                                     taxon = "sphere") {


  taxa <- c("sphere", "frog", "lizard", "flyinginsect", "spider")

  stopifnot(u  >= 0, 
            D  >= 0, 
            K  >= 0, 
            nu >= 0, 
            length(taxon) == 1,
            taxon %in% taxa)

  # Dimensionless constant (Cl)
  Cls <- c(0.34, 0.196, 0.56, 0.0714, 0.52)
  ns <- c(0.6, 0.667, 0.6, 0.78, 0.5)

  # Find index
  ind <- match(taxon, taxa)

  Re <- u * D / nu # Reynolds number
  Nu <- Cls[ind] * Re^ns[ind]  # Nusselt number

  Nu * K / D

}


#' @title Estimate the Heat Transfer Coefficient using Simple Relationships
#'
#' @description The function estimates the heat transfer coefficient \insertCite{Mitchell1976}{TrenchR} using either the relationship in \insertCite{Spotila1992;textual}{TrenchR} or that in \insertCite{Gates1980;textual}{TrenchR}.
#'
#' @param u \code{numeric} air velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param D \code{numeric} characteristic dimension (e.g., diameter or snout-vent length) (m).
#'
#' @param type \code{character} choice between \code{"Spotila"} and \code{"Gates"} for equation to use.
#'
#' @return \code{numeric} heat transfer coefficient, H_L (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   heat_transfer_coefficient_simple(u    = 0.5,
#'                                    D    = 0.05,
#'                                    type = "Gates")
#'
heat_transfer_coefficient_simple <- function(u,
                                             D,
                                             type) {

  stopifnot(u >= 0, 
            D >= 0,
            length(type) == 1,
            type %in% c("Spotila", "Gates"))

  if(type == "Spotila") {

    H_L <- 6.77 * u^0.6 * D^(-0.4)

  }

  if(type == "Gates") {

    H_L <- 3.49 * u^0.5 * D^(-0.5)

  }

  H_L

}

#' @title Absorbed Solar and Thermal Radiation
#'
#' @description The function estimates solar and thermal radiation (W) absorbed by the surface of an animal following \insertCite{Gates1980}{TrenchR} and \insertCite{Spotila1992}{TrenchR}.
#'
#' @param a \code{numeric} solar absorptivity of animal surface (proportion), default value is for reptiles (0-1).
#'
#' @param A \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @param psa_dir \code{numeric} proportion surface area exposed to direct solar radiation (0-1).
#'
#' @param psa_dif \code{numeric} proportion surface area exposed to diffuse solar radiation (0-1).
#'
#' @param psa_ref \code{numeric} proportion surface area exposed to reflected solar radiation (0-1).
#'
#' @param S_dir \code{numeric} direct solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#'
#' @param S_dif \code{numeric} diffuse solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#'
#' @param S_ref \code{numeric} reflected solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}), either provided or estimated if surface albedo is provided instead
#'
#' @param rho \code{numeric} is surface albedo (proportion), optional (not used) if reflected radiation is provided. Values available in \insertCite{Gates1980}{TrenchR} Table 8.2.
#'
#' @return \code{numeric} solar radiation absorbed (W)
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qradiation_absorbed(a       = 0.9,
#'                       A       = 1,
#'                       psa_dir = 0.5,
#'                       psa_dif = 0.5,
#'                       psa_ref = 0.5,
#'                       S_dir   = 1000,
#'                       S_dif   = 200,
#'                       rho     = 0.5)
#'
Qradiation_absorbed <- function (a = 0.9,
                                 A,
                                 psa_dir,
                                 psa_dif,
                                 psa_ref,
                                 S_dir,
                                 S_dif,
                                 S_ref = NA,
                                 rho   = NA) {

  stopifnot(a       >= 0, 
            a       <= 1, 
            A       >  0, 
            psa_dir >= 0, 
            psa_dir <= 1, 
            psa_dif >= 0, 
            psa_dif <= 1, 
            psa_ref >= 0, 
            psa_ref <= 1, 
            S_dir   >  0, 
            S_dif   >  0)

  #Calculate S_ref if not provided
  if(is.na(S_ref)) {

    S_ref <- rho * S_dir

  }

  # Areas
  A_dir <- A * psa_dir
  A_dif <- A * psa_dif
  A_ref <- A * psa_ref

  # Solar radiation
  a * A_dir * S_dir + a * A_dif * S_dif + a * A_ref * S_ref

}

#' @title Effective radiant temperature of sky (K)
#' 
#' @description This function estimates the effective radiant temperature of the sky (K) using either the Brunt or Swinbank formulations \insertCite{Gates1980}{TrenchR}. 
#' 
#' @param T_a \code{numeric} ambient air temperature (K)
#' 
#' @param formula \code{character} Formula to use, current choices are: \code{"Brunt"}, \code{"Swinbank"}. 
#'
#' @return \code{numeric} T_sky (K).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   T_sky(T_a=293, formula="Swinbank")
#'
T_sky <- function (T_a, formula) {
  
  # Stefan-Boltzmann constant
  sigma <- stefan_boltzmann_constant()
  
  #Gates 1980, 7.1
  if(formula=="Swinbank") T_sky= 1.22*sigma*T_a^4-171
  
  #Gates 1980, 7.12
  if(formula=="Brunt") T_sky= 1.22*(T_a-273.15)-20.4 +273.15
  
  T_sky
}

#' @title Emitted Thermal Radiation
#'
#' @description The function estimates the net thermal radiation (W) emitted by the surface of an animal \insertCite{Gates1980,Spotila1992}{TrenchR}.
#'
#' @param epsilon \code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.
#'
#' @param A \code{numeric} surface area ((\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @param psa_dir \code{numeric} view factor indicating the proportion of surface area exposed to sky (or enclosure) (0-1)
#'
#' @param psa_ref \code{numeric} view factor indicating the proportion surface area exposed to ground (0-1).
#'
#' @param T_b \code{numeric} body surface temperature (K).
#'
#' @param T_g \code{numeric} ground surface temperature (K).
#' 
#' @param T_sky \code{numeric} Estimate effective radiant temperature of sky (K)
#'
#' @param T_a \code{numeric} ambient air temperature (K), only required if the animal is in an enclosed environment.
#'
#' @param enclosed \code{logical} whether the animal is an enclosed environment or not.
#'
#' @return \code{numeric} emitted thermal radiation, \code{Qemit} (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qemitted_thermal_radiation(epsilon  = 0.96,
#'                              A        = 1,
#'                              psa_dir  = 0.4,
#'                              psa_ref  = 0.5,
#'                              T_b      = 303,
#'                              T_g      = 293,
#'                              T_a      = 298,
#'                              enclosed = FALSE)
#'
Qemitted_thermal_radiation <- function (epsilon  = 0.96,
                                        A,
                                        psa_dir,
                                        psa_ref,
                                        T_b,
                                        T_g,
                                        T_sky = NA,
                                        T_a,
                                        enclosed = FALSE){

  stopifnot(epsilon >= 0, 
            epsilon <= 1, 
            A       >  0, 
            psa_dir >= 0, 
            psa_dir <= 1, 
            psa_ref >= 0, 
            psa_ref <= 1, 
            T_b     >  200, 
            T_b     <  400, 
            T_g     >  200, 
            T_g     <  400, 
            T_a     >  200, 
            T_a     <  400, 
            is.logical(enclosed))

  # Stefan-Boltzmann constant
  sigma <- stefan_boltzmann_constant()

  # Areas
  A_s <- A * psa_dir
  A_r <- A * psa_ref

  # Estimate effective radiant temperature of sky using Brunt equation if not provided
  if(is.na(T_sky)) T_sky <- (1.22 * (T_a-273.15) - 20.4) + 273.15 # K, Gates eq 7.12

  if(enclosed){

    Qemit <- A_r * epsilon * sigma * (T_b^4 - T_a^4)

  } else {

    Qemit <- epsilon * sigma * (A_s * (T_b^4 - T_sky^4) + A_r * (T_b^4 - T_g^4))

  }

  Qemit

}

#' @title Heat Loss Associated with Evaporative Water Loss
#'
#' @description The function estimates heat loss associated with evaporative water loss for an amphibian \insertCite{Spotila1992}{TrenchR} or lizard. The lizard estimation is based on empirical measurements in \insertCite{Porter1973;textual}{TrenchR}).
#'
#' @param A \code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
#'
#' @param T_b \code{numeric} body temperature (K).
#'
#' @param taxon \code{character} organism type. Current choices: \code{"lizard"}, \code{"amphibian_wetskin"} (fully wet skin), \code{"amphibian"} (not fully wet skin).
#'
#' @param e_s \code{numeric} saturation water vapor density at skin surface (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) (needed if amphibian).
#'
#' @param e_a \code{numeric} saturation water vapor density in ambient air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) (needed if amphibian).
#'
#' @param hp \code{numeric} relative humidity (0-1) (needed if amphibian).
#'
#' @param H \code{numeric} convective heat transfer coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}) (needed if amphibian).
#'
#' @param r_i \code{numeric} internal (cutaneous) resistance to vapor transport (\ifelse{html}{\out{s m<sup>-1</sup>}}{\eqn{s m^-1}{ASCII}}) (needed if amphibian).
#'
#' @return \code{numeric} evaporative heat loss (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qevaporation(A      = 0.1,
#'                T_b    = 293,
#'                taxon = "amphibian",
#'                e_s = 0.003,
#'                e_a = 0.002,
#'                hp     = 0.5,
#'                H     = 20,
#'                r_i   = 50)
#'   Qevaporation(A     = 0.1,
#'                T_b   = 293,
#'                taxon = "lizard")
#'
Qevaporation <- function (A,
                          T_b,
                          taxon,
                          e_s = NA,
                          e_a = NA,
                          hp     = NA,
                          H     = NA,
                          r_i   = NA) {
  
  stopifnot(A   > 0, 
            T_b > 200, 
            T_b < 400, 
            length(taxon) == 1,
            taxon %in% c("lizard", "amphibian_wetskin", "amphibian"))
  
  if(taxon %in% c("amphibian_wetskin", "amphibian")) {
    
    stopifnot(e_s >  0,
              e_a >  0, 
              hp     >= 0, 
              hp     <= 1, 
              H     >  0, 
              r_i   >  0)
    
  }
  
  # Porter et al. 1973 in Gates 1980 Biophysical ecology
  if (taxon == "lizard") {
    if(T_b < 293) {
      
      E_kg <- 0.27
      
    }
    
    if(T_b >= 293 & T_b <= 309) {
      
      E_kg <- 0.08 * exp(0.586) * (T_b - 273.5)
      
    }
    
    if(T_b > 309) {
      
      E_kg <- 2.97 * 10^(-3) * exp(0.1516) * (T_b - 273.5)
      
    }
    
    # convert from W/kg to W/m2
    E <- E_kg * 0.067 / 0.018 #for 0.067kg lizard with 0.018m^2 surface area
    
    # multiply by surface area
    Qevap <- E * A
    
  }
  
  # Spotila et al. 1992
  evap_heat <- 2.44 * 10^(6) # J/kg at most temperatures
  
  rhocp <- 1200 # J*m^(-3)*K^(-1)
  # external (convective) resistance to water vapor transport (s/m), Lewis rule
  r_e <- 0.93 * rhocp / H
  
  if(taxon == "amphibian_wetskin") {
    
    Ec <- A * (e_s - hp * e_a) / r_e # rate of water transport (kg/s)
    Qevap <- Ec * evap_heat #to W
    
  }
  
  if(taxon == "amphibian") {
    
    Ec= A * (e_s-hp*e_a)/(r_i+r_e) # rate of water transport (kg/s)
    Qevap= Ec*evap_heat # to W
    
  }
  
  Qevap
  
}

#' @title Saturation Water Vapor Pressure
#'
#' @description The function approximates saturation water vapor pressure as a function of ambient temperature for temperatures from 0 to 40 C using \insertCite{Rosenberg1974;textual}{TrenchR} in \insertCite{Spotila1992;textual}{TrenchR}. See also NicheMapR \code{\link{WETAIR}} and \code{\link{DRYAIR}} \insertCite{Kearney2020}{TrenchR}.
#'
#' @param T_a \code{numeric} air temperature (C).
#'
#' @return \code{numeric} Saturation water vapor pressure, \code{e_s} (Pa).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   saturation_water_vapor_pressure(T_a = 20)
#'
saturation_water_vapor_pressure <- function (T_a) {

  10^(0.02604 * T_a + 2.82488)

}


#' @title External Resistance to Water Vapor Transfer
#'
#' @description The function estimate external resistance to water vapor transfer using the Lewis rule relating heat and mass transport \insertCite{Spotila1992}{TrenchR}
#'
#' @param H \code{numeric} heat transfer (convection) coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}).
#'
#' @param ecp \code{numeric} aggregate parameter (\ifelse{html}{\out{J m<sup>-3</sup> K<sup>-1</sup>}}{\eqn{J m^-3 K^-1}{ASCII}}) that is the product of the density of air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) and the specific heat of air at constant pressure (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}). Default of 12000 (\ifelse{html}{\out{J m<sup>-3</sup> K<sup>-1</sup>}}{\eqn{J m^-3 K^-1}{ASCII}}) commonly assumed.
#'
#' @return \code{numeric} external resistance to water vapor transfer (\ifelse{html}{\out{s m<sup>-1</sup>}}{\eqn{s m^-1}{ASCII}}).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   external_resistance_to_water_vapor_transfer(H = 20)
#'
external_resistance_to_water_vapor_transfer <- function (H,
                                                         ecp = 12000) {

  stopifnot(H > 0)

  0.93 * ecp / H

}


#' @title Metabolism as a Function of Mass
#'
#' @description The function estimates the field metabolic rate (W) of various taxa as a function of mass (g). The function does not account for temperature and is based on empirical relationships from \insertCite{Nagy2005;textual}{TrenchR}.
#'
#' @param m \code{numeric} mass (grams).
#'
#' @param taxon \code{character} taxon for calculation. Options: \code{"reptile"}, \code{"bird"}, \code{"mammal"}.
#'
#' @return \code{numeric} metabolism (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qmetabolism_from_mass(m     = 12,
#'                         taxon = "reptile")
#'
Qmetabolism_from_mass <- function(m,
                                  taxon = "reptile") {

  stopifnot(m > 0, 
            length(taxon) == 1,
            taxon %in% c("reptile", "bird", "mammal"))

  # FMR in W, M is mass (g)
  # Convert 1 kJ/day = 0.0115741 W

  # Reptile
  if(taxon == "reptile") {

    Qmet <- 0.196 * m^0.889 * 0.0115741

  }

  # Mammal
  if(taxon == "mammal") {

    Qmet <- 4.82 * m^0.734 * 0.0115741

  }

  # Bird
  if(taxon == "bird") {

    Qmet <- 10.5 * m^0.681 * 0.0115741

  }

  Qmet

}


#' @title Metabolism as a Function of Mass and Body Temperature
#'
#' @description The function estimates basal (or resting) metabolic rate (W) as a function of mass (g) and temperature (C). The function is based on empirical data and the metabolic theory of ecology (assumes a 3/4 scaling exponent) \insertCite{Gillooly2001}{TrenchR}.
#'
#' @param m \code{numeric} mass (grams).
#'
#' @param T_b \code{numeric} body temperature (C).
#'
#' @param taxon \code{character} organism type. Options: \code{"bird"}, \code{"mammal"}, \code{"reptile"}, \code{"amphibian"}, \code{"invertebrate"}.
#'
#' @return \code{numeric} basal metabolism (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Qmetabolism_from_mass_temp(m     = 100,
#'                              T_b   = 30,
#'                              taxon = "reptile")
#'
Qmetabolism_from_mass_temp <- function (m,
                                        T_b,
                                        taxon) {

  stopifnot(m   > 0, 
            T_b > -50, 
            T_b < 100, 
            length(taxon) == 1,
            taxon %in% c("bird", "mammal", "reptile", "amphibian", "invertebrate"))

  #Convert to K
  T_b= celsius_to_kelvin(T_b)
  
  if (taxon %in% c("bird", "mammal")) {

    Qmet <- exp(-9100 / T_b + 29.49) * m^0.75 / 60

  }

  if(taxon == "reptile") {

    Qmet <- exp(-8780 / T_b + 26.85) * m^0.75 / 60

  }

  if(taxon == "amphibian") {

    Qmet <- exp(-5760 / T_b + 16.68) * m^0.75 / 60

  }

  if(taxon == "invertebrate") {

    Qmet <- exp(-9150 / T_b + 27.62) * m^0.75 / 60

  }

  Qmet

}


#' @title Actual Vapor Pressure from Dewpoint Temperature
#'
#' @description The function calculates actual vapor pressure from dewpoint temperature based on \insertCite{Stull2000,Riddell2018;textual}{TrenchR}.
#'
#' @param T_dewpoint \code{numeric} dewpoint temperature (K).
#'
#' @return \code{numeric} actual vapor pressure (kPa).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @author Eric Riddell
#'
#' @examples
#'   actual_vapor_pressure(T_dewpoint = 293)
#'
actual_vapor_pressure <- function (T_dewpoint) {

  0.611 * (2.71828182845904^(((1.0 / 273.0) - (1.0 / T_dewpoint)) * 5422.9939))

}


#' @title Saturation Vapor Pressure
#'
#' @description The function calculates saturation vapor pressure (kPa) based on the Clausius-Clapeyron equation \insertCite{Stull2000,Riddell2018}{TrenchR}.
#'
#' @param T_a \code{numeric} air temperature (K).
#'
#' @return \code{numeric} saturation vapor pressure, \code{e_s} (kPa).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @author Eric Riddell
#'
#' @examples
#'   saturation_vapor_pressure(T_a = 293)
#'
saturation_vapor_pressure <- function (T_a) {

  stopifnot(T_a > 200, 
            T_a < 400)

  Rv <- 461.5     # J*K^-1*kg^-1, ideal gas constant for water vapor
  L <- 2.5 * 10^6 # J per kg, latent heat of vaporization
  e_o <- 0.611    # kPa

  e_o * exp((L / Rv) * ((1. / 273.15) - (1. / T_a)))

}


#' @title Boundary Layer Resistance
#'
#' @description The function estimates boundary layer resistance under free convection based on the function in \insertCite{Riddell2018;textual}{TrenchR}.
#'
#' @param T_a \code{numeric} air temperature (K).
#'
#' @param e_s \code{numeric} saturation vapor pressure (kPa).
#'
#' @param e_a \code{numeric} actual vapor pressure (kPa).
#'
#' @param elev \code{numeric} elevation (m).
#'
#' @param D \code{numeric} characteristic dimension (e.g., body diameter) (m).
#'
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}), if not provided assume free convection; if provided, use forced convection if appropriate.
#'
#' @return \code{numeric} boundary layer resistance (\ifelse{html}{\out{s cm<sup>-1</sup>}}{\eqn{s cm^-1}{ASCII}}).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @author Eric Riddell
#'
#' @examples
#'   boundary_layer_resistance(T_a  = 293,
#'                             e_s  = 2.5,
#'                             e_a  = 2.0,
#'                             elev = 500,
#'                             D    = 0.007,
#'                             u    = 2)
#'
boundary_layer_resistance <- function (T_a,
                                       e_s,
                                       e_a,
                                       elev,
                                       D,
                                       u = NA) {
  
  stopifnot(T_a  > 200, 
            T_a  < 400, 
            e_s  > 0, 
            e_a  > 0, 
            elev > 0, 
            D    > 0)
  
  if(e_s < e_a) {
    
    stop("Actual vapor pressure, e_a, must be lower that saturation vapor pressure, e_s")
    
  }
  
  gravity = 9.8 # m/s
  
  air_pressure <- 101325. * (1. - (2.2569 * 10^-5) * elev)^5.2553
  air_density <- air_pressure / (287.04 * T_a)
  dynamic_viscosity <- (1.8325 * 10^-5) * ((296.16 + 120.) / (T_a + 120.)) * ((T_a / 296.16)^1.5) #Tracy et al. 2010
  kinematic_viscosity <- dynamic_viscosity / air_density
  T_surface <- (T_a) * (1. + 0.38 * ((e_s * 1000.) / air_pressure)) #soil surface temperature in steady state heat balance
  T_air <- T_a * (1. + 0.38 * ((e_a * 1000.) / air_pressure)) #air temperature in steady state heat balance
  coef_thermal_expansion <- 1.0 / T_a
  
  # Grashof and Nusselt numbers
  Grashof <- (coef_thermal_expansion * gravity * (D^3) * (abs(T_surface - T_air))) / (kinematic_viscosity^2)
  Nusselt <- 0.48 * (Grashof)^0.25
  
  thermal_conductivity <- (2.4525*10^-2) + ((7.038 * 10^-5) * (T_a - 273.15))
  mixing_ratio <- (0.6257 * (e_a * 1000)) / (air_pressure - (1.006 * (e_a * 1000)))
  
  # free convection
  hc <- (Nusselt * thermal_conductivity) / D #free convective heat transfer coefficient
  
  if(!is.na(u)){ #check if wind speed is provided
    
    # estimate Reynolds number- ratio of interval viscous forces
    Re <- u * D / kinematic_viscosity
    
    # forced convection
    # use if Gr< 16 * Re^2
    if(Grashof <= 16 * Re^2) {
      
      hc <- 0.923 * (u^0.333 * D^(-0.666))
      
    }
    
  }
  
  # calculate boundary layer resistance
  specific_heat <- (1004.84 + (1846.4 * mixing_ratio)) / (1 + mixing_ratio)
  
  ((specific_heat * air_density) / hc) / 100
  
}

#' @title Humid Operative Environmental Temperature of a Salamander
#'
#' @description The function estimates the humid body temperature (C, operative environmental temperature) using an adaptation of \insertCite{Campbell1998;textual}{TrenchR} described in \insertCite{Riddell2018;textual}{TrenchR}.
#'
#' @param r_i \code{numeric} internal (skin) resistance (\ifelse{html}{\out{s cm<sup>-1</sup>}}{\eqn{s cm^-1}{ASCII}}).
#'
#' @param r_b \code{numeric} boundary layer resistance (\ifelse{html}{\out{s cm<sup>-1</sup>}}{\eqn{s cm^-1}{ASCII}}).
#'
#' @param D \code{numeric} body diameter (m); can estimate as diameter = 0.0016*log(mass) + 0.0061 for mass(g).
#'
#' @param T_a \code{numeric} ambient temperature (C).
#'
#' @param elev \code{numeric} elevation (m).
#'
#' @param e_s \code{numeric} saturation vapor pressure (kPa).
#'
#' @param e_a \code{numeric} actual vapor pressure (kPa).
#'
#' @param Qabs \code{numeric} Solar and thermal radiation absorbed (W).
#'
#' @param epsilon \code{numeric} emissivity of salamander skin (proportion).
#'
#' @return \code{numeric} humid operative temperature (C).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @author Eric Riddell
#'
#' @examples
#'   Tb_salamander_humid(r_i     = 4,
#'                       r_b     = 1,
#'                       D       = 0.01,
#'                       T_a     = 20,
#'                       elev    = 500,
#'                       e_a     = 2.0,
#'                       e_s     = 2.5,
#'                       Qabs    = 400,
#'                       epsilon = 0.96)
#'
Tb_salamander_humid <- function (r_i,
                                 r_b,
                                 D,
                                 T_a,
                                 elev,
                                 e_a,
                                 e_s,
                                 Qabs,
                                 epsilon = 0.96) {

  stopifnot(r_i     >  0, 
            r_b     >  0, 
            D       >  0, 
            elev    >  0, 
            e_s     >  0, 
            e_a     >  0, 
            epsilon >  0.5, 
            epsilon <= 1)

  if(e_s < e_a) {

    stop("Actual vapor pressure, e_a, must be lower that saturation vapor pressure, e_s.")

  }

  # Stefan-Boltzmann constant
  sigma <- stefan_boltzmann_constant()

  vpd <- e_s - e_a # vapor pressure deficit

  #radiative conductance function, Campbell and Norman 1998
  radiative_conductance <- (4 * (5.670373 * 10^-8) * (T_a + 273.15)^3) / 29.3

  gamma <- 0.000666
  gvs <- 1/((r_i*100.0)/41.4)
  gva <- 1/(((r_b*0.93)*100)/41.4)
  gHa <- 1/((r_b*100)/41.4)
  gamma_naut <- gamma * (1 / gvs + 1 / gva) / (1 / gHa + 1 / radiative_conductance)
  s <- ((((17.502 * 240.97)) * 0.611 * exp((17.502 * T_a) / (T_a + 240.97))) / (240.97 + T_a)^2) / (101.3 * exp(-elev / 8200))

  T_a+(gamma_naut/(gamma_naut+s))*(((Qabs - (epsilon*(sigma)*((T_a+273.15)^4)))/(29.3*(radiative_conductance+gHa)))-(vpd/(gamma_naut*(101.3*exp(-elev/8200)))))
}


#' @title Absorbed Thermal Radiation
#'
#' @description The function estimates longwave (thermal) radiation (W) absorbed from the sky and the ground \insertCite{Campbell1998,Riddell2018}{TrenchR}.
#'
#' @param T_a \code{numeric} air temperature (C).
#'
#' @param T_g \code{numeric} ground temperature (C).
#'
#' @param epsilon_ground \code{numeric} emissivity (proportion) for more soil types \insertCite{Campbell1998}{TrenchR}.
#'
#' @param a_longwave \code{numeric} absorptance (proportion) of organism to longwave radiation \insertCite{Bartlett1967,Buckley2008}{TrenchR}.
#'
#' @return \code{numeric} thermal radiation absorbed (W).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @author Eric Riddell
#'
#' @examples
#'   Qthermal_radiation_absorbed(T_a            = 20,
#'                               T_g            = 25,
#'                               epsilon_ground = 0.97,
#'                               a_longwave     = 0.965)
#'
Qthermal_radiation_absorbed <- function (T_a,
                                         T_g,
                                         epsilon_ground = 0.97,
                                         a_longwave = 0.965) {

  stopifnot(epsilon_ground >= 0, 
            epsilon_ground <= 1, 
            a_longwave     >= 0, 
            a_longwave     <= 1)

  # Stefan-Boltzmann constant
  sigma <- stefan_boltzmann_constant()

  # longwave radiation from sky function, Campbell and Norman 1998
  Slongwave_sky <- 53.1 * 10^-14 * (T_a + 273.15)^6.

  # longwave radiation from ground function, Campbell and Norman 1998
  Slongwave_ground <- epsilon_ground * sigma * (T_g + 273.15)^4.

  # radiation absorbed function, adapted from Campbell and Norman 1998
  0.5 * a_longwave * (Slongwave_sky + Slongwave_ground)

}


#' @title Approximate Soil Temperature
#'
#' @description The function estimates soil temperature (C) at a given depth and hour by approximating diurnal variation as sinusoidal. The function is adapted from \insertCite{Campbell1998;textual}{TrenchR} and described in \insertCite{Riddell2018;textual}{TrenchR}.
#'
#' @param T_g_max \code{numeric} daily maximum soil surface temperature (C).
#'
#' @param T_g_min \code{numeric} daily minimum soil surface temperature (C).
#'
#' @param hour \code{numeric} hour of the day.
#'
#' @param depth \code{numeric} depth (cm).
#'
#' @return \code{numeric} soil temperature (C).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @author Eric Riddell
#'
#' @examples
#'   Tsoil(T_g_max = 30,
#'         T_g_min = 15,
#'         hour    = 12,
#'         depth   = 5)
#'
Tsoil <- function (T_g_max,
                   T_g_min,
                   hour,
                   depth) {

  stopifnot(T_g_max > T_g_min, 
            hour   >= 0,
            hour   <= 24,
            depth  >= 0)

  offset <- ifelse(hour %in% c(0, 1, 2, 3), -13, 11)

  ((T_g_max + T_g_min) / 2.0) + ((T_g_max - T_g_min) / 2.0) * (2.71^(-depth / 5.238)) * sin((3.14 / 12.) * (hour - offset) - depth / 5.238)

}


#' @title Nusselt Number
#'
#' @description The function estimates the Nusselt Number, which describes dimensionless conductance \insertCite{Gates1980}{TrenchR}.
#'
#' @param H_L \code{numeric} convective heat transfer coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}).
#'
#' @param D \code{numeric} characteristic dimension (e.g., body diameter) (m).
#'
#' @param K \code{numeric} thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}).
#'
#' @return \code{numeric} Nusselt number.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Nusselt_number(H_L = 20,
#'                  D   = 0.01,
#'                  K   = 0.5)
#'
Nusselt_number <- function(H_L,
                           D,
                           K) {

  stopifnot(H_L > 0, 
            D   > 0, 
            K   > 0)

  H_L * D / K #eq 9.24

}


#' @title Prandtl Number
#'
#' @description The function estimates the Prandtl Number, which describes the ratio of kinematic viscosity to thermal diffusivity \insertCite{Gates1980}{TrenchR}.
#'
#' @param c_p \code{numeric} specific heat at constant pressure (\ifelse{html}{\out{J mol<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J mol^-1 K^-1}{ASCII}}).
#'
#' @param mu \code{numeric} dynamic viscosity (\ifelse{html}{\out{mol s<sup>-1</sup> m<sup>-1</sup>}}{\eqn{mol s^-1 m^-1}{ASCII}}).
#'
#' @param K \code{numeric} thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}).
#'
#' @return \code{numeric} Prandtl number.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Prandtl_number(c_p = 29.3,
#'                  mu  = 0.00001,
#'                  K   = 0.5)
#'
Prandtl_number <- function (c_p,
                            mu,
                            K) {

  stopifnot(c_p > 0, 
            mu  > 0, 
            K   > 0)

  c_p * mu / K #eq 9.26

}


#' @title Reynolds Number
#'
#' @description The function estimates the Reynolds Number, which describes the dynamic properties of the fluid surrounding the animal as the ratio of internal viscous forces \insertCite{Gates1980}{TrenchR}.
#'
#' @param D \code{numeric} characteristic dimension (e.g., body diameter) (m)
#'
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param nu \code{numeric} the kinematic viscosity, ratio of dynamic viscosity to density of the fluid (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}); can calculate from \code{\link{DRYAIR}} or \code{\link{WETAIR}}.
#'
#' @return \code{numeric} Reynolds number.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Reynolds_number(u  = 1,
#'                   D  = 0.001,
#'                   nu = 1.2)
#'
Reynolds_number <- function(u,
                            D,
                            nu) {

  stopifnot(D  >= 0, 
            u  >= 0, 
            nu >= 0)

  u * D / nu #eq 9.25

}


#' @title Grashof Number
#'
#' @description The function estimates the Grashof Number, which describes the ability of a parcel of fluid warmer or colder than the surrounding fluid to rise against or fall with the attractive force of gravity. The Grashof Number is estimated as the ratio of a buoyant force times an inertial force to the square of a viscous force \insertCite{Campbell1998}{TrenchR}.
#'
#' @param T_a \code{numeric} Air temperature (C).
#'
#' @param T_g \code{numeric} ground (surface) temperature (C).
#'
#' @param D \code{numeric} characteristic dimension (e.g., body diameter) (meters).
#'
#' @param nu \code{numeric} the kinematic viscosity, ratio of dynamic viscosity to density of the fluid (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}); can calculate from \code{\link{DRYAIR}} or \code{\link{WETAIR}}.
#'
#' @return \code{numeric} Grashof number.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Grashof_number(T_a = 30,
#'                  T_g = 35,
#'                  D  = 0.001,
#'                  nu = 1.2)
#'
Grashof_number <- function (T_a,
                            T_g,
                            D,
                            nu) {

  stopifnot(D  >= 0, 
            nu >= 0)

  # constant
  gravity <- 9.8 # meters per second

  gravity * D^3 * abs(T_g - T_a) / (T_a * nu^2)

}


#' @title Grashof Number as in Gates (1980)
#'
#' @description The function estimates the Grashof Number, which describes the ability of a parcel of fluid warmer or colder than the surrounding fluid to rise against or fall with the attractive force of gravity \insertCite{Gates1980}{TrenchR}. The Grashof Number is estimated as the ratio of a buoyant force times an inertial force to the square of a viscous force.
#'
#' @param T_a \code{numeric} Air temperature (C).
#'
#' @param T_g \code{numeric} Ground (surface) temperature (C).
#'
#' @param beta \code{numeric} coefficient of volumetric thermal expansion, \code{beta} = 3.67 x \ifelse{html}{\out{10<sup>-3</sup> C<sup>-1</sup>}}{\eqn{10^-3 C^-1}{ASCII}}  in air and 41.9 x \ifelse{html}{\out{10<sup>-4</sup> C<sup>-1</sup>}}{\eqn{10^-4 C^-1}{ASCII}} in water.
#'
#' @param D \code{numeric} is characteristic dimension (e.g., body diameter) (m)
#'
#' @param nu \code{numeric} is the kinematic viscosity, the ratio of dynamic viscosity to density of the fluid (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}); can calculate from \code{\link{DRYAIR}} or \code{\link{WETAIR}}.
#'
#' @return \code{numeric} Grashof number.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Grashof_number_Gates(T_a   = 30,
#'                        T_g   = 35,
#'                        beta = 0.00367,
#'                        D    = 0.001,
#'                        nu   = 1.2)
#'
Grashof_number_Gates <- function (T_a,
                                  T_g,
                                  beta,
                                  D,
                                  nu) {

  stopifnot(beta >  0, 
            D    >= 0, 
            nu   >  0)

  gravity <- 9.8 

  gravity * beta * D^3 * abs(T_g - T_a) / nu^2

}


#' @title Nusselt Number from the Reynolds Number
#'
#' @description The function estimates the Nusselt number from the Reynolds number for various taxa using \insertCite{Mitchell1976;textual}{TrenchR} (Table 1: Convective Heat Transfer Relations for Animal Shapes).
#'
#' @param Re \code{numeric} Reynolds Number (dimensionless).
#'
#' @param taxon \code{character} which class of organism. Current choices: \code{"sphere"}, \code{"cylinder"}, \code{"frog"}, \code{"lizard_traverse_to_air_flow"}, \code{"lizard_parallel_to_air_flow"}, \code{"lizard_surface"}, \code{"lizard_elevated"}, \code{"flyinginsect"}, \code{"spider"}.
#'
#' @return \code{numeric} Nusselt number (dimensionless).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Nusselt_from_Reynolds(Re    = 5,
#'                         taxon = "cylinder")
#'
Nusselt_from_Reynolds <- function (Re,
                                   taxon = "cylinder") {

  taxa <- c("sphere", "cylinder", "frog", "lizard_traverse_to_air_flow", "lizard_parallel_to_air_flow", "lizard_surface", "lizard_elevated", "flyinginsect", "spider")
  stopifnot(length(taxon) == 1,
            taxon %in% taxa)

  # Dimensionless constant (Cl)
  Cls <- c(0.37, 0.615, 0.258, 0.35, 0.1, 1.36, 1.91, 0.0749, 0.47)
  ns <- c(0.6, 0.466, 0.667, 0.6, 0.74, 0.39, 0.45, 0.78, 0.5)

  # find index
  ind <- match(taxon, taxa)

  Cls[ind] * Re^(ns[ind])

}


#' @title Nusselt Number from the Grashof Number
#'
#' @description The function estimates the Nusselt number from the Grashof Number \insertCite{Gates1980}{TrenchR}.
#'
#' @param Gr \code{numeric} Grashof Number (dimensionless).
#'
#' @return \code{numeric} Nusselt number (dimensionless).
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Nusselt_from_Grashof(Gr = 5)
#'
Nusselt_from_Grashof <- function (Gr) {

  0.48 * Gr^0.25

}


#' @title Determine if Convection is Free or Forced
#'
#' @description The function compares the Grashof and Reynolds numbers to determine whether convection is free or forced \insertCite{Gates1980}{TrenchR}.
#'
#' @param Gr \code{numeric} Grashof Number (dimensionless).
#'
#' @param Re \code{numeric} Reynolds Number (dimensionless).
#'
#' @return \code{character} \code{"free"}, \code{"forced"}, or \code{"intermediate"}.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   free_or_forced_convection(Gr = 100,
#'                             Re = 5)
#'
free_or_forced_convection <- function (Gr,
                                       Re) {

  conv <- "intermediate condition, mixed convection based on Nusselt numbers is appropriate"

  if(Gr < 0.1 * Re^2) {

    conv <- "forced convection" #P284

  }

  if(Gr > 16 * Re^2) {

    conv <- "free convection"

  }

  conv

}

context("energybalance_functions")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("Qconduction_animal function works as expected", {
  expect_equal(Qconduction_animal(T_g= 293,T_b=303,d=10^-6,K=0.5,A=10^-3, proportion=0.2), 1000)
  expect_error(Qconduction_animal(T_g= 293,T_b=303,d=10^-6,K=0.5,A=10^-3, proportion=-0.2), "proportion >= 0 is not TRUE")
})

test_that("Qconduction_substrate function works as expected", {
  expect_equal(Qconduction_substrate(T_g= 293, T_b=303, D=0.01, K_g=0.3, A=10^-2, proportion=0.2),1.2)
})

test_that("Qconvection function works as expected", {
  expect_similar(Qconvection(T_a= 293, T_b= 303, H_L=10.45, A=0.0025, proportion=0.85),0.2731369)
})

test_that("heat_transfer_coefficient function works as expected", {
  expect_similar(heat_transfer_coefficient(u=0.5,D=0.05,K= 25.7 * 10^(-3), nu= 15.3 * 10^(-6), "cylinder"),9.936011)
  expect_similar(heat_transfer_coefficient(u=0.5,D=0.05,K= 25.7 * 10^(-3), nu= 15.3 * 10^(-6), "lizard_surface"),12.52186)
})

test_that("heat_transfer_coefficient_approximation function works as expected", {
  expect_similar(heat_transfer_coefficient_approximation(u=3,D=0.05,K= 25.7 * 10^(-3),nu= 15.3 * 10^(-6), "sphere"),43.37924)
  expect_similar(heat_transfer_coefficient_approximation(u=3,D=0.05,K= 25.7 * 10^(-3),nu= 15.3 * 10^(-6), "frog"),46.28954)
})

test_that("heat_transfer_coefficient_simple function works as expected", {
  expect_similar(heat_transfer_coefficient_simple(u=0.5,D=0.05, type = "Spotila"), 14.80412)
  expect_similar(heat_transfer_coefficient_simple(u=0.5,D=0.05, type = "Gates"), 11.03635)
})

test_that("Qradiation_absorbed function works as expected", {
  expect_equal(Qradiation_absorbed(a=0.9, A=1, psa_dir=0.4, psa_dif=0.5, psa_ref=0.5, S_dir=1000, S_dif=200, rho=0.5), 675)
})

test_that("Qemitted_thermal_radiation function works as expected", {
  expect_similar(Qemitted_thermal_radiation(epsilon=0.96, A=1, psa_dir=0.4, psa_ref=0.6, T_b=303, T_g=293, T_a=298, enclosed=FALSE),78.35451)
  expect_similar(Qemitted_thermal_radiation(epsilon=0.96, A=1, psa_dir=0.4, psa_ref=0.6, T_b=303, T_g=293, T_a=298, enclosed=TRUE),17.7349)
})

test_that("Qevaporation function works as expected", {
  expect_similar(Qevaporation(A=0.1, T_b=293, taxon="amphibian", e_s=0.003, e_a=0.002, hp=0.5, H=20, r_i=50),4.612476)
  expect_similar(Qevaporation(A=0.1, T_b=293, taxon="amphibian_wetskin", e_s=0.003, e_a=0.002, hp=0.5, H=20, r_i=50),8.74552)
  expect_similar(Qevaporation(A=0.1, T_b=293, taxon="lizard"),1.043334)
  expect_similar(Qevaporation(A=0.1, T_b=290, taxon="lizard"),0.1005)
  expect_similar(Qevaporation(A=0.1, T_b=310, taxon="lizard"), 0.04695595)
})

test_that("Qevaporation errors work as expected", {
  expect_error(Qevaporation(A=-0.1, T_b=293, taxon="amphibian", e_s=0.003, e_a=0.002, h=0.5, H=20, r_i=50))
  expect_error(Qevaporation(A=0.1, T_b=410, taxon="amphibian", e_s=0.003, e_a=0.002, h=0.5, H=20, r_i=50))
  expect_error(Qevaporation(A=0.1, T_b=293, taxon="frog", e_s=0.003, e_a=0.002, h=0.5, H=20, r_i=50))
  expect_error(Qevaporation(A=0.1, T_b=293, taxon=c("frog","amphibian"), e_s=0.003, e_a=0.002, h=0.5, H=20, r_i=50))
  expect_error(Qevaporation(A=0.1, T_b=293, taxon="amphibian", e_s=-0.003, e_a=0.002, h=0.5, H=20, r_i=50))
  expect_error(Qevaporation(A=0.1, T_b=293, taxon="amphibian", e_s=0.003, e_a=-0.002, h=0.5, H=20, r_i=50))   
  expect_error(Qevaporation(A=0.1, T_b=293, taxon="amphibian", e_s=0.003, e_a=0.002, h=50, H=20, r_i=50)) 
  expect_error(Qevaporation(A=0.1, T_b=293, taxon="amphibian", e_s=0.003, e_a=0.002, h=0.5, H=-20, r_i=50)) 
  expect_error(Qevaporation(A=0.1, T_b=293, taxon="amphibian", e_s=0.003, e_a=0.002, h=0.5, H=20, r_i=-50)) 
})

test_that("saturation_water_vapor_pressure function works as expected", {
  expect_similar(saturation_water_vapor_pressure(T_a=20), 2216.563)
})

test_that("external_resistance_to_water_vapor_transfer function works as expected", {
  expect_equal(external_resistance_to_water_vapor_transfer(H=20), 558)
})

test_that("Qmetabolism_from_mass function expect_similar function works as expected", {
  expect_similar(Qmetabolism_from_mass(m=12,"reptile"), 0.02066024)
  expect_similar(Qmetabolism_from_mass(60000, taxon = "mammal"), 179.3483)
  expect_similar(Qmetabolism_from_mass(200, taxon="bird"), 4.484128)
})

test_that("Qmetabolism_from_mass_temp function works as expected", {
  expect_similar(Qmetabolism_from_mass_temp(m=100, T_b=30, "reptile"), 0.06282736)
  expect_similar(Qmetabolism_from_mass_temp(m=1, T_b=17, "invertebrate"), 0.0003268306)
  expect_similar(Qmetabolism_from_mass_temp(m=50, T_b=15, "amphibian"), 0.0113297)
  expect_similar(Qmetabolism_from_mass_temp(m=50, T_b=15, "bird"),0.03809967)

})

test_that("actual_vapor_pressure function works as expected", {
  expect_similar(actual_vapor_pressure(T_dewpoint=293), 2.370917)
  expect_similar(actual_vapor_pressure(299), 3.437295)
})

test_that("saturation_vapor_pressure function works as expected", {
  expect_similar(saturation_vapor_pressure(T_a=293), 2.341779)
})

test_that("expect_similar(boundary_layer_resistance function works as expected", {
  expect_similar(boundary_layer_resistance(T_a=293, e_s=2.5, e_a=2.4, elev=500, D=0.007, u=2), 0.3649484)
  expect_similar(boundary_layer_resistance(T_a=293, e_s=2.5, e_a=2.4, elev=500, D=0.007),4.302272)
  expect_error(boundary_layer_resistance(T_a=293, e_s=2.3, e_a=2.4, elev=500, D=0.007))
})

test_that("Tb_salamander_humid function works as expected", {
  expect_similar(Tb_salamander_humid(r_i = 4, r_b = 1, D = 0.01, T_a = 20, elev = 500, e_a = 2.0, e_s = 2.5, Qabs = 400,epsilon=0.96), 17.90782)
  expect_error(Tb_salamander_humid(r_i = 4, r_b = 1, D = 0.01, T_a = 20, elev = 500, e_a = 2.8, e_s = 2.5, Qabs = 400,epsilon=0.96))

})

test_that("Qthermal_radiation_absorbed function works as expected", {
  expect_similar(Qthermal_radiation_absorbed(T_a=20, T_g=25, epsilon_ground=0.97, a_longwave=0.965), 372.4115)
})

test_that("Tsoil function works as expected", {
  expect_similar(Tsoil(T_g_max=30, T_g_min=15, hour=12, depth=5), 20.65027)
})

test_that("Nusselt_number function works as expected", {
  expect_equal(Nusselt_number(H_L=20, D=0.01, K=0.5), 0.4)
})

test_that("Prandtl_number function works as expected", {
  expect_similar(Prandtl_number(c_p=29.3, mu=0.00001, K=0.5), 0.000586)
})

test_that("Reynolds_number function works as expected", {
  expect_similar(Reynolds_number(u=1, D=0.001, nu=1.2), 0.0008333333)
})

test_that("Grashof_number function works as expected", {
  expect_similar(Grashof_number(T_a=30, T_g=35, D=0.001, nu=1.2), 1.134259e-09)
})

test_that("Grashof_number_Gates function works as expected", {
  expect_similar(Grashof_number_Gates(T_a=30, T_g=35, beta=0.00367, D=0.001, nu=1.2), 1.248819e-10)
})

test_that("Nusselt_from_Reynolds function works as expected", {
  expect_similar(Nusselt_from_Reynolds(Re=5, taxon="cylinder"), 1.301952)
  expect_similar(Nusselt_from_Reynolds(Re=5, taxon="frog"), 0.7548014)
  expect_similar(Nusselt_from_Reynolds(Re=5, taxon="lizard_parallel_to_air_flow"), 0.3290317)
})

test_that("Nusselt_from_Grashof function works as expected", {
  expect_similar(Nusselt_from_Grashof(5), 0.7177674)
})

test_that("free_or_forced_convection function works as expected", {
  expect_identical(free_or_forced_convection(Gr=100, Re=5), "intermediate condition, mixed convection based on Nusselt numbers is appropriate")
  expect_identical(free_or_forced_convection(1,10), "forced convection")
  expect_identical(free_or_forced_convection(1000,1), "free convection")
})

#' @title Calendar Day from Date
#' 
#' @description The function converts a date (day, month, year) to Calendar Day (day of year).
#'
#' @param day \code{character} numerical date in standard format (e.g. \code{"2017-01-02"}, \code{"01-02"}, \code{"01/02/2017"} etc). 
#'
#' @param format \code{character} date format following \code{\link[base]{as.POSIXlt}} conventions. Default value = \code{"\%Y-\%m-\%d"}. 
#'
#' @return \code{numeric} Julian day number, 1-366 (e.g. 1 for January 1st).
#' 
#' @family utility functions  
#'  
#' @export
#' 
#' @examples
#'   day_of_year(day    = "2017-04-22", 
#'               format = "%Y-%m-%d")
#'   day_of_year(day    = "2017-04-22")
#'   day_of_year(day    = "04/22/2017", 
#'               format = "%m/%d/%Y")
#'
day_of_year <- function (day, 
                         format = "%Y-%m-%d") {
  
  day <- as.POSIXlt(day, format = format)
  as.numeric(strftime(day, format = "%j"))
  
}

#' @title Solar Declination in Radians
#' 
#' @description The function calculates solar declination, which is the angular distance of the sun north or south of the earth’s equator, based on the day of year \insertCite{Campbell1998}{TrenchR}.
#' 
#' @param doy \code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.
#' 
#' @return \code{numeric} declination angle (radians).
#' 
#' @family utility functions
#'
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' 
#' @examples
#'   dec_angle(doy = 112)
#'   dec_angle(doy = 360)
#' 
dec_angle <- function (doy) {
  
  stopifnot(doy > 0, 
            doy < 367)
  
  RevAng <- 0.21631 + 2 * atan(0.967 * tan (0.0086 * (-186 + doy))) 
  asin(0.39795 * cos (RevAng)) 
  
}

#' @title Day Length
#' 
#' @description The function calculates daylength in hours as a function of latitude and day of year. Uses the CMB model \insertCite{Campbell1998}{TrenchR}.
#'
#' @param lat \code{numeric} latitude (decimal degrees).
#' 
#' @param doy \code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.
#' 
#' @return \code{numeric} day length (hours).
#'
#' @references
#'   \insertAllCited{}
#' 
#' @family utility functions
#'
#' @export
#'
#' @examples
#'   daylength(lat = 47.61, 
#'             doy = 112)
#'
daylength <- function(lat,
                      doy){
  
  stopifnot(lat >= -90, 
            lat <= 90, 
            doy >  0, 
            doy <  367)
  
  lat_rad <- degrees_to_radians(lat)
  DecAng <- dec_angle(doy)
  subset <- (sin (6 * pi / 180) + sin (lat_rad) * sin (DecAng)) / (cos (lat_rad) * cos (DecAng))
  subset[which(subset>1)] <- 1
  subset[which(subset< -1)] <- -1
  24 - (24 / pi) * acos(subset) 
  
}

#' @title Time of Solar Noon
#' 
#' @description The function calculates the time of solar noon in hours as a function of the day of year and longitude \insertCite{Campbell1998}{TrenchR}.
#'
#' @param lon \code{numeric} longitude (decimal degrees).
#'
#' @param doy \code{numeric} day of year (1-366). This can be obtained from a standard date via \code{\link{day_of_year}}.
#'
#' @param offset \code{numeric} number of hours to add to UTC (Coordinated Universal Time) to get local time (improves accuracy but not always necessary). Defaults to NA.
#'
#' @return \code{numeric} time of solar noon (hours).
#'
#' @references
#'   \insertAllCited{}
#'
#' @family utility functions
#'
#' @export
#'
#' @examples
#'   solar_noon(lon = -122.335,
#'              doy = 112)
#'
solar_noon <- function (lon,
                        doy, 
                        offset = NA){
  
  stopifnot(lon >= -180, 
            lon <= 180, 
            doy >  0, 
            doy <  367)
  
  # Calculates the time of solar noon for each day using longitude correction (LC), equation of time (ET), and a conversion (f)
  f <- (279.575 + 0.9856 * doy)  # f in degrees as a function of day of year, p.169 Campbell & Norman 1998
  f[f > 360] = f[f > 360] - 360  # ensure 0 to 360 degrees
  f <- f * pi / 180              # convert f in degrees to radians
  
  # (11.4) Equation of time: ET is a 15-20 minute correction which depends on calendar day
  ET <- (-104.7 * sin(f) + 596.2 * sin(2*f) + 4.3 * sin(3*f) - 12.7 * sin(4*f) - 429.3 * cos(f) - 2.0 * cos(2*f) + 19.3 * cos(3*f)) / 3600
  
  lon[lon < 0] <- 360 + lon[lon < 0] # convert to 0 to 360
  LC <- 1 / 15 * (lon %% 15) # longitude correction, 1/15h for each degree of standard meridian
  LC[LC > 0.5] <- LC[LC > 0.5] - 1
  t_0 <- 12 - LC - ET # solar noon, p168, 11.3, Campbell &Norman 1998 
  
  # Check if offset is as expected. (Is the timezone of the location the same as that of the meridian 
  # that's within 7.5 degrees from that location?)
  lon[lon > 180] <- lon[lon > 180] - 360
  if (!is.na(offset)) {
    
    offset_theory <- as.integer(lon / 15) + lon / abs(lon) * as.integer(abs(lon) %% 15 / 7.5)
    t_0 = t_0 - offset_theory + offset
    
  }
  
  t_0
}


#' @title Zenith Angle
#' 
#' @description The function calculates the zenith angle, the location of the sun as an angle (in degrees) measured from vertical \insertCite{Campbell1998}{TrenchR}.
#' 
#' @param doy \code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.
#' 
#' @param lat \code{numeric} latitude (decimal degrees).
#' 
#' @param lon \code{numeric} longitude (decimal degrees).
#' 
#' @param hour \code{numeric} hour of the day (0-24).
#' 
#' @param offset \code{numeric} the number of hours to add to UTC (Coordinated Universal Time) to get local time (improves accuracy but not always necessary). Optional. Defaults to NA.
#' 
#' @return \code{numeric} zenith angle (degrees)
#' 
#' @family utility functions
#' 
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   zenith_angle(doy  = 112, 
#'                lat  = 47.61, 
#'                lon  = -122.33, 
#'                hour = 12)
#' 
zenith_angle <- function (doy, 
                          lat, 
                          lon, 
                          hour, 
                          offset = NA) {

  stopifnot(doy  >  0, 
            doy  <  367, 
            lat  >= -90, 
            lat  <= 90, 
            lon  >= -180, 
            lon  <= 180, 
            hour >= 0, 
            hour <= 24)
  
  lat <- lat * pi / 180 # to radians
    
  RevAng <- 0.21631 + 2 * atan(0.967 * tan(0.0086 * (-186 + doy))) # Revolution angle in radians
  DecAng <- asin(0.39795 * cos(RevAng))                            # Declination angle in radians           
    
  f <- 279.575 + 0.9856 * doy # f in degrees as a function of day of year, p.169 Campbell & Norman 1998
  f <- f * pi / 180 # convert f in degrees to radians
  
  # (11.4) Equation of time: ET is a 15-20 minute correction which depends on calendar day
  ET <- (-104.7 * sin(f) + 596.2 * sin(2 * f) + 4.3 * sin(3 * f) - 12.7 * sin(4 * f) - 429.3 * cos(f) - 2.0 * cos(2 * f) + 19.3 * cos(3 * f)) / 3600   
  
  lon[lon < 0] <- 360 + lon[lon < 0] #convert to 0 to 360
  LC <- 1 / 15 * (lon %% 15) # longitude correction, 1/15h for each degree of standard meridian
  LC[LC > 0.5] <- LC[LC > 0.5] - 1
  t_0 <- 12 - LC - ET # solar noon
  
  # Check if offset is as expected. (Is the timezone of the location the same as that of the meridian 
  # that's within 7.5 degrees from that location?)
  lon[lon > 180] <- lon[lon > 180] - 360
  if (!is.na(offset)) {
    offset_theory <- as.integer(lon / 15) + lon / abs(lon) * as.integer(abs(lon) %% 15 / 7.5)
    t_0 <- t_0 - offset_theory + offset
  }
  
  cos.zenith <- sin(DecAng) * sin(lat) + cos(DecAng) * cos(lat) * cos(pi / 12 * (hour - t_0)) # cos of zenith angle in radians
  zenith <- acos(cos.zenith) * 180 / pi # zenith angle in degrees
  zenith[zenith > 90] <- 90 # if measured from the vertical psi can't be greater than pi/2 (90 degrees)

  zenith
}

#' @title Azimuth Angle
#'
#' @description The function calculates the azimuth angle, the angle (degrees) from which the sunlight is coming measured from true north or south measured in the horizontal plane. The azimuth angle is measured with respect to due south, increasing in the counter clockwise direction so 90 degrees is east \insertCite{Campbell1998}{TrenchR}.
#' 
#' @param doy \code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.
#' 
#' @param lat \code{numeric} latitude (decimal degrees).
#' 
#' @param lon \code{numeric} longitude (decimal degrees).
#' 
#' @param hour \code{numeric} hour of the day.
#' 
#' @param offset \code{numeric} number of hours to add to UTC (Coordinated Universal Time) to get local time (to improve accuracy but not always necessary). Optional. Defaults to NA.
#' 
#' @return \code{numeric} azimuth angle (degrees).
#'
#' @references
#'   \insertAllCited{}
#'
#' @family utility functions
#' 
#' @export
#' 
#' @examples
#'   azimuth_angle(doy    = 112, 
#'                 lat    = 47.61, 
#'                 lon    = -122.33, 
#'                 hour   = 12, 
#'                 offset = -8)
#' 
azimuth_angle <- function (doy,
                           lat, 
                           lon, 
                           hour, 
                           offset = NA) {
  
  stopifnot(doy  >  0, 
            doy  <  367, 
            lat  >= -90, 
            lat  <= 90, 
            lon  >= -180, 
            lon  <= 180, 
            hour >= 0, 
            hour <= 24)
  
  lat <- lat * pi / 180 #to radians
  
  RevAng <- 0.21631 + 2 * atan(0.967 * tan(0.0086 * (-186 + doy))) # Revolution angle in radians
  DecAng <- asin(0.39795 * cos(RevAng))                            # Declination angle in radians           

  f <- 279.575 + 0.9856 * doy  # f in degrees as a function of day of year, p.169 Campbell & Norman 2000
  f <- f * pi / 180 # convert f in degrees to radians
  
  # (11.4) Equation of time: ET is a 15-20 minute correction which depends on calendar day
  ET <- (-104.7 * sin(f) + 596.2 * sin(2 * f) + 4.3 * sin(3 * f) - 12.7 * sin(4 * f) - 429.3 * cos(f) - 2.0 * cos(2 * f) + 19.3 * cos(3 * f)) / 3600  
  lon[lon < 0] <- 360 + lon[lon < 0] # convert to 0 to 360

  # Set up two Booleans indicating whether we need to apply a correction on azimuth angle at the end
  azi_corr1 <- FALSE
  azi_corr2 <- TRUE
  
  LC <- 1 / 15 * (lon %% 15) # longitude correction, 1/15h for each degree of standard meridian

  if (LC > 0.5) {
    
    LC = LC - 1
  
  } else {
    
    azi_corr1 = TRUE
    
  }
  
  t_0 <- 12 - LC - ET # solar noon
  
  # Check if offset is as expected. (Is the timezone of the location the same as that of the meridian 
  # that's within 7.5 degrees from that location?)
  lon[lon > 180] <- lon[lon > 180] - 360
  if (!is.na(offset)) {
    
    offset_theory <- as.integer(lon / 15) + lon / abs(lon) * as.integer(abs(lon) %% 15 / 7.5)
    
    if (offset_theory != offset) {
      
      t_0 <- t_0 - offset_theory + offset
      azi_corr2 <- FALSE
      
    }
    
  }
  
  cos.zenith <- sin(DecAng) * sin(lat) + cos(DecAng) * cos(lat) * cos(pi / 12 * (hour - t_0)) #cos of zenith angle in radians
  zenith <- acos(cos.zenith) # zenith angle in radians

  if (zenith > pi / 2){
    
    zenith <- pi / 2 # if measured from the vertical psi can't be greater than pi/2 (90 degrees)
    
  } 
  
  cos.azimuth <- -(sin(DecAng) - cos(zenith) * sin(lat)) / (cos(lat) * sin(zenith))
  azimuth <- acos(cos.azimuth) * 180 / pi # azimuth angle in degrees
  
  if (azi_corr1 && azi_corr2) {
    
    azimuth = 360 - azimuth
    
  }
  
  azimuth
}

#' @title Air Pressure from Elevation 
#'
#' @description The function estimates air pressure (kPa) as a function of elevation \insertCite{engingeeringtoolbox}{TrenchR}.
#' 
#' @param elev \code{numeric} elevation (meters).
#' 
#' @family utility functions
#' 
#' @return \code{numeric} air pressure (kPa).
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @examples
#'   airpressure_from_elev(elev = 1500)
#'
airpressure_from_elev <- function (elev) {  
 
  stopifnot(elev >= 0)
  
  101.3 * exp (-elev / 8200)  

}

#' @title Convert Among Temperature Scales 
#' 
#' @description The function converts temperatures among Celsius, Fahrenheit, and Kelvin \insertCite{Blischak}{TrenchR}.
#' 
#' @param temperature \code{numeric} temperature (Celsius, Fahrenheit, or Kelvin).
#' 
#' @family utility functions
#' 
#' @return \code{numeric} temperature (Celsius, Fahrenheit, or Kelvin).
#' 
#' @export
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @name temperature conversions
#'
#' @examples
#'   kelvin_to_celsius(temperature = 270)
#'   fahrenheit_to_kelvin(temperature = 85)
#'   fahrenheit_to_celsius(temperature = 85)
#'   celsius_to_kelvin(temperature = -10)
#' 
fahrenheit_to_kelvin <- function (temperature) {
  
  ((temperature - 32) * (5 / 9)) + 273.15
  
}

#' @name temperature conversions
#' 
#' @export
#' 
kelvin_to_celsius <- function (temperature) {
  
  temperature - 273.15
  
}

#' @name temperature conversions
#' 
#' @export
#' 
celsius_to_kelvin <- function (temperature) {
  
  temperature + 273.15
  
}

#' @name temperature conversions
#' 
#' @export
#' 
fahrenheit_to_celsius <- function (temperature) {
  
  temp_k <- fahrenheit_to_kelvin(temperature)
  kelvin_to_celsius(temp_k)
  
}



#' @title Convert Angles Between Radians and Degrees
#' 
#' @description The function converts angles in radians to degrees or degrees to radians.
#' 
#' @param rad \code{numeric} angle (radians).
#' 
#' @param deg \code{numeric} angle (degrees).
#' 
#' @return \code{numeric} angle (degrees or radians).
#' 
#' @name angle conversions
#'
#' @export
#' 
#' @examples
#'   radians_to_degrees(0.831)
#'   degrees_to_radians(47.608)
#' 
radians_to_degrees <- function (rad) {
  
  rad * 180 / pi
  
}

#' @name angle conversions
#' 
#' @export
#' 
degrees_to_radians <- function (deg) {
  
  deg * pi / 180
  
}

context("Zenith Angle")
test_that("zenith_angle function works as expected", {
  # Vancouver - Approx Solar Noon
  june10 = day_of_year("2017-06-10", format= "%Y-%m-%d")
  za_van = zenith_angle(june10, 49.267, -123.117, 12)
  
  # Expect cos(za) to be 0.8966
  expect_gt(cos(za_van*pi/180), 0.895)
  expect_lt(cos(za_van*pi/180), 0.897)

  # Seattle - Approx Solar Noon
  za_sea = zenith_angle(june10, 47.606, -122.331, 12)
  
  # Expect cos(za) to be 0.9093
  expect_gt(cos(za_sea*pi/180), 0.90)
  expect_lt(cos(za_sea*pi/180), 0.91)
  
  # Madrid - Approx Solar Noon
  za_mad = zenith_angle(june10, 40.4167, -3.703, 12, 1)
  
  # Expect cos(za) to be 0.9179
  expect_gt(cos(za_mad*pi/180), 0.917)
  expect_lt(cos(za_mad*pi/180), 0.919)
})

context("Day of the year")
test_that("day_of_year function works as expected", {
  # First day of the year
  dayofyear = day_of_year("2017-01-01", format= "%Y-%m-%d")
  expect_equal(dayofyear, 1)
  # Summer Equinox
  dayofyear =  day_of_year("2017-06-20", format= "%Y-%m-%d")
  expect_equal(dayofyear, 171)
  # Last day of the year
  dayofyear = day_of_year("2017-12-31", format= "%Y-%m-%d")
  expect_equal(dayofyear, 365)
})

context("Utility functions:Air pressure from elevation")
test_that("airpressure_from_elev works as expected", {
  #101.3kPa at 0m elevation
  expect_equal(airpressure_from_elev(0),101.3)
  #89.8kPa at 1000m
  expect_lt(airpressure_from_elev(1000),90)
  #26.4kPa at 10000m
  expect_lt(airpressure_from_elev(10000),30)
})

context("Day length")
test_that("daylength function works as expected", {
  # Day Length - Norway - Should be at least 20 hours
  norway = day_of_year("2017-06-10", format= "%Y-%m-%d")
  norway_dl = daylength(58.88, norway)
  expect_gte(norway_dl,20)
  
  # Day Length - US (Seattle) - Should be at least 17 hours
  seattle = day_of_year("2017-06-10", format= "%Y-%m-%d")
  seattle_dl = daylength(47.43, seattle)
  expect_gte(seattle_dl,17)
  
  # TODO -
  # Day Length - Northern Sweden - Should be at least 23 hours
  # Currently an issue (to debug)
  # Warning message in acos((sin(6 * pi/180) + sin(lat) * sin(DecAng))/(cos(lat) * cos(DecAng))):
  #  “NaNs produced”
  
  sweden = day_of_year("2017-06-10", format= "%Y-%m-%d")
  sweden_dl = daylength(66.86, sweden)
  expect_gte(sweden_dl, 23)
  
})

context("utility functions")
  
expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}


test_that("Temperature conversion functions work as expected", {

  expect_similar(kelvin_to_celsius(temperature = 270), -3.15)
  expect_similar(fahrenheit_to_kelvin(temperature = 85), 302.5944)
  expect_similar(fahrenheit_to_celsius(temperature = 85), 29.44444)
  expect_similar(celsius_to_kelvin(temperature = -10), 263.15)
})


test_that("angle conversion functions work as expected", {

  expect_similar(radians_to_degrees(0.831),  47.61279)
  expect_similar(degrees_to_radians(47.608), 0.8309164)

})

context("Declination Angle")
test_that("dec_angle function works as expected", {
  # Winter Solstice (Should be negative around -23.45 degrees , approx -0.41 radians)
  winsol = day_of_year("2017-12-21", format= "%Y-%m-%d")
  dec_angle_win_sols = dec_angle(winsol)
  expect_lte(dec_angle_win_sols, -.40)
  # Summer Equinox (Should be positive 23.45 degrees , approx 0.41 radians))
  summerequi =  day_of_year("2017-06-20", format= "%Y-%m-%d")
  expect_gte(summerequi, .40)
  
})

context("Solar noon")
test_that("solar_noon function works as expected", {
  # Vancouver - Solar noon in the month of June
  june10 = day_of_year("2017-06-10", format= "%Y-%m-%d")
  solar_noon_van = solar_noon(-123.117, june10, -8)
  
  # we would expect 12:11:57 ~= 12.2
  expect_gt(solar_noon_van, 12.1)
  expect_lt(solar_noon_van, 12.3)
  
  # Sydney - Solar noon in the month of January (daylight saving time applied)
  jan15 = day_of_year("2018-01-15", format= "%Y-%m-%d")
  solar_noon_syd = solar_noon(151.217, jan15, 11)
  
  # we would expect 13:04:13 ~= 13.07
  expect_gt(solar_noon_syd, 13.06)
  expect_lt(solar_noon_syd, 13.08)
  
  # Madrid - on June 10th daylight saving
  solar_noon_mad = solar_noon(-3.703, june10, 1)
  
  # we would expect 13:14 ~= 13.23
  expect_gt(solar_noon_mad, 13.2)
  expect_lt(solar_noon_mad, 13.3)
})

context("Azimuth Angle")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("azimuth_angle function works as expected", {
  # Vancouver - Approx Solar Noon expect 6.08
  june10 = day_of_year("2017-06-10", format= "%Y-%m-%d")
  aa_van = azimuth_angle(june10, 49.25, -123.119, 12)
  
  expect_gt(aa_van, 6)
  expect_lt(aa_van, 7)
  
  # Sydney - Approx Solar Noon Expect 181.5
  aa_syd = azimuth_angle(june10, -33.87, 151.217, 12, 10)
  
  expect_gt(aa_syd, 181)
  expect_lt(aa_syd, 182)

  # Beijing expect 10.8
  aa_bei = azimuth_angle(june10, 39.9075, 116.397, 12)
  
  expect_gt(aa_bei, 10)
  expect_lt(aa_bei, 11)
  
  # saudi expect 314
  aa_sau = azimuth_angle(june10, 24.633, 46.7167, 12)
  
  expect_gt(aa_sau, 313)
  expect_lt(aa_sau, 315)
  
  # Madrid offset necessary since timezone is completely off from what it is supposed to be. (expect 47.6)
  aa_mad = azimuth_angle(june10, 40.4167, -3.703, 12, 1)
  
  expect_gt(aa_mad, 47)
  expect_lt(aa_mad, 48)

   expect_similar(azimuth_angle(doy    = 112, 
                 lat    = 47.61, 
                 lon    = -122.33, 
                 hour   = 0, 
                 offset = -8), 107.8561)

})

#' @title Operative Environmental Temperature of a Limpet Based on a Model by Helmuth  
#'
#' @description The function predicts body temperatures (C, operative environmental temperatures) of a limpet. The function was provided by Brian Helmuth -- although radiation and convection are altered from his original model -- and based on \insertCite{Denny2006;textual}{TrenchR}.
#'
#' @param T_a \code{numeric} air temperature (C).
#'
#' @param T_r \code{numeric} rock surface temperature (C) in the sunlight.
#'
#' @param l \code{numeric} limpet length (anterior/posterior axis) (m).
#'
#' @param h \code{numeric} limpet height (dorsal/ventral axis) (m).
#'
#' @param S \code{numeric} solar irradiance (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#'
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param s_aspect \code{numeric} solar aspect angle (degree), the angle between the limpet's length dimension and the vector to the Sun. Generally between 70 and 110 degrees.
#'
#' @param s_slope \code{numeric} solar elevation angle (degree), the altitude of the sun, which is the angle between the horizon and the sun.
#'
#' @param c \code{numeric} fraction of the sky covered by clouds.
#'
#' @return \code{numeric} predicted body (operative environmental) temperature (C).
#'
#' @family biophysical models
#'
#' @author Brian Helmuth et al.
#'
#' @details The original equation uses a finite-difference approach where they divide the rock into series of chunks, and calculate the temperature at each node to derive the conductive heat. For simplification, here it takes the rock temperature as a parameter, and conductive heat is calculated by the product of the area, thermal conductivity of rock and the difference in temperatures of the rock and the body.
#'   \cr \cr
#'   Limpets are simulated as cones following and using solar emissivity values from \insertCite{Campbell1998;textual}{TrenchR}.
#'   \cr \cr
#'   The area of the limpet's shell (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) is projected in the direction at which sunlight strikes the organism \insertCite{Pennell1989;textual}{TrenchR}.
#'   \cr \cr
#'   Air conductivity values (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) are calculated following \insertCite{Denny2006;textual}{TrenchR}.
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Tb_limpetBH(T_a = 25,
#'               T_r = 30,
#'               l = 0.0176,
#'               h = 0.0122,
#'               S = 1300,
#'               u = 1,
#'               s_aspect = 90,
#'               s_slope = 60,
#'               c = 1)
#'
Tb_limpetBH <- function (T_a,
                         T_r,
                         l,
                         h,
                         S,
                         u,
                         s_aspect,
                         s_slope,
                         c) {

  stopifnot(l        >  0, 
            h        >  0, 
            S        >  0, 
            u        >= 0, 
            s_slope  >= 0, 
            s_slope  <= 90, 
            s_aspect >= 70, 
            s_aspect <= 110, 
            c        >= 0, 
            c        <= 1)

  # Conversions

    T_a <- celsius_to_kelvin(T_a)
    T_r <- celsius_to_kelvin(T_r)

    s_aspect <- degrees_to_radians(s_aspect)
    s_slope  <- degrees_to_radians(s_slope)
    r <- l / 2

  # Calculations

    # Adjust solar radiation for sun angles

      r_aspect <- 257* pi / 180
      r_slope <- 44.5* pi / 180

      delta_i <- cos(r_slope) * cos(s_slope) * cos(s_aspect - r_aspect) + sin(r_slope) * sin(s_slope)
      S <- S*delta_i

    # Short wave heat transfer

      # Area of the limpet's shell (m^2) projected in the direction at which sunlight strikes the organism (Pennell and Deignan 1989)

        Ap <- pi * r^2

      # short-wave absorptivity of the shell (the fraction of light energy that is absorbed)
      # Absorptivity from Luke Miller

        if (l >= 0.037) {

          alpha_sw <- 0.615

        } else if (l <= 0.02225) {

          alpha_sw <-0.689

        } else {

          alpha_sw <- 0.68

        }

        q1 = Ap * alpha_sw * S

    # Long-wave energy transfer

      # View factor simulating limpets as a cone (Campbell and Norman 1998).

        Vs <- 0.7

      # lateral area of a limpet shell (m^2)

        Al <- pi * r * sqrt(h^2 + r^2)

      #  long-wave emissivity of the shell

        eps_ws <- 0.97

        sigma <- stefan_boltzmann_constant()

      # clear sky emissivity (Campbell and Norman 1998, 10.11)

        eps_ac <- 9.2 * 10^-6 * T_a^2

      # emissivity of air with clouds (same as above, 10.12)

        eps_wa <- (1 - 0.84 * c) * eps_ac + 0.84 * c

      q2 <- Vs * Al * eps_ws * sigma * T_a^4 * (eps_wa - 1)
      q3 <- 4 * Vs * Al * eps_ws * sigma * T_a^3


    # Convective heat transfer

      # conductivity of air (W m^-1 K^-1)

        Ka <- 0.00501 + 7.2 * 10^-5 * T_a

      # kinematic viscosity of air (m^2 s^-1)

        v <- -1.25 * 10^-5 + 9.2 * 10^-8 * T_a

      # Reynolds number

        Re <- u * l / v

     # Absorptivity from Luke Miller

      if (l >= 0.037) {

        a <- 0.447
        b <- 0.516

      } else if (l <= 0.02225) {

        a <-0.1515
        b <- 0.6184

      } else  {

        a <- 0.1658
        b <- 0.6206

      }

      # Nusselt number

        Nu <- a * Re^b

      # Heat transfer coefficient (W m^-2 K^-1)

        hc <- a * Ka * ((u / v)^b) * (l^(b - 1))

      # area of the shell in convective contact with the air (m^2)

        A_cv <- Al
        q4 <- hc * A_cv

  # Conductive heat transfer

    # area of conductive contact between the limpet's foot and the rock (m^2)

      A_cd <- pi * r^2

    # thermal conductivity of rock (W m^-1 K^-1)

      Kr <- 3.06
      q5 <- A_cd * Kr


  T_b <- (q1 + q2 + (q3 + q4) * T_a + q5 * T_r) / (q3 + q4 + q5)

  kelvin_to_celsius(T_b)

}

context("biophysmodel_LimpetBH")


test_that("Tb_limpetBH function works as expected", {
  expect_equal(Tb_limpetBH(T_a = 25, T_r = 30, l = 0.0176, h = 0.0122, S = 1300, u = 1, s_aspect = 90, s_slope = 60, c = 1), 30.6807, tol = 1e-4)
  expect_equal(Tb_limpetBH(T_a = 25, T_r = 30, l = 0.03, h = 0.0122, S = 1300, u = 1, s_aspect = 90, s_slope = 60, c = 1), 32.9128, tol = 1e-4)
  expect_equal(Tb_limpetBH(T_a = 25, T_r = 30, l = 0.04, h = 0.0122, S = 1300, u = 1, s_aspect = 90, s_slope = 60, c = 1), 32.53228, tol = 1e-4)

})

#' @title Gaussian-Quadratic Function Thermal Performance Curve 
#'     
#' @description The function constructs a thermal performance curve by combining as a Gaussian function to describe the rise in performance up to the optimal temperature and a quadratic decline to zero performance at critical thermal maxima and higher temperatures \insertCite{Deutsch2008}{TrenchR}.
#' 
#' @param T_b \code{numeric} vector of temperature range (C).
#' 
#' @param T_opt \code{numeric} thermal optima (C), the temperature at which peak performance occurs.
#' 
#' @param CT_min,CT_max \code{numeric} critical thermal minimum and maximum (C), the lower and upper temperature limits for performance.
#' 
#' @return performance
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' 
#' @examples
#'   TPC(T_b    = 0:60, 
#'       T_opt  = 30, 
#'       CT_min = 10, 
#'       CT_max = 40)
#'
TPC <- function (T_b, 
                 T_opt, 
                 CT_min, 
                 CT_max) {

  stopifnot(CT_max >= CT_min)
  
  perf   <- T_b
  perf[] <- NA
  sigma <- (T_opt - CT_min) / 4

  perf[T_b <= T_opt & !is.na(T_b)] <- exp(-((T_b[T_b <= T_opt & !is.na(T_b)] - T_opt) / (2 * sigma))^2) 
  perf[T_b > T_opt & !is.na(T_b)]  <- 1 - ((T_b[T_b > T_opt & !is.na(T_b)] - T_opt) / (T_opt - CT_max))^2 
  perf[perf < 0]                  <- 0
  
  perf
  
}


#' @title Beta Function Thermal Performance Curve 
#'  
#' @description The function constructs a thermal performance curve based on a beta function \insertCite{Asbury2010}{TrenchR}.
#' 
#' @param T_b \code{numeric} temperature (C).
#' 
#' @param shift \code{numeric} mode of the thermal performance curve.
#' 
#' @param breadth \code{numeric} breadth of the thermal performance curve. 
#' 
#' @param aran \code{numeric} scale performance value. If \code{0}, no scaling; if \code{1}, include a thermodynamic effect on mean performance.
#' 
#' @param tolerance \code{numeric} maximal breath (C) of the thermal performance curve.
#' 
#' @param skew \code{numeric} skewness of the thermal performance curve (0-1). 
#' 
#' @return \code{numeric} performance.
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' 
#' @examples
#'   TPC_beta(T_b       = 0:60, 
#'            shift     = -1, 
#'            breadth   = 0.1, 
#'            aran      = 0, 
#'            tolerance = 43, 
#'            skew      = 0.7)
#'
TPC_beta <- function (T_b, 
                      shift     = -1, 
                      breadth   = 0.1, 
                      aran      = 0, 
                      tolerance = 43, 
                      skew      = 0.7) { 
  
  stopifnot(breadth   > 0, 
            tolerance > 0, 
            skew      >= 0, 
            skew      <= 1, 
            aran %in% c(0, 1))
  
  T_b <- celsius_to_kelvin(T_b)
  shift <- celsius_to_kelvin(shift)
  z <- rep(0.01, length(T_b))
  z[which(is.na(T_b))] <- NA 
  
  sel <- which(T_b - shift >= 0 & T_b - shift <= tolerance)
  z[sel] <- ((T_b[sel] - shift) / tolerance)^(skew / breadth - 1) * (1 - (T_b[sel] - shift) / tolerance)^((1 - skew) / breadth - 1) / beta(skew / breadth, (1 - skew) / breadth) 
  
  if (aran == 1) {
    
    z[sel] <- z[sel] * exp(-0.6 / (T_b[sel] * 8.61734 * 10^(-5))) * 10^10 # scaling factor
    
  }
  
  z
  
}

context("TPCfunctions")

expect_similar <- function(input, expected) {
  eval(bquote(expect_lt(abs(input - expected), 0.01)))
}

test_that("TPC function works as expected", {
  expect_equal(max(TPC(T_b = 0:60, T_opt = 30, CT_min = 10, CT_max = 40)),1)
  expect_equal(min(TPC(T_b = 0:60, T_opt = 30, CT_min = 10, CT_max = 40)),0)
  expect_equal(length(TPC(T_b = 0:60, T_opt = 30, CT_min = 10, CT_max = 40)),61)
  expect_similar(TPC(T_b = 0:60, T_opt = 30, CT_min = 10, CT_max = 40)[1], 0.0001234098)
})

test_that("TPC.beta function works as expected", {
  expect_equal(length(TPC_beta(T_b = 0:60, shift = -1, breadth = 0.1, aran = 0, tolerance = 43, skew = 0.7)), 61)
  expect_similar(TPC_beta(T_b = 0:60, shift = -1, breadth = 0.1, aran = 0, tolerance = 43, skew = 0.7)[1], 3.80322e-08)
  expect_similar(TPC_beta(T_b = 0:60, shift = -1, breadth = 0.1, aran = 1, tolerance = 43, skew = 0.7)[1], 3.2345e-09)
})

#' @title Operative Environmental Temperature of a Grasshopper
#'
#' @description The function estimates body temperatures (C, operative environmental temperatures) of a grasshopper based on \insertCite{Lactin1998;textual}{TrenchR}. Part of the model is based on \insertCite{Swinbank1963;textual}{TrenchR}, following \insertCite{Gates1962;textual}{TrenchR} in \insertCite{Kingsolver1983;textual}{TrenchR}.
#' 
#' @param T_a \code{numeric} air temperature (C).
#'
#' @param T_g  \code{numeric} surface temperature (C). \insertCite{Kingsolver1983;textual}{TrenchR} assumes \code{T_g - T_a = 8.4}.
#'
#' @param u \code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
#'
#' @param S \code{numeric} total (direct + diffuse) solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
#'
#' @param K_t \code{numeric} clearness index (dimensionless), which is the ratio of the global solar radiation measured at the surface to the total solar radiation at the top of the atmosphere.
#'
#' @param psi \code{numeric} solar zenith angle (degrees).
#'
#' @param l \code{numeric} grasshopper length (m).
#'
#' @param Acondfact \code{numeric} the proportion of the grasshopper surface area that is in contact with the ground.
#'
#' @param z \code{numeric} distance from the ground to the grasshopper (m).
#'
#' @param abs \code{numeric} absorptivity of the grasshopper to solar radiation (proportion). See \insertCite{Anderson1979;textual}{TrenchR}.
#'
#' @param r_g \code{numeric} substrate solar reflectivity (proportion). See \insertCite{Kingsolver1983;textual}{TrenchR}.
#'
#' @return \code{numeric} predicted body (operative environmental) temperature (C).
#'
#' @details 
#'   Total radiative flux is calculated as thermal radiative heat flux plus convective heat flux, following \insertCite{Kingsolver1983;textual}{TrenchR}, with the \insertCite{Erbs1982;textual}{TrenchR} model from \insertCite{Wong2001;textual}{TrenchR}.
#'   \cr \cr
#'   Energy balance is based on \insertCite{Kingsolver1983;textual}{TrenchR}.
#'   \cr \cr
#'   Radiation is calculated without area dependence \insertCite{Anderson1979}{TrenchR}.
#'   \cr \cr
#'   The body of a grasshopper female is approximated by a rotational ellipsoid with half the body length as the semi-major axis  \insertCite{Samietz2005}{TrenchR}.
#'   \cr \cr
#'   The diffuse fraction is corrected following \insertCite{Olyphant1984;textual}{TrenchR}.
#'
#' @family biophysical models
#'
#' @export
#'
#' @references
#'   \insertAllCited{}
#'
#' @examples
#'   Tb_grasshopper(T_a       = 25, 
#'                  T_g       = 25,      
#'                  u         = 0.4, 
#'                  S         = 400, 
#'                  K_t       = 0.7, 
#'                  psi       = 30, 
#'                  l         = 0.02, 
#'                  Acondfact = 0.25, 
#'                  z         = 0.001, 
#'                  abs       = 0.7, 
#'                  r_g       = 0.3)
#' 
Tb_grasshopper <- function (T_a, 
                            T_g, 
                            u, 
                            S, 
                            K_t, 
                            psi, 
                            l, 
                            Acondfact = 0.25, 
                            z         = 0.001, 
                            abs       = 0.7, 
                            r_g       = 0.3) {

  stopifnot(u         >= 0, 
            S         >= 0, 
            K_t       >= 0, 
            K_t       <= 1, 
            psi       >= -90, 
            psi       <= 90, 
            l         >= 0, 
            Acondfact >= 0, 
            Acondfact <= 1, 
            z         >= 0, 
            abs       >= 0, 
            abs       <= 1, 
            r_g       >= 0, 
            r_g       <= 1)
    
  # conversions

    T_a <- celsius_to_kelvin(T_a)
    T_g <- celsius_to_kelvin(T_g)

  # Butterfly Parameters

    sigma <- stefan_boltzmann_constant() 

    # IR emissivity of surface to longwave 

      epsilon <- 1 

    # thermal conductivity of fluid
    #   Kf <- 0.024 + 0.00007 * T_a[k] 

      Kf <- 0.025  

    #  kinematic viscosity of air (m^2/s)
    #    http://users.wpi.edu/~ierardi/PDF/air_nu_plot.PDF
    #  m^2/s, kinematic viscosity of air,  at 300K 
    #    http://www.engineeringtoolbox.com/air-absolute-kinematic-viscosity-d_601.html

      v <- 15.68*10^-6  

  # Calculations

    # AREAS
    #   c: semi-major axis
    #   a: semi-minor axis

      c <- l / 2
      a <- (0.365 + 0.241 * l * 1000) / 1000
      e <- sqrt(1 - a^2 / c^2)
      A <- 2 * pi * a^2 + 2 * pi * a * c / e * asin(e)

    # SOLAR RADIATIVE HEAT FLUX   
    #   Separate total radiation into components
    #  Use Erbs et al model from Wong and Chow (2001, Applied Energy 69:1991-224)
    #  Anderson 1979 - calculates radiation as W without area dependence 
    #  kd - diffuse fraction
    #    if(K_t<= 0.22) 
    #    kd = 0.125 Correction from 16.5 for CO from Olyphant 1984

      kd <- 1 - 0.09 * K_t 
      kd[K_t > 0.22 & K_t <= 0.8] <- 0.9511 - 0.1604 * K_t + 4.388 * K_t^2 - 16.638 * K_t^3 + 12.336 * K_t^4
      kd[K_t > 0.8] <- 0.165 

      Sttl <- S
      Sdir <- Sttl * (1-kd)
      Sdif <- Sttl * kd

      psi_r <- degrees_to_radians(psi)

    # Convection

      #   Reynolds number- ratio of interval viscous forces
      #   L: Characteristic dimension (length)
      #   u = windspeed 
      #   Lactin and Johnson add 1m/s to account for cooling by passive convection

        Re <- u * l / v

      # Nusselt number- dimensionless conductance
      #   Anderson 1979 empirical
      #   h_c: heat transfer coefficient, Wm^{-2}C^{-1} #reported in Lactin and Johnson 1998
      #   hc_s: heat transfer coefficient in turbulent air 

        Nu   <- 0.41* Re^0.5 
        h_c  <- Nu * Kf / l 
        hc_s <- h_c * (-0.007 * z / l + 1.71) 


    # Conduction 
    #   cuticle thickness (m)
    #   hcut: W m^-1 K^-1
    #   Qcond = hcut * Acond * (Tb - (T_a)) / Thick
 
     Thick <- 6*10^(-5)
     hcut  <- 0.15
     Acond <- A * Acondfact 


    # Energy balance 

      # Thermal radiative flux

      # silhouette area / total area
      # empirical from Anderson 1979, psi in degrees
  
        sa <- 0.19 - 0.00173 * psi 

        Adir <- A*sa
        Aref <- Adir 

    # Qabs (W)

      Qdir <- abs * Adir * Sdir / cos(psi_r)
      Qdif <- abs * Aref * Sdif
      Qref <- r_g * Aref *Sttl
      Qabs <- Qdir + Qdif + Qref  

   # black body sky temperature in Kelvin
   #   from Swinbank (1963), Kingsolver 1983 estimates using Brunt equation
 
    T_sky <- 0.0552 * (T_a)^1.5
               

  # Solution
  # t solved in wolfram alpha #Solve[a t^4 +b t -d, t]

    a <- A * epsilon * sigma
    b <- hc_s * A + hcut * Acond / Thick
    d <- hc_s * A * T_a + 0.5 * A * epsilon * sigma * (T_sky^4 + T_g^4)+ hcut * Acond * T_g / Thick + Qabs

    # Define the energy balance residual as a function of T_b (in Kelvin)
    # f(T_b) = 0 at steady-state body temperature
    energy_balance <- function(T_b) a * T_b^4 + b * T_b - d
    
    # Solve numerically using Brent's method
    # Bounds: 200K (-73°C) to 500K (227°C) — physically safe for any ectotherm
    T_b <- tryCatch(
      uniroot(energy_balance, interval = c(200, 500), tol = 1e-6)$root,
      error = function(e)
        stop("Could not solve energy balance for T_b. ",
             "Check that inputs produce a physically plausible temperature. ",
             "Original error: ", conditionMessage(e))
    )
    
  kelvin_to_celsius(T_b)

}

context("biophysmodel_Grasshopper")

test_that("Tb_grasshopper function works as expected", {
  expect_equal(Tb_grasshopper(T_a = 25, T_g = 25, u = 0.4, S = 400, K_t = 0.7, psi = 30, l = 0.02, Acondfact = 0.25, z = 0.001, abs = 0.7, r_g = 0.3), 25.03167, tol = 1e-4)
})

---
title: "Allometries and conversions"
author: "Lauren Buckley and the TrEnCh project, Department of Biology, University of Washington"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
bibliography: bibliography.bib
vignette: >
  %\VignetteIndexEntry{Allometries and conversions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

```{r}

# Load TrenchR package

  library(TrenchR)

```

This tutorial reviews tools that use allometries to estimate the dimensions of organisms needed for energy balances and other analyses. 
The tutorial also documents convenience functions for converting units. 
Finally, we present functions that describe thermal performance curves.

# Allometries

We provide tools for estimating additional dimensions of organisms from measured dimensions. 

### Mass

The function `mass_from_length()` encompasses empirically-derived relationships to estimate mass as a power law function of mass for six taxonomic classifications. 
The relationships are derived by relating length to mass for specimens of numerous species. We use the `mass_from_length()` function to illustrate the allometries for different taxa:

```{r, fig.height = 3, fig.width = 4}

oldpar<- par()

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0.01, 0.25, 0.01), 
     y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                             taxon = "lizard"), 
     type = "l", 
     xlab = "length (m)", 
     ylab = "mass (g)", 
     ylim = c(0, 200), 
     xlim = c(0, 0.25), 
     las  = 1, 
     bty  = "L", 
     lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "salamander"), 
       type = "l", 
       col  = "blue", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01),
                               taxon = "frog"), 
       type = "l", 
       col  = "red", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "snake"), 
       type = "l", 
       col  = "green", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "turtle"), 
       type = "l", 
       col  = "orange", 
       lwd  = 2)

points(x    = seq(0.01, 0.25, 0.01), 
       y    = mass_from_length(l     = seq(0.01, 0.25, 0.01), 
                               taxon = "insect"), 
       type = "l", 
       col  = "purple", 
       lwd  = 2)

```

### Surface Area

Surface area is needed for several aspects of biophysical models but is rarely measured. 
We provide functions to estimate surface area ($m^3$) from length, mass, or volume for a variety of taxa. 

The function `surface_area_from_mass()` encompasses empirically-derived relationship to estimate surface area as a power law function of mass for lizards, frogs, and insects. 

```{r, fig.height = 3, fig.width = 4}

par(mar = c(5, 5, 2, 2))

plot(x    = 1:50, 
     y    = surface_area_from_mass(m     = 1:50, 
                                   taxon = "lizard"), 
     type = "l", 
     xlab = "mass (g)", 
     ylab = "", 
     las  = 1, 
     bty  = "L", 
     lwd  = 2)

mtext(text = expression("surface area" ~ (m^{2})), 
      side = 2, 
      line = 3.5)

points(x    = 1:50, 
       y    = surface_area_from_mass(m     = 1:50, 
                                     taxon = "frog"), 
       type = "l", 
       lty  = "dashed", 
       lwd  = 2)

points(x    = 1:50, 
       y    = surface_area_from_mass(m     = 1:50,  
                                     taxon = "salamander"), 
       type = "l", 
       lty  = "dotted", 
       lwd  = 2)
points(x    = seq(0.1, 5, 0.2), 
       y    = surface_area_from_mass(m     = seq(0.1,5,0.2),  
                                     taxon = "insect"), 
       type = "l", 
       lty  = "dotdash", 
       lwd   = 2)

box(bty = "L",
    lwd = 2)

```

We also provide empirical relationships estimating surface area as power law function of volume. 
The functions were developed by Mitchell [-@mitchell1976heat] for use in approximating convective heat transfer:

```{r, fig.height = 3, fig.width = 4}

plot(x    = seq(0.001, 0.01, 0.001), 
     y    = surface_area_from_volume(V     = seq(0.001, 0.01, 0.001), 
                                     taxon = "lizard"), 
     type = "l", 
     xlab = expression(volume ~ (m^{3})),
     ylab = expression("surface area" ~ (m^{2})))

points(x    = seq(0.001, 0.01, 0.001), 
       y    = surface_area_from_volume(V     = seq(0.001, 0.01, 0.001), 
                                       taxon = "frog"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0.001, 0.01, 0.001),
       y    = surface_area_from_volume(V     = seq(0.001, 0.01, 0.001), 
                                       taxon = "sphere"),
       type = "l", 
       lty  = "dotted")

```

For the same study, we additionally provide a function to estimate volume from length for several taxa or by approximating the organism as a sphere. 
Finally, we provide a function for estimating surface area from length by approximating the animal's body as a rotational ellipsoid. 
The allometric functions are available in R as follows:

```{r, fig.height = 3, fig.width = 4}

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0.01, 0.05, 0.001), 
     y    = volume_from_length(l     = seq(0.01, 0.05, 0.001), 
                               taxon = "lizard"), 
     type = "l", 
     xlab = "length (m)", 
     ylab = expression(volume ~ (m^{3})))

points(x    = seq(0.01, 0.05, 0.001), 
       y    = volume_from_length(l     = seq(0.01, 0.05, 0.001), 
                                 taxon = "frog"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0.01, 0.05, 0.001), 
       y    = volume_from_length(l     = seq(0.01, 0.05, 0.001), 
                                 taxon = "sphere"), 
       type = "l", 
       lty  = "dotted")

```

```{r, fig.height = 3, fig.width = 4}

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0.01, 0.05, 0.001), 
     y    = surface_area_from_length(l = seq(0.01, 0.05, 0.001)), 
     type = "l", 
     xlab = "length (m)", 
     ylab = expression("surface area" ~ (m^{2})))

```

Estimating the amount of solar radiation that an animal absorbs requires knowledge of the silhouette area, the proportion of an animal's surface area that is projected onto a horizontal surface. 
Silhouette area is a function of the sun's angle. 
We offer empirically-derived relationships to predict the proportional silhouette area for frogs, lizards, and grasshoppers as a polynomial function of zenith angle (`proportion_silhouette_area()`). 
We additionally offer a function (`proportion_silhouette_area_shapes()`) to estimate silhouette area by approximating the animal as a sphere or cylinder as a function of theta, the angle between the solar beam and the longitudinal axis. 
The R functions are parameterized as follows:

```{r, fig.height = 3, fig.width = 4}

par(mar = c(5, 5, 2, 2))

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area(psi   = seq(0, 90, 10), 
                                       taxon = "frog"), 
     type = "l", 
     xlab = "zenith angle (°)", 
     ylab = "proportion silhouette area", 
     ylim = range(0, 0.5))

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area(psi     = seq(0, 90, 10), 
                                         taxon   = "lizard", 
                                         raz     = 0, 
                                         posture = "prostrate"), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area(psi     = seq(0, 90, 10), 
                                         taxon   = "lizard", 
                                         raz     = 0, 
                                         posture = "elevated"), 
       type = "l", 
       lty  = "dotted")
points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area(psi   = seq(0, 90, 10), 
                                         taxon =  "grasshopper"), 
       type = "l", 
       lty  = "dotdash")

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area_shapes(shape = "spheroid", 
                                              theta = seq(0, 90, 10), 
                                              h     = 0.4, 
                                              d     = 0.39), 
     type = "l", 
     xlab = "theta (°)", 
     ylab = "proportion silhouette area", 
     ylim = c(0, 0.35), 
     lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "spheroid", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.3), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "spheroid", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.2), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "spheroid",
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.1), 
       type = "l", 
       lty  = "dotdash")

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                              theta = seq(0, 90, 10), 
                                              h     = 0.4, 
                                              d     = 0.4), 
     type = "l", 
     lty  = "dashed", 
     xlab = "theta (°)", 
     ylab = "proportion silhouette area", 
     ylim = c(0, 0.35))

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.2), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.1), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder flat ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.05), 
       type = "l", 
       lty  = "dotdash")

plot(x    = seq(0, 90, 10), 
     y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                              theta = seq(0, 90, 10), 
                                              h     = 0.4, 
                                              d     = 0.4), 
     type = "l", 
     xlab = "theta (°)", 
     ylab = "proportion silhouette area", 
     ylim = c(0, 0.35))

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.2), 
       type = "l", 
       lty  = "dashed")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.1), 
       type = "l", 
       lty  = "dotted")

points(x    = seq(0, 90, 10), 
       y    = proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends", 
                                                theta = seq(0, 90, 10), 
                                                h     = 0.4, 
                                                d     = 0.05), 
       type = "l", 
       lty  = "dotdash")

```

# Conversions

We offer convenience functions to convert among units for angles and temperatures:

```{r}

degrees_to_radians(47.608)
radians_to_degrees(0.831)
fahrenheit_to_celsius(85)
fahrenheit_to_kelvin(85)
kelvin_to_celsius(270)

```

# Thermal Performance Curve Functions

Thermal performance curves (TPCs) describe the temperature dependence of organismal performance such as locomotion and digestion [@huey1989evolution]. 
They have become an important tool to understand organismal responses to environmental conditions, particularly responses to climate change [@sinclair2016can]. 
TrenchR includes two forms of TPCs. 
We include a function `TPC()` to describe a Gaussian-quadratic TPC as a function of thermal optima and critical thermal minima and maxima [@deutsch2008impacts]. 
The `TPC_beta()` function uses a beta function to define a TPC based on mode, breadth, and skew. 
A scale factor enables including a thermodynamic effect on mean performance [@asbury2010thermodynamic]. Additional TPC resources are available in the R packages rTPC and thermPref.



```{r, fig.height = 3, fig.width = 4}

plot(x    = 0:60, 
     y    = TPC(T      = 0:60, 
                T_opt  = 30, 
                CT_min = 10, 
                CT_max = 40), 
     type = "l", 
     ylim = c(0,3), 
     ylab = "performance", 
     xlab = "temperature (°C)")

points(x    = 0:60, 
       y    = TPC_beta(T         = 0:60, 
                       shift     = -1, 
                       breadth   = 0.1, 
                       aran      = 0, 
                       tolerance = 43, 
                       skew      = 0.7), 
       type = "l", 
       lty  = "dashed")

suppressWarnings(par(oldpar))

```

# References

---
title: "Estimating microclimates"
author: "Lauren Buckley and the TrEnCh project, Department of Biology, University of Washington"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
bibliography: bibliography.bib
vignette: >
  %\VignetteIndexEntry{Estimating microclimates}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

```{r}

# Load TrenchR package

  library(TrenchR)

```

This tutorial covers approaches and functions for preparing environmental data for analyses such as examining organismal energy balances. 
The tutorial first addresses estimating the intensity and partitioning of solar radiation, including estimating solar angles and day length. 
We then turn to estimating temperature and wind speed profiles that can scale weather station data to the conditions experienced by organisms. 
We review functions for modeling diurnal variation in temperature and solar radiation as well as heat accumulation (degree days). 
The tutorial concludes with a microclimate model for estimating soil temperatures. 
Additional resources are available in R packages including NicheMapR, microclima, and microclimc. Functions are generally from Campbell and Norman [-@campbell2012introduction] unless other resources are specified. More detailed source information is available in each function.

# Radiation

## Solar angles and day length

Sun position and day length are often estimated based on the calendar date. 
We provide a function `day_of_year()` that facilitates translating a date into a calendar date.

```{r}

day_of_year(day    = "2017-04-22", 
            format = "%Y-%m-%d")

```

We next present functions to calculate sun angles. 
We use the calendar date (J) to estimate solar declination (radian), which is the angular distance of the sun north or south of the earth's equator, as follows (`dec_angle()`):

$$ \delta= \arcsin[0.39795 \cos(0.21631+ 2 \arctan (0.967 \tan [0.0086(-186+J)]))].$$

The solar declination angle can be calculated in TrenchR as follows:

```{r, fig.height = 3, fig.width = 4}

plot(x    = 1:365, 
     y    = dec_angle(1:365), 
     type = "l", 
     xlab = "day of year", 
     ylab = "declination angle (radian)")

```

Zenith angle $\psi$ (&deg;), the location of the sun as an angle measured from vertical, can be estimated as: 

$$\cos \psi = \sin( \delta) \sin( \phi) + \cos(\delta)\cos(\phi)\cos(\pi/12*(h-h_0)),$$ 

where $J$ is calendar date, $\phi$ is latitude (radians), $h$ is hour, and $h_0$ is the time of solar noon. 
We illustrate how zenith angles (&deg;) change over the day throughout the year in Seattle, WA, USA:

```{r, fig.height = 3, fig.width = 6}

zenith <- zenith_angle(hour = 1:24, 
                       doy  = 200, 
                       lat  = 47.61, 
                       lon  = -122.33)

plot(x    = 1:24, 
     y    = zenith, 
     type = "l", 
     ylab = "zenith angle (°)", 
     xlab = "hour")

zenith <- zenith_angle(hour = 1:24, 
                       doy  = 1, 
                       lat  = 47.61, 
                       lon  = -122.33)  
points(x    = 1:24, 
       y    = zenith, 
       type = "l", 
       lty  = "dotted")

zenith <- zenith_angle(hour = 1:24, 
                       doy  = 100, 
                       lat  = 47.61, 
                       lon  = -122.33)
points(x    = 1:24, 
       y    = zenith, 
       type = "l", 
       lty  = "dotdash")
  
zenith <- zenith_angle(hour = 1:24, 
                       doy  = 300, 
                       lat  = 47.61, 
                       lon  = -122.33)
points(x    = 1:24, 
       y    = zenith, 
       type = "l", 
       lty  = "dashed")

legend(x      = "bottomleft", 
       title  = "day of year", 
       legend = c(1, 100, 200, 300), 
       lty    = c("dotted", "dotdash", "solid", "dashed"))

```

We also examine how zenith angles (&deg;) at noon change over the course of the year for latitudes along the longitude of Seattle, WA USA:

```{r, fig.height = 3, fig.width = 6}

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 60, 
                       lon  = -122.33)

plot(x    = 1:365, 
     y    = zenith, 
     type = "l", 
     ylim = range(0,90), 
     ylab = "zenith angle (°)", 
     xlab = "day of year")

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 40, 
                       lon  = -122.33)
points(x    = 1:365, 
       y    = zenith, 
       type = "l", 
       lty  = "dotted")

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 20, 
                       lon  = -122.33)

points(x    = 1:365, 
       y    = zenith, 
       type = "l", 
       lty  = "dotdash")

zenith <- zenith_angle(doy  = 1:365, 
                       hour = 12, 
                       lat  = 0, 
                       lon  = -122.33)

points(x    = 1:365, 
       y    = zenith, 
       type = "l", 
       lty  = "dashed")

legend(x      = "topleft", 
       title  = "latitude (°)", 
       legend = c(0, 20, 40, 60), 
       lty    = c("dashed", "dotdash", "dotted", "solid"))

```

The time of solar noon (`solar_noon()`) can be estimated as $h_0= 12 - LC -ET$, where $LC$ in the longitude correction (+4 minutes for each degree east of the standard meridian and -4 minutes for each degree west of the standard meridian) and $ET$ is the equation of time. 
The $ET$ is estimated as: $$ ET= \frac{-104.7\sin (f)+596.2\sin (2f)+4.3\sin (3f)-12.7\sin (4f)-429.3\cos (f)-2.0\cos (2f)+19.3\cos (3f)}{3600}, $$where $f=279.575 + 0.9856 J$, in degrees. The time of solar noon can be estimated across the year for several longitudes as follows:

```{r, fig.height = 3, fig.width = 4}

plot(x    = 1:365, 
     y    = solar_noon(lon = 150, doy = 1:365), 
     type = "l", 
     ylim = c(11.3,12.7), 
     ylab = "time of solar noon (hour)", 
     xlab = "day of year")

points(x    = 1:365, 
       y    = solar_noon(lon = 155, doy = 1:365), 
       type = "l", 
       lty  = "dotted")

points(x    = 1:365, 
       y    = solar_noon(lon = 145, doy = 1:365), 
       type = "l", 
       lty  = "dashed")

abline(a = 12,
       b = 0)

legend(x      = "topright", 
       title  = "longitude (°)", 
       legend = c(145, 150, 155), 
       lty    = c("dashed", "solid", "dotted"))

```

The azimuth angle is the angle (&deg;) from which the sunlight is coming measured from true north or south measured in the horizontal plane. 
The azimuth angle ($AZ$, &deg;) is estimated with respect to due south, increasing in the counter-clockwise direction such that $90^\circ$ is east. 
$AZ$ can be calculated as a function of declination angle $\delta$ (&deg;), zenith angle $\psi$ (&deg;), and latitude $\phi$ (&deg;) as follows:

$$\cos AZ= \frac{-(\sin \delta - \cos \psi \sin \phi)}{\cos \phi \sin \psi}.$$

TrenchR includes a function to estimate azimuth angle as follows. We first examine how azimuth angle varies across the day:

```{r, fig.height = 3, fig.width = 6}

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 173, 
                    lat = 47.61, 
                    lon = -122.33))

plot(x    = 5:18, 
     y    = az, 
     type = "l", 
     xlab = "hour", 
     ylab = "azimuth angle (°)")

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 356, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 5:18, 
       y    = az, 
       type = "l",
       lty  = "dashed")

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 266, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 5:18, 
       y    = az, 
       type = "l",
       lty  = "dotted")

az <- unlist(lapply(5:18, 
                    FUN = azimuth_angle, 
                    doy = 228, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 5:18, 
       y    = az, 
       type = "l",
       lty  = "dotdash")

legend(x      = "topleft", 
       title  = "day of year", 
       legend = c(173, 228, 266, 356), 
       lty    = c("solid", "dotdash", "dotted", "dashed"))

```

We then examine how azimuth angle varies across the year:

```{r, fig.height = 3, fig.width = 4}

az <- unlist(lapply(1:365, 
                    FUN = azimuth_angle, 
                    hour = 12, 
                    lat = 47.61, 
                    lon = -122.33))

plot(x    = 1:365, 
     y    = az, 
     type = "l", 
     xlab = "day of year", 
     ylab = "azimuth angle (°)", ylim=c(0,120))

az <- unlist(lapply(1:365, 
                    FUN = azimuth_angle, 
                    hour = 9, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 1:365, 
       y    = az, 
       type = "l",
       lty  = "dashed")

az <- unlist(lapply(1:365, 
                    FUN = azimuth_angle, 
                    hour = 6, 
                    lat = 47.61, 
                    lon = -122.33))

points(x    = 1:365, 
       y    = az, 
       type = "l",
       lty  = "dotted")

legend(x      = "topright", 
       title  = "hour", 
       legend = c(6, 9, 12), 
       lty    = c("dotted", "dashed", "solid"))

```

These sun angles can be used to solve for day length. 
The equation for half day length $h_s$, which is the time (&deg;) from sunrise to solar noon, is used as the basis for our `daylength()` function:

$$ h_s= \cos^{-1}\left( \frac{\cos \psi-sin \phi \sin \delta}{\cos \phi \cos \delta} \right).$$

We examine how day length varies over the year for sites at several latitudes.

```{r, fig.height = 3, fig.width = 4}

plot(x    = 1:365,  
     y    = daylength(lat = 10, doy = 1:365), 
     type = "l", 
     ylim = c(8, 22), 
     xlab = "day of year",
     ylab = "day length (hour)")

points(x    = 1:365,  
       y    = daylength(lat = 35, doy = 1:365),  
       type = "l",  
       lty  = "dashed")

points(x    = 1:365, 
       y    = daylength(lat = 60, doy = 1:365),  
       type = "l",  
       lty   = "dotted")

legend(x      = "topright", 
       title  = "latitude (°)", 
       legend = c(10, 35, 60), 
       lty    = c("solid", "dashed", "dotted"))

```

## Radiation components

We provide functions to estimate solar (shortwave) radiation by discounting incoming solar radiation due to scattering as the radiation travels through the atmosphere. 
The primary function `solar_radiation()` returns estimates of the direct, diffuse, and reflected components of solar radiation ($W m^{-2}$). 
The radiation streams are the direct irradiance on a surface perpendicular to the beam $S_p$, the diffuse sky irradiance on a horizontal plane $S_d$ ($W m^{-2}$), and the reflected radiation from the ground $S_r$. 
The sum of these three streams represents the total irradiance of a horizontal surface $S_t$.

Calculation of these flux densities requires introducing several additional quantities. 
The atmospheric transmissivity, $\tau$, ranges between 0.6 and 0.7 for typical clear sky conditions [@gates2012biophysical]. 
The solar constant, $S_{p0}$, approximates extraterrestrial flux density as $1,360 W m^{−2}$. 
The optical air mass number, $m_a$, is the ratio of slant path length through the atmosphere to zenith path length and is a function of atmospheric pressure: $m_a=p_a/(101.3\cos(\psi))$, where $p_a$ is air pressure (kPa) and $\psi$ is zenith angle (&deg;, see above). 
The function `airpressure_from_elev()` estimates air pressure as $p_a (kPa)= 101.3 e ^{\frac{-E}{8200}}$, where $E$ is the elevation in meters above sea level. 
TrenchR provides a function to estimate air pressure from elevation (m).

```{r, fig.height = 3, fig.width = 4}

plot(x    = 1:4000, 
     y    = airpressure_from_elev(elev = 1:4000), 
     type = "l", 
     xlab = "elevation (m)", 
     ylab = "air pressure (kPa)")

```

Direct irradiance is a function of the distance a solar beam travels through the atmosphere; the transmittance of the atmosphere, $\tau$; and the incident flux density, $S_{p0}$: $S_p=S_{p0}\tau^{m_a}$. 
We approximate diffuse radiation using an empirical relation [@liu1960interrelationship], $S_d=0.3(1-\tau^{m_a}) S_{p0}\cos\psi$.
Finally, reflected radiation is the product of albedo, $\rho_S$, and the sum of direct and diffuse radiation: $S_r= \rho_S (S_p+S_d)$. 
We estimate the direct, diffuse, and reflected components of solar radiation ($W m^{-2}$) as follows.

We estimate how solar radiation components vary diurnally.

```{r, fig.height = 3, fig.width = 6}

oldpar<-par()

par(mar=c(5, 5, 3, 2))

zen <- unlist(lapply(1:24, 
                     FUN = zenith_angle, 
                     doy = 200, 
                     lat = 47.61, 
                     lon = -122.33))

zen <- zen * 2 * pi / 360

rd <- lapply(zen, 
             FUN  = solar_radiation, 
             doy  = 200, 
             tau  = 0.6, 
             elev = 1500, 
             rho  = 0.7)

rd <- matrix(unlist(rd), 
             nrow  = 3, 
             byrow = FALSE)

plot(x    = 1:24, 
     y    = rd[1,], 
     type = "l", 
     xlab = "hour", 
     ylab = expression(radiation ~ (W/m^{2})), 
     ylim = c(0, 800))

points(x    = 1:24, 
       y    = rd[2,], 
       type = "l", 
       lty = "dotted")

points(x    = 1:24, 
       y    = rd[3,], 
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "solar radiation flux", 
       legend = c("direct", "diffuse", "reflected"), 
       lty    = c("solid", "dotted", "dashed"))

```

We then examine how solar radiation components vary as a function of zenith angle.

```{r, fig.height = 3, fig.width = 4}

par(mar = c(5, 5, 3, 2))

psi.seq <- seq(from = 0, 
               to   = 90, 
               by   = 5)

rd <- lapply(psi.seq * 2 * pi / 360, 
             FUN  = solar_radiation, 
             doy  = 200, 
             tau  = 0.75, 
             elev = 1500, 
             rho  = 0.7)

rd <- matrix(unlist(rd), 
             nrow  = 3, 
             byrow = FALSE)

plot(x    = psi.seq, 
     y    = rd[1,], 
     type = "l", 
     xlab = "zenith angle (°)", 
     ylab = expression(radiation ~ (W/m^{2})), 
     ylim = c(0, 1200))

points(x    = psi.seq, 
       y    = rd[2,], 
       type = "l", 
       lty = "dotted")

points(x    = psi.seq, 
       y    = rd[3,], 
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "solar radiation flux", 
       legend = c("direct", "diffuse", "reflected"), 
       lty    = c("solid", "dotted", "dashed"))

```

Similar but alternative algorithms for estimating hourly direct solar radiation ($W m^{-2}$) were adapted from a comparison of the methods by Tracy et al. [-@tracy1983estimating]. 
Algorithms from earlier versions of Campbell and Norman [-@campbell2012introduction] and Gates [-@gates2012biophysical] are implemented. 
Both estimate radiation as a function of latitude, day of year, elevation, and time.

```{r, fig.height = 3, fig.width = 6}

par(mar = c(5, 5, 3, 2))

r.seq <- lapply(seq(4, 20), 
                FUN    = direct_solar_radiation, 
                lat    = 43.57,
                doy    = 112,
                elev   = 866,
                t0     = 12, 
                method = "Campbell 1977")

r.seq <- unlist(r.seq)

plot(x    = seq(4, 20), 
     y    = r.seq, 
     type = "l", 
     xlab = "hour", 
     ylab = expression(radiation ~ (W/m^{2})))

r.seq <- lapply(seq(4, 20), 
                FUN    = direct_solar_radiation, 
                lat    = 43.57,
                doy    = 112,
                elev   = 866,
                t0     = 12, 
                method = "Gates 1962")

points(x    = seq(4, 20), 
       y    = r.seq,
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "Radiation algorithm", 
       legend = c("Campbell 1977", "Gates 1962"), 
       lty    = c("solid", "dashed"))

```

We additionally provide a function (`monthly_solar_radiation()`) implementing an algorithm to estimate average monthly solar radiation ($W m^{-2} day^{-1}$) using basic topographic and climatic information as input. Cloudiness is stochastically modeled, so output will vary between functional calls and the relationship in the plot deviates from a smooth curve.

```{r, fig.height = 3, fig.width = 4}

par(mar = c(5, 5, 3, 2))

r.seq <- lapply(seq(1, 365, 31), 
                FUN  = monthly_solar_radiation, 
                lat  = 43.57,
                lon  = -116.22,
                elev = 866,
                T    = 20,
                hp   = 15,
                P    = 15)

r.seq <- unlist(r.seq)

plot(x    = seq(1, 365, 31), 
     y    = r.seq, 
     type = "l", 
     xlab = "day of year", 
     ylab = expression(radiation ~ (W/m^{2})))

```

We provide a function `partition_solar_radiation()` to estimate the diffuse fraction $k_d$ that can be used to partition measures of solar radiation into direct and diffuse components. 
The partitioning is based on empirically-derived relations estimated as a function of the clearness index $k_t$, which is the ratio of the global solar radiation measured at the surface to the total solar radiation at the top of the atmosphere. 
The relations are compiled in Wong and Chow [-@wong2001solar].

```{r, fig.height = 3, fig.width = 6}

par(mar=c(5, 5, 3, 2))

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Erbs", 
                lat      = 40, 
                sol.elev = 60)

plot(x    = seq(0, 1, 0.1), 
     y    = r.seq, 
     type = "l", 
     xlab = expression("clearness index" ~ k[t]), 
     ylab = "diffuse fraction", 
     ylim = c(0, 1))

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Liu_Jordan", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "dashed")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Orgill_Hollands", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "dotted")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Olyphant", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "dotdash")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Reindl-1", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "longdash")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Reindl-2", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "longdash")

r.seq <- lapply(seq(0, 1, 0.1), 
                FUN      = partition_solar_radiation, 
                method   = "Lam_Li", 
                lat      = 40, 
                sol.elev = 60)

points(x    = seq(0, 1, 0.1), 
       y    = r.seq, 
       type = "l",
       lty  = "twodash")

legend(x      = "topright", 
       title  = "Partitioning method", 
       legend = c("Erbs", "Liu_Jordan", "Orgill_Hollands", "Olyphant", "Reindl-1", "Reindl-2", "Lam_Li"), 
       lty    = c("solid", "dashed", "dotted", "dotdash", "longdash", "longdash", "twodash"))

```

The functions above are fairly coarse approximations. 
The SOLRAD routine by McCullough and Porter [-@mccullough1971computing] accounts for additional determinations of the diffuse fractions such as zenith angle, atmospheric pressure, and surface albedo. 
The SOLRAD routine is implemented in  NicheMapR. 
TrenchR implements a numeric approximation to the SOLRAD routine from Tracy et al. [-@tracy1983estimating]. 
We illustrate how the diffuse proportion shifts as a function of zenith angle for several different levels of atmospheric pressure and note the function general estimates a lower fraction of diffuse radiation than the algorithms above.

```{r, fig.height = 3, fig.width = 6}

par(mar = c(5, 5, 3, 2))
r.seq <- lapply(seq(20, 85), 
                FUN = proportion_diffuse_solar_radiation, 
                p_a = 86.1, 
                rho = 0.25)

plot(x    = seq(20,85), 
     y    = r.seq,
     type = "l", 
     xlab = "zenith angle (°)", 
     ylab = "diffuse fraction")

r.seq <- lapply(seq(20, 85), 
                FUN = proportion_diffuse_solar_radiation, 
                p_a = 96.1, 
                rho = 0.25)

points(x    = seq(20,85), 
       y    = r.seq, 
       type = "l", 
       lty  = "dashed")

r.seq <- lapply(seq(20, 85), 
                FUN = proportion_diffuse_solar_radiation, 
                p_a = 76.1, 
                rho = 0.25)

points(x    = seq(20,85), 
       y    = r.seq, 
       type = "l", 
       lty = "dotted")

```

# Temperature and wind speed profiles

Temperatures and wind speeds, standardly collected at a height of 1-2 m, can differ dramatically from conditions near the ground where most organisms live. 
Turbulent airflow influenced by the ground surface roughness sets up temperature and wind speed profiles. 
These profiles can be estimated to scale temperatures and wind speeds to organism height. 
We scale across height by estimating a wind speed profile using the relationship: 

$$u(z)= \frac{u^*}{0.41}ln \frac{z-d}{z_m} $$ 

where $u(z)$ is the wind speed ($m s^{-1}$) at height $z$ (m), $u^*$ is the friction velocity ($m s^{-1}$), $d$ is the zero-plane displacement (m, the height at which wind speed is zero), and the constant 0.41 is the von Karman constant.

We provide a function to use this relationship to estimate surface roughness from wind speed measurements at a vector of heights  (`surface_roughness()`). 
First, the zero-plane displacement is estimated as the intercept of the relationship between the wind velocities and the log of measurement height. 
Surface roughness $z_0$ can then be estimated by repeating the regression accounting for the zero-plane displacement (by subtracting the zero-plane displacement from the measurement heights). 
$z_0$ is estimated as the exponential of the ratio of the intercept and slope of the regression relationship as follows.

We illustrate estimating surface roughness.

```{r, fig.height = 3, fig.width = 4}

u_r <- c(0.01, 0.025, 0.05, 0.1, 0.2)
zr  <- c(0.1, 0.25, 0.5, 0.75, 1)

surface_roughness(u_r = u_r, 
                  zr  = zr)

```

We use the following relationship to estimate air temperature and wind speed at height $z$ under neutral conditions (no buoyancy associated with differential density):

$$\frac{u_z}{u_r}=\frac{ln(z/z_0+1)}{ln(z_r/z_0+1)}=\frac{T_z-T_0}{T_r-T_0} $$

where $u_z$ and $u_r$ are the wind speeds ($m s^{-1}$) at height $z$ (m) and reference height $r$ (m), respectively. 
$T_z$, $T_r$, and $T_0$ are air and soil temperatures at heights $z$ and $r$ and soil surface temperature, respectively. 
We provide this relationship in the `wind_speed_profile_neutral()` function, which estimates wind speed at a specified height. 
We additionally provide this relationship in the `air_temp_profile_neutral()` function, which estimates temperature at a specified height.

We examine how the wind speed profiles varies with wind speed (m/s).

```{r, fig.height = 3, fig.width = 4}

zs <- seq(0, 2, 0.1) 

us <- wind_speed_profile_neutral(u_r = 1, 
                                 zr  = 2, 
                                 z0  = 0.2,
                                 z   = zs)

plot(x    = us,
     y    = zs, 
     type = "l", 
     xlab = "wind speed, u (m/s)", 
     ylab = "height, z (m)")

points(x    = wind_speed_profile_neutral(u_r = 0.5,
                                         zr  = 2, 
                                         z0  = 0.2, 
                                         z   = zs), 
       y    = zs, 
       type = "l", 
       lty  = "dashed")

points(x    = wind_speed_profile_neutral(u_r = 0.25, 
                                         zr  = 2, 
                                         z0  = 0.2, 
                                         z   = zs), 
       y    = zs, 
       type = "l", 
       lty  = "dotted")

legend(x      = "bottomright", 
       title  = "wind speed at 2m", 
       legend = c(0.25, 5, 1), 
       lty    = c("dotted", "dashed", "solid"))

```

We next illustrate the influence of surface roughness on air temperature profiles.

```{r, fig.height = 3, fig.width = 4}

z.seq <- seq(0, 2, by = 0.1)
t.seq <- air_temp_profile_neutral(T_r = 20, 
                                  zr  = 1, 
                                  z0  = 0.02, 
                                  z   = z.seq, 
                                  T_s = 25)

plot(x    = t.seq, 
     y    = z.seq, 
     type = "l", 
     xlab = "Temperature (°C)", 
     ylab = "z (height above ground in m)")

t.seq <- air_temp_profile_neutral(T_r = 20, 
                                  zr  = 1, 
                                  z0  = 0.05, 
                                  z   = z.seq, 
                                  T_s = 25)

points(x    = t.seq, 
       y    = z.seq, 
       type = "l", 
       lty  = "dotted")

t.seq <- air_temp_profile_neutral(T_r = 20, 
                                  zr  = 1, 
                                  z0  = 0.08, 
                                  z   = z.seq, 
                                  T_s = 25)
points(x    = t.seq, 
       y    = z.seq, 
       type = "l", 
       lty  = "dashed")

legend(x      = "topright", 
       title  = "surface roughness", 
       legend = c(0.02, 0.05, 0.08), 
       lty    = c("solid", "dotted", "dashed"))

```

Temperature at a given height is computed by the function `air_temp_profile()` by first estimating the temperature at roughness height: 

$$T_{z_0}= \frac{T_r S_{t_b}-T_0 S_{t_s}}{S_{t_b}-S_{t_s}}. $$

Where $S_{t_s}$ is the sublayer Stanton Number $S_{t_s}=0.62/ (\frac{z_0u^*}{12})^{0.45}$ and $S_{t_b}$ is the bulk Stanton Number $S_{t_b}=0.64/ (\frac{z}{z_0}+1)$. 
The Stanton numbers are dimensionless numbers that measures the ratio of the heat transferred in and the thermal capacity. 
The temperature at height $z$ can then be estimated as: 

$$T_z= T_{z_0}+(T_r-T_{z_0})ln(\frac{z}{z_0}+1) .$$

 We estimate air temperature profiles in TrenchR as:

```{r, fig.height = 3, fig.width = 4}

z.seq <- seq(0, 2, by = 0.1)

t.seq <- air_temp_profile(T_r = 20, 
                          u_r = 0.5, 
                          zr  = 2, 
                          z0  = 0.02, 
                          z   = z.seq, 
                          T_s = 25)

plot(x    = t.seq, 
     y    = z.seq, 
     type = "l", 
     xlab = "Temperature (°C)", 
     ylab = "z (height above ground in m)")

t.seq <- air_temp_profile(T_r = 20, 
                          u_r = 0.5, 
                          zr  = 2, 
                          z0  = 0.05, 
                          z   = z.seq, 
                          T_s = 25)

points(x    = t.seq, 
       y    = z.seq, 
       type = "l", 
       lty  = "dotted")

t.seq <- air_temp_profile(T_r = 20, 
                          u_r = 0.5, 
                          zr  = 2, 
                          z0  = 0.08, 
                          z   = z.seq, 
                          T_s = 25)

points(t.seq, 
       z.seq, 
       type = "l", 
       lty = "dashed")

legend(x      = "topright", 
       title  = "surface roughness", 
       legend = c(0.02, 0.05, 0.08), 
       lty    = c("solid", "dotted", "dashed"))

```

TrenchR also adapts functions from NicheMapR [@Kearney2017] to refine wind speed and air temperature profiles by using multiple segments:

```{r, fig.height = 3, fig.width = 4}

z.seq <- seq(0, 2, by = 0.1)
t.seq <- lapply(z.seq, 
                FUN = air_temp_profile_segment, 
                T_r = c(25, 22, 20),
                u_r = c(0.01, 0.025, 0.05), 
                zr  = c(0.05, 0.25, 0.5), 
                z0  = c(0.01, 0.05, 0.1), 
                T_s = 27)

plot(x    = t.seq, 
     y    = z.seq, 
     type = "l",  
     xlab = "Temperature (°C)",  
     ylab = "z (height above ground in m)")

points(x = c(27, 25, 22, 20),
       y = c(0, 0.05, 0.25, 0.5))

```

```{r, fig.height = 3, fig.width = 4}

z.seq <- seq(0, 2, by = 0.1)
u.seq <- lapply(z.seq, 
                FUN = wind_speed_profile_segment, 
                u_r = c(0.01, 0.025, 0.05), 
                zr  = c(0.05, 0.25, 0.5),
                z0  = c(0.01, 0.05, 0.1))

plot(x    = u.seq, 
     y    = z.seq, 
     type = "l", 
     xlab = "Wind speed (m/s)", 
     ylab = "z (height above ground in m)",
     xlim=c(0,0.05))

points(x = c(0.01, 0.025, 0.05),
       y = c(0.05, 0.25, 0.5))

```

# Diurnal variation in temperature and radiation

We provide three forms of equations to estimate diurnal temperature variation: a sine model, a sine exponential model, and a sine square root model. 
The models assume that maximum temperature $T_x$ will occur sometime between sunrise $t_r$ and sunset $t_s$ and that the minimum temperature $T_n$ will occur in the early morning.

The sine model (`diurnal_temp_variation_sine()`) estimates temperature $T$ at hour $h$ as a function of minimum and maximum temperature 

$$T(h)= T_x -T_n(1-\gamma), $$

where $\gamma=0.44-0.46 \sin (0.9+WH_r)+0.11 \sin(0.9+2WH_r)$ and $W=\pi/12$.

For the sine exponential model (`diurnal_temp_variation_sineexp()`), hourly temperatures are estimated as a function of three empirically determined parameters: $\alpha$ is the time difference between the hour of maximum temperature $h_x$ and midday; $\beta$ is the time difference between the hour of minimum temperature $h_n$ and sunrise; and $\gamma$ is a decay parameter that determines the rate of temperature change from sunset to $h'_n$ of the next day.

The time of maximum temperature can be calculated as $h_x=1/2(t_r+t_s)+\alpha$. 
The time of minimum temperature can be calculated as $t_n=t_r+\beta$. 
The model estimates temperature at time $h$ as follows:

$$  T(h)=\left \{
                \begin{array}{ll}
                  T_n+(T_x-T_n)\sin \frac{\pi(h-h_r-\beta)}{l +2(\alpha-\beta)}, h_n \leq h \leq h_s\\
                  T_n'+[T(h_s)-T'_n]\exp \frac{\gamma(h-h_s)}{24-l+\beta}, h_s \leq h \leq h'_n 
                \end{array}
              \right. $$ 

where $h'_n$ is the time of the minimum temperature, $T'_n$, for the next day. 
Day length $l$ is calculated as $h_s-h_r$.

The sine square root model (`diurnal_temp_variation_sinesqrt()`) estimates temperature assuming the time of maximum temperature $t_x$ occurs 4 hours before sunset ($t_x=t_s -4$): 

$$  T(h)=\left \{
                \begin{array}{ll}
                  T_n+\alpha \pi /2 \left \{ \frac{h-h_r}{h_x-h_r} \right \}, h_r < h \leq h_x\\
                  T_r+R \sin [\pi/2+(\frac{h-h_x}{4})\pi/2], h_x < h < h_s\\ 
                  T_r+b \sqrt{h-h_s}, h_s < h \leq h_p
                \end{array}
              \right. $$ 

where $\alpha=T_x-T_n$,$R=T_x-T_s$, and $b=\frac{T_p-T_s}{\sqrt{h_p - h_s}}$. 
Sunset temperature is estimated as $T_s=T_x-c(T_x-T_p)$ where $c$ is an empirically estimated parameter ($c=0.39$).

TrenchR provides the functions as follows:

```{r, fig.height = 3, fig.width = 6}

plot(x    = 1:23, 
     y    = diurnal_temp_variation_sine(T_max = 30, T_min = 10, t=1:23), 
     type = "l", 
     xlab = "hour", 
     ylab = "temperature (°C)")

t.seq <- lapply(1:23, 
                FUN   = diurnal_temp_variation_sineexp, 
                T_max = 30, 
                T_min = 10, 
                t_r   = 6, 
                t_s   = 18)

points(x    = 1:23, 
       y    = t.seq, 
       type = "l", 
       lty  = "dotted")

points(1:23, 
       y = diurnal_temp_variation_sinesqrt(t      = 1:23, 
                                           t_r    = 6, 
                                           t_s    = 18, 
                                           T_max  = 30, 
                                           T_min  = 10, 
                                           T_minp = 12), 
       type = "l", 
       lty = "dashed")

legend(x      = "topleft", 
       title  = "function", 
       legend = c("sine", "sine-exp", "sine-sqrt"),
       lty    = c("solid", "dotted", "dashed"))

```

We adapted equations from Tham et al. [-@tham2010estimation] to estimate hourly solar radiation ($W m^{-2} hr^{-1}$) as a function of daily global solar radiation (in $W m^{-2} d^{-1}$). 
The function `diurnal_radiation_variation()` partitions daily solar radiation $S$.
The function estimates the ratio of hourly to daily global irradiation,$S_G$, as follows:

$$S_G= \frac{\pi}{24}(a'+b'\cos \omega)\frac{\cos \omega - \cos \omega_s}{\sin \omega_s - \omega_s \cos \omega_s}, $$

where $a'= 0.409+0.5016\sin(\omega_s-1.047)$ and $b'=0.6609-0.4767\sin(\omega_s-1.047)$. 

The function utilizes two angles in radians: $\omega$ is the hour angle of the sun and $\omega_s$ is the hour angle at sunset. 
The function calculates these angles as a function of calendar date $J$ and location (latitude and longitude). 
We depict how the diurnal trends in radiation vary across the year (also assuming differences in total radiation).

```{r, fig.height = 3, fig.width = 4}

par(mar=c(5, 5, 3, 2))

r.seq <- lapply(1:24, 
                FUN = diurnal_radiation_variation, 
                doy = 172, 
                S = 8000, 
                lon = -112.07, 
                lat = 33.45)

plot(x    = 1:24, 
     y    = r.seq, 
     type = "l", 
     xlab = "hour", 
     ylab = expression(radiation ~ (W/m^{2})))

r.seq <- lapply(1:24, 
                FUN    = diurnal_radiation_variation, 
                doy    = 356, 
                S      = 4000, 
                lon    = -112.07, 
                lat    = 33.45)

points(x    = 1:24, 
       y    = r.seq, 
       type = "l", 
       lty  = "dotted")

r.seq <- lapply(1:24, 
                FUN    = diurnal_radiation_variation, 
                doy    = 266, 
                S      = 6000, 
                lon    = -112.07, 
                lat    = 33.45)

points(x    = 1:24, 
       y    = r.seq, 
       type = "l", 
       lty  = "dashed")

r.seq <- lapply(1:24, 
                FUN    = diurnal_radiation_variation, 
                doy    = 228, 
                S      = 6000, 
                lon    = -112.07, 
                lat    = 33.45)

points(x    = 1:24, 
       y    = r.seq, 
       type = "l", 
       lty  = "dotdash")

legend(x      = "topright", 
       title  = "day of year", 
       legend = c(172, 228, 266, 356), 
       lty    = c("solid", "dotdash", "dashed", "dotted"))

```

# Degree days

Many organismal processes and rates are temperature dependent.
Accounting for temperature dependence allows translating calendar time into a more biologically relevant concept known as degree days. 
Degree days indicate the accumulated heat available for processes such as development and are estimated as the accumulated product of time and temperature. 
For the frequent application of estimating development, degree days are calculated within physiological bounds known as the lower and upper developmental temperatures, LDT and UDT, respectively.
We provide four approaches for estimating degree days that differ in the complexity with which they aggregate heat: "single.sine", "double.sine", "single.triangulation" or "double.triangulation". 

We implement the methods detailed here: <https://ipm.ucanr.edu/WEATHER/ddfigindex.html>.
The methods account for whether the temperature time series is intercepted by the LDT and / or the UDT. 
Briefly, we present the equations for each method for the case of the temperature time series being intercepted by the lower threshold. 
For the single-sine method, degree days $D$ can be estimated as

$$D= \frac{1}{\pi} \left[ \left( \frac{T_x+T_n}{2}-LDT \right) +\alpha\cos(\theta_1) \right] \\
\theta_1= \sin^{-1}\left[ 1/ \alpha \left( LDT- \frac{T_x+T_n}{2} \right) \right]$$

For the double-sine method, degree days $D$ can be estimated as

$$D= \frac{1}{2\pi} \left[ \left( \frac{T_x+T_n}{2}-LDT \right)(\pi/2- \theta_1 ) +\alpha\cos(\theta_1) \right] \\
\theta_1= \sin^{-1}\left[ 1/ \alpha \left( LDT- \frac{T_x+T_n}{2} \right) \right].$$

For the single triangulation method, degree days $D$ can be estimated as

$$D= \frac{(T_x-LDT)^2}{2(T_x-T_n)} .$$ 

For the double triangulation method, degree days $D$ can be estimated as

$$D= \frac{(T_x-LDT)^2}{4(T_x-T_n)}. $$

The methods are available in the following R function. 
Note the function calculates degree days for a full day assuming symmetry in minimum temperatures.

```{r}

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "single.sine")

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "double.sine")

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "single.triangulation")

degree_days(T_min  = 7, 
            T_max  = 14, 
            LDT    = 12, 
            UDT    = 33, 
            method = "double.triangulation")

```

# Soil temperatures

TrenchR includes a microclimate model that uses finite-difference methods to solve heat balance equations describing soil temperatures at the surface and specified depths. 
The function requires air temperature and radiation data as input. 
Our approach follows Porter et al. [-@porter1973behavioral] and is similar to that described for the NicheMapR R package [@Kearney2017]. 
The primary function to call to calculate soil temperature is `soil_temperature()`, which is a wrapper to solve the differential equations below for equilibrium conditions. 
`soil_temperature_function()` contains the majority of the soil temperature model and calls two helper functions that implement an equation for calculating soil temperature adapted from Beckman et al. [-@beckman1973thermal]: `soil_temperature_integrand()` includes the integrand in the equation and is called by `soil_temperature_equation()`, which represents the full equation. 

Below we outline the soil temperature model encapsulated by the functions.

The change in soil temperature ($T_s$) through time $t$ and depth $z$ can be approximated with a differential equation 

$$\frac{dT_s}{dt}= \frac{k_s}{\rho_s c_s} \frac{\delta^2T_s}{\delta z^2}$$

where $k_s$ is the thermal conductivity of soil (W/m·K), $\rho_s$ is the density of soil, and $c_s$ is the specific heat of soil (J/kg·K). 
We solve the differential equation by breaking the soil profile into depth nodes exchanging heat. 
We estimate the steady-state temperature profile, which is independent of initial conditions. 
The choice of initial conditions only affects the rate of convergence to the steady-state solution. 
A finite difference approximation of the differential equation yields a transient equation for the temperature at depth $i$ and time $p$ where we let $a= \frac{k_s}{\rho_s c_s}$:

$$\frac{T_i^{p+1}-T_i^p}{dt}= a \frac{T_{i+dz}^p+T_{i-dz}^p-2T_i^p}{dz^2}. $$

Solving for $T_i^{p+1}$ produces as expression for solving soil temperature at depth $i$ at any time:

$$T_i^{p+1} = a \frac{dt}{dz^2}(T_{i+dz}^p+T_{i-dz}^p)+(1-2a \frac{dt}{dz^2})T_i^p$$

## Soil boundary conditions 

Solving the equation requires two boundary conditions: 1) the deep soil temperature, and 2) the soil surface temperature. 
We assume that the soil temperature at 60cm is constant diurnally. 
The soil surface temperature at time $p$ is estimated using an energy balance equating the energy conducted to the soil surface the net heat transfer to the surface by solar radiation, infrared radiation, and conduction [@beckman1973thermal]:

$$Q_{net_s}= Q_{sol_s} + Q_{emit_s} + Q_{conv_s} +Q_{cond_s}$$ 

where the solar radiation absorbed by the soil surface is the product of incoming radiation $S$ and the proportion of radiation absorbed $\alpha_s$, $Q_{sol_s}= \alpha_s S$. 

The net thermal radiation to the soil surface is determined by the balance of radiation emitted by the soil surface $\sigma T_s^4$ and sky radiation absorbed by the soil surface $\sigma T_{sky}^4$: $Q_{emit_s}= \epsilon_s \sigma [(T^p_{sky})^4-(T^p_0)^4]$, where $\epsilon_s$ is the thermal emissivity of the soil surface and $T^p_{sky}=0.552 T^{1.5}$. 
Convection between the soil surface and air is estimated as a function of the temperature difference $Q_{conv_s}= h_s(T^p-T^p_s)$, where $h_s$ is the surface heat transfer coefficient. 
Consideration of temperature profiles and surface roughness yields an expression during neutral conditions [@sellers1965physical]:

$$h_s= \frac{\rho_s c_s k^2 V_r}{ln(z_r/z_o+1)^2},$$ 

where $V$ is the wind speed at reference height $z_r$ and $z_0$ is roughness length.
Conduction within the soil profile is estimated as $Q_{cond_s}=k_s \frac{\delta T_s}{\delta z}$, at $z=0$. 
Approximating $Q_{cond_s}$ by finite difference yields $Q_{cond_s}=\frac{k_s}{dz}(T^p_{dz}-T^p_s)$, at $z=0$.

The net heat transfer through the profile across time is expressed as

$$Q_{net_s}= \frac{\rho_s c_s dz}{2 dt} (T_0^{p+1}-T_0^p).$$ 

Letting $a=\frac{2dt}{\rho_s c_s dz}$ and solving for $T_0^{p+1}$ in the soil energy budget yields the following boundary condition for soil surface temperature:

$$T_0^{p+1}= a \alpha_s S + a \epsilon_S \sigma [(T^p_{sky})^4-(T^p_0)^4]+a h_s (T^p-T_0^p) +a \frac{k_s}{dz}(T^p_{dz}-T^p_0)+T_0^p$$

Soil temperature can be estimated in TrenchR as follows. The solid line depicts air temperature (&deg;C).

```{r, fig.height = 3, fig.width = 4}

temp_vector <- diurnal_temp_variation_sine(T_max = 20, 
                                           T_min = -10, 
                                           t     = rep(1:24,4))

wind_speed_vector <- runif(n   = 96,
                           min = 0, 
                           max = 0.4)

time_vector <- rep(1:24, 4)
solrad_vector <- unlist(lapply(1:24, 
                               FUN    = diurnal_radiation_variation,
                               doy    = 172, 
                               S = 8000, 
                               lon    = -112.07, 
                               lat    = 33.45))

solrad_vector <- rep(solrad_vector, 4)

params <- list(SSA        = 0.7, 
               epsilon_so = 0.98, 
               k_so       = 0.293, 
               c_so       = 800, 
               dz         = 0.05, 
               z_r        = 1.5, 
               z0         = 0.02, 
               S          = solrad_vector, 
               T_a        = temp_vector, 
               u          = wind_speed_vector, 
               rho_a      = 1.177,
               rho_so     = 1620, 
               c_a        = 1006, 
               TimeIn     = time_vector, 
               dt         = 60*60, 
               shade      = FALSE)

plot(x    = 1:96, 
     y    = temp_vector, 
     type = "l", 
     xlab = "hour", 
     ylab = "temperature (°C)", 
     ylim = c(-10,50))

T_soil <- soil_temperature(z_r.intervals = 12,
                           z_r           = 1.5, 
                           z             = 2, 
                           T_a           = temp_vector, 
                           u             = wind_speed_vector, 
                           Tsoil0        = 20, 
                           z0            = 0.02, 
                           SSA           = 0.7, 
                           TimeIn        = time_vector, 
                           S             = solrad_vector, 
                           water_content = 0.2, 
                           air_pressure  = 85, 
                           rho_so        = 1620, 
                           shade         = FALSE)

points(x    = 1:96, 
       y    = T_soil, 
       lty  = "dashed", 
       type = "l")

T_soil <- soil_temperature(z_r.intervals = 12,
                           z_r           = 1.5, 
                           z             = 4, 
                           T_a           = temp_vector, 
                           u             = wind_speed_vector, 
                           Tsoil0        = 20, 
                           z0            = 0.02, 
                           SSA           = 0.7, 
                           TimeIn        = time_vector, 
                           S             = solrad_vector, 
                           water_content = 0.2, 
                           air_pressure  = 85, 
                           rho_so        = 1620, 
                           shade         = FALSE)

points(x    = 1:96, 
       y    = T_soil, 
       lty  = "dotted", 
       type = "l")

T_soil <- soil_temperature(z_r.intervals = 12,
                           z_r           = 1.5, 
                           z             = 6, 
                           T_a           = temp_vector, 
                           u             = wind_speed_vector, 
                           Tsoil0        = 20, 
                           z0            = 0.02, 
                           SSA           = 0.7, 
                           TimeIn        = time_vector, 
                           S             = solrad_vector, 
                           water_content = 0.2, 
                           air_pressure  = 85, 
                           rho_so        = 1620, 
                           shade         = FALSE)

points(x    = 1:96, 
       y    = T_soil, 
       lty  = "dotdash", 
       type = "l")

legend(x      = "topright", 
       title  = "soil depth interval", 
       legend = c(2, 4, 6), 
       lty    = c("dashed", "dotted", "dotdash"))

```

## Soil specific heat and conductivity

We can approximate soil specific heat, $c_s$, as the weighted sum of the specific heats of soil constituents [@campbell2012introduction]: 

$$\rho_s c_s= rho_m c_m x_m + rho_o c_o x_o + rho_w c_w x_w,$$

where $\rho_s$ is the particle density of soil (bulk density), $x$ is the volume fraction, and the subscripts $m$, $o$,and $w$ stand for minerals, organic matter, and water, respectively. 

We provide a function to calculate soil specific heat given the volume fractions and $\rho_s$:

```{r, fig.height = 3, fig.width = 4}

plot(x    = seq(500,1700,100), 
     y    = soil_specific_heat(x_o    = 0.01, 
                               x_m    = 0.6, 
                               x_w    = 0.2, 
                               rho_so = seq(500,1700,100)), 
     type = "l", 
     xlab = expression("bulk density" ~ (kg/m^{3})), 
     ylab = expression("soil specific heat" ~ (J ~ kg^{-1} ~ K^{-1})))

suppressWarnings(par(oldpar))

```

Various methods of approximating soil thermal conductivity $k_s$ are reviewed in Boguslaw and Lukasz [-@boguslaw2004thermal]. 
We provide a function `soil_conductivity()` to estimate $k_s$ using the methods of de Vries [-@de1963physics]. 
The function calculates $k_s$ as the weighted average of the thermal conductivities $k$ of soil components $i$:

$$k_s= \frac{\sum j_i k_i x_i}{\sum j_i x_i},$$ 

where quantity $j_i$ is the ratio of the average temperature gradient in the granules and the corresponding quantity in the medium (water or air):

$$j_i= \frac{1}{3} \sum_{p=a,b,c}[1+\left( \frac{k_i}{k_0}-1 \right)g_{p,i}]^{-1}, $$

where $k_0$ is the conductivity of the continuous medium in which soil particles are suspended (water or air) and $g$ is a shape factor for the soil particles. 
The soil particles are assumed to be ellipsoids with axes $g_a$, $g_b$, and $g_c$, where $g_a +g_b +g_c=1$. 
de Vries [-@de1952nonstationary] suggests $g_a=g_b=0.125$. 
We provide the function as follows:

```{r}

soil_conductivity(x      = c(0.10,0.40,0.11,0.01,0.2, 0.18), 
                  lambda = c(0.10,0.40,0.11,0.01,0.2, 0.18), 
                  g_a    = 0.125)

```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DRYWETAIR.R
\name{WETAIR}
\alias{WETAIR}
\title{Properties of Wet Air}
\usage{
WETAIR(db, wb = db, rh = 0, dp = 999, bp = 101325)
}
\arguments{
\item{db}{\code{numeric} dry bulb temperature (C).}

\item{wb}{\code{numeric} wet bulb temperature (C).}

\item{rh}{\code{numeric} relative humidity (\%).}

\item{dp}{\code{numeric} dew point temperature (C).}

\item{bp}{\code{numeric} barometric pressure (Pa).}
}
\value{
Named \code{list} with elements:
  \describe{
    \item{\code{e}}{\code{numeric} saturation vapor pressure (Pa)}
    \item{\code{vd}}{\code{numeric} vapor density (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}})}
    \item{\code{rw}}{\code{numeric} mixing ratio (\ifelse{html}{\out{kg kg<sup>-1</sup>}}{\eqn{kg kg^-1}{ASCII}})}
    \item{\code{tvir}}{\code{numeric} virtual temperature (K)}
    \item{\code{tvinc}}{\code{numeric} virtual temperature increment (K)}
    \item{\code{denair}}{\code{numeric} density of the air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}})}
    \item{\code{cp}}{\code{numeric} specific heat of air at constant pressure (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}})}
    \item{\code{wtrpot}}{\code{numeric} water potential (Pa)}
    \item{\code{rh}}{\code{numeric} relative humidity (\%)}
  }
}
\description{
The function calculates several properties of humid air described as output variables below. The program is based on equations from \insertCite{List1971;textual}{TrenchR} and code implementation from NicheMapR \insertCite{Kearney2017,Kearney2020}{TrenchR}.
 \cr 
 WETAIR must be used in conjunction with \code{\link{VAPPRS}}. Input variables are shown below. See Details.
}
\details{
The user must supply known values for DB and BP (BP at one standard
  atmosphere is 101,325 pascals). Values for the remaining variables are
  determined by whether the user has either (1) psychrometric data (WB or RH),
  or (2) hygrometric data (DP):
  \describe{
    \item{Psychrometric data}{If WB is known but not RH, then set RH = -1
      and DP = 999. If RH is known but not WB then set WB = 0 and DP = 999.}
    \item{Hygrometric data}{If DP is known, set WB = 0 and RH = 0.}
  }
}
\examples{
  WETAIR(db = 30, 
         wb = 28, 
         rh = 60, 
         bp = 100 * 1000)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TsoilFunctions.R
\name{soil_temperature}
\alias{soil_temperature}
\title{Calculate Soil Temperature using ODEs}
\usage{
soil_temperature(
  z_r.intervals = 12,
  z_r,
  z,
  T_a,
  u,
  Tsoil0,
  z0,
  SSA,
  TimeIn,
  S,
  water_content = 0.2,
  air_pressure,
  rho_so = 1620,
  shade = FALSE
)
}
\arguments{
\item{z_r.intervals}{\code{numeric} the number of intervals in the soil profile to calculate, defaults to 12.}

\item{z_r}{\code{numeric} reference height (m).}

\item{z}{\code{numeric} interval of the soil profile to return (1 to \code{z_r.intervals}).}

\item{T_a}{\code{numeric} vector of air temperature (degrees C), Note: missing values will be linearly interpolated.}

\item{u}{\code{numeric} vector of wind speeds (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{Tsoil0}{\code{numeric} initial soil temperature (degrees C).}

\item{z0}{\code{numeric} surface roughness (m).}

\item{SSA}{\code{numeric} solar absorptivity of soil surface as a fraction.}

\item{TimeIn}{\code{numeric} vector of time periods for the model.}

\item{S}{\code{numeric} vector of solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{water_content}{\code{numeric} percent water content (percent).}

\item{air_pressure}{\code{numeric} air pressure (kPa).}

\item{rho_so}{\code{numeric} particle density of soil.}

\item{shade}{\code{logical} whether or not soil temperature should be calculated in the shade.}
}
\value{
\code{numeric} soil temperature (C).
}
\description{
This function is called to calculate soil temperature (C) as in \insertCite{Beckman1973;textual}{TrenchR}. This function calls \code{\link{soil_temperature_function}}, which uses ODEs to calculate a soil profile using equations from \insertCite{deVries1963;textual}{TrenchR}
}
\examples{
  set.seed(123)
  temp_vector       <- runif(48, min = -10, max = 10)
  wind_speed_vector <- runif(48, min = 0, max = 0.4)
  time_vector       <- rep(1:24, 2)
  solrad_vector     <- rep(c(rep(0, 6), 
                             seq(10, 700, length.out = 6), 
                             seq(700, 10, length.out = 6), 
                             rep(0, 6)),
                           2)

  soil_temperature(z_r.intervals = 12, 
                   z_r           = 1.5, 
                   z             = 2, 
                   T_a           = temp_vector, 
                   u             = wind_speed_vector, 
                   Tsoil0        = 20, 
                   z0            = 0.02, 
                   SSA           = 0.7, 
                   TimeIn        = time_vector, 
                   S             = solrad_vector, 
                   water_content = 0.2, 
                   air_pressure  = 85, 
                   rho_so        = 1620, 
                   shade         = FALSE)

}
\references{
\insertAllCited{}
}
\seealso{
Other soil temperature functions: 
\code{\link{soil_conductivity}()},
\code{\link{soil_specific_heat}()},
\code{\link{soil_temperature_equation}()},
\code{\link{soil_temperature_function}()},
\code{\link{soil_temperature_integrand}()}
}
\author{
Joseph Grigg
}
\concept{soil temperature functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OperativeTemperatureFunctions.R
\name{Tb_CampbellNorman}
\alias{Tb_CampbellNorman}
\title{Operative Environmental Temperature of an Ectotherm based on Campbell and Norman (1988)}
\usage{
Tb_CampbellNorman(
  T_a,
  T_g,
  S,
  a_s = 0.7,
  a_l = 0.96,
  epsilon = 0.96,
  c_p = 29.3,
  D,
  u
)
}
\arguments{
\item{T_a}{\code{numeric} air temperature (C).}

\item{T_g}{\code{numeric} ground temperature (C).}

\item{S}{\code{numeric} flux density of solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}), combining direct, diffuse, and reflected radiation accounting for view factors.}

\item{a_s}{\code{numeric} organismal solar absorptivity (proportion).}

\item{a_l}{\code{numeric} organismal thermal absorptivity (proportion); 0.965 for lizards \insertCite{Bartlett1967}{TrenchR}.}

\item{epsilon}{\code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.}

\item{c_p}{\code{numeric} specific heat of air (\ifelse{html}{\out{J mol<sup>-1</sup> C<sup>-1</sup>}}{\eqn{J mol^-1 C^-1}{ASCII}}).}

\item{D}{\code{numeric} characteristic dimension of the animal (m).}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}
}
\value{
\code{numeric} operative environmental temperature, \code{T_e} (C).
}
\description{
The function estimates body temperatures (C, operative environmental temperature) of an ectotherm using an approximation based on \insertCite{Campbell1998;textual}{TrenchR} and \insertCite{Mitchell1976;textual}{TrenchR}.
}
\details{
Boundary conductance uses a factor of 1.4 to account for increased convection \insertCite{Mitchell1976}{TrenchR}. The function assumes forced conduction.
}
\examples{
Tb_CampbellNorman (T_a     = 30, 
                   T_g     = 30, 
                   S       = 823, 
                   a_s = 0.7, 
                   a_l = 0.96, 
                   epsilon = 0.96, 
                   c_p     = 29.3, 
                   D       = 0.17, 
                   u       = 1)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qconduction_animal}
\alias{Qconduction_animal}
\title{Conductance Assuming Animal Thermal Conductivity is Rate Limiting}
\usage{
Qconduction_animal(T_g, T_b, d, K = 0.5, A, proportion)
}
\arguments{
\item{T_g}{\code{numeric} ground surface temperature (K).}

\item{T_b}{\code{numeric} body temperature (K).}

\item{d}{\code{numeric} mean thickness of the animal skin (surface) in (meters). The function assumes a well mixed interior.}

\item{K}{\code{numeric} thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}). \code{K = 0.5} for naked skin and \code{K = 0.15} for insect cuticle \insertCite{Galushko2005}{TrenchR}. The conductivity of the ground is generally greater than that of animal tissues, so animal thermal conductivity is generally the rate limiting step.}

\item{A}{\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{proportion}{\code{numeric} proportion of body in contact with the surface (0-1).}
}
\value{
\code{numeric} conductance (W).
}
\description{
The function calculates conductance (W) of an ectothermic animal to its substrate. Method assumes the major resistance to conduction is within surface layers of the animal and that the interior of the animal is equal in temperature to its surface (thermally well mixed) \insertCite{Spotila1992}{TrenchR}.
}
\examples{
  Qconduction_animal(T_g        = 293,
                     T_b        = 303,
                     d          = 10^-6,
                     K          = 0.5,
                     A          = 10^-3,
                     proportion = 0.2)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OperativeTemperatureFunctions.R
\name{Tb_Gates}
\alias{Tb_Gates}
\title{Operative Environmental Temperature of an Ectotherm Based on Gates (1980)}
\usage{
Tb_Gates(
  A,
  D,
  psa_dir,
  psa_ref,
  psa_air,
  psa_g,
  T_g,
  T_a,
  Qabs,
  epsilon,
  H_L,
  ef = 1.3,
  K
)
}
\arguments{
\item{A}{\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{D}{\code{numeric} characteristic dimension for conduction (meters).}

\item{psa_dir}{\code{numeric} view factor for the proportion surface area exposed to direct radiation from the sky (or enclosure) (0-1).}

\item{psa_ref}{\code{numeric} view factor for proportion surface area exposed to reflected radiation from the ground (0-1).}

\item{psa_air}{\code{numeric} proportion surface area exposed to air (0-1).}

\item{psa_g}{\code{numeric} proportion surface area in contact with substrate (0-1).}

\item{T_g}{\code{numeric} ground surface temperature (C).}

\item{T_a}{\code{numeric} ambient air temperature (C).}

\item{Qabs}{\code{numeric} Solar radiation absorbed (W).}

\item{epsilon}{\code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.}

\item{H_L}{\code{numeric} Convective heat transfer coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}).}

\item{ef}{\code{numeric} enhancement factor used to adjust H_L to field conditions using h_L approximation from \insertCite{Mitchell1976;textual}{TrenchR}.  Approximated as 1.23 by default, but see \insertCite{Mitchell1976;textual}{TrenchR} for relationship.}

\item{K}{\code{numeric} Thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}), K = 0.5 \ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}} for naked skin, K = 0.15 \ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}for insect cuticle \insertCite{Galushko2005;textual}{TrenchR}; conductivity of the ground is generally greater than that of animal tissues, so animal thermal conductivity is generally the rate limiting step.}
}
\value{
\code{numeric} operative environmental temperature, \code{T_e} (C).
}
\description{
The function predicts body temperatures (C, operative environmental temperature) of an ectotherm using the approximation in \insertCite{Gates1980;textual}{TrenchR}. The functions omits evaporative and metabolic heat loss \insertCite{Mitchell1976,Kingsolver1983}{TrenchR}.
}
\examples{
 Tb_Gates (A       = 0.1, 
           D       = 0.025, 
           psa_dir = 0.6, 
           psa_ref = 0.4, 
           psa_air = 0.5, 
           psa_g   = 0.1, 
           T_g     = 30, 
           T_a     = 37, 
           Qabs    = 2, 
           epsilon = 0.95, 
           H_L     = 10, 
           ef      = 1.23, 
           K       = 0.5)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Tsoil}
\alias{Tsoil}
\title{Approximate Soil Temperature}
\usage{
Tsoil(T_g_max, T_g_min, hour, depth)
}
\arguments{
\item{T_g_max}{\code{numeric} daily maximum soil surface temperature (C).}

\item{T_g_min}{\code{numeric} daily minimum soil surface temperature (C).}

\item{hour}{\code{numeric} hour of the day.}

\item{depth}{\code{numeric} depth (cm).}
}
\value{
\code{numeric} soil temperature (C).
}
\description{
The function estimates soil temperature (C) at a given depth and hour by approximating diurnal variation as sinusoidal. The function is adapted from \insertCite{Campbell1998;textual}{TrenchR} and described in \insertCite{Riddell2018;textual}{TrenchR}.
}
\examples{
  Tsoil(T_g_max = 30,
        T_g_min = 15,
        hour    = 12,
        depth   = 5)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\author{
Eric Riddell
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Grashof_number}
\alias{Grashof_number}
\title{Grashof Number}
\usage{
Grashof_number(T_a, T_g, D, nu)
}
\arguments{
\item{T_a}{\code{numeric} Air temperature (C).}

\item{T_g}{\code{numeric} ground (surface) temperature (C).}

\item{D}{\code{numeric} characteristic dimension (e.g., body diameter) (meters).}

\item{nu}{\code{numeric} the kinematic viscosity, ratio of dynamic viscosity to density of the fluid (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}); can calculate from \code{\link{DRYAIR}} or \code{\link{WETAIR}}.}
}
\value{
\code{numeric} Grashof number.
}
\description{
The function estimates the Grashof Number, which describes the ability of a parcel of fluid warmer or colder than the surrounding fluid to rise against or fall with the attractive force of gravity. The Grashof Number is estimated as the ratio of a buoyant force times an inertial force to the square of a viscous force \insertCite{Campbell1998}{TrenchR}.
}
\examples{
  Grashof_number(T_a = 30,
                 T_g = 35,
                 D  = 0.001,
                 nu = 1.2)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TempWindProfileFunctions.R
\name{wind_speed_profile_neutral}
\alias{wind_speed_profile_neutral}
\title{Wind Speed at a Specific Height Under Neutral Conditions}
\usage{
wind_speed_profile_neutral(u_r, zr, z0, z)
}
\arguments{
\item{u_r}{\code{numeric} wind velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at reference height.}

\item{zr}{\code{numeric} initial reference height (m).}

\item{z0}{\code{numeric} surface roughness (m).}

\item{z}{\code{numeric} height to scale (m).}
}
\value{
\code{numeric} windspeed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
}
\description{
The function calculates wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at a specified height (m) within a boundary layer near the surface.  The profile assumes neutral conditions. The velocity profile is the neutral profile described by \insertCite{Sellers1965;textual}{TrenchR}. Function is equations (2) and (3) of \insertCite{Porter1973;textual}{TrenchR}.
}
\examples{
  wind_speed_profile_neutral(u_r = 0.1, 
                             zr  = 0.1, 
                             z0  = 0.2, 
                             z   = 0.15)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TsoilFunctions.R
\name{soil_temperature_function}
\alias{soil_temperature_function}
\title{Core Function for Calculating Soil Temperature}
\usage{
soil_temperature_function(j, T_so, params)
}
\arguments{
\item{j}{\code{numeric} the number of the iteration of running the model.}

\item{T_so}{\code{numeric} the initial soil temperature profile in C.}

\item{params}{\code{list} containing the following param, which are described or calculated in \code{\link{soil_temperature}}: \code{SSA, epsilon_so, k_so, c_so, dz, z_r, z0, S, T_a, u, rho_a, rho_so, c_a, TimeIn, dt, shade}.}
}
\value{
Soil temperature profile as a \code{list}.
}
\description{
This function is called to calculate soil temperature as in \insertCite{Beckman1973;textual}{TrenchR}. Parameters are passed as a list to facilitating solving the equations. This function is not intended to be called directly. The energy balance equations are from \insertCite{Porter1973;textual}{TrenchR} and \insertCite{Kingsolver1979;textual}{TrenchR}
}
\examples{
  set.seed(123)
  temp_vector       <- runif(96, min = -10, max = 10)
  wind_speed_vector <- runif(96, min = 0, max = 0.4)
  time_vector       <- rep(1:24, 4)
  solrad_vector     <- rep(c(rep(0, 6), 
                             seq(10, 700, length.out = 6), 
                             seq(700, 10, length.out = 6), 
                             rep(0, 6)),
                           4)
  params            <- list(SSA        = 0.7, 
                            epsilon_so = 0.98, 
                            k_so       = 0.293, 
                            c_so       = 800, 
                            dz         = 0.05, 
                            z_r        = 1.5, 
                            z0         = 0.02, 
                            S          = solrad_vector, 
                            T_a        = temp_vector, 
                            u          = wind_speed_vector, 
                            rho_a      = 1.177, 
                            rho_so     = 1620,
                            c_a        = 1006, 
                            TimeIn     = time_vector, 
                            dt         = 60 * 60, 
                            shade      = FALSE)

soil_temperature_function(j      = 1, 
                          T_so   = rep(20,13), 
                          params = params)



}
\references{
\insertAllCited{}
}
\seealso{
Other soil temperature functions: 
\code{\link{soil_conductivity}()},
\code{\link{soil_specific_heat}()},
\code{\link{soil_temperature_equation}()},
\code{\link{soil_temperature_integrand}()},
\code{\link{soil_temperature}()}
}
\author{
Joseph Grigg
}
\concept{soil temperature functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TPCfunctions.R
\name{TPC}
\alias{TPC}
\title{Gaussian-Quadratic Function Thermal Performance Curve}
\usage{
TPC(T_b, T_opt, CT_min, CT_max)
}
\arguments{
\item{T_b}{\code{numeric} vector of temperature range (C).}

\item{T_opt}{\code{numeric} thermal optima (C), the temperature at which peak performance occurs.}

\item{CT_min, CT_max}{\code{numeric} critical thermal minimum and maximum (C), the lower and upper temperature limits for performance.}
}
\value{
performance
}
\description{
The function constructs a thermal performance curve by combining as a Gaussian function to describe the rise in performance up to the optimal temperature and a quadratic decline to zero performance at critical thermal maxima and higher temperatures \insertCite{Deutsch2008}{TrenchR}.
}
\examples{
  TPC(T_b    = 0:60, 
      T_opt  = 30, 
      CT_min = 10, 
      CT_max = 40)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qconduction_substrate}
\alias{Qconduction_substrate}
\title{Conductance Assuming Substrate Thermal Conductivity is Rate Limiting}
\usage{
Qconduction_substrate(T_g, T_b, D, K_g = 0.5, A, proportion)
}
\arguments{
\item{T_g}{\code{numeric} surface temperature (K).}

\item{T_b}{\code{numeric} body temperature (K).}

\item{D}{\code{numeric} characteristic dimension of the animal (m).}

\item{K_g}{\code{numeric} thermal conductivity of substrate (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}).}

\item{A}{\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{proportion}{\code{numeric} proportion in contact to the surface.}
}
\value{
\code{numeric} conductance (W).
}
\description{
The function calculates conductance (W) of an ectothermic animal to its substrate. The method assumes the major resistance to conduction is the substrate and that the interior of the animal is equal in temperature to its surface (thermally well mixed) \insertCite{Spotila1992}{TrenchR}.
}
\examples{
  Qconduction_substrate(T_g        = 293,
                        T_b        = 303,
                        D          = 0.01,
                        K_g        = 0.3,
                        A          = 10^-2,
                        proportion = 0.2)


}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qemitted_thermal_radiation}
\alias{Qemitted_thermal_radiation}
\title{Emitted Thermal Radiation}
\usage{
Qemitted_thermal_radiation(
  epsilon = 0.96,
  A,
  psa_dir,
  psa_ref,
  T_b,
  T_g,
  T_sky = NA,
  T_a,
  enclosed = FALSE
)
}
\arguments{
\item{epsilon}{\code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.}

\item{A}{\code{numeric} surface area ((\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{psa_dir}{\code{numeric} view factor indicating the proportion of surface area exposed to sky (or enclosure) (0-1)}

\item{psa_ref}{\code{numeric} view factor indicating the proportion surface area exposed to ground (0-1).}

\item{T_b}{\code{numeric} body surface temperature (K).}

\item{T_g}{\code{numeric} ground surface temperature (K).}

\item{T_sky}{\code{numeric} Estimate effective radiant temperature of sky (K)}

\item{T_a}{\code{numeric} ambient air temperature (K), only required if the animal is in an enclosed environment.}

\item{enclosed}{\code{logical} whether the animal is an enclosed environment or not.}
}
\value{
\code{numeric} emitted thermal radiation, \code{Qemit} (W).
}
\description{
The function estimates the net thermal radiation (W) emitted by the surface of an animal \insertCite{Gates1980,Spotila1992}{TrenchR}.
}
\examples{
  Qemitted_thermal_radiation(epsilon  = 0.96,
                             A        = 1,
                             psa_dir  = 0.4,
                             psa_ref  = 0.5,
                             T_b      = 303,
                             T_g      = 293,
                             T_a      = 298,
                             enclosed = FALSE)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{daylength}
\alias{daylength}
\title{Day Length}
\usage{
daylength(lat, doy)
}
\arguments{
\item{lat}{\code{numeric} latitude (decimal degrees).}

\item{doy}{\code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.}
}
\value{
\code{numeric} day length (hours).
}
\description{
The function calculates daylength in hours as a function of latitude and day of year. Uses the CMB model \insertCite{Campbell1998}{TrenchR}.
}
\examples{
  daylength(lat = 47.61, 
            doy = 112)

}
\references{
\insertAllCited{}
}
\seealso{
Other utility functions: 
\code{\link{airpressure_from_elev}()},
\code{\link{azimuth_angle}()},
\code{\link{day_of_year}()},
\code{\link{dec_angle}()},
\code{\link{solar_noon}()},
\code{\link{temperature conversions}},
\code{\link{zenith_angle}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_Snail.R
\name{Tb_snail}
\alias{Tb_snail}
\title{Operative Environmental Temperature of a Marine Snail}
\usage{
Tb_snail(temp, l, S, u, CC, WL, WSH)
}
\arguments{
\item{temp}{\code{numeric} air temperature (C).}

\item{l}{\code{numeric} snail length (m).}

\item{S}{\code{numeric} direct solar flux density (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{CC}{\code{numeric} fraction of the sky covered by cloud (0-1).}

\item{WL}{\code{numeric} water loss rate (\ifelse{html}{\out{kg s<sup>-1</sup>}}{\eqn{kg s^-1}{ASCII}}), 5 percent loss of body mass over one hour is a reasonable maximum level \insertCite{Helmuth1999}{TrenchR}.}

\item{WSH}{\code{numeric} wind sensor height (m).}
}
\value{
\code{numeric} predicted body (operative environmental) temperature (C).
}
\description{
The function estimates body temperature (C, operative environmental temperature) of a marine snail. The function implements a steady-state model, which assumes unchanging environmental conditions and is based on \insertCite{Iacarella2012}{TrenchR}. Body temperature and desiccation constrain the activity of Littoraria irrorata within the Spartina alterniflora canopy. The function was provided by Brian Helmuth and is a simplified version of the published model.
}
\details{
Thermal radiative flux is calculated following \insertCite{Helmuth1998;textual}{TrenchR}, \insertCite{Helmuth1999;textual}{TrenchR}, and \insertCite{Idso1969;textual}{TrenchR}.
}
\examples{
  Tb_snail(temp  = 25, 
           l     = 0.012, 
           S = 800, 
           u    = 1, 
           CC    = 0.5, 
           WL    = 0, 
           WSH   = 10)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\author{
Brian Helmuth et al.
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_Mussel.R
\name{Tb_mussel}
\alias{Tb_mussel}
\title{Operative Environmental Temperature of a Mussel}
\usage{
Tb_mussel(l, h, T_a, T_g, S, k_d, u, psi, cl, evap = FALSE, group = "solitary")
}
\arguments{
\item{l}{\code{numeric} mussel length (anterior/posterior axis, m).}

\item{h}{\code{numeric} mussel height (dorsal/ventral axis, m). It is reasonable to assume \code{h = 0.5 * l}.}

\item{T_a}{\code{numeric} air temperature (C).}

\item{T_g}{\code{numeric} ground temperature (C).}

\item{S}{\code{numeric} direct solar flux density (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{k_d}{\code{numeric} diffuse fraction, proportion of solar radiation that is diffuse.}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{psi}{\code{numeric} solar zenith angle (degrees): can be calculated from \code{\link{zenith_angle}}.}

\item{cl}{\code{numeric} fraction of the sky covered by cloud.}

\item{evap}{\code{logical} Whether mussel is gaping to evaporatively cool. If \code{TRUE}, the function assumes a constant mass loss rate of 5 percent of the initial body mass per hour.}

\item{group}{\code{character}; options are \code{"aggregated"}: mussels living in beds; \code{"solitary"}: solitary individual, anterior or posterior end facing upwind; and \code{"solitary_valve"}: solitary individual, valve facing upwind.}
}
\value{
\code{numeric} predicted body (operative environmental) temperature (C).
}
\description{
The function estimates body temperature (C, operative environmental temperature) of a mussel. The function implements a steady-state model, which assumes unchanging environmental conditions.
}
\details{
Thermal radiative flux is calculated following \insertCite{Helmuth1998;textual}{TrenchR}, \insertCite{Helmuth1999;textual}{TrenchR}, and \insertCite{Idso1969;textual}{TrenchR}.
}
\examples{

Tb_mussel(l     = 0.1, 
          h     = 0.05, 
          T_a   = 25, 
          T_g   = 30, 
          S     = 500, 
          k_d   = 0.2, 
          u     = 2, 
          psi   = 30, 
          evap  = FALSE, 
          cl    = 0.5, 
          group = "solitary")

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_Grasshopper.R
\name{Tb_grasshopper}
\alias{Tb_grasshopper}
\title{Operative Environmental Temperature of a Grasshopper}
\usage{
Tb_grasshopper(
  T_a,
  T_g,
  u,
  S,
  K_t,
  psi,
  l,
  Acondfact = 0.25,
  z = 0.001,
  abs = 0.7,
  r_g = 0.3
)
}
\arguments{
\item{T_a}{\code{numeric} air temperature (C).}

\item{T_g}{\code{numeric} surface temperature (C). \insertCite{Kingsolver1983;textual}{TrenchR} assumes \code{T_g - T_a = 8.4}.}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{S}{\code{numeric} total (direct + diffuse) solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{K_t}{\code{numeric} clearness index (dimensionless), which is the ratio of the global solar radiation measured at the surface to the total solar radiation at the top of the atmosphere.}

\item{psi}{\code{numeric} solar zenith angle (degrees).}

\item{l}{\code{numeric} grasshopper length (m).}

\item{Acondfact}{\code{numeric} the proportion of the grasshopper surface area that is in contact with the ground.}

\item{z}{\code{numeric} distance from the ground to the grasshopper (m).}

\item{abs}{\code{numeric} absorptivity of the grasshopper to solar radiation (proportion). See \insertCite{Anderson1979;textual}{TrenchR}.}

\item{r_g}{\code{numeric} substrate solar reflectivity (proportion). See \insertCite{Kingsolver1983;textual}{TrenchR}.}
}
\value{
\code{numeric} predicted body (operative environmental) temperature (C).
}
\description{
The function estimates body temperatures (C, operative environmental temperatures) of a grasshopper based on \insertCite{Lactin1998;textual}{TrenchR}. Part of the model is based on \insertCite{Swinbank1963;textual}{TrenchR}, following \insertCite{Gates1962;textual}{TrenchR} in \insertCite{Kingsolver1983;textual}{TrenchR}.
}
\details{
Total radiative flux is calculated as thermal radiative heat flux plus convective heat flux, following \insertCite{Kingsolver1983;textual}{TrenchR}, with the \insertCite{Erbs1982;textual}{TrenchR} model from \insertCite{Wong2001;textual}{TrenchR}.
  \cr \cr
  Energy balance is based on \insertCite{Kingsolver1983;textual}{TrenchR}.
  \cr \cr
  Radiation is calculated without area dependence \insertCite{Anderson1979}{TrenchR}.
  \cr \cr
  The body of a grasshopper female is approximated by a rotational ellipsoid with half the body length as the semi-major axis  \insertCite{Samietz2005}{TrenchR}.
  \cr \cr
  The diffuse fraction is corrected following \insertCite{Olyphant1984;textual}{TrenchR}.
}
\examples{
  Tb_grasshopper(T_a       = 25, 
                 T_g       = 25,      
                 u         = 0.4, 
                 S         = 400, 
                 K_t       = 0.7, 
                 psi       = 30, 
                 l         = 0.02, 
                 Acondfact = 0.25, 
                 z         = 0.001, 
                 abs       = 0.7, 
                 r_g       = 0.3)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{solar_noon}
\alias{solar_noon}
\title{Time of Solar Noon}
\usage{
solar_noon(lon, doy, offset = NA)
}
\arguments{
\item{lon}{\code{numeric} longitude (decimal degrees).}

\item{doy}{\code{numeric} day of year (1-366). This can be obtained from a standard date via \code{\link{day_of_year}}.}

\item{offset}{\code{numeric} number of hours to add to UTC (Coordinated Universal Time) to get local time (improves accuracy but not always necessary). Defaults to NA.}
}
\value{
\code{numeric} time of solar noon (hours).
}
\description{
The function calculates the time of solar noon in hours as a function of the day of year and longitude \insertCite{Campbell1998}{TrenchR}.
}
\examples{
  solar_noon(lon = -122.335,
             doy = 112)

}
\references{
\insertAllCited{}
}
\seealso{
Other utility functions: 
\code{\link{airpressure_from_elev}()},
\code{\link{azimuth_angle}()},
\code{\link{day_of_year}()},
\code{\link{daylength}()},
\code{\link{dec_angle}()},
\code{\link{temperature conversions}},
\code{\link{zenith_angle}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OperativeTemperatureFunctions.R
\name{Tb_Gates2}
\alias{Tb_Gates2}
\title{Operative Environmental Temperature of an Ectotherm Based on a Variant of Gates (1980)}
\usage{
Tb_Gates2(A, D, T_g, T_a, Qabs, u, epsilon)
}
\arguments{
\item{A}{\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{D}{\code{numeric} characteristic dimension for conduction (meters).}

\item{T_g}{\code{numeric} ground surface temperature (C).}

\item{T_a}{\code{numeric} ambient air temperature (C).}

\item{Qabs}{\code{numeric} Solar radiation absorbed (W).}

\item{u}{\code{numeric} Wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{epsilon}{\code{numeric} longwave infrared emissivity of skin (proportion), 0.95 to 1 for most animals \insertCite{Gates1980}{TrenchR}.}
}
\value{
\code{numeric} operative environmental temperature (C).
}
\description{
The function predicts body temperatures (C, operative environmental temperature) of an ectotherm using the approximation in \insertCite{Gates1980;textual}{TrenchR}. The function omits evaporative and metabolic heat loss.
}
\examples{
  Tb_Gates2(A       = 1, 
            D       = 0.001, 
            T_g     = 27, 
            T_a     = 37, 
            Qabs    = 2, 
            u       = 0.1, 
            epsilon = 1) 

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RadiationPartitioningFunctions.R
\name{partition_solar_radiation}
\alias{partition_solar_radiation}
\title{Diffuse Fraction for Partitioning Solar Radiation}
\usage{
partition_solar_radiation(method, kt, lat = NA, sol.elev = NA)
}
\arguments{
\item{method}{\code{character} method to use for estimating the diffuse fraction, currently available: \code{"Liu_Jordan"}, \code{"Orgill_Hollands"}, \code{"Erbs"}, \code{"Olyphant"}, \code{"Spencer"}, \code{"Reindl-1"}, \code{"Reindl-2"}, \code{"Lam_Li"}.}

\item{kt}{\code{numeric} the clearness index (dimensionless), which is the ratio of the global solar radiation measured at the surface to the total solar radiation at the top of the atmosphere. (0-1)}

\item{lat}{\code{numeric} latitude (degrees). Needed only if method is \code{"Spencer"}.}

\item{sol.elev}{\code{numeric} the solar elevation angles (degrees). Needed only if method is \code{"Reindl-2"}.}
}
\value{
\code{numeric} diffuse fraction.
}
\description{
The function partitions solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}) into direct and diffuse components by estimating the diffuse fraction (k_d). The function uses the models presented in \insertCite{Wong2001;textual}{TrenchR}.
}
\examples{
  partition_solar_radiation(method   = "Erbs", 
                            kt       = 0.5, 
                            lat      = 40, 
                            sol.elev = 60)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qmetabolism_from_mass}
\alias{Qmetabolism_from_mass}
\title{Metabolism as a Function of Mass}
\usage{
Qmetabolism_from_mass(m, taxon = "reptile")
}
\arguments{
\item{m}{\code{numeric} mass (grams).}

\item{taxon}{\code{character} taxon for calculation. Options: \code{"reptile"}, \code{"bird"}, \code{"mammal"}.}
}
\value{
\code{numeric} metabolism (W).
}
\description{
The function estimates the field metabolic rate (W) of various taxa as a function of mass (g). The function does not account for temperature and is based on empirical relationships from \insertCite{Nagy2005;textual}{TrenchR}.
}
\examples{
  Qmetabolism_from_mass(m     = 12,
                        taxon = "reptile")

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_LizardFei.R
\name{Tb_lizard_Fei}
\alias{Tb_lizard_Fei}
\title{Operative Temperature of a Lizard Using Fei et al. (2012)}
\usage{
Tb_lizard_Fei(T_a, T_g, S, lw, shade, m, Acondfact, Agradfact)
}
\arguments{
\item{T_a}{\code{numeric} air temperature at lizard height (C).}

\item{T_g}{\code{numeric} surface temperature (C).}

\item{S}{\code{numeric} total (direct + diffuse) solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m-^2}{ASCII}}).}

\item{lw}{\code{numeric} downward flux of near-infrared radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m-^2}{ASCII}}).}

\item{shade}{\code{numeric} proportion of shade.}

\item{m}{\code{numeric} lizard mass (g).}

\item{Acondfact}{\code{numeric} proportion of the lizard projected area that is in contact with the ground. \code{Acondfact = 0.1} for standing and \code{Acondfact = 0.4} for lying on ground.}

\item{Agradfact}{\code{numeric} proportion of the lizard projected area exposed to radiation from the ground. \code{Agradfact = 0.3} for standing and \code{Agradfact = 0.0} for lying on ground.}
}
\value{
\code{numeric} predicted body (operative environmental) temperature (K).
}
\description{
The function predicts body temperature (C, operative environmental temperature) of a lizard based on \insertCite{Fei2012;textual}{TrenchR}.
}
\details{
Thermal radiative flux is calculated following \insertCite{Fei2012;textual}{TrenchR} based on \insertCite{Bartlett1967;textual}{TrenchR} and \insertCite{Porter1973;textual}{TrenchR}.
}
\examples{
  Tb_lizard_Fei(T_a       = 20,
                T_g       = 27,
                S         = 1300, 
                lw        = 60, 
                shade     = 0.5, 
                m         = 10.5, 
                Acondfact = 0.1, 
                Agradfact = 0.3)


}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\author{
Ofir Levy
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TrenchR-package.R
\docType{package}
\name{TrenchR-package}
\alias{TrenchR}
\alias{TrenchR-package}
\title{Tools for Microclimate and Biophysical Ecology}
\description{
Tools for translating environmental change into organismal
  response. Microclimate models to vertically scale weather station data
  to organismal heights. The biophysical modeling tools include both
  general models for heat flows and specific models to predict body
  temperatures for a variety of ectothermic taxa. Additional functions
  model and temporally partition air and soil temperatures and solar
  radiation. Utility functions estimate the organismal and environmental
  parameters needed for biophysical ecology. 'TrenchR' focuses on
  relatively simple and modular functions so users can create transparent
  and flexible biophysical models. Many functions are derived from
  Gates (1980) and Campbell and Norman (1988).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://trenchproject.github.io/TrenchR/}
  \item \url{https://github.com/trenchproject/TrenchR}
  \item Report bugs at \url{https://github.com/trenchproject/TrenchR/issues}
}

}
\author{
\strong{Maintainer}: Lauren Buckley \email{lbuckley@uw.edu} (\href{https://orcid.org/0000-0003-1315-3818}{ORCID})

Authors:
\itemize{
  \item Bryan Briones Ortiz
  \item Isaac Caruso
  \item Aji John (\href{https://orcid.org/0000-0002-4401-1401}{ORCID})
  \item Ofir Levy (\href{https://orcid.org/0000-0003-0920-1207}{ORCID})
  \item Abigail Meyer
  \item Eric Riddell (\href{https://orcid.org/0000-0002-4229-4911}{ORCID})
  \item Yutaro Sakairi
  \item Juniper Simonis (\href{https://orcid.org/0000-0001-9798-0460}{ORCID})
}

Other contributors:
\itemize{
  \item Brian Helmuth (\href{https://orcid.org/0000-0003-0180-3414}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TsoilFunctions.R
\name{soil_temperature_integrand}
\alias{soil_temperature_integrand}
\title{Solve Equation for Soil Temperature}
\usage{
soil_temperature_integrand(x, L, z0)
}
\arguments{
\item{x}{\code{numeric} vector of volume fractions of soil constituents (e.g., clay, quartz, minerals other than quartz, organic matter, water, air).  The volume fractions should sum to 1. Note that x and lambda values in the example correspond to these soil constituents.}

\item{L}{\code{numeric} Monin-Obukhov length, a measure of the instability of heat flow \insertCite{Beckman1973}{TrenchR}.}

\item{z0}{\code{numeric} surface roughness (m).}
}
\value{
\code{numeric} integrand for soil temperature function.
}
\description{
This function is called by \code{\link{soil_temperature_equation}} to solve the equation for soil temperature from \insertCite{Beckman1973;textual}{TrenchR}. The function represents the integrand in the equation. It is not intended to be called directly.
}
\examples{
  soil_temperature_integrand(x  = c(0.10, 0.40, 0.11, 0.01, 0.2, 0.18), 
                             L  = -10, 
                             z0 = 0.2)

}
\references{
\insertAllCited{}
}
\seealso{
Other soil temperature functions: 
\code{\link{soil_conductivity}()},
\code{\link{soil_specific_heat}()},
\code{\link{soil_temperature_equation}()},
\code{\link{soil_temperature_function}()},
\code{\link{soil_temperature}()}
}
\author{
Joseph Grigg
}
\concept{soil temperature functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DDFunctions.R
\name{degree_days}
\alias{degree_days}
\title{Degree Days}
\usage{
degree_days(T_min, T_max, LDT = NA, UDT = NA, method = "single.sine")
}
\arguments{
\item{T_min}{\code{numeric} Minimum temperature of the day (C).}

\item{T_max}{\code{numeric} Maximum temperature of the day (C).}

\item{LDT}{\code{numeric} lower developmental threshold (C).}

\item{UDT}{\code{numeric} upper developmental threshold (C).}

\item{method}{\code{character} type of method being used. Current choices: \code{"single.sine"}, \code{"double.sine"}, \code{"single.triangulation"}, and \code{"double.triangulation"}.}
}
\value{
\code{numeric} degree days (C).
}
\description{
The function calculates degree days using the following approximations: single or double sine wave, single or double triangulation \insertCite{ucipm}{TrenchR}. Double approximation methods assume symmetry, such that a day's thermal minimum is equal to that of the previous day. Double sine wave approximation of degree days from \insertCite{Allen1976;textual}{TrenchR}.
}
\examples{
  degree_days(T_min  = 7, 
              T_max  = 14, 
              LDT    = 12, 
              UDT    = 33, 
              method = "single.sine")
  degree_days(T_min  = 7, 
              T_max  = 14, 
              LDT    = 12, 
              UDT    = 33, 
              method = "single.triangulation")

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DRYWETAIR.R
\name{VAPPRS}
\alias{VAPPRS}
\title{Saturation Vapor Pressure}
\usage{
VAPPRS(db)
}
\arguments{
\item{db}{\code{numeric} Dry bulb temperature (C)}
}
\value{
esat \code{numeric} Saturation vapor pressure (Pa)
}
\description{
The function calculates saturation vapor pressure for a given air temperature. The program is based on equations from \insertCite{List1971;textual}{TrenchR} and code implementation from NicheMapR \insertCite{Kearney2017,Kearney2020}{TrenchR}.
}
\examples{
  VAPPRS(db = 30)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllometricFunctions.R
\name{surface_area_from_mass}
\alias{surface_area_from_mass}
\title{Organism Surface Area from Mass}
\usage{
surface_area_from_mass(m, taxon)
}
\arguments{
\item{m}{\code{numeric} vector of mass (g).}

\item{taxon}{\code{character} taxonomic classification of organism, current choices: \code{"lizard"}, \code{"salamander"}, \code{"frog"}, \code{"insect"}.}
}
\value{
\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
}
\description{
The function estimates surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) from mass (g) for one of a variety of taxa.
}
\details{
All models follow (\ifelse{html}{\out{SA = a M<sup>b</sup>}}{\eqn{SA = a M^b}{ASCII}}) with mass in grams and surface area in \ifelse{html}{\out{meters<sup>2</sup>}}{\eqn{meters^2}{ASCII}}.
\cr
 \itemize{
  \item Lizards \insertCite{Norris1965,Porter1979,Roughgarden1981,OConnor1999,Fei2012}{TrenchR}:
    \cr 
    \cr \eqn{a = 0.000314 \pi} 
    \cr \eqn{b = 2/3}
  \item Salamanders \insertCite{Whitford1967,Riddell2017}{TrenchR}:
    \cr 
    \cr \eqn{a = 0.000842} 
    \cr \eqn{b = 0.694}
  \item Frogs \insertCite{McClanahan1969}{TrenchR}:
    \cr 
    \cr \eqn{a = 0.00099} 
    \cr \eqn{b = 0.56}
  \item Insects \insertCite{Lactin1997}{TrenchR}:
    \cr 
    \cr \eqn{a = 0.0013} 
    \cr \eqn{b = 0.8}
 }
}
\examples{
  surface_area_from_mass(m     = 1:50, 
                         taxon = "lizard")
  surface_area_from_mass(m     = 1:50,  
                         taxon = "salamander")
  surface_area_from_mass(m     = 1:50,  
                         taxon = "frog")
  surface_area_from_mass(m     = seq(0.1, 5, 0.1),  
                         taxon = "insect")

}
\references{
\insertAllCited{}
}
\seealso{
Other allometric functions: 
\code{\link{mass_from_length}()},
\code{\link{proportion_silhouette_area_shapes}()},
\code{\link{proportion_silhouette_area}()},
\code{\link{surface_area_from_length}()},
\code{\link{surface_area_from_volume}()},
\code{\link{volume_from_length}()}
}
\concept{allometric functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{day_of_year}
\alias{day_of_year}
\title{Calendar Day from Date}
\usage{
day_of_year(day, format = "\%Y-\%m-\%d")
}
\arguments{
\item{day}{\code{character} numerical date in standard format (e.g. \code{"2017-01-02"}, \code{"01-02"}, \code{"01/02/2017"} etc).}

\item{format}{\code{character} date format following \code{\link[base]{as.POSIXlt}} conventions. Default value = \code{"\%Y-\%m-\%d"}.}
}
\value{
\code{numeric} Julian day number, 1-366 (e.g. 1 for January 1st).
}
\description{
The function converts a date (day, month, year) to Calendar Day (day of year).
}
\examples{
  day_of_year(day    = "2017-04-22", 
              format = "\%Y-\%m-\%d")
  day_of_year(day    = "2017-04-22")
  day_of_year(day    = "04/22/2017", 
              format = "\%m/\%d/\%Y")

}
\seealso{
Other utility functions: 
\code{\link{airpressure_from_elev}()},
\code{\link{azimuth_angle}()},
\code{\link{daylength}()},
\code{\link{dec_angle}()},
\code{\link{solar_noon}()},
\code{\link{temperature conversions}},
\code{\link{zenith_angle}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TempWindProfileFunctions.R
\name{air_temp_profile_segment}
\alias{air_temp_profile_segment}
\title{Air Temperature at a Specified Height}
\usage{
air_temp_profile_segment(T_r, u_r, zr, z0, z, T_s)
}
\arguments{
\item{T_r}{\code{numeric} a vector of air temperatures (C) at the 3 reference heights.}

\item{u_r}{\code{numeric} a vector of wind speeds (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at the 3 reference heights.}

\item{zr}{\code{numeric} a vector of 3 reference heights (meters).}

\item{z0}{\code{numeric} a vector of 3 experimentally determined roughness heights (meters).}

\item{z}{\code{numeric} height for air temperature estimation (meters).}

\item{T_s}{\code{numeric} surface temperatures (C).}
}
\value{
\code{numeric} air temperature (C).
}
\description{
The function calculates air temperature (C) at a specified height (m). Estimates a three segment velocity and temperature profile based on user-specified, experimentally determined values for 3 roughness heights and reference heights. Multiple heights are appropriate in heterogenous areas with, for example, a meadow, bushes, and rocks. Implements the MICROSEGMT routine from NicheMapR as described in \insertCite{Kearney2017;textual}{TrenchR}.
}
\examples{
  air_temp_profile_segment(T_r = c(25, 22, 20), 
                           u_r = c(0.01, 0.025, 0.05), 
                           zr  = c(0.05, 0.25, 0.5), 
                           z0  = c(0.01, 0.15, 0.2), 
                           z   = 0.3, 
                           T_s = 27)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RadiationFunctions.R
\name{monthly_solar_radiation}
\alias{monthly_solar_radiation}
\title{Average Monthly Solar Radiation}
\usage{
monthly_solar_radiation(lat, lon, doy, elev, T_a, hp, P)
}
\arguments{
\item{lat}{\code{numeric} latitude (degrees).}

\item{lon}{\code{numeric} longitude (degrees).}

\item{doy}{\code{numeric} day of year (1-366).}

\item{elev}{\code{numeric} elevation (meters).}

\item{T_a}{\code{numeric} mean monthly air temperature (C).}

\item{hp}{\code{numeric} mean month relative humidity (percentage).}

\item{P}{\code{numeric} total monthly precipitation (mm).}
}
\value{
\code{numeric} average monthly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
}
\description{
The function estimates average monthly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> d<sup>-1</sup>}}{\eqn{W m^-2 d^-1}{ASCII}}) using basic topographic and climatic information as input. Cloudiness is stochastically modeled, so output will vary between functional calls. Based on \insertCite{Nikolov1992;textual}{TrenchR}.
}
\examples{
  monthly_solar_radiation(lat  = 47.61, 
                          lon  = -122.33, 
                          doy  = 112, 
                          elev = 1500, 
                          T_a  = 15, 
                          hp   = 50, 
                          P   = 50)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DTRFunctions.R
\name{diurnal_temp_variation_sine}
\alias{diurnal_temp_variation_sine}
\title{Hourly Temperature Variation assuming a Sine Interpolation}
\usage{
diurnal_temp_variation_sine(T_max, T_min, t)
}
\arguments{
\item{T_max, T_min}{\code{numeric} maximum and minimum daily temperatures (C).}

\item{t}{\code{numeric} time for temperature estimate (hour).}
}
\value{
\code{numeric} temperature (C) at a specified hour.
}
\description{
The function estimates temperature for a specified hour using the sine interpolation in \insertCite{Campbell1998;textual}{TrenchR}.
}
\examples{
  diurnal_temp_variation_sine(T_max = 30, 
                              T_min = 10, 
                              t     = 11:15)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{heat_transfer_coefficient_approximation}
\alias{heat_transfer_coefficient_approximation}
\title{Estimate the Heat Transfer Coefficient Using a Spherical Approximation}
\usage{
heat_transfer_coefficient_approximation(u, D, K, nu, taxon = "sphere")
}
\arguments{
\item{u}{\code{numeric} air velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{D}{\code{numeric} characteristic dimension (e.g., diameter or snout-vent length) (meters).}

\item{K}{\code{numeric} thermal conductivity of air (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}), can calculate using \code{\link{DRYAIR}} or \code{\link{WETAIR}}.}

\item{nu}{\code{numeric} kinematic Viscosity of air (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}), can calculate using \code{\link{DRYAIR}} or \code{\link{WETAIR}}.}

\item{taxon}{\code{character} of which class of organism, current choices: \code{"sphere"}, \code{"frog"}, \code{"lizard"}, \code{"flyinginsect"}, \code{"spider"}.}
}
\value{
\code{numeric} heat transfer coefficient, \code{H_L} (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 C^-1}{ASCII}}).
}
\description{
The function estimates the heat transfer coefficient for various taxa. Approximates forced convective heat transfer for animal shapes using the convective relationship for a sphere \insertCite{Mitchell1976}{TrenchR}.
}
\examples{
 heat_transfer_coefficient_approximation(u     = 3,
                                         D     = 0.05,
                                         K     = 25.7 * 10^(-3),
                                         nu    = 15.3 * 10^(-6),
                                         taxon = "sphere")

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_limpetBH.R
\name{Tb_limpetBH}
\alias{Tb_limpetBH}
\title{Operative Environmental Temperature of a Limpet Based on a Model by Helmuth}
\usage{
Tb_limpetBH(T_a, T_r, l, h, S, u, s_aspect, s_slope, c)
}
\arguments{
\item{T_a}{\code{numeric} air temperature (C).}

\item{T_r}{\code{numeric} rock surface temperature (C) in the sunlight.}

\item{l}{\code{numeric} limpet length (anterior/posterior axis) (m).}

\item{h}{\code{numeric} limpet height (dorsal/ventral axis) (m).}

\item{S}{\code{numeric} solar irradiance (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{s_aspect}{\code{numeric} solar aspect angle (degree), the angle between the limpet's length dimension and the vector to the Sun. Generally between 70 and 110 degrees.}

\item{s_slope}{\code{numeric} solar elevation angle (degree), the altitude of the sun, which is the angle between the horizon and the sun.}

\item{c}{\code{numeric} fraction of the sky covered by clouds.}
}
\value{
\code{numeric} predicted body (operative environmental) temperature (C).
}
\description{
The function predicts body temperatures (C, operative environmental temperatures) of a limpet. The function was provided by Brian Helmuth -- although radiation and convection are altered from his original model -- and based on \insertCite{Denny2006;textual}{TrenchR}.
}
\details{
The original equation uses a finite-difference approach where they divide the rock into series of chunks, and calculate the temperature at each node to derive the conductive heat. For simplification, here it takes the rock temperature as a parameter, and conductive heat is calculated by the product of the area, thermal conductivity of rock and the difference in temperatures of the rock and the body.
  \cr \cr
  Limpets are simulated as cones following and using solar emissivity values from \insertCite{Campbell1998;textual}{TrenchR}.
  \cr \cr
  The area of the limpet's shell (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) is projected in the direction at which sunlight strikes the organism \insertCite{Pennell1989;textual}{TrenchR}.
  \cr \cr
  Air conductivity values (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) are calculated following \insertCite{Denny2006;textual}{TrenchR}.
}
\examples{
  Tb_limpetBH(T_a = 25,
              T_r = 30,
              l = 0.0176,
              h = 0.0122,
              S = 1300,
              u = 1,
              s_aspect = 90,
              s_slope = 60,
              c = 1)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\author{
Brian Helmuth et al.
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TempWindProfileFunctions.R
\name{surface_roughness}
\alias{surface_roughness}
\title{Surface Roughness from Empirical Measurements}
\usage{
surface_roughness(u_r, zr)
}
\arguments{
\item{u_r}{\code{numeric} wind velocity (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at a vector of reference heights.}

\item{zr}{\code{numeric} vector of reference heights (m).}
}
\value{
\code{numeric} surface roughness (m).
}
\description{
The function estimates surface roughness (m) from empirical wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) data collected at a vector of heights (m) \insertCite{Kingsolver2015,Campbell1998,Porter1979}{TrenchR}.
}
\examples{
  surface_roughness(u_r = c(0.01, 0.025, 0.05, 0.1, 0.2), 
                    zr  = c(0.05, 0.25, 0.5, 0.75, 1))

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\name{constants}
\alias{constants}
\alias{specific_heat_h2o}
\alias{latent_heat_vaporization_h2o}
\alias{stefan_boltzmann_constant}
\alias{von_karman_constant}
\title{General Use Constants}
\usage{
specific_heat_h2o(units = "J_kg-1_K-1")

latent_heat_vaporization_h2o(units = "J_kg-1")

stefan_boltzmann_constant(units = "W_m-2_K-4")

von_karman_constant(units = "")
}
\arguments{
\item{units}{\code{character} indicating units \itemize{
  \item{specific_heat_h2o}: \itemize{
    \item{\code{"J_kg-1_K-1"}}: {\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}}
  }
  \item{latent_heat_vaporization_h2o}: \itemize{
    \item{\code{"J_kg-1"}}: {\ifelse{html}{\out{J kg<sup>-1</sup>}}{\eqn{J kg^-1}{ASCII}}}
  }
  \item{stefan_boltzmann_constant}: \itemize{
    \item{\code{"W_m-2_K-4"}}: {\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-4</sup>}}{\eqn{W m^-2 K^-4}{ASCII}}}
    \item{\code{"mW_cm-2_K-4"}}: {\ifelse{html}{\out{mW cm<sup>-2</sup> K<sup>-4</sup>}}{\eqn{mW cm^-2 K^-4}{ASCII}}}
  }
  \item{con_karman_constant}: \itemize{
    \item{\code{""}}: dimensionless
  }
}}
}
\value{
\code{numeric} values in \code{units}.
}
\description{
Basic functions for numerical constants for conversions.
}
\examples{
  specific_heat_h2o()
  latent_heat_vaporization_h2o()
  stefan_boltzmann_constant()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllometricFunctions.R
\name{mass_from_length}
\alias{mass_from_length}
\title{Organism Mass from Length}
\usage{
mass_from_length(l, taxon)
}
\arguments{
\item{l}{\code{numeric} vector of length (m). Can be 1 or more values.
\cr
Snout-vent length is used for amphibians and reptiles, except turtles where length is carapace length.}

\item{taxon}{\code{character} taxon of organism, current choices: \code{"insect"}, \code{"lizard"}, \code{"salamander"}, \code{"frog"}, \code{"snake"}, \code{"turtle"}.}
}
\value{
\code{numeric} mass (g).
}
\description{
The function estimates mass (g) from length (m) for a variety of taxa.
}
\details{
All models follow (\ifelse{html}{\out{m = a l<sup>b</sup>}}{\eqn{m = a * l^b}{ASCII}}) with mass in grams and length in meters.
 \itemize{
  \item Lizards: \insertCite{Meiri2010;textual}{TrenchR}:
    \cr 
    \cr \eqn{a = 16368.17} 
    \cr \eqn{b = 3.022}
  \item Salamanders: \insertCite{Pough1980;textual}{TrenchR}:
    \cr 
    \cr \eqn{a = 13654.4} 
    \cr \eqn{b = 2.94}
  \item Frogs: \insertCite{Pough1980;textual}{TrenchR}:
    \cr 
    \cr \eqn{a = 181197.1} 
    \cr \eqn{b = 3.24}
  \item Snakes: \insertCite{Pough1980;textual}{TrenchR}:
    \cr 
    \cr \eqn{a = 723.6756} 
    \cr \eqn{b = 3.02}
  \item Turtles: \insertCite{Pough1980;textual}{TrenchR}:
    \cr 
    \cr \eqn{a = 93554.48} 
    \cr \eqn{b = 2.69}
  \item Insects: \insertCite{Sample1993;textual}{TrenchR}:
    \cr 
    \cr \eqn{a = 806.0827} 
    \cr \eqn{b = 2.494}
 }
}
\examples{
  mass_from_length(l     = 0.04,
                   taxon = "insect")
  mass_from_length(l     = 0.04,
                   taxon = "lizard")
  mass_from_length(l     = 0.04,
                   taxon = "salamander")
  mass_from_length(l     = 0.04,
                   taxon = "frog")
  mass_from_length(l     = 0.04, 
                   taxon = "snake")
  mass_from_length(l     = 0.04, 
                   taxon = "turtle")

}
\references{
\insertAllCited{}
}
\seealso{
Other allometric functions: 
\code{\link{proportion_silhouette_area_shapes}()},
\code{\link{proportion_silhouette_area}()},
\code{\link{surface_area_from_length}()},
\code{\link{surface_area_from_mass}()},
\code{\link{surface_area_from_volume}()},
\code{\link{volume_from_length}()}
}
\concept{allometric functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RadiationFunctions.R
\name{direct_solar_radiation}
\alias{direct_solar_radiation}
\title{Direct Solar Radiation}
\usage{
direct_solar_radiation(lat, doy, elev, t, t0, method = "Campbell 1977")
}
\arguments{
\item{lat}{\code{numeric} latitude (degrees).}

\item{doy}{\code{numeric} day of year (1-366).}

\item{elev}{\code{numeric} elevation (m).}

\item{t}{\code{numeric} local time (decimal hours).}

\item{t0}{\code{numeric} time of local noon (decimal hours), can be estimated using \code{\link{solar_noon}}.}

\item{method}{\code{character} method for estimating direct solar radiation, options: \code{"Campbell 1977"} (default), \code{"Gates 1962"}.}
}
\value{
\code{numeric} direct solar radiation (\ifelse{html}{\out{W/m<sup>2</sup>}}{\eqn{W/m^2}{ASCII}}).
}
\description{
The function estimates direct solar radiation (\ifelse{html}{\out{W/m<sup>2</sup>}}{\eqn{W/m^2}{ASCII}}) based on latitude, day of year, elevation, and time. The function uses two methods \insertCite{McCullough1971,Campbell1998}{TrenchR} compiled in \insertCite{Tracy1983;textual}{TrenchR}.
}
\examples{
  direct_solar_radiation(lat    = 47.61, 
                         doy    = 112, 
                         elev   = 1500, 
                         t      = 9, 
                         t0     = 12, 
                         method = "Campbell 1977")

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_Sceloporus.R
\name{Tb_lizard}
\alias{Tb_lizard}
\title{Operative Environmental Temperature of a Lizard}
\usage{
Tb_lizard(
  T_a,
  T_g,
  u,
  svl,
  m,
  psi,
  rho_s,
  elev,
  doy,
  sun = TRUE,
  surface = TRUE,
  a_s = 0.9,
  a_l = 0.965,
  epsilon_s = 0.965,
  F_d = 0.8,
  F_r = 0.5,
  F_a = 0.5,
  F_g = 0.5
)
}
\arguments{
\item{T_a}{\code{numeric} air temperature (C).}

\item{T_g}{\code{numeric} surface temperature (C).}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{svl}{\code{numeric} lizard snout vent length (mm).}

\item{m}{\code{numeric} lizard mass (g); note that it can be estimated as in \code{\link{mass_from_length}}: \ifelse{html}{\out{3.55 x 10<sup>-5</sup> x length<sup>3</sup>}}{\eqn{3.55*10^-5 * length^3}{ASCII}}}

\item{psi}{\code{numeric} solar zenith angle (degrees).}

\item{rho_s}{\code{numeric} surface albedo (proportion). ~ 0.25 for grass, ~ 0.1 for dark soil, > 0.75 for fresh snow \insertCite{Campbell1998}{TrenchR}.}

\item{elev}{\code{numeric} elevation (m).}

\item{doy}{\code{numeric} day of year (1-366).}

\item{sun}{\code{logical} indicates whether lizard is in sun (\code{TRUE}) or shade (\code{FALSE}).}

\item{surface}{\code{logical} indicates whether lizard is on ground surface (\code{TRUE}) or above the surface (\code{FALSE}, e.g. in a tree).}

\item{a_s}{\code{numeric} lizard solar absorptivity (proportion), \code{a_s = 0.9} \insertCite{Gates1980}{TrenchR} (Table 11.4).}

\item{a_l}{\code{numeric} lizard thermal absorptivity (proportion), \code{a_l = 0.965} \insertCite{Bartlett1967}{TrenchR}.}

\item{epsilon_s}{\code{numeric} surface emissivity of lizards (proportion), \code{epsilon_s = 0.965} \insertCite{Bartlett1967}{TrenchR}.}

\item{F_d}{\code{numeric} the view factor between the surface of the lizard and diffuse solar radiation (proportion). i.e., the portion of the lizard surface that is exposed to diffuse solar radiation \insertCite{Bartlett1967}{TrenchR}.}

\item{F_r}{\code{numeric} the view factor between the surface of the lizard and reflected solar radiation (proportion).}

\item{F_a}{\code{numeric} the view factor between the surface of the lizard and atmospheric radiation (proportion).}

\item{F_g}{\code{numeric} the view factor between the surface of the lizard and ground thermal radiation (proportion).}
}
\value{
T_e \code{numeric} predicted body (operative environmental) temperature (C).
}
\description{
The function estimates body temperature (C, operative environmental temperature) of a lizard based on \insertCite{Campbell1998;textual}{TrenchR}. The function was designed for Sceloporus lizards and described in \insertCite{Buckley2008;textual}{TrenchR}.
}
\details{
The proportion of radiation that is direct is determined following \insertCite{Sears2011;textual}{TrenchR}.
  \cr \cr 
  Boundary conductance uses a factor of 1.4 to account for increased convection \insertCite{Mitchell1976}{TrenchR}.
}
\examples{
  Tb_lizard(T_a       = 25, 
            T_g       = 30, 
            u         = 0.1, 
            svl       = 60, 
            m         = 10, 
            psi       = 34, 
            rho_s     = 0.24, 
            elev      = 500, 
            doy       = 200, 
            sun       = TRUE, 
            surface   = TRUE, 
            a_s   = 0.9, 
            a_l   = 0.965, 
            epsilon_s = 0.965, 
            F_d       = 0.8, 
            F_r       = 0.5, 
            F_a       = 0.5, 
            F_g       = 0.5)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TsoilFunctions.R
\name{soil_specific_heat}
\alias{soil_specific_heat}
\title{Soil Specific Heat}
\usage{
soil_specific_heat(x_o, x_m, x_w, rho_so)
}
\arguments{
\item{x_o}{\code{numeric} volume fraction of organic material (0-1).}

\item{x_m}{\code{numeric} volume fraction of minerals (0-1).}

\item{x_w}{\code{numeric} volume fraction of water (0-1).}

\item{rho_so}{\code{numeric} particle density of soil in (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) (bulk density).}
}
\value{
\code{numeric} soil specific heat (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}).
}
\description{
The function estimates soil specific heat (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}) using the methods of \insertCite{deVries1963;textual}{TrenchR}. The function incorporates the volume fraction of organic material, minerals, and water in soil.
}
\examples{
  soil_specific_heat(x_o    = 0.01, 
                     x_m    = 0.6, 
                     x_w    = 0.2, 
                     rho_so = 1620)

}
\references{
\insertAllCited{}
}
\seealso{
Other soil temperature functions: 
\code{\link{soil_conductivity}()},
\code{\link{soil_temperature_equation}()},
\code{\link{soil_temperature_function}()},
\code{\link{soil_temperature_integrand}()},
\code{\link{soil_temperature}()}
}
\author{
Joseph Grigg
}
\concept{soil temperature functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qconvection}
\alias{Qconvection}
\title{Organismal Convection}
\usage{
Qconvection(T_a, T_b, A, proportion, H_L = 10.45, ef = 1.23)
}
\arguments{
\item{T_a}{\code{numeric} air temperature (K).}

\item{T_b}{\code{numeric} initial body temperature (K).}

\item{A}{\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{proportion}{\code{numeric} proportion of surface area exposed to air.}

\item{H_L}{\code{numeric} convective heat transfer coefficient (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-2</sup>}}{\eqn{W K^-1 m^-2}{ASCII}}).}

\item{ef}{\code{numeric} is the enhancement factor, used to adjust H to field conditions.  Approximated as mean value of 1.23 by default, but see \insertCite{Mitchell1976;textual}{TrenchR} for further information.}
}
\value{
\code{numeric} convection (W).
}
\description{
The function calculates convection from an organism to its environment as in \insertCite{Mitchell1976;textual}{TrenchR}. It includes an enhancement factor associated with outdoor environments.
}
\examples{
  Qconvection(T_a        = 293,
              T_b        = 303,
              H_L        = 10.45,
              A          = 0.0025,
              proportion = 0.85)


}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TempWindProfileFunctions.R
\name{air_temp_profile}
\alias{air_temp_profile}
\title{Air Temperature Profile using MICRO Routine}
\usage{
air_temp_profile(T_r, u_r, zr, z0, z, T_s)
}
\arguments{
\item{T_r}{\code{numeric} air temperature (C) at reference height.}

\item{u_r}{\code{numeric} windspeed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at reference height.}

\item{zr}{\code{numeric} initial reference height (m).}

\item{z0}{\code{numeric} surface roughness (m).}

\item{z}{\code{numeric} height to scale (m).}

\item{T_s}{\code{numeric} surface temperatures (C).}
}
\value{
\code{numeric} air temperature (C).
}
\description{
The function estimates air temperature (C) at a specified height (m). Estimates a single, unsegmented temperature profile using the MICRO routine from NicheMapR \insertCite{Kearney2017}{TrenchR}.
}
\examples{
  air_temp_profile(T_r = 20,
                   u_r = 0.1, 
                   zr  = 0.1, 
                   z0  = 0.2, 
                   z   = 0.15, 
                   T_s = 25)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{angle conversions}
\alias{angle conversions}
\alias{radians_to_degrees}
\alias{degrees_to_radians}
\title{Convert Angles Between Radians and Degrees}
\usage{
radians_to_degrees(rad)

degrees_to_radians(deg)
}
\arguments{
\item{rad}{\code{numeric} angle (radians).}

\item{deg}{\code{numeric} angle (degrees).}
}
\value{
\code{numeric} angle (degrees or radians).
}
\description{
The function converts angles in radians to degrees or degrees to radians.
}
\examples{
  radians_to_degrees(0.831)
  degrees_to_radians(47.608)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllometricFunctions.R
\name{proportion_silhouette_area}
\alias{proportion_silhouette_area}
\title{Organism Silhouette Area}
\usage{
proportion_silhouette_area(psi, taxon, raz = 0, posture = "prostrate")
}
\arguments{
\item{psi}{\code{numeric} zenith angle in degrees between \code{0} and \code{360}.}

\item{taxon}{\code{character} organism name. Current choices are \code{"lizard"}, \code{"frog"}, and \code{"grasshopper"}.}

\item{raz}{\code{numeric} relative solar azimuth angle (in degrees). Required if \code{taxon = "lizard"}. This is the horizontal angle of the sun relative to the head and frontal plane of the lizard and options currently include \code{0} (in front), \code{90} (to side), and \code{180} (behind) degrees.}

\item{posture}{\code{character} value describing posture. Required if \code{taxon = "lizard"}. Options include \code{"prostrate"} (default) and \code{"elevated"}.}
}
\value{
\code{numeric} silhouette area as a proportion.
}
\description{
The function estimates the projected (silhouette) area as a portion of the surface area of the organism as a function of zenith angle. The function is useful for estimating absorbed solar radiation.
}
\details{
Relationships come from \itemize{
  \item Lizards: \insertCite{Muth1977;textual}{TrenchR}
  \item Frogs: \insertCite{Tracy1976;textual}{TrenchR}
  \item Grasshoppers: \insertCite{Anderson1979;textual}{TrenchR}
 }
}
\examples{
  proportion_silhouette_area(psi     = 60,   
                             taxon = "frog")
  proportion_silhouette_area(psi     = 60, 
                             taxon = "grasshopper")
  proportion_silhouette_area(psi       = 60, 
                             taxon   = "lizard", 
                             posture = "prostrate", 
                             raz     = 90)
  proportion_silhouette_area(psi       = 60, 
                             taxon   = "lizard", 
                             posture = "elevated", 
                             raz     = 180)

}
\references{
\insertAllCited{}
}
\seealso{
Other allometric functions: 
\code{\link{mass_from_length}()},
\code{\link{proportion_silhouette_area_shapes}()},
\code{\link{surface_area_from_length}()},
\code{\link{surface_area_from_mass}()},
\code{\link{surface_area_from_volume}()},
\code{\link{volume_from_length}()}
}
\concept{allometric functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TempWindProfileFunctions.R
\name{wind_speed_profile_segment}
\alias{wind_speed_profile_segment}
\title{Wind Speed at a Specified Height}
\usage{
wind_speed_profile_segment(u_r, zr, z0, z)
}
\arguments{
\item{u_r}{\code{numeric} a vector of wind speeds (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at the 3 reference heights.}

\item{zr}{\code{numeric} a vector of 3 reference heights (m).}

\item{z0}{\code{numeric} a vector of 3 experimentally determined roughness heights (m).}

\item{z}{\code{numeric} height to scale (m).}
}
\value{
\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).
}
\description{
The function calculates wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}) at a specified height (m). The function estimates a three segment velocity and temperature profile based on user-specified, experimentally determined values for 3 roughness heights and reference heights. Multiple heights are appropriate in heterogenous areas with, for example, a meadow, bushes, and rocks. Implements the MICROSEGMT routine from NicheMapR as described in \insertCite{Kearney2017;textual}{TrenchR}.
}
\examples{
  wind_speed_profile_segment(u_r = c(0.01, 0.025, 0.05), 
                             zr  = c(0.05, 0.25, 0.5), 
                             z0  = c(0.01, 0.15, 0.2), 
                             z   = 0.3)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllometricFunctions.R
\name{volume_from_length}
\alias{volume_from_length}
\title{Organism Volume from Length}
\usage{
volume_from_length(l, taxon)
}
\arguments{
\item{l}{\code{numeric} length (m). 
\cr
Use snout-vent length for lizards and frogs.}

\item{taxon}{\code{character} taxon of organism, current choices: \code{"lizard"}, \code{"frog"}, \code{"sphere"}.}
}
\value{
\code{numeric} volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}).
}
\description{
The function estimates volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}) from length (m) for a variety of taxa following \insertCite{Mitchell1976;textual}{TrenchR}.
}
\details{
Relationships come from \itemize{
  \item Lizards: \insertCite{Norris1965;textual}{TrenchR}
  \item Frogs: \insertCite{Tracy1972;textual}{TrenchR}
  \item Sphere: \insertCite{Mitchell1976;textual}{TrenchR}
 }
}
\examples{
  volume_from_length(l     = 0.05,  
                     taxon = "lizard")
  volume_from_length(l     = 0.05,   
                     taxon = "frog")
  volume_from_length(l     = 0.05,   
                     taxon = "sphere")

}
\references{
\insertAllCited{}
}
\seealso{
Other allometric functions: 
\code{\link{mass_from_length}()},
\code{\link{proportion_silhouette_area_shapes}()},
\code{\link{proportion_silhouette_area}()},
\code{\link{surface_area_from_length}()},
\code{\link{surface_area_from_mass}()},
\code{\link{surface_area_from_volume}()}
}
\concept{allometric functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Reynolds_number}
\alias{Reynolds_number}
\title{Reynolds Number}
\usage{
Reynolds_number(u, D, nu)
}
\arguments{
\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{D}{\code{numeric} characteristic dimension (e.g., body diameter) (m)}

\item{nu}{\code{numeric} the kinematic viscosity, ratio of dynamic viscosity to density of the fluid (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}}); can calculate from \code{\link{DRYAIR}} or \code{\link{WETAIR}}.}
}
\value{
\code{numeric} Reynolds number.
}
\description{
The function estimates the Reynolds Number, which describes the dynamic properties of the fluid surrounding the animal as the ratio of internal viscous forces \insertCite{Gates1980}{TrenchR}.
}
\examples{
  Reynolds_number(u  = 1,
                  D  = 0.001,
                  nu = 1.2)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DTRFunctions.R
\name{diurnal_temp_variation_sineexp}
\alias{diurnal_temp_variation_sineexp}
\title{Hourly Temperature Variation assuming Sine and Exponential Components}
\usage{
diurnal_temp_variation_sineexp(
  T_max,
  T_min,
  t,
  t_r,
  t_s,
  alpha = 2.59,
  beta = 1.55,
  gamma = 2.2
)
}
\arguments{
\item{T_max, T_min}{\code{numeric} maximum and minimum daily temperatures (C).}

\item{t}{\code{numeric} time for temperature estimate (hour).}

\item{t_r, t_s}{\code{numeric} times of sunrise and sunset (hour).}

\item{alpha}{\code{numeric} time difference between \code{t_x} (time of maximum temperature) and noon (hour).}

\item{beta}{\code{numeric} time difference between \code{t_x} and sunrise (hour).}

\item{gamma}{\code{numeric} decay parameter for rate of \code{t} change from sunset to \code{t_n} (time of minimum temp).}
}
\value{
\code{numeric} temperature (C) at a specified hour.
}
\description{
The function estimates temperature across hours using a diurnal temperature variation function incorporating sine and exponential components \insertCite{Parton1981}{TrenchR}.
}
\details{
Default \code{alpha}, \code{beta}, and \code{gamma} values are the
  average of 5 North Carolina sites \insertCite{Wann1985}{TrenchR}.
  \cr \cr
  Other \code{alpha}, \code{beta}, and \code{gamma} parameterizations include
  values for Denver, Colorado from \insertCite{Parton1981;textual}{TrenchR}:
  \describe{
    \item{150 cm air temperature}{\code{alpha} = 1.86, \code{beta} = 2.20, \code{gamma} = -0.17}
    \item{10 cm air temperature}{\code{alpha} = 1.52, \code{beta} = 2.00, \code{gamma} = -0.18}
    \item{Soil surface temperature}{\code{alpha} = 0.50, \code{beta} = 1.81, \code{gamma} = 0.49}
    \item{10 cm soil temperature}{\code{alpha} = 0.45, \code{beta} = 2.28, \code{gamma} = 1.83}
  }
}
\examples{
  diurnal_temp_variation_sineexp(T_max = 30, 
                                 T_min = 10, 
                                 t     = 11:15, 
                                 t_r   = 6, 
                                 t_s   = 18)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RadiationFunctions.R
\name{diurnal_radiation_variation}
\alias{diurnal_radiation_variation}
\title{Hourly Solar Radiation}
\usage{
diurnal_radiation_variation(doy, S, hour, lon, lat)
}
\arguments{
\item{doy}{\code{numeric} the day of year.}

\item{S}{\code{numeric} solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> d<sup>-1</sup>}}{\eqn{W m^-2 d^-1}{ASCII}}).}

\item{hour}{\code{numeric} hour (0-24).}

\item{lon}{\code{numeric} longitude (degrees).}

\item{lat}{\code{numeric} latitude (degrees).}
}
\value{
\code{numeric} hourly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
}
\description{
The function estimates hourly solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> hr<sup>-1</sup>}}{\eqn{W m^-2 hr^-1}{ASCII}}) as a function of daily global solar radiation (\ifelse{html}{\out{W m<sup>-2</sup> d<sup>-1</sup>}}{\eqn{W m^-2 d^-1}{ASCII}}). Based on \insertCite{Tham2010;textual}{TrenchR} and \insertCite{AlRawahi2011;textual}{TrenchR}.
}
\examples{
diurnal_radiation_variation(doy    = 112, 
                            S = 8000, 
                            hour   = 12, 
                            lon    = -122.33, 
                            lat    = 47.61)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{airpressure_from_elev}
\alias{airpressure_from_elev}
\title{Air Pressure from Elevation}
\usage{
airpressure_from_elev(elev)
}
\arguments{
\item{elev}{\code{numeric} elevation (meters).}
}
\value{
\code{numeric} air pressure (kPa).
}
\description{
The function estimates air pressure (kPa) as a function of elevation \insertCite{engingeeringtoolbox}{TrenchR}.
}
\examples{
  airpressure_from_elev(elev = 1500)

}
\references{
\insertAllCited{}
}
\seealso{
Other utility functions: 
\code{\link{azimuth_angle}()},
\code{\link{day_of_year}()},
\code{\link{daylength}()},
\code{\link{dec_angle}()},
\code{\link{solar_noon}()},
\code{\link{temperature conversions}},
\code{\link{zenith_angle}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Nusselt_number}
\alias{Nusselt_number}
\title{Nusselt Number}
\usage{
Nusselt_number(H_L, D, K)
}
\arguments{
\item{H_L}{\code{numeric} convective heat transfer coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}).}

\item{D}{\code{numeric} characteristic dimension (e.g., body diameter) (m).}

\item{K}{\code{numeric} thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}}).}
}
\value{
\code{numeric} Nusselt number.
}
\description{
The function estimates the Nusselt Number, which describes dimensionless conductance \insertCite{Gates1980}{TrenchR}.
}
\examples{
  Nusselt_number(H_L = 20,
                 D   = 0.01,
                 K   = 0.5)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qradiation_absorbed}
\alias{Qradiation_absorbed}
\title{Absorbed Solar and Thermal Radiation}
\usage{
Qradiation_absorbed(
  a = 0.9,
  A,
  psa_dir,
  psa_dif,
  psa_ref,
  S_dir,
  S_dif,
  S_ref = NA,
  rho = NA
)
}
\arguments{
\item{a}{\code{numeric} solar absorptivity of animal surface (proportion), default value is for reptiles (0-1).}

\item{A}{\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{psa_dir}{\code{numeric} proportion surface area exposed to direct solar radiation (0-1).}

\item{psa_dif}{\code{numeric} proportion surface area exposed to diffuse solar radiation (0-1).}

\item{psa_ref}{\code{numeric} proportion surface area exposed to reflected solar radiation (0-1).}

\item{S_dir}{\code{numeric} direct solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{S_dif}{\code{numeric} diffuse solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{S_ref}{\code{numeric} reflected solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}), either provided or estimated if surface albedo is provided instead}

\item{rho}{\code{numeric} is surface albedo (proportion), optional (not used) if reflected radiation is provided. Values available in \insertCite{Gates1980}{TrenchR} Table 8.2.}
}
\value{
\code{numeric} solar radiation absorbed (W)
}
\description{
The function estimates solar and thermal radiation (W) absorbed by the surface of an animal following \insertCite{Gates1980}{TrenchR} and \insertCite{Spotila1992}{TrenchR}.
}
\examples{
  Qradiation_absorbed(a       = 0.9,
                      A       = 1,
                      psa_dir = 0.5,
                      psa_dif = 0.5,
                      psa_ref = 0.5,
                      S_dir   = 1000,
                      S_dif   = 200,
                      rho     = 0.5)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllometricFunctions.R
\name{surface_area_from_volume}
\alias{surface_area_from_volume}
\title{Organism Surface Area from Volume}
\usage{
surface_area_from_volume(V, taxon)
}
\arguments{
\item{V}{\code{numeric} vector of volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}). Can be one or more values.}

\item{taxon}{\code{character} taxon of organism, current choices: \code{"lizard"}, \code{"frog"}, \code{"sphere"}.}
}
\value{
\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
}
\description{
The function estimates surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) from volume (\ifelse{html}{\out{m<sup>3</sup>}}{\eqn{m^3}{ASCII}}) for a variety of taxa following \insertCite{Mitchell1976;textual}{TrenchR}.
}
\details{
All models follow (\ifelse{html}{\out{SA = Ka V<sup>2/3</sup>}}{\eqn{SA = Ka * V^{2/3}}{ASCII}}) with surface area and volume in meters.
 \itemize{
  \item Lizards: \insertCite{Norris1965;textual}{TrenchR}:
    \cr 
    \cr \eqn{Ka = 11.0} 
  \item Frogs: \insertCite{Tracy1972;textual}{TrenchR}:
    \cr 
    \cr \eqn{Ka = 11.0} 
  \item Sphere: \insertCite{Mitchell1976;textual}{TrenchR}:
    \cr 
    \cr \eqn{Ka = 4.83} 
 }
}
\examples{
  surface_area_from_volume(V     = 0.001, 
                           taxon = "lizard")
  surface_area_from_volume(V     = 0.001,  
                           taxon = "frog")
  surface_area_from_volume(V     = 0.001,  
                           taxon = "sphere")

}
\references{
\insertAllCited{}
}
\seealso{
Other allometric functions: 
\code{\link{mass_from_length}()},
\code{\link{proportion_silhouette_area_shapes}()},
\code{\link{proportion_silhouette_area}()},
\code{\link{surface_area_from_length}()},
\code{\link{surface_area_from_mass}()},
\code{\link{volume_from_length}()}
}
\concept{allometric functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{zenith_angle}
\alias{zenith_angle}
\title{Zenith Angle}
\usage{
zenith_angle(doy, lat, lon, hour, offset = NA)
}
\arguments{
\item{doy}{\code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.}

\item{lat}{\code{numeric} latitude (decimal degrees).}

\item{lon}{\code{numeric} longitude (decimal degrees).}

\item{hour}{\code{numeric} hour of the day (0-24).}

\item{offset}{\code{numeric} the number of hours to add to UTC (Coordinated Universal Time) to get local time (improves accuracy but not always necessary). Optional. Defaults to NA.}
}
\value{
\code{numeric} zenith angle (degrees)
}
\description{
The function calculates the zenith angle, the location of the sun as an angle (in degrees) measured from vertical \insertCite{Campbell1998}{TrenchR}.
}
\examples{
  zenith_angle(doy  = 112, 
               lat  = 47.61, 
               lon  = -122.33, 
               hour = 12)

}
\references{
\insertAllCited{}
}
\seealso{
Other utility functions: 
\code{\link{airpressure_from_elev}()},
\code{\link{azimuth_angle}()},
\code{\link{day_of_year}()},
\code{\link{daylength}()},
\code{\link{dec_angle}()},
\code{\link{solar_noon}()},
\code{\link{temperature conversions}}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TPCfunctions.R
\name{TPC_beta}
\alias{TPC_beta}
\title{Beta Function Thermal Performance Curve}
\usage{
TPC_beta(T_b, shift = -1, breadth = 0.1, aran = 0, tolerance = 43, skew = 0.7)
}
\arguments{
\item{T_b}{\code{numeric} temperature (C).}

\item{shift}{\code{numeric} mode of the thermal performance curve.}

\item{breadth}{\code{numeric} breadth of the thermal performance curve.}

\item{aran}{\code{numeric} scale performance value. If \code{0}, no scaling; if \code{1}, include a thermodynamic effect on mean performance.}

\item{tolerance}{\code{numeric} maximal breath (C) of the thermal performance curve.}

\item{skew}{\code{numeric} skewness of the thermal performance curve (0-1).}
}
\value{
\code{numeric} performance.
}
\description{
The function constructs a thermal performance curve based on a beta function \insertCite{Asbury2010}{TrenchR}.
}
\examples{
  TPC_beta(T_b       = 0:60, 
           shift     = -1, 
           breadth   = 0.1, 
           aran      = 0, 
           tolerance = 43, 
           skew      = 0.7)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_Limpet.R
\name{Tb_limpet}
\alias{Tb_limpet}
\title{Operative Environmental Temperature of a Limpet}
\usage{
Tb_limpet(T_a, T_r, l, h, S, u, psi, c, position = "anterior")
}
\arguments{
\item{T_a}{\code{numeric} air temperature (C).}

\item{T_r}{\code{numeric} rock surface temperature (C) in the sunlight.}

\item{l}{\code{numeric} limpet length (anterior/posterior axis, m).}

\item{h}{\code{numeric} limpet height (dorsal/ventral axis, m).}

\item{S}{\code{numeric} solar irradiance (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{psi}{\code{numeric} solar zenith angle (degrees). Can be calculated from \code{\link{zenith_angle}} function.}

\item{c}{\code{numeric} fraction of the sky covered by cloud (proportion).}

\item{position}{\code{character} direction of the limpet that is facing upwind. Options are \code{"anterior"}, \code{"posterior"}, and \code{"broadside"}.}
}
\value{
\code{numeric} predicted body (operative environmental) temperature (C).
}
\description{
The function estimates body temperatures (C, operative environmental temperatures) of a limpet based on \insertCite{Denny2006;textual}{TrenchR}.
}
\details{
The original equation uses a finite-difference approach where they divide the rock into series of chunks, and calculate the temperature at each node to derive the conductive heat. For simplification, here it takes the rock temperature as a parameter, and conductive heat is calculated as a product of the area, thermal conductivity of rock and the temperature difference between the rock and the body.
  \cr \cr
  Limpets are simulated as cones following and using solar emissivity values from \insertCite{Campbell1998;textual}{TrenchR}.
  \cr \cr
  The area of the limpet's shell (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) is projected according to the direction at which sunlight strikes the organism \insertCite{Pennell1989}{TrenchR}.
  \cr \cr
  Air conductivity values (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) are calculated following \insertCite{Denny2006;textual}{TrenchR}.
}
\examples{
  Tb_limpet(T_a      = 25, 
            T_r      = 30, 
            l        = 0.0176, 
            h        = 0.0122, 
            S        = 1300, 
            u        = 1, 
            psi      = 30, 
            c        = 1, 
            position = "anterior")

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TempWindProfileFunctions.R
\name{air_temp_profile_neutral}
\alias{air_temp_profile_neutral}
\title{Air Temperature at a Specified Height Under Neutral Conditions}
\usage{
air_temp_profile_neutral(T_r, zr, z0, z, T_s)
}
\arguments{
\item{T_r}{\code{numeric} air temperature (C) at reference height.}

\item{zr}{\code{numeric} initial reference height (m).}

\item{z0}{\code{numeric} surface roughness (m).}

\item{z}{\code{numeric} height to scale to (m).}

\item{T_s}{\code{numeric} surface temperatures (C).}
}
\value{
\code{numeric} air temperature (C).
}
\description{
The function calculates air temperature (C) at a specified height (m) within a boundary layer near the surface.  The velocity profile is the neutral profile described by \insertCite{Sellers1965;textual}{TrenchR}. Function is included as equations (2) and (3) of \insertCite{Porter1973;textual}{TrenchR}.
}
\examples{
  air_temp_profile_neutral(T_r = 20, 
                           zr  = 0.1, 
                           z0  = 0.2, 
                           z   = 0.15, 
                           T_s = 25)


}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{dec_angle}
\alias{dec_angle}
\title{Solar Declination in Radians}
\usage{
dec_angle(doy)
}
\arguments{
\item{doy}{\code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.}
}
\value{
\code{numeric} declination angle (radians).
}
\description{
The function calculates solar declination, which is the angular distance of the sun north or south of the earth’s equator, based on the day of year \insertCite{Campbell1998}{TrenchR}.
}
\examples{
  dec_angle(doy = 112)
  dec_angle(doy = 360)

}
\references{
\insertAllCited{}
}
\seealso{
Other utility functions: 
\code{\link{airpressure_from_elev}()},
\code{\link{azimuth_angle}()},
\code{\link{day_of_year}()},
\code{\link{daylength}()},
\code{\link{solar_noon}()},
\code{\link{temperature conversions}},
\code{\link{zenith_angle}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{free_or_forced_convection}
\alias{free_or_forced_convection}
\title{Determine if Convection is Free or Forced}
\usage{
free_or_forced_convection(Gr, Re)
}
\arguments{
\item{Gr}{\code{numeric} Grashof Number (dimensionless).}

\item{Re}{\code{numeric} Reynolds Number (dimensionless).}
}
\value{
\code{character} \code{"free"}, \code{"forced"}, or \code{"intermediate"}.
}
\description{
The function compares the Grashof and Reynolds numbers to determine whether convection is free or forced \insertCite{Gates1980}{TrenchR}.
}
\examples{
  free_or_forced_convection(Gr = 100,
                            Re = 5)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Tb_salamander_humid}
\alias{Tb_salamander_humid}
\title{Humid Operative Environmental Temperature of a Salamander}
\usage{
Tb_salamander_humid(r_i, r_b, D, T_a, elev, e_a, e_s, Qabs, epsilon = 0.96)
}
\arguments{
\item{r_i}{\code{numeric} internal (skin) resistance (\ifelse{html}{\out{s cm<sup>-1</sup>}}{\eqn{s cm^-1}{ASCII}}).}

\item{r_b}{\code{numeric} boundary layer resistance (\ifelse{html}{\out{s cm<sup>-1</sup>}}{\eqn{s cm^-1}{ASCII}}).}

\item{D}{\code{numeric} body diameter (m); can estimate as diameter = 0.0016*log(mass) + 0.0061 for mass(g).}

\item{T_a}{\code{numeric} ambient temperature (C).}

\item{elev}{\code{numeric} elevation (m).}

\item{e_a}{\code{numeric} actual vapor pressure (kPa).}

\item{e_s}{\code{numeric} saturation vapor pressure (kPa).}

\item{Qabs}{\code{numeric} Solar and thermal radiation absorbed (W).}

\item{epsilon}{\code{numeric} emissivity of salamander skin (proportion).}
}
\value{
\code{numeric} humid operative temperature (C).
}
\description{
The function estimates the humid body temperature (C, operative environmental temperature) using an adaptation of \insertCite{Campbell1998;textual}{TrenchR} described in \insertCite{Riddell2018;textual}{TrenchR}.
}
\examples{
  Tb_salamander_humid(r_i     = 4,
                      r_b     = 1,
                      D       = 0.01,
                      T_a     = 20,
                      elev    = 500,
                      e_a     = 2.0,
                      e_s     = 2.5,
                      Qabs    = 400,
                      epsilon = 0.96)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\author{
Eric Riddell
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qevaporation}
\alias{Qevaporation}
\title{Heat Loss Associated with Evaporative Water Loss}
\usage{
Qevaporation(A, T_b, taxon, e_s = NA, e_a = NA, hp = NA, H = NA, r_i = NA)
}
\arguments{
\item{A}{\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).}

\item{T_b}{\code{numeric} body temperature (K).}

\item{taxon}{\code{character} organism type. Current choices: \code{"lizard"}, \code{"amphibian_wetskin"} (fully wet skin), \code{"amphibian"} (not fully wet skin).}

\item{e_s}{\code{numeric} saturation water vapor density at skin surface (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) (needed if amphibian).}

\item{e_a}{\code{numeric} saturation water vapor density in ambient air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) (needed if amphibian).}

\item{hp}{\code{numeric} relative humidity (0-1) (needed if amphibian).}

\item{H}{\code{numeric} convective heat transfer coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}) (needed if amphibian).}

\item{r_i}{\code{numeric} internal (cutaneous) resistance to vapor transport (\ifelse{html}{\out{s m<sup>-1</sup>}}{\eqn{s m^-1}{ASCII}}) (needed if amphibian).}
}
\value{
\code{numeric} evaporative heat loss (W).
}
\description{
The function estimates heat loss associated with evaporative water loss for an amphibian \insertCite{Spotila1992}{TrenchR} or lizard. The lizard estimation is based on empirical measurements in \insertCite{Porter1973;textual}{TrenchR}).
}
\examples{
  Qevaporation(A      = 0.1,
               T_b    = 293,
               taxon = "amphibian",
               e_s = 0.003,
               e_a = 0.002,
               hp     = 0.5,
               H     = 20,
               r_i   = 50)
  Qevaporation(A     = 0.1,
               T_b   = 293,
               taxon = "lizard")

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OperativeTemperatureFunctions.R
\name{Qnet_Gates}
\alias{Qnet_Gates}
\title{Net Energy Exchange Between an Animal and the Environment}
\usage{
Qnet_Gates(Qabs, Qemit, Qconv, Qcond, Qmet, Qevap)
}
\arguments{
\item{Qabs}{\code{numeric} solar radiation absorbed (W).}

\item{Qemit}{\code{numeric} thermal radiation emitted (W).}

\item{Qconv}{\code{numeric} energy exchange due to convection; Energy exchange from an animal to its surrounding environment (air or water) (W).}

\item{Qcond}{\code{numeric} energy exchange due to conduction; Energy exchange from animal to a surface if they are in contact  (W).}

\item{Qmet}{\code{numeric} energy emitted due to metabolism (W).}

\item{Qevap}{\code{numeric} energy emitted due to evaporative water loss (W).}
}
\value{
\code{numeric} net energy exchange (W).
}
\description{
The function estimates the net energy exchange (W) between an animal and the environment. The function follows \insertCite{Gates1980;textual}{TrenchR} and others.
}
\examples{
Qnet_Gates(Qabs  = 500,
           Qemit = 10, 
           Qconv = 100, 
           Qcond = 100, 
           Qmet  = 10, 
           Qevap = 5)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TsoilFunctions.R
\name{soil_temperature_equation}
\alias{soil_temperature_equation}
\title{Core Function Called to Solve Equation for Soil Temperature}
\usage{
soil_temperature_equation(L, rho_a, c_a, u_inst, z_r, z0, T_inst, T_s)
}
\arguments{
\item{L}{\code{numeric} Monin-Obukhov length, a measure of the instability of heat flow (see \insertCite{Beckman1973;textual}{TrenchR}).}

\item{rho_a}{\code{numeric} density of air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}).}

\item{c_a}{\code{numeric} specific heat of air (\ifelse{html}{\out{J kg<sup>-1</sup> C<sup>-1</sup>}}{\eqn{J kg^-1 C^-1}{ASCII}}).}

\item{u_inst}{\code{numeric} instantaneous wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{z_r}{\code{numeric} reference height (m).}

\item{z0}{\code{numeric} surface roughness (m).}

\item{T_inst}{\code{numeric} instantaneous air temperature (C).}

\item{T_s}{\code{numeric} initial soil surface temperature (C).}
}
\value{
\code{numeric} soil temperature (C).
}
\description{
The function called by \code{\link{soil_temperature_function}} to solve equation for soil temperature from \insertCite{Beckman1973;textual}{TrenchR}.
}
\examples{
  soil_temperature_equation(L      = -10, 
                            rho_a  = 1.177, 
                            c_a    = 1006, 
                            u_inst = 0.3, 
                            z_r    = 1.5, 
                            z0     = 0.02, 
                            T_inst = 8, 
                            T_s    = 20)

}
\references{
\insertAllCited{}
}
\seealso{
Other soil temperature functions: 
\code{\link{soil_conductivity}()},
\code{\link{soil_specific_heat}()},
\code{\link{soil_temperature_function}()},
\code{\link{soil_temperature_integrand}()},
\code{\link{soil_temperature}()}
}
\author{
Joseph Grigg
}
\concept{soil temperature functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TsoilFunctions.R
\name{soil_conductivity}
\alias{soil_conductivity}
\title{Soil Thermal Conductivity}
\usage{
soil_conductivity(x, lambda, g_a)
}
\arguments{
\item{x}{\code{numeric} vector of volume fractions of soil constituents (e.g., clay, quartz, minerals other than quartz, organic matter, water, air). The volume fractions should sum to 1. Note that \code{x} and \code{lambda} values in the example correspond to these soil constituents.}

\item{lambda}{\code{numeric} vector of the thermal conductivities (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) of the soil constituents.}

\item{g_a}{\code{numeric} shape factor on soil particles. The soil particles are assumed to be ellipsoids with axes \code{g_a}, \code{g_b}, and \code{g_c}, where \code{g_a + g_b + g_c = 1} and \code{g_a = g_b}. \insertCite{deVries1952;textual}{TrenchR} suggests \code{g_a = g_b = 0.125}.}
}
\value{
\code{numeric} soil thermal conductivity (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}).
}
\description{
The function estimates soil thermal conductivity (\ifelse{html}{\out{W m<sup>-1</sup> K<sup>-1</sup>}}{\eqn{W m^-1 K^-1}{ASCII}}) using the methods of \insertCite{deVries1963;textual}{TrenchR}.
}
\examples{
  soil_conductivity(x      = c(0.10, 0.40, 0.11, 0.01, 0.2, 0.18), 
                    lambda = c(0.10, 0.40, 0.11, 0.01, 0.2, 0.18), 
                    g_a     = 0.125)
  
}
\references{
\insertAllCited{}
}
\seealso{
Other soil temperature functions: 
\code{\link{soil_specific_heat}()},
\code{\link{soil_temperature_equation}()},
\code{\link{soil_temperature_function}()},
\code{\link{soil_temperature_integrand}()},
\code{\link{soil_temperature}()}
}
\author{
Joseph Grigg
}
\concept{soil temperature functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{boundary_layer_resistance}
\alias{boundary_layer_resistance}
\title{Boundary Layer Resistance}
\usage{
boundary_layer_resistance(T_a, e_s, e_a, elev, D, u = NA)
}
\arguments{
\item{T_a}{\code{numeric} air temperature (K).}

\item{e_s}{\code{numeric} saturation vapor pressure (kPa).}

\item{e_a}{\code{numeric} actual vapor pressure (kPa).}

\item{elev}{\code{numeric} elevation (m).}

\item{D}{\code{numeric} characteristic dimension (e.g., body diameter) (m).}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}), if not provided assume free convection; if provided, use forced convection if appropriate.}
}
\value{
\code{numeric} boundary layer resistance (\ifelse{html}{\out{s cm<sup>-1</sup>}}{\eqn{s cm^-1}{ASCII}}).
}
\description{
The function estimates boundary layer resistance under free convection based on the function in \insertCite{Riddell2018;textual}{TrenchR}.
}
\examples{
  boundary_layer_resistance(T_a  = 293,
                            e_s  = 2.5,
                            e_a  = 2.0,
                            elev = 500,
                            D    = 0.007,
                            u    = 2)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\author{
Eric Riddell
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DRYWETAIR.R
\name{DRYAIR}
\alias{DRYAIR}
\title{Properties of Dry Air}
\usage{
DRYAIR(db, bp = 0, alt = 0)
}
\arguments{
\item{db}{\code{numeric} dry bulb temperature (C).}

\item{bp}{\code{numeric} barometric pressure (Pa).}

\item{alt}{\code{numeric} altitude (m).}
}
\value{
Named \code{list} with elements:
  \describe{
    \item{\code{patmos}}{\code{numeric} standard atmospheric pressure (Pa)}
    \item{\code{density}}{\code{numeric} density (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}})}
    \item{\code{visdyn}}{\code{numeric} dynamic viscosity (\ifelse{html}{\out{kg m<sup>-1</sup> s<sup>-1</sup>}}{\eqn{kg m^-1 s^-1}{ASCII}})}
    \item{\code{viskin}}{\code{numeric} kinematic viscosity (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}})}
    \item{\code{difvpr}}{\code{numeric} diffusivity of water vapor in air (\ifelse{html}{\out{m<sup>2</sup> s<sup>-1</sup>}}{\eqn{m^2 s^-1}{ASCII}})}
    \item{\code{thcond}}{\code{numeric} thermal conductivity (\ifelse{html}{\out{W K<sup>-1</sup> m<sup>-1</sup>}}{\eqn{W K^-1 m^-1}{ASCII}})}
    \item{\code{htovpr}}{\code{numeric} latent heat of vaporization of water (\ifelse{html}{\out{J kg<sup>-1</sup>}}{\eqn{J kg^-1}{ASCII}})}
    \item{\code{tcoeff}}{\code{numeric} temperature coefficient of volume expansion (\ifelse{html}{\out{K<sup>-1</sup>}}{\eqn{K^-1}{ASCII}})}
    \item{\code{ggroup}}{\code{numeric} group of variables in Grashof number (\ifelse{html}{\out{m<sup>-3</sup> K<sup>-1</sup>}}{\eqn{m^-3 K^-1}{ASCII}})}
    \item{\code{bbemit}}{\code{numeric} black body emittance (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}})}
    \item{\code{emtmax}}{\code{numeric} wave length of maximum emittance (m)}
  }
}
\description{
The function calculates several properties of dry air and related characteristics shown as output variables below. The program is based on equations from \insertCite{List1971;textual}{TrenchR} and code implementation from NicheMapR \insertCite{Kearney2017,Kearney2020}{TrenchR}.
  \cr \cr
 The user must supply values for the input variables \code{db}, \code{bp}, and \code{alt}. If \code{alt} is known (-1000 < alt < 20000) but not BP, then set \code{bp = 0}.
}
\examples{
  DRYAIR(db = 30, 
         bp = 100*1000, 
         alt = 0)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RadiationFunctions.R
\name{solar_radiation}
\alias{solar_radiation}
\title{Estimate the Three Components of Solar Radiation (Direct, Diffuse and Reflected)}
\usage{
solar_radiation(doy, psi, tau, elev, rho = 0.7)
}
\arguments{
\item{doy}{\code{numeric} the day of year; \code{\link{day_of_year}}.}

\item{psi}{\code{numeric} zenith angle (radians).}

\item{tau}{\code{numeric} atmospheric transmissivity (proportion), which is ratio of global solar radiation at ground level to extra-terrestrial solar radiation.}

\item{elev}{\code{numeric} elevation (meters).}

\item{rho}{\code{numeric} albedo as a proportion (0-1).}
}
\value{
\code{numeric} radiation components - direct, diffused and reflected (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).
}
\description{
The function estimate direct, diffuse, and reflected components of solar radiation (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}) as a function of day of year using the model in \insertCite{Campbell1998;textual}{TrenchR}.
}
\examples{
  solar_radiation(doy  = 112, 
                  psi  = 1, 
                  tau  = 0.6, 
                  elev = 1500, 
                  rho  = 0.7)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DTRFunctions.R
\name{diurnal_temp_variation_sinesqrt}
\alias{diurnal_temp_variation_sinesqrt}
\title{Hourly Temperature Variation using Sine and Square Root Functions}
\usage{
diurnal_temp_variation_sinesqrt(t, t_r, t_s, T_max, T_min, T_minp)
}
\arguments{
\item{t}{\code{numeric} hour or hours for temperature estimate.}

\item{t_r, t_s}{\code{numeric} sunrise and sunset hours (0-23).}

\item{T_max, T_min}{\code{numeric} maximum and minimum temperatures of current day (C).}

\item{T_minp}{\code{numeric} minimum temperature of following day (C).}
}
\value{
\code{numeric} temperature (C) at a specified hour.
}
\description{
The function estimates temperature for a specified hour using sine and square root functions \insertCite{Cesaraccio2001}{TrenchR}.
}
\examples{
  diurnal_temp_variation_sinesqrt(t      = 11:15, 
                                  t_r    = 6, 
                                  t_s    = 18, 
                                  T_max  = 30, 
                                  T_min  = 10, 
                                  T_minp = 12)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{proportion_diffuse_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biophysmodel_Colias.R
\name{Tb_butterfly}
\alias{Tb_butterfly}
\title{Operative Environmental Temperature of a Butterfly}
\usage{
Tb_butterfly(
  T_a,
  T_g,
  T_sh,
  u,
  S_sdir,
  S_sdif,
  z,
  D,
  delta,
  alpha,
  r_g = 0.3,
  shade = FALSE
)
}
\arguments{
\item{T_a}{\code{numeric} air temperature (C).}

\item{T_g}{\code{numeric} surface temperature (C) in the sunlight.}

\item{T_sh}{\code{numeric} surface temperature (C) in the shade.}

\item{u}{\code{numeric} wind speed (\ifelse{html}{\out{m s<sup>-1</sup>}}{\eqn{m s^-1}{ASCII}}).}

\item{S_sdir}{\code{numeric} direct solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{S_sdif}{\code{numeric} diffuse solar radiation flux (\ifelse{html}{\out{W m<sup>-2</sup>}}{\eqn{W m^-2}{ASCII}}).}

\item{z}{\code{numeric} solar zenith angle (degrees).}

\item{D}{\code{numeric} thoracic diameter (cm).}

\item{delta}{\code{numeric} thoracic fur thickness (mm).}

\item{alpha}{\code{numeric} wing solar absorptivity (proportion). The range for Colias butterflies is 0.4 to 0.7.}

\item{r_g}{\code{numeric} substrate solar reflectivity (proportion). See \insertCite{Kingsolver1983;textual}{TrenchR}.}

\item{shade}{\code{logical} whether body temperature should be calculated in sun (\code{FALSE}) or shade (\code{TRUE}).}
}
\value{
\code{numeric} predicted body (operative environmental) temperature (C).
}
\description{
The function estimates body temperatures (C, operative environmental temperatures) of a butterfly based on \insertCite{Kingsolver1983;textual}{TrenchR} and \insertCite{Buckley2012;textual}{TrenchR}. The function is designed for butterflies that bask with closed wings such as Colias.
}
\details{
Thermal radiative flux is calculated following \insertCite{Gates1980;textual}{TrenchR} based on \insertCite{Swinbank1960;textual}{TrenchR}. \insertCite{Kingsolver1983;textual}{TrenchR} estimates using the Brunt equation with black body sky temperature from \insertCite{Swinbank1963;textual}{TrenchR}.
}
\examples{
  Tb_butterfly(T_a    = 25, 
               T_g    = 25, 
               T_sh   = 20, 
               u      = 0.4, 
               S_sdir = 300, 
               S_sdif = 100, 
               z      = 30, 
               D      = 0.36, 
               delta  = 1.46, 
               alpha  = 0.6, 
               r_g    = 0.3)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{Qmetabolism_from_mass_temp}
\alias{Qmetabolism_from_mass_temp}
\title{Metabolism as a Function of Mass and Body Temperature}
\usage{
Qmetabolism_from_mass_temp(m, T_b, taxon)
}
\arguments{
\item{m}{\code{numeric} mass (grams).}

\item{T_b}{\code{numeric} body temperature (C).}

\item{taxon}{\code{character} organism type. Options: \code{"bird"}, \code{"mammal"}, \code{"reptile"}, \code{"amphibian"}, \code{"invertebrate"}.}
}
\value{
\code{numeric} basal metabolism (W).
}
\description{
The function estimates basal (or resting) metabolic rate (W) as a function of mass (g) and temperature (C). The function is based on empirical data and the metabolic theory of ecology (assumes a 3/4 scaling exponent) \insertCite{Gillooly2001}{TrenchR}.
}
\examples{
  Qmetabolism_from_mass_temp(m     = 100,
                             T_b   = 30,
                             taxon = "reptile")

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{external_resistance_to_water_vapor_transfer}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllometricFunctions.R
\name{surface_area_from_length}
\alias{surface_area_from_length}
\title{Organism Surface Area from Length}
\usage{
surface_area_from_length(l)
}
\arguments{
\item{l}{\code{numeric} length (m).}
}
\value{
\code{numeric} surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}).
}
\description{
This function estimates surface area (\ifelse{html}{\out{m<sup>2</sup>}}{\eqn{m^2}{ASCII}}) from length (m) by approximating the animal's body as a rotational ellipsoid with half the body length as the semi-major axis.
}
\details{
Following \insertCite{Samietz2005;textual}{TrenchR} and \insertCite{Lactin1998;textual}{TrenchR}.
}
\examples{
  surface_area_from_length(l = 0.04)

}
\references{
\insertAllCited{}
}
\seealso{
Other allometric functions: 
\code{\link{mass_from_length}()},
\code{\link{proportion_silhouette_area_shapes}()},
\code{\link{proportion_silhouette_area}()},
\code{\link{surface_area_from_mass}()},
\code{\link{surface_area_from_volume}()},
\code{\link{volume_from_length}()}
}
\concept{allometric functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/energybalance_functions.R
\name{external_resistance_to_water_vapor_transfer}
\alias{external_resistance_to_water_vapor_transfer}
\title{External Resistance to Water Vapor Transfer}
\usage{
external_resistance_to_water_vapor_transfer(H, ecp = 12000)
}
\arguments{
\item{H}{\code{numeric} heat transfer (convection) coefficient (\ifelse{html}{\out{W m<sup>-2</sup> K<sup>-1</sup>}}{\eqn{W m^-2 K^-1}{ASCII}}).}

\item{ecp}{\code{numeric} aggregate parameter (\ifelse{html}{\out{J m<sup>-3</sup> K<sup>-1</sup>}}{\eqn{J m^-3 K^-1}{ASCII}}) that is the product of the density of air (\ifelse{html}{\out{kg m<sup>-3</sup>}}{\eqn{kg m^-3}{ASCII}}) and the specific heat of air at constant pressure (\ifelse{html}{\out{J kg<sup>-1</sup> K<sup>-1</sup>}}{\eqn{J kg^-1 K^-1}{ASCII}}). Default of 12000 (\ifelse{html}{\out{J m<sup>-3</sup> K<sup>-1</sup>}}{\eqn{J m^-3 K^-1}{ASCII}}) commonly assumed.}
}
\value{
\code{numeric} external resistance to water vapor transfer (\ifelse{html}{\out{s m<sup>-1</sup>}}{\eqn{s m^-1}{ASCII}}).
}
\description{
The function estimate external resistance to water vapor transfer using the Lewis rule relating heat and mass transport \insertCite{Spotila1992}{TrenchR}
}
\examples{
  external_resistance_to_water_vapor_transfer(H = 20)

}
\references{
\insertAllCited{}
}
\seealso{
Other biophysical models: 
\code{\link{Grashof_number_Gates}()},
\code{\link{Grashof_number}()},
\code{\link{Nusselt_from_Grashof}()},
\code{\link{Nusselt_from_Reynolds}()},
\code{\link{Nusselt_number}()},
\code{\link{Prandtl_number}()},
\code{\link{Qconduction_animal}()},
\code{\link{Qconduction_substrate}()},
\code{\link{Qconvection}()},
\code{\link{Qemitted_thermal_radiation}()},
\code{\link{Qevaporation}()},
\code{\link{Qmetabolism_from_mass_temp}()},
\code{\link{Qmetabolism_from_mass}()},
\code{\link{Qnet_Gates}()},
\code{\link{Qradiation_absorbed}()},
\code{\link{Qthermal_radiation_absorbed}()},
\code{\link{Reynolds_number}()},
\code{\link{T_sky}()},
\code{\link{Tb_CampbellNorman}()},
\code{\link{Tb_Gates2}()},
\code{\link{Tb_Gates}()},
\code{\link{Tb_butterfly}()},
\code{\link{Tb_grasshopper}()},
\code{\link{Tb_limpetBH}()},
\code{\link{Tb_limpet}()},
\code{\link{Tb_lizard_Fei}()},
\code{\link{Tb_lizard}()},
\code{\link{Tb_mussel}()},
\code{\link{Tb_salamander_humid}()},
\code{\link{Tb_snail}()},
\code{\link{Tbed_mussel}()},
\code{\link{Tsoil}()},
\code{\link{actual_vapor_pressure}()},
\code{\link{boundary_layer_resistance}()},
\code{\link{free_or_forced_convection}()},
\code{\link{heat_transfer_coefficient_approximation}()},
\code{\link{heat_transfer_coefficient_simple}()},
\code{\link{heat_transfer_coefficient}()},
\code{\link{saturation_vapor_pressure}()},
\code{\link{saturation_water_vapor_pressure}()}
}
\concept{biophysical models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{azimuth_angle}
\alias{azimuth_angle}
\title{Azimuth Angle}
\usage{
azimuth_angle(doy, lat, lon, hour, offset = NA)
}
\arguments{
\item{doy}{\code{numeric} day of year (1-366). This can be obtained from standard date via \code{\link{day_of_year}}.}

\item{lat}{\code{numeric} latitude (decimal degrees).}

\item{lon}{\code{numeric} longitude (decimal degrees).}

\item{hour}{\code{numeric} hour of the day.}

\item{offset}{\code{numeric} number of hours to add to UTC (Coordinated Universal Time) to get local time (to improve accuracy but not always necessary). Optional. Defaults to NA.}
}
\value{
\code{numeric} azimuth angle (degrees).
}
\description{
The function calculates the azimuth angle, the angle (degrees) from which the sunlight is coming measured from true north or south measured in the horizontal plane. The azimuth angle is measured with respect to due south, increasing in the counter clockwise direction so 90 degrees is east \insertCite{Campbell1998}{TrenchR}.
}
\examples{
  azimuth_angle(doy    = 112, 
                lat    = 47.61, 
                lon    = -122.33, 
                hour   = 12, 
                offset = -8)

}
\references{
\insertAllCited{}
}
\seealso{
Other utility functions: 
\code{\link{airpressure_from_elev}()},
\code{\link{day_of_year}()},
\code{\link{daylength}()},
\code{\link{dec_angle}()},
\code{\link{solar_noon}()},
\code{\link{temperature conversions}},
\code{\link{zenith_angle}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RadiationPartitioningFunctions.R
\name{proportion_diffuse_solar_radiation}
\alias{proportion_diffuse_solar_radiation}
\title{Ratio of Diffuse to Direct Solar Radiation}
\usage{
proportion_diffuse_solar_radiation(psi, p_a, rho)
}
\arguments{
\item{psi}{\code{numeric} Zenith angle of the sun (degrees).}

\item{p_a}{\code{numeric} Atmospheric pressure (kPa).}

\item{rho}{\code{numeric} albedo of the substrate (fraction of 1).}
}
\value{
\code{numeric} diffuse fraction.
}
\description{
The function estimates the ratio of diffuse to direct solar radiation based on the approximation of the SOLRAD model \insertCite{McCullough1971}{TrenchR} described in \insertCite{Tracy1983;textual}{TrenchR}.
}
\examples{
  proportion_diffuse_solar_radiation(psi = 60, 
                                     p_a = 86.1, 
                                     rho   = 0.25)

}
\references{
\insertAllCited{}
}
\seealso{
Other microclimate functions: 
\code{\link{air_temp_profile_neutral}()},
\code{\link{air_temp_profile_segment}()},
\code{\link{air_temp_profile}()},
\code{\link{degree_days}()},
\code{\link{direct_solar_radiation}()},
\code{\link{diurnal_radiation_variation}()},
\code{\link{diurnal_temp_variation_sineexp}()},
\code{\link{diurnal_temp_variation_sinesqrt}()},
\code{\link{diurnal_temp_variation_sine}()},
\code{\link{monthly_solar_radiation}()},
\code{\link{partition_solar_radiation}()},
\code{\link{solar_radiation}()},
\code{\link{surface_roughness}()},
\code{\link{wind_speed_profile_neutral}()},
\code{\link{wind_speed_profile_segment}()}
}
\concept{microclimate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UtilityFunctions.R
\name{temperature conversions}
\alias{temperature conversions}
\alias{fahrenheit_to_kelvin}
\alias{kelvin_to_celsius}
\alias{celsius_to_kelvin}
\alias{fahrenheit_to_celsius}
\title{Convert Among Temperature Scales}
\usage{
fahrenheit_to_kelvin(temperature)

kelvin_to_celsius(temperature)

celsius_to_kelvin(temperature)

fahrenheit_to_celsius(temperature)
}
\arguments{
\item{temperature}{\code{numeric} temperature (Celsius, Fahrenheit, or Kelvin).}
}
\value{
\code{numeric} temperature (Celsius, Fahrenheit, or Kelvin).
}
\description{
The function converts temperatures among Celsius, Fahrenheit, and Kelvin \insertCite{Blischak}{TrenchR}.
}
\examples{
  kelvin_to_celsius(temperature = 270)
  fahrenheit_to_kelvin(temperature = 85)
  fahrenheit_to_celsius(temperature = 85)
  celsius_to_kelvin(temperature = -10)

}
\references{
\insertAllCited{}
}
\seealso{
Other utility functions: 
\code{\link{airpressure_from_elev}()},
\code{\link{azimuth_angle}()},
\code{\link{day_of_year}()},
\code{\link{daylength}()},
\code{\link{dec_angle}()},
\code{\link{solar_noon}()},
\code{\link{zenith_angle}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllometricFunctions.R
\name{proportion_silhouette_area_shapes}
\alias{proportion_silhouette_area_shapes}
\title{Organism Silhouette Area using Shape Approximations}
\usage{
proportion_silhouette_area_shapes(shape, theta, h, d)
}
\arguments{
\item{shape}{\code{character} Shape to use to approximate an organism. Shapes are assumed to be prolate or have the longest axis parallel with the ground. Current choices are \code{"spheroid"}, \code{"cylinder flat ends"}, and \code{"cylinder hemisphere ends"}.}

\item{theta}{\code{numeric} angle between the solar beam and the longitudinal axis (degrees).}

\item{h}{\code{numeric} height (long axis in m). Cross section length for spheroid.}

\item{d}{\code{numeric} diameter (short axis in m). Cross section length for spheroid.}
}
\value{
\code{numeric} silhouette area as a proportion.
}
\description{
The function estimates the projected (silhouette) area as a portion of the surface area of the organism. The function estimates the projected area as a function of the dimensions and the angle between the solar beam and the longitudinal axis of the solid, using Figure 11.6 in \insertCite{Campbell1998;textual}{TrenchR}. The function is useful for estimating absorbed solar radiation.
}
\examples{
  proportion_silhouette_area_shapes(shape = "spheroid", 
                                    theta = 60,  
                                    h     = 0.01,  
                                    d     = 0.001)
  proportion_silhouette_area_shapes(shape = "cylinder flat ends",  
                                    theta = 60,  
                                    h     = 0.01, 
                                    d     = 0.001)
  proportion_silhouette_area_shapes(shape = "cylinder hemisphere ends",  
                                    theta = 60,  
                                    h     = 0.01, 
                                    d     = 0.001)

}
\references{
\insertAllCited{}
}
\seealso{
Other allometric functions: 
\code{\link{mass_from_length}()},
\code{\link{proportion_silhouette_area}()},
\code{\link{surface_area_from_length}()},
\code{\link{surface_area_from_mass}()},
\code{\link{surface_area_from_volume}()},
\code{\link{volume_from_length}()}
}
\concept{allometric functions}
